/** { ttc_interrupt_interface.h *************************************************
 *
 *                           The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Interface layer between high- and low-level driver for INTERRUPT device.
 *
 *  *_interface_*() functions may be redirected to low-level implementations in
 *  two ways:
 *
 *  1) Macro Definition
 *     This applies if only one architecture can be available during runtime.
 *     One example is a GPIO driver for current microcontroller architecture.
 *     Only one type of microcontroller can be available during runtime.
 *
 *  2) Selector Function
 *     Applies if more than one architecture can be availabe during runtime.
 *     A function will select the corresponding low-level driver via a switch-,
 *     if-else construct of via a vector table.
 *     An example can be a display driver. It is possible to connect more than
 *     one different displays to a microcontroller. So every interface function has
 *     to choose, which low-level driver should be used.
 *
 *  Adding a new low-level implementation requires changes in this file.
 *
 *  Created from template ttc_device_interface.h revision 20 at 20140224 21:37:20 UTC
 *
 *  Authors: Gregor Rebel
 *
}*/

#ifndef TTC_INTERRUPT_INTERFACE_H
#define TTC_INTERRUPT_INTERFACE_H

//{ Includes *************************************************************

#include "../ttc_basic.h"
//{ BEGIN_ARCHITECTURE_INCLUDES (DO NOT DELETE THIS LINE!)

#ifdef EXTENSION_interrupt_stm32w1xx
    #include "../interrupt/interrupt_stm32w1xx.h"
#endif
#ifdef EXTENSION_interrupt_stm32f1xx
    #include "../interrupt/interrupt_stm32f1xx.h"  // low-level driver for interrupt devices on stm32f1xx architecture
#endif
#ifdef EXTENSION_interrupt_stm32l1xx
    #include "../interrupt/interrupt_stm32l1xx.h"  // low-level driver for interrupt devices on stm32l1xx architecture
#endif
#ifdef EXTENSION_interrupt_cortexm3
    #include "../interrupt/interrupt_cortexm3.h"  // low-level driver for interrupt devices on cortexm3 architecture
#endif
#ifdef EXTENSION_interrupt_stm32f30x
    #include "../interrupt/interrupt_stm32f30x.h"  // low-level driver for interrupt devices on stm32f30x architecture
#endif
//InsertArchitectureIncludes above (DO NOT DELETE THIS LINE!)

//} END_ARCHITECTURE_INCLUDES (DO NOT DELETE THIS LINE!)

#include "../ttc_interrupt_types.h"
//? #include "../ttc_gpio.h"
//? #include "../ttc_usart.h"

//} Includes
/** static declaration of interface between high- and low-level implementation ********{
 *
 * For each _driver_interrupt_*() declaration in ../ttc_interrupt.h, a corresponding
 * interface part is required here. The interface will forward the declaration either
 * to a low-level implementation or a default implementation in ttc_interrupt_interface.c .
 * A warning is generated for each missing driver implementation.
 *
 * Note: This code is generated automatically. Check ../ttc_interrupt.h for prototypes of low-level driver functions!
 *
 */
void ttc_interrupt_interface_prepare();

// lines below are automatically generated by ./create_DeviceDriver.pl interrupt UPDATE

#ifdef EXTENSION_ttc_gpio //{

    void ttc_interrupt_interface_enable_gpio( t_ttc_interrupt_config_gpio* Config_ISRs ); //{  macro _driver_interrupt_interface_enable_gpio(Config_ISRs)
    #ifdef ttc_driver_interrupt_enable_gpio
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_enable_gpio(Config_ISRs) ttc_driver_interrupt_enable_gpio(Config_ISRs)
    #else
        #define _driver_interrupt_enable_gpio(Config_ISRs) ttc_interrupt_interface_enable_gpio(Config_ISRs)
    #endif
    //}
    e_ttc_interrupt_errorcode ttc_interrupt_interface_init_gpio( t_ttc_interrupt_config_gpio* Config_ISRs ); //{  macro _driver_interrupt_interface_init_gpio(Config_ISRs)
    #ifdef ttc_driver_interrupt_init_gpio
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_init_gpio(Config_ISRs) ttc_driver_interrupt_init_gpio(Config_ISRs)
    #else
        #define _driver_interrupt_init_gpio(Config_ISRs) ttc_interrupt_interface_init_gpio(Config_ISRs)
    #endif
    //}
    void ttc_interrupt_interface_deinit_gpio( t_ttc_interrupt_config_gpio* Config_ISRs ); //{  macro _driver_interrupt_interface_deinit_gpio(Config_ISRs)
    #ifdef ttc_driver_interrupt_deinit_gpio
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_deinit_gpio(Config_ISRs) ttc_driver_interrupt_deinit_gpio(Config_ISRs)
    #else
        #define _driver_interrupt_deinit_gpio(Config_ISRs) ttc_interrupt_interface_deinit_gpio(Config_ISRs)
    #endif
    //}
    t_u8 ttc_interrupt_interface_gpio_calc_config_index( t_base PhysicalIndex ); //{
    #ifdef ttc_driver_interrupt_gpio_calc_config_index
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_gpio_calc_config_index(PhysicalIndex) ttc_driver_interrupt_gpio_calc_config_index(PhysicalIndex)
    #else
        #define _driver_interrupt_gpio_calc_config_index(PhysicalIndex) ttc_interrupt_interface_gpio_calc_config_index(PhysicalIndex)
    #endif
    //}
    void ttc_interrupt_interface_deinit_all_gpio(); //{  macro _driver_interrupt_interface_deinit_all_gpio()
    #ifdef ttc_driver_interrupt_deinit_all_gpio
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_deinit_all_gpio() ttc_driver_interrupt_deinit_all_gpio()
    #else
        #define _driver_interrupt_deinit_all_gpio() ttc_interrupt_interface_deinit_all_gpio()
    #endif
    //}
    void ttc_interrupt_interface_disable_gpio( t_ttc_interrupt_config_gpio* Config_ISRs ); //{  macro _driver_interrupt_interface_disable_gpio(Config_ISRs)
    #ifdef ttc_driver_interrupt_disable_gpio
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_disable_gpio(Config_ISRs) ttc_driver_interrupt_disable_gpio(Config_ISRs)
    #else
        #define _driver_interrupt_disable_gpio(Config_ISRs) ttc_interrupt_interface_disable_gpio(Config_ISRs)
    #endif
    //}
    t_bool ttc_interrupt_interface_is_enablight_gpio( t_ttc_interrupt_config_gpio* Config_ISRs ); //{  macro _driver_interrupt_interface_is_enablight_gpio(Config_ISRs)
    #ifdef ttc_driver_interrupt_is_enablight_gpio
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_is_enablight_gpio ttc_driver_interrupt_is_enablight_gpio
    #else
        #define _driver_interrupt_is_enablight_gpio ttc_interrupt_interface_is_enablight_gpio
    #endif
    //}
    void ttc_interrupt_interface_generate_gpio( t_ttc_interrupt_config_gpio* Config_ISRs ); //{  macro _driver_interrupt_interface_generate_gpio(Handle)
    #ifdef ttc_driver_interrupt_generate_gpio
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_generate_gpio(Config_ISRs) ttc_driver_interrupt_generate_gpio(Config_ISRs)
    #else
        #define _driver_interrupt_generate_gpio(Config_ISRs) ttc_interrupt_interface_generate_gpio(Config_ISRs)
    #endif
    //}

#endif //}

#ifdef EXTENSION_ttc_timer

    e_ttc_interrupt_errorcode ttc_interrupt_interface_init_timer( t_ttc_interrupt_config_timer* Config_ISRs ); //{  macro _driver_interrupt_interface_init_timer(Config_ISRs)
    #ifdef ttc_driver_interrupt_init_timer
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_init_timer(Config_ISRs) ttc_driver_interrupt_init_timer(Config_ISRs)
    #else
        #define _driver_interrupt_init_timer(Config_ISRs) ttc_interrupt_interface_init_timer(Config_ISRs)
    #endif
    //}
    void ttc_interrupt_interface_enable_timer( t_ttc_interrupt_config_timer* Config_ISRs ); //{  macro _driver_interrupt_interface_enable_timer(Config_ISRs)
    #ifdef ttc_driver_interrupt_enable_timer
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_enable_timer(Config_ISRs) ttc_driver_interrupt_enable_timer(Config_ISRs)
    #else
        #define _driver_interrupt_enable_timer(Config_ISRs) ttc_interrupt_interface_enable_timer(Config_ISRs)
    #endif
    //}
    void ttc_interrupt_interface_disable_timer( t_ttc_interrupt_config_timer* Config_ISRs ); //{  macro _driver_interrupt_interface_disable_timer(Config_ISRs)
    #ifdef ttc_driver_interrupt_disable_timer
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_disable_timer(Config_ISRs) ttc_driver_interrupt_disable_timer(Config_ISRs)
    #else
        #define _driver_interrupt_disable_timer(Config_ISRs) ttc_interrupt_interface_disable_timer(Config_ISRs)
    #endif
    //}
    t_bool ttc_interrupt_interface_is_enabled_timer( t_ttc_interrupt_config_timer* Config_ISRs, e_ttc_interrupt_type Type ); //{  macro _driver_interrupt_interface_is_enabled_timer(Config_ISRs, Type)
    #ifdef ttc_driver_interrupt_is_enabled_timer
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_is_enabled_timer ttc_driver_interrupt_is_enabled_timer
    #else
        #define _driver_interrupt_is_enabled_timer ttc_interrupt_interface_is_enabled_timer
    #endif
    //}
    void ttc_interrupt_interface_deinit_timer( t_ttc_interrupt_config_timer* Config_ISRs ); //{  macro _driver_interrupt_interface_deinit_timer(Config_ISRs)
    #ifdef ttc_driver_interrupt_deinit_timer
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_deinit_timer(Config_ISRs) ttc_driver_interrupt_deinit_timer(Config_ISRs)
    #else
        #define _driver_interrupt_deinit_timer(Config_ISRs) ttc_interrupt_interface_deinit_timer(Config_ISRs)
    #endif
    //}
    void ttc_interrupt_interface_deinit_all_timer(); //{  macro _driver_interrupt_interface_deinit_all_timer()
    #ifdef ttc_driver_interrupt_deinit_all_timer
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_deinit_all_timer() ttc_driver_interrupt_deinit_all_timer()
    #else
        #define _driver_interrupt_deinit_all_timer() ttc_interrupt_interface_deinit_all_timer()
    #endif
    //}

#endif
#ifdef EXTENSION_ttc_usart

    e_ttc_interrupt_errorcode ttc_interrupt_interface_init_usart( t_ttc_interrupt_config_usart* Config_ISRs, e_ttc_interrupt_type Type );  //{  macro _driver_interrupt_interface_init_usart(Config_ISRs)
    #ifdef ttc_driver_interrupt_init_usart
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_init_usart(Config_ISRs, Type) ttc_driver_interrupt_init_usart(Config_ISRs, Type)
    #else
        #define _driver_interrupt_init_usart(Config_ISRs, Type) ttc_interrupt_interface_init_usart(Config_ISRs, Type)
    #endif
    //}
    void ttc_interrupt_interface_enable_usart( t_ttc_interrupt_config_usart* Config_ISRs, e_ttc_interrupt_type Type ); //{  macro _driver_interrupt_interface_enable_usart(Config_ISRs)
    #ifdef ttc_driver_interrupt_enable_usart
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_enable_usart(Config_ISRs, Type) ttc_driver_interrupt_enable_usart(Config_ISRs, Type)
    #else
        #define _driver_interrupt_enable_usart(Config_ISRs, Type) ttc_interrupt_interface_enable_usart(Config_ISRs, Type)
    #endif
    //}
    void ttc_interrupt_interface_deinit_usart( t_ttc_interrupt_config_usart* Config_ISRs ); //{  macro _driver_interrupt_interface_deinit_usart(Config_ISRs)
    #ifdef ttc_driver_interrupt_deinit_usart
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_deinit_usart(Config_ISRs) ttc_driver_interrupt_deinit_usart(Config_ISRs)
    #else
        #define _driver_interrupt_deinit_usart(Config_ISRs) ttc_interrupt_interface_deinit_usart(Config_ISRs)
    #endif
    //}
    void ttc_interrupt_interface_deinit_all_usart(); //{  macro _driver_interrupt_interface_deinit_all_usart()
    #ifdef ttc_driver_interrupt_deinit_all_usart
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_deinit_all_usart() ttc_driver_interrupt_deinit_all_usart()
    #else
        #define _driver_interrupt_deinit_all_usart() ttc_interrupt_interface_deinit_all_usart()
    #endif
    //}
    void ttc_interrupt_interface_disable_usart( t_ttc_interrupt_config_usart* Config_ISRs, e_ttc_interrupt_type Type ); //{  macro _driver_interrupt_interface_disable_usart(Config_ISRs)
    #ifdef ttc_driver_interrupt_disable_usart
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_disable_usart(Config_ISRs, Type) ttc_driver_interrupt_disable_usart(Config_ISRs, Type)
    #else
        #define _driver_interrupt_disable_usart(Config_ISRs, Type) ttc_interrupt_interface_disable_usart(Config_ISRs, Type)
    #endif
    //}
    t_bool ttc_interrupt_interface_is_enabled_usart( t_ttc_interrupt_config_usart* Config_ISRs, e_ttc_interrupt_type Type ); //{  macro _driver_interrupt_interface_is_enabled_usart(Config_ISRs)
    #ifdef ttc_driver_interrupt_is_enabled_usart
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_is_enabled_usart(Config_ISRs, Type) ttc_driver_interrupt_is_enabled_usart(Config_ISRs, Type)
    #else
        #define _driver_interrupt_is_enabled_usart(Config_ISRs, Type) ttc_interrupt_interface_is_enabled_usart(Config_ISRs, Type)
    #endif
    //}
#endif
#ifdef EXTENSION_ttc_spi

    void ttc_interrupt_interface_enable_spi( t_ttc_interrupt_config_spi* Config_ISRs ); //{  macro _driver_interrupt_interface_enable_spi(Config_ISRs)
    #ifdef ttc_driver_interrupt_enable_spi
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_enable_spi(Config_ISRs) ttc_driver_interrupt_enable_spi(Config_ISRs)
    #else
        #define _driver_interrupt_enable_spi(Config_ISRs) ttc_interrupt_interface_enable_spi(Config_ISRs)
    #endif
    //}
    e_ttc_interrupt_errorcode ttc_interrupt_interface_init_spi( t_ttc_interrupt_config_spi* Config_ISRs ); //{  macro _driver_interrupt_interface_init_spi(Config_ISRs)
    #ifdef ttc_driver_interrupt_init_spi
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_init_spi(Config_ISRs) ttc_driver_interrupt_init_spi(Config_ISRs)
    #else
        #define _driver_interrupt_init_spi(Config_ISRs) ttc_interrupt_interface_init_spi(Config_ISRs)
    #endif
    //}
    void ttc_interrupt_interface_deinit_all_spi(); //{  macro _driver_interrupt_interface_deinit_all_spi()
    #ifdef ttc_driver_interrupt_deinit_all_spi
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_deinit_all_spi() ttc_driver_interrupt_deinit_all_spi()
    #else
        #define _driver_interrupt_deinit_all_spi() ttc_interrupt_interface_deinit_all_spi()
    #endif
    //}
    void ttc_interrupt_interface_deinit_spi( t_ttc_interrupt_config_spi* Config_ISRs ); //{  macro _driver_interrupt_interface_deinit_spi(Config_ISRs)
    #ifdef ttc_driver_interrupt_deinit_spi
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_deinit_spi(Config_ISRs) ttc_driver_interrupt_deinit_spi(Config_ISRs)
    #else
        #define _driver_interrupt_deinit_spi(Config_ISRs) ttc_interrupt_interface_deinit_spi(Config_ISRs)
    #endif
    //}
    void ttc_interrupt_interface_disable_spi( t_ttc_interrupt_config_spi* Config_ISRs ); //{  macro _driver_interrupt_interface_disable_spi(Config_ISRs)
    #ifdef ttc_driver_interrupt_disable_spi
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_disable_spi(Config_ISRs) ttc_driver_interrupt_disable_spi(Config_ISRs)
    #else
        #define _driver_interrupt_disable_spi(Config_ISRs) ttc_interrupt_interface_disable_spi(Config_ISRs)
    #endif
    //}
    t_bool ttc_interrupt_interface_is_enabled_spi( t_ttc_interrupt_config_spi* Config_ISRs, e_ttc_interrupt_type Type ); //{  macro _driver_interrupt_interface_is_enabled_spi(Config_ISRs, Type)
    #ifdef ttc_driver_interrupt_is_enabled_spi
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_is_enabled_spi ttc_driver_interrupt_is_enabled_spi
    #else
        #define _driver_interrupt_is_enabled_spi ttc_interrupt_interface_is_enabled_spi
    #endif
    //}

#endif
#ifdef EXTENSION_ttc_i2c

    e_ttc_interrupt_errorcode ttc_interrupt_interface_init_i2c( t_ttc_interrupt_config_i2c* Config_ISRs ); //{  macro _driver_interrupt_interface_init_i2c(Config_ISRs)
    #ifdef ttc_driver_interrupt_init_i2c
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_init_i2c(Config_ISRs) ttc_driver_interrupt_init_i2c(Config_ISRs)
    #else
        #define _driver_interrupt_init_i2c(Config_ISRs) ttc_interrupt_interface_init_i2c(Config_ISRs)
    #endif
    //}
    void ttc_interrupt_interface_enable_i2c( t_ttc_interrupt_config_i2c* Config_ISRs ); //{  macro _driver_interrupt_interface_enable_i2c(Config_ISRs)
    #ifdef ttc_driver_interrupt_enable_i2c
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_enable_i2c(Config_ISRs) ttc_driver_interrupt_enable_i2c(Config_ISRs)
    #else
        #define _driver_interrupt_enable_i2c(Config_ISRs) ttc_interrupt_interface_enable_i2c(Config_ISRs)
    #endif
    //}
    void ttc_interrupt_interface_deinit_i2c( t_ttc_interrupt_config_i2c* Config_ISRs ); //{  macro _driver_interrupt_interface_deinit_i2c(Config_ISRs)
    #ifdef ttc_driver_interrupt_deinit_i2c
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_deinit_i2c(Config_ISRs) ttc_driver_interrupt_deinit_i2c(Config_ISRs)
    #else
        #define _driver_interrupt_deinit_i2c(Config_ISRs) ttc_interrupt_interface_deinit_i2c(Config_ISRs)
    #endif
    //}
    void ttc_interrupt_interface_disable_i2c( t_ttc_interrupt_config_i2c* Config_ISRs ); //{  macro _driver_interrupt_interface_disable_i2c(Config_ISRs)
    #ifdef ttc_driver_interrupt_disable_i2c
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_disable_i2c(Config_ISRs) ttc_driver_interrupt_disable_i2c(Config_ISRs)
    #else
        #define _driver_interrupt_disable_i2c(Config_ISRs) ttc_interrupt_interface_disable_i2c(Config_ISRs)
    #endif
    //}
    t_bool ttc_interrupt_interface_is_enabled_i2c( t_ttc_interrupt_config_i2c* Config_ISRs, e_ttc_interrupt_type Type ); //{  macro _driver_interrupt_interface_is_enabled_i2c(Config_ISRs, Type)
    #ifdef ttc_driver_interrupt_is_enabled_i2c
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_is_enabled_i2c ttc_driver_interrupt_is_enabled_i2c
    #else
        #define _driver_interrupt_is_enabled_i2c ttc_interrupt_interface_is_enabled_i2c
    #endif
    //}
    void ttc_interrupt_interface_deinit_all_i2c(); //{  macro _driver_interrupt_interface_deinit_all_i2c()
    #ifdef ttc_driver_interrupt_deinit_all_i2c
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_deinit_all_i2c() ttc_driver_interrupt_deinit_all_i2c()
    #else
        #define _driver_interrupt_deinit_all_i2c() ttc_interrupt_interface_deinit_all_i2c()
    #endif
    //}

#endif
#ifdef EXTENSION_ttc_can

    void ttc_interrupt_interface_enable_can( t_ttc_interrupt_config_can* Config_ISRs ); //{  macro _driver_interrupt_interface_enable_can(Config_ISRs)
    #ifdef ttc_driver_interrupt_enable_can
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_enable_can(Config_ISRs) ttc_driver_interrupt_enable_can(Config_ISRs)
    #else
        #define _driver_interrupt_enable_can(Config_ISRs) ttc_interrupt_interface_enable_can(Config_ISRs)
    #endif
    //}
    e_ttc_interrupt_errorcode ttc_interrupt_interface_init_can( t_ttc_interrupt_config_can* Config_ISRs ); //{  macro _driver_interrupt_interface_init_can(Config_ISRs)
    #ifdef ttc_driver_interrupt_init_can
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_init_can(Config_ISRs) ttc_driver_interrupt_init_can(Config_ISRs)
    #else
        #define _driver_interrupt_init_can(Config_ISRs) ttc_interrupt_interface_init_can(Config_ISRs)
    #endif
    //}
    void ttc_interrupt_interface_deinit_all_can(); //{  macro _driver_interrupt_interface_deinit_all_can()
    #ifdef ttc_driver_interrupt_deinit_all_can
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_deinit_all_can() ttc_driver_interrupt_deinit_all_can()
    #else
        #define _driver_interrupt_deinit_all_can() ttc_interrupt_interface_deinit_all_can()
    #endif
    //}
    void ttc_interrupt_interface_deinit_can( t_ttc_interrupt_config_can* Config_ISRs ); //{  macro _driver_interrupt_interface_deinit_can(Config_ISRs)
    #ifdef ttc_driver_interrupt_deinit_can
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_deinit_can(Config_ISRs) ttc_driver_interrupt_deinit_can(Config_ISRs)
    #else
        #define _driver_interrupt_deinit_can(Config_ISRs) ttc_interrupt_interface_deinit_can(Config_ISRs)
    #endif
    //}
    void ttc_interrupt_interface_disable_can( t_ttc_interrupt_config_can* Config_ISRs ); //{  macro _driver_interrupt_interface_disable_can(Config_ISRs)
    #ifdef ttc_driver_interrupt_disable_can
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_disable_can(Config_ISRs) ttc_driver_interrupt_disable_can(Config_ISRs)
    #else
        #define _driver_interrupt_disable_can(Config_ISRs) ttc_interrupt_interface_disable_can(Config_ISRs)
    #endif
    //}
    t_bool ttc_interrupt_interface_is_enabled_can( t_ttc_interrupt_config_can* Config_ISRs, e_ttc_interrupt_type Type ); //{  macro _driver_interrupt_interface_is_enabled_can(Config_ISRs, Type)
    #ifdef ttc_driver_interrupt_is_enabled_can
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_is_enabled_can ttc_driver_interrupt_is_enabled_can
    #else
        #define _driver_interrupt_is_enabled_can ttc_interrupt_interface_is_enabled_can
    #endif
    //}

#endif

// void ttc_interrupt_interface_all_disable(); //{  macro _driver_interrupt_interface_all_disable()
#ifdef ttc_driver_interrupt_all_disable
    // enable following line to forward interface function as a macro definition
    #define _driver_interrupt_all_disable() ttc_driver_interrupt_all_disable()
#else
    #define _driver_interrupt_all_disable() ttc_interrupt_interface_all_disable()
#endif
//}
// void ttc_interrupt_interface_all_enable(); //{  macro _driver_interrupt_interface_all_enable()
#ifdef ttc_driver_interrupt_all_enable
    // enable following line to forward interface function as a macro definition
    #define _driver_interrupt_all_enable() ttc_driver_interrupt_all_enable()
#else
    #define _driver_interrupt_all_enable() ttc_interrupt_interface_all_enable()
#endif



#ifdef EXTENSION_ttc_rtc

    e_ttc_interrupt_errorcode ttc_interrupt_interface_init_rtc( t_ttc_interrupt_config_rtc* Config_ISRs ); //{  macro _driver_interrupt_interface_init_rtc(Config_ISRs)
    #ifdef ttc_driver_interrupt_init_rtc
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_init_rtc(Config_ISRs) ttc_driver_interrupt_init_rtc(Config_ISRs)
    #else
        #define _driver_interrupt_init_rtc(Config_ISRs) ttc_interrupt_interface_init_rtc(Config_ISRs)
    #endif
    //}
    void ttc_interrupt_interface_enable_rtc( t_ttc_interrupt_config_rtc* Config_ISRs ); //{  macro _driver_interrupt_interface_enable_rtc(Config_ISRs)
    #ifdef ttc_driver_interrupt_enable_rtc
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_enable_rtc(Config_ISRs) ttc_driver_interrupt_enable_rtc(Config_ISRs)
    #else
        #define _driver_interrupt_enable_rtc(Config_ISRs) ttc_interrupt_interface_enable_rtc(Config_ISRs)
    #endif
    //}
    void ttc_interrupt_interface_deinit_rtc( t_ttc_interrupt_config_rtc* Config_ISRs ); //{  macro _driver_interrupt_interface_deinit_rtc(Config_ISRs)
    #ifdef ttc_driver_interrupt_deinit_rtc
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_deinit_rtc(Config_ISRs) ttc_driver_interrupt_deinit_rtc(Config_ISRs)
    #else
        #define _driver_interrupt_deinit_rtc(Config_ISRs) ttc_interrupt_interface_deinit_rtc(Config_ISRs)
    #endif
    //}
    void ttc_interrupt_interface_deinit_all_rtc(); //{  macro _driver_interrupt_interface_deinit_all_rtc()
    #ifdef ttc_driver_interrupt_deinit_all_rtc
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_deinit_all_rtc() ttc_driver_interrupt_deinit_all_rtc()
    #else
        #define _driver_interrupt_deinit_all_rtc() ttc_interrupt_interface_deinit_all_rtc()
    #endif
    //}
    void ttc_interrupt_interface_disable_rtc( t_ttc_interrupt_config_rtc* Config_ISRs ); //{  macro _driver_interrupt_interface_disable_rtc(Config_ISRs)
    #ifdef ttc_driver_interrupt_disable_rtc
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_disable_rtc(Config_ISRs) ttc_driver_interrupt_disable_rtc(Config_ISRs)
    #else
        #define _driver_interrupt_disable_rtc(Config_ISRs) ttc_interrupt_interface_disable_rtc(Config_ISRs)
    #endif
    //}
    t_bool ttc_interrupt_interface_is_enabled_rtc( t_ttc_interrupt_config_rtc* Config_ISRs, e_ttc_interrupt_type Type ); //{  macro _driver_interrupt_interface_is_enabled_rtc(Config_ISRs, Type)
    #ifdef ttc_driver_interrupt_is_enabled_rtc
        // enable following line to forward interface function as a macro definition
        #define _driver_interrupt_is_enabled_rtc ttc_driver_interrupt_is_enabled_rtc
    #else
        #define _driver_interrupt_is_enabled_rtc ttc_interrupt_interface_is_enabled_rtc
    #endif
    //}

#endif
//}
BOOL ttc_interrupt_interface_check_inside_isr(); //{  macro _driver_interrupt_interface_check_inside_isr()
#ifdef ttc_driver_interrupt_check_inside_isr
    // enable following line to forward interface function as a macro definition
    #define _driver_interrupt_check_inside_isr ttc_driver_interrupt_check_inside_isr
#else
    #define _driver_interrupt_check_inside_isr ttc_interrupt_interface_check_inside_isr
#endif
//}
BOOL ttc_interrupt_interface_check_all_disabled(); //{  macro _driver_interrupt_interface_check_all_disabled()
#ifdef ttc_driver_interrupt_check_all_disabled
    // enable following line to forward interface function as a macro definition
    #define _driver_interrupt_check_all_disabled ttc_driver_interrupt_check_all_disabled
#else
    #define _driver_interrupt_check_all_disabled ttc_interrupt_interface_check_all_disabled
#endif
//}

// Note: If create_DeviceDriver.pl adds function+macro blocks below, you have to move them into corresponding #ifdef EXTENSION_ttc_* section above to avoid compile errors!
// Hint: Edit this file with jEdit and set fold marks to { } !

//InsertDriverPrototypes above (DO NOT DELETE THIS LINE!)

//} Function prototypes
//{ private Function prototypes ******************************************
//
// Convention: Private functions start with an underscore "_".
// Example:    void _ttc_interrupt_interface_foo(ttc_foo_isrs_t* Config_ISRs)

//InsertPrivateFunctionDeclarations above (DO NOT DELETE THIS LINE!)

//}PrivateFunctions

#endif //TTC_INTERRUPT_INTERFACE_H
