/** { ttc_light_interface.h *************************************************
 *
 *                           The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Interface layer between high- and low-level driver for LED device.
 *
 *  *_interface_*() functions may be redirected to low-level implementations in
 *  two ways:
 *
 *  1) Macro Definition
 *     This applies if only one architecture can be available during runtime.
 *     One example is a GPIO driver for current microcontroller architecture.
 *     Only one type of microcontroller can be available during runtime.
 *
 *  2) Selector Function
 *     Applies if more than one architecture can be availabe during runtime.
 *     A function will select the corresponding low-level driver via a switch-,
 *     if-else construct of via a vector table.
 *     An example can be a display driver. It is possible to connect more than
 *     one different displays to a microcontroller. So every interface function has
 *     to choose, which low-level driver should be used.
 *
 *  Adding a new low-level implementation requires changes in this file.
 *
 *  Created from template ttc_device_interface.h revision 24 at 20191218 12:21:38 UTC
 *
 *  Authors: Gregor Rebel
 *
}*/

#ifndef TTC_LIGHT_INTERFACE_H
#define TTC_LIGHT_INTERFACE_H

//{ Includes *************************************************************
//
// Header files include all header files that are required to include this
// header file. Includes that provide function declarations should be placed
// in "ttc_light_interface.c"
//

#include "../ttc_basic.h"
#include "compile_options.h" // only required to help IDE to know constant definitions

//{ BEGIN_ARCHITECTURE_INCLUDES (DO NOT DELETE THIS LINE!)

#ifdef EXTENSION_light_gpio
    #include "../light/light_gpio.h"  // low-level driver for led devices on gpio architecture
#endif
//InsertArchitectureIncludes above (DO NOT DELETE THIS LINE!)

// Check if at least one low-level driver could be activated
#ifndef EXTENSION_LED_DRIVER_AVAILABLE
    #  error Missing low-level driver for TTC_LIGHT! Did you forget to provide a feature? Check your activation stage: ./clean.sh ; ./activate_project.sh 2>&1 | grep _light_
#endif

//} END_ARCHITECTURE_INCLUDES (DO NOT DELETE THIS LINE!)

#include "../ttc_light_types.h"

//} Includes
/** static declaration of interface between high- and low-level implementation ********{
 *
 * For each _driver_light_*() declaration in ../ttc_light.h, a corresponding
 * interface part is required here. The interface will forward the declaration either
 * to a low-level implementation or a default implementation in ttc_light_interface.c .
 * A warning is generated for each missing driver implementation.
 *
 * Note: This code is generated automatically. Check ../ttc_light.h for prototypes of low-level driver functions!
 *
 */

void ttc_light_interface_configuration_check( t_ttc_light_config* Config ); //{  macro _driver_light_interface_configuration_check(Config)
#ifdef ttc_driver_light_configuration_check
    // enable following line to forward interface function as a macro definition
    #define _driver_light_configuration_check ttc_driver_light_configuration_check
#else
    #define _driver_light_configuration_check ttc_light_interface_configuration_check
#endif
//}
e_ttc_light_errorcode ttc_light_interface_deinit( t_ttc_light_config* Config ); //{  macro _driver_light_interface_deinit(Config)
#ifdef ttc_driver_light_deinit
    // enable following line to forward interface function as a macro definition
    #define _driver_light_deinit ttc_driver_light_deinit
#else
    #define _driver_light_deinit ttc_light_interface_deinit
#endif
//}
e_ttc_light_errorcode ttc_light_interface_init( t_ttc_light_config* Config ); //{  macro _driver_light_interface_init(Config)
#ifdef ttc_driver_light_init
    // enable following line to forward interface function as a macro definition
    #define _driver_light_init ttc_driver_light_init
#else
    #define _driver_light_init ttc_light_interface_init
#endif
//}
e_ttc_light_errorcode ttc_light_interface_load_defaults( t_ttc_light_config* Config ); //{  macro _driver_light_interface_load_defaults(Config)
void ttc_light_interface_prepare(); //{  macro _driver_light_interface_prepare()
#ifdef ttc_driver_light_prepare
    // enable following line to forward interface function as a macro definition
    #define _driver_light_prepare ttc_driver_light_prepare
#else
    #define _driver_light_prepare ttc_light_interface_prepare
#endif
//}
void ttc_light_interface_reset( t_ttc_light_config* Config ); //{  macro _driver_light_interface_reset(Config)
#ifdef ttc_driver_light_reset
    // enable following line to forward interface function as a macro definition
    #define _driver_light_reset ttc_driver_light_reset
#else
    #define _driver_light_reset ttc_light_interface_reset
#endif
//}
void ttc_light_interface_channel_off( t_ttc_light_config* Config ); //{  macro _driver_light_interface_off(Config)
#ifdef ttc_driver_light_off
    // enable following line to forward interface function as a macro definition
    #define _driver_light_off ttc_driver_light_off
#else
    #define _driver_light_off ttc_light_interface_channel_off
#endif
//}
void ttc_light_interface_channel_on( t_ttc_light_config* Config ); //{  macro _driver_light_interface_on(Config)
#ifdef ttc_driver_light_on
    // enable following line to forward interface function as a macro definition
    #define _driver_light_on ttc_driver_light_on
#else
    #define _driver_light_on ttc_light_interface_channel_on
#endif
//}
void ttc_light_interface_brightness_update( t_ttc_light_config* Config ); //{  macro _driver_light_interface_brightness_update(Config)
#ifdef ttc_driver_light_brightness_update
    // enable following line to forward interface function as a macro definition
    #define _driver_light_brightness_update ttc_driver_light_brightness_update
#else
    #define _driver_light_brightness_update ttc_light_interface_brightness_update
#endif
//}
//InsertDriverPrototypes above (DO NOT DELETE THIS LINE!)

//} Function prototypes
//{ private Function prototypes ******************************************
//
// Convention: Private functions start with an underscore "_".
// Example:    void _ttc_light_interface_foo(t_ttc_light_config* Config)

//InsertPrivateFunctionDeclarations above (DO NOT DELETE THIS LINE!)

//}PrivateFunctions

#endif //TTC_LIGHT_INTERFACE_H
