/** { ttc_register_interface.h *************************************************
 *
 *                           The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Interface layer between high- and low-level driver for REGISTER device.
 *
 *  *_interface_*() functions may be redirected to low-level implementations in
 *  two ways:
 *
 *  1) Macro Definition
 *     This applies if only one architecture can be available during runtime.
 *     One example is a GPIO driver for current microcontroller architecture.
 *     Only one type of microcontroller can be available during runtime.
 *
 *  2) Selector Function
 *     Applies if more than one architecture can be availabe during runtime.
 *     A function will select the corresponding low-level driver via a switch-,
 *     if-else construct of via a vector table.
 *     An example can be a display driver. It is possible to connect more than
 *     one different displays to a microcontroller. So every interface function has
 *     to choose, which low-level driver should be used.
 *
 *  Adding a new low-level implementation requires changes in this file.
 *
 *  Created from template ttc_device_interface.h revision 20 at 20140223 10:54:39 UTC
 *
 *  Authors: Gregor Rebel
 *
}*/

#ifndef TTC_REGISTER_INTERFACE_H
#define TTC_REGISTER_INTERFACE_H

//{ Includes *************************************************************

#include "compile_options.h"
#include "../ttc_basic.h"

//{ BEGIN_ARCHITECTURE_INCLUDES (DO NOT DELETE THIS LINE!)

#ifdef EXTENSION_register_stm32w1xx
    #include "../register/register_stm32w1xx.h"
#endif
#ifdef EXTENSION_register_stm32f1xx
    #include "../register/register_stm32f1xx.h"  // low-level driver for register devices on stm32f1xx architecture
#endif
#ifdef EXTENSION_register_stm32l1xx
    #include "../register/register_stm32l1xx.h"  // low-level driver for register devices on stm32f1xx architecture
#endif
#ifdef EXTENSION_register_stm32f30x
    #include "../register/register_stm32f30x.h"  // low-level driver for register devices on stm32f30x architecture
#endif
#ifdef EXTENSION_register_cortexm3
    #include "../register/register_cortexm3.h"  // low-level driver for register devices on cortexm3 architecture
#endif
#ifdef EXTENSION_register_stm32l0xx
    #include "../register/register_stm32l0xx.h"  // low-level driver for register devices on stm32l0xx architecture
#endif
//InsertArchitectureIncludes above (DO NOT DELETE THIS LINE!)

//} END_ARCHITECTURE_INCLUDES (DO NOT DELETE THIS LINE!)

#include "../ttc_register_types.h"

//} Includes
/** static declaration of interface between high- and low-level implementation ********{
 *
 * For each _driver_register_*() declaration in ../ttc_register.h, a corresponding
 * interface part is required here. The interface will forward the declaration either
 * to a low-level implementation or a default implementation in ttc_register_interface.c .
 * A warning is generated for each missing driver implementation.
 *
 * Note: This code is generated automatically. Check ../ttc_register.h for prototypes of low-level driver functions!
 *
 */

//}

#ifndef t_ttc_register_architecture
    #warning Missing low-level definition for t_ttc_register_architecture
#endif

t_ttc_register_architecture* ttc_register_interface_get_configuration(); //{  macro _driver_register_interface_get_configuration()
#ifdef ttc_driver_register_get_configuration
    // enable following line to forward interface function as a macro definition
    #define _driver_register_get_configuration() ttc_driver_register_get_configuration()
#else
    #define _driver_register_get_configuration() ttc_register_interface_get_configuration()
#endif
//}
void ttc_register_interface_prepare(); //{  macro _driver_register_interface_prepare()
#ifdef ttc_driver_register_prepare
    // enable following line to forward interface function as a macro definition
    #define _driver_register_prepare() ttc_driver_register_prepare()
#else
    #define _driver_register_prepare() ttc_register_interface_prepare()
#endif
//}
e_ttc_register_errorcode ttc_register_interface_reset( void* Register ); //{  macro _driver_register_interface_reset(Register)
#ifdef ttc_driver_register_reset
    // enable following line to forward interface function as a macro definition
    #define _driver_register_reset(Register) ttc_driver_register_reset(Register)
#else
    #define _driver_register_reset(Register) ttc_register_interface_reset(Register)
#endif
//}
//InsertDriverPrototypes above (DO NOT DELETE THIS LINE!)

//} Function prototypes
//{ private Function prototypes ******************************************
//
// Convention: Private functions start with an underscore "_".
// Example:    void _ttc_register_interface_foo(t_ttc_register_config* Config)

//InsertPrivateFunctionDeclarations above (DO NOT DELETE THIS LINE!)

//}PrivateFunctions

#endif //TTC_REGISTER_INTERFACE_H
