/** { interrupt_stm32l1xx.c ************************************************
 *
 *                          The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Low-Level driver for interrupt devices on stm32l1xx architectures.
 *  Implementation of low-level driver.
 *
 *  Created from template device_architecture.c revision 21 at 20140424 04:22:44 UTC
 *
 *  Note: See ttc_interrupt.h for description of stm32l1xx independent INTERRUPT implementation.
 *
 *  Authors: Greg Knoll, Gregor Rebel
 *
 *  ToDo: Replace all Standard Peripheral code by direct register access and remove dependency on stm32l1xx_usart.c
}*/

#include "interrupt_cortexm3.h"
#include "interrupt_stm32l1xx.h"
#include "../basic/basic_cm3.h"
#include "../ttc_heap.h"

//{ Global Variables ***********************************************************
// All initializers use same configuration variable to safe memory and speed up initialization.
// Note: This makes interrupt initialization thread unsafe! Always enclose your initialisation calls in critical sections!

NVIC_InitTypeDef interrupt_stm32l1xx_NVIC_Cfg; // ToDo: get rid of this!
const t_u8 interrupt_stm32l1xx_IRQChannel_USART[TTC_INTERRUPT_USART_AMOUNT] = { USART1_IRQn, USART2_IRQn, USART3_IRQn };

// provided by ttc_interrupt.c
#ifdef EXTENSION_ttc_usart
volatile t_register_stm32l1xx_usart* const interrupt_stm32l1xx_BaseRegister_USART[TTC_INTERRUPT_USART_AMOUNT] = {
    ( volatile t_register_stm32l1xx_usart* ) USART1,
    ( volatile t_register_stm32l1xx_usart* ) USART2,
    ( volatile t_register_stm32l1xx_usart* ) USART3
};

A_dynamic_extern( t_ttc_interrupt_config_usart*, ttc_interrupt_config_usart );
#endif
#ifdef EXTENSION_ttc_gpio
    A_dynamic_extern( t_ttc_interrupt_config_gpio*,  ttc_interrupt_config_gpio );
#endif
#ifdef EXTENSION_ttc_rtc
    A_dynamic_extern( t_ttc_interrupt_config_rtc*,  ttc_interrupt_config_rtc );
#endif
#ifdef EXTENSION_ttc_timer
    A_dynamic_extern( t_ttc_interrupt_config_timer*,  ttc_interrupt_config_timer );
#endif
#ifdef EXTENSION_ttc_i2c
    A_dynamic_extern( t_ttc_interrupt_config_i2c*, ttc_interrupt_config_i2c );
#endif
#ifdef EXTENSION_ttc_can
    # warning stm32l1xx has no CAN Interface.
#endif
//}Global Variables
//{ Function Definitions *******************************************************


//------------------------------GPIO----------------------------------------//
#ifdef EXTENSION_ttc_gpio //{

const t_u8 interrupt_stm32l1xx_IRQChannel_GPIO[TTC_INTERRUPT_GPIO_AMOUNT] = { EXTI0_IRQn,
                                                                              EXTI1_IRQn,
                                                                              EXTI2_IRQn,
                                                                              EXTI3_IRQn,
                                                                              EXTI4_IRQn,
                                                                              EXTI9_5_IRQn,
                                                                              EXTI9_5_IRQn,
                                                                              EXTI9_5_IRQn,
                                                                              EXTI9_5_IRQn,
                                                                              EXTI9_5_IRQn,
                                                                              EXTI15_10_IRQn,
                                                                              EXTI15_10_IRQn,
                                                                              EXTI15_10_IRQn,
                                                                              EXTI15_10_IRQn,
                                                                              EXTI15_10_IRQn,
                                                                              EXTI15_10_IRQn
                                                                            };
e_ttc_interrupt_errorcode interrupt_stm32l1xx_init_gpio( t_ttc_interrupt_config_gpio* Config ) {
    t_ttc_gpio_bank GPIOx;
    t_u8 Pin;

    ttc_gpio_from_index( Config->PhysicalIndex, &GPIOx, &Pin );
    Assert_INTERRUPT( GPIOx, E_ttc_assert_origin_auto );
    Assert_INTERRUPT( Pin < TTC_GPIO_MAX_PINS, E_ttc_assert_origin_auto );

    // mask will allow fast bit masking in different registers
    Config->LowLevel.Mask = 1 << Pin;

    // ensure interrupt is disabled before reconfiguring it
    interrupt_stm32l1xx_disable_gpio( Config );

    // initialize GPIO pin as input
    ttc_gpio_init( Config->PhysicalIndex, E_ttc_gpio_mode_input_floating, E_ttc_gpio_speed_max );

    // calculate bit banding address of EXTI-line (==Pin) in first register of
    t_u32* BitBand_EXTI = cm3_calc_peripheral_BitBandAddress( &register_stm32l1xx_EXTI, Pin );

    // calculate bit banding adresses of corresponding bits in EXTI sub registers by adding constant register offsets
    t_u32* BitBand_RTSR = ( t_u32* )( ( ( t_u32 ) BitBand_EXTI )
                                      + 32 * ( ( ( t_u32 ) &register_stm32l1xx_EXTI.RTSR ) - ( ( t_u32 ) &register_stm32l1xx_EXTI ) )
                                    );
    t_u32* BitBand_FTSR = ( t_u32* )( ( ( t_u32 ) BitBand_EXTI )
                                      + 32 * ( ( ( t_u32 ) &register_stm32l1xx_EXTI.FTSR ) - ( ( t_u32 ) &register_stm32l1xx_EXTI ) )
                                    );

    // EXTI_RTSR + EXTI_FTSR
    switch ( Config->Common.Type ) {

        case  E_ttc_interrupt_type_GPIO_Active_Low:         // voltage on input pin is low
        case  E_ttc_interrupt_type_GPIO_Falling:            // voltage on input pin falls from logical 1 to 0

            *BitBand_RTSR = 0; // disable rising trigger
            *BitBand_FTSR = 1; // enable falling trigger
            break;

        case  E_ttc_interrupt_type_GPIO_Active_High:        // voltage on input pin is high
        case  E_ttc_interrupt_type_GPIO_Rising:             // voltage on input pin rises from logical 0 to 1

            *BitBand_FTSR = 0; // disable falling trigger
            *BitBand_RTSR = 1; // enable rising trigger
            break;

        case  E_ttc_interrupt_type_GPIO_Rising_Falling:     // voltage on input pin falls or rises
            *BitBand_FTSR = 1; // enable falling trigger
            *BitBand_RTSR = 1; // enable rising trigger
            break;

        default: ttc_assert_halt_origin( ec_basic_InvalidImplementation ); break; // unknown/ invalid interrupt type!
    }

    t_u8 IRQ_Channel = interrupt_stm32l1xx_IRQChannel_GPIO[Pin];
    Assert_INTERRUPT( IRQ_Channel < 57, E_ttc_assert_origin_auto );  // only interrupt lines 0..56 available in STM32L1xx (->RM0038 p,181,182)

    // set interrupt priority of corresponding IRQ channel in NVIC
    interrupt_cortexm3_priority( IRQ_Channel, 0, 3 );

    // enable selected NVIC IRQ channel
    interrupt_cortexm3_enable( IRQ_Channel, 1 );

    TTC_RETURN ec_interrupt_OK;
}
void   interrupt_stm32l1xx_enable_gpio( t_ttc_interrupt_config_gpio* Config ) {
#if (TTC_Assert_INTERRUPTS==1)
    Assert_INTERRUPT( !A_is_null( ttc_interrupt_config_gpio ), E_ttc_assert_origin_auto );  // init interrupt first!
    t_ttc_interrupt_config_gpio* GPIO_ISR_Entry = A( ttc_interrupt_config_gpio, Pin );
    Assert_INTERRUPT( GPIO_ISR_Entry, E_ttc_assert_origin_auto );  // init interrupt first!
#endif

    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

    // set corresponding bit in Interrupt Mask Register
    register_stm32l1xx_EXTI.IMR.All |= Config->LowLevel.Mask;

    // re-set pending bit (required to re-activate if interrupt already occured)
    register_stm32l1xx_EXTI.PR.All = Config->LowLevel.Mask;
}
t_bool interrupt_stm32l1xx_is_enablight_gpio( t_ttc_interrupt_config_gpio* Config ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)

    t_bool IsEnabled = ( register_stm32l1xx_EXTI.IMR.All & Config->LowLevel.Mask ) ? TRUE : FALSE;

    TTC_RETURN( IsEnabled );   // will perform stack overflow check and return given value
}
void   interrupt_stm32l1xx_deinit_gpio( t_ttc_interrupt_config_gpio* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

    interrupt_stm32l1xx_disable_gpio( Config );

    if ( 1 ) { // disable interrupt channel in NVIC ICER register
        t_ttc_gpio_bank* GPIOx;
        t_u8 Pin;
        ttc_gpio_from_index( Config->PhysicalIndex, ( t_ttc_gpio_bank* ) &GPIOx, &Pin );

        t_u8 IRQ_Channel = interrupt_stm32l1xx_IRQChannel_GPIO[Pin];
        t_u8 RegisterIndex = IRQ_Channel / 32;
        Assert_INTERRUPT( RegisterIndex < 3, E_ttc_assert_origin_auto );  // each NVIC register has three 32 bit fields!

        // disable selected IRQ channel
        volatile t_u32* ICER = &register_cortexm3_NVIC.ICER0.All;
        t_u32* BitBand_ICER = cm3_calc_peripheral_BitBandAddress( ICER + RegisterIndex, IRQ_Channel );
        *BitBand_ICER = 1; // set corresponding bit in ICER register

        // disble selected NVIC IRQ channel
        interrupt_cortexm3_enable( IRQ_Channel, 0 );
    }
}
void   interrupt_stm32l1xx_deinit_all_gpio() {

    ttc_memory_set( ( t_u8* ) & register_stm32l1xx_EXTI, 0, sizeof( register_stm32l1xx_EXTI ) );
    t_ttc_interrupt_config_gpio Config_ISR;
    Config_ISR.PhysicalIndex = E_ttc_gpio_pin_a0;

    for ( t_u8 IndexEXTI = 0; IndexEXTI < 32; IndexEXTI++ ) { // deinit all 32 EXTI lines
        interrupt_stm32l1xx_deinit_gpio( &Config_ISR );
        Config_ISR.PhysicalIndex += 4;
    }
}
void   interrupt_stm32l1xx_disable_gpio( t_ttc_interrupt_config_gpio* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

    // clear corresponding bit in Interrupt Mask Register
    volatile t_u32 IMR = * ( ( t_u32* ) &register_stm32l1xx_EXTI.IMR.All );
    IMR &= ~Config->LowLevel.Mask;
    register_stm32l1xx_EXTI.IMR.All = IMR;
}
void   interrupt_stm32l1xx_generate_gpio( t_ttc_interrupt_config_gpio* Config ) {
#if (TTC_Assert_INTERRUPTS==1)
    Assert_INTERRUPT( !A_is_null( ttc_interrupt_config_gpio ), E_ttc_assert_origin_auto );  // init interrupt first!
    t_ttc_interrupt_config_gpio* GPIO_ISR_Entry = A( ttc_interrupt_config_gpio, Pin );
    Assert_INTERRUPT( GPIO_ISR_Entry, E_ttc_assert_origin_auto );  // init interrupt first!
#endif

    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

    // re-set pending bit (required to re-activate if interrupt already occured)
    register_stm32l1xx_EXTI.PR.All = Config->LowLevel.Mask;

    // set corresponding bit in Interrupt Mask Register to trigger interrupt
    register_stm32l1xx_EXTI.SWIER.All = Config->LowLevel.Mask;
}
t_u8   interrupt_stm32l1xx_gpio_calc_config_index( t_base interrupt_stm32l1xx_PhysicalIndex ) {

    e_ttc_gpio_pin GPIO = ( e_ttc_gpio_pin ) interrupt_stm32l1xx_PhysicalIndex; // we know that physical index is bit banding address of gpio pin
    t_u8 PinNo = cm3_calc_peripheral_BitNumber( GPIO );

    return PinNo; // stm32l1xx has one interrupt line for each pin over all banks (E_ttc_gpio_pin_a1 == tgpb1 == ...) so only pin # is important
}

// interrupt handlers for GPIO
void GPIO_General_IRQHandler( t_physical_index InterruptLine ) {
    t_ttc_interrupt_config_gpio* Entry = A( ttc_interrupt_config_gpio, InterruptLine );
    Assert_INTERRUPT_Writable( Entry, E_ttc_assert_origin_auto );  // no entry found for this interrupt: call ttc_interrupt_init() before!
    if ( Entry && Entry->isr )
    { Entry->isr( Entry->PhysicalIndex, Entry->Argument ); }
}
void EXTI0_IRQHandler() {
    *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR ) = 1 << 0;
    GPIO_General_IRQHandler( 0 );
}
void EXTI1_IRQHandler() {
    *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR ) = 1 << 1;
    GPIO_General_IRQHandler( 1 );
}
void EXTI2_IRQHandler() {
    *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR ) = 1 << 2;
    GPIO_General_IRQHandler( 2 );
}
void EXTI3_IRQHandler() {
    *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR ) = 1 << 3;
    GPIO_General_IRQHandler( 3 );
}
void EXTI4_IRQHandler() {
    *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR ) = 1 << 4;
    GPIO_General_IRQHandler( 4 );
}
void EXTI9_5_IRQHandler() {

    t_u32 PR = *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR );

    if ( PR & 1 << 5 ) { // EXTI5 interrupt active
        *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR ) = 1 << 5;
        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 5 );
    }
    if ( PR & 1 << 6 ) { // EXTI6 interrupt active
        *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR ) = 1 << 6;
        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 6 );
    }
    if ( PR & 1 << 7 ) { // EXTI7 interrupt active
        *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR ) = 1 << 7;
        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 7 );
    }
    if ( PR & 1 << 8 ) { // EXTI8 interrupt active
        *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR ) = 1 << 8;
        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 8 );
    }
    if ( PR & 1 << 9 ) { // EXTI9 interrupt active
        *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR ) = 1 << 9;
        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 9 );
    }
}
void EXTI15_10_IRQHandler() {

    t_u32 PR = *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR );

    if ( PR & 1 << 10 ) { // EXTI10 interrupt active
        *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR ) = 1 << 10;
        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 10 );
    }
    if ( PR & 1 << 11 ) { // EXTI11 interrupt active
        *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR ) = 1 << 11;
        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 11 );
    }
    if ( PR & 1 << 12 ) { // EXTI12 interrupt active
        *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR ) = 1 << 12;
        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 12 );
    }
    if ( PR & 1 << 13 ) { // EXTI13 interrupt active
        *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR ) = 1 << 13;
        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 13 );
    }
    if ( PR & 1 << 14 ) { // EXTI14 interrupt active
        *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR ) = 1 << 14;
        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 14 );
    }
    if ( PR & 1 << 15 ) { // EXTI15 interrupt active
        *( ( volatile t_u32* ) & register_stm32l1xx_EXTI.PR ) = 1 << 15;
        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 15 );
    }
}

#endif //}EXTENSION_ttc_gpio
//------------------------------USART---------------------------------------//
#ifdef EXTENSION_ttc_usart //{

e_ttc_interrupt_errorcode interrupt_stm32l1xx_init_usart( t_ttc_interrupt_config_usart* Config, e_ttc_interrupt_type Type ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL
    Assert_INTERRUPT( Config->PhysicalIndex < 3, E_ttc_assert_origin_auto );  // stm32l1xx has only three USARTs

    Config->LowLevel.BaseRegister = interrupt_stm32l1xx_BaseRegister_USART[Config->PhysicalIndex];
    Assert_INTERRUPT( Config->LowLevel.BaseRegister->CR1.Bits.UE == 1, E_ttc_assert_origin_auto ); // device unit is switched off. Switch it on or initialize it first!

    if ( 1 ) { // calculate bit banding address of interrupt enable bits for fast and atomic enable/disable
        volatile t_u32* ControlRegister1 = NULL;
        t_u8 BitNumber1 = 0;
        volatile t_u32* ControlRegister2 = NULL;
        t_u8 BitNumber2 = 0;

        switch ( Type ) { // enable individual USART interrupt source

            case E_ttc_interrupt_type_USART_Cts: {               // CR3.Bits.CTSIE
                ControlRegister1 = ( volatile t_u32* ) & ( Config->LowLevel.BaseRegister->CR3 );
                BitNumber1 = 10;
                break;
            }
            case E_ttc_interrupt_type_USART_Idle: {              // CR1.Bits.IDLEIE
                ControlRegister1 = ( volatile t_u32* ) & ( Config->LowLevel.BaseRegister->CR1 );
                BitNumber1 = 4;
                break;
            }
            case E_ttc_interrupt_type_USART_TxComplete: {        // CR1.Bits.TCIE
                ControlRegister1 = ( volatile t_u32* ) & ( Config->LowLevel.BaseRegister->CR1 );
                BitNumber1 = 6;
                break;
            }
            case E_ttc_interrupt_type_USART_TransmitDataEmpty: { // CR1.Bits.TXEIE
                ControlRegister1 = ( volatile t_u32* ) & ( Config->LowLevel.BaseRegister->CR1 );
                BitNumber1 = 7;
                break;
            }
            case E_ttc_interrupt_type_USART_RxNE: {              // CR1.Bits.RXNEIE
                ControlRegister1 = ( volatile t_u32* ) & ( Config->LowLevel.BaseRegister->CR1 );
                BitNumber1 = 5;
                break;
            }
            case E_ttc_interrupt_type_USART_LinBreak: {          // CR2.Bits.LBDIE
                ControlRegister1 = ( volatile t_u32* ) & ( Config->LowLevel.BaseRegister->CR2 );
                BitNumber1 = 6;
                break;
            }
            case E_ttc_interrupt_type_USART_Error: {             // CR1.Bits.PEIE, CR3.Bits.EIE
                ControlRegister1 = ( volatile t_u32* ) & ( Config->LowLevel.BaseRegister->CR1 );
                BitNumber1 = 8;
                ControlRegister2 = ( volatile t_u32* ) & ( Config->LowLevel.BaseRegister->CR3 );
                BitNumber2 = 0;
                break;
            }
            default: ttc_assert_halt_origin( ec_interrupt_InvalidImplementation ); break; // invalid type given!
        }

        Assert_INTERRUPT_EXTRA_Writable( ( void* ) ControlRegister1, E_ttc_assert_origin_auto );
        volatile t_u32* EnableBit1 = cm3_calc_peripheral_BitBandAddress( ControlRegister1, BitNumber1 );
        volatile t_u32* EnableBit2 = NULL;
        if ( ControlRegister2 ) {
            EnableBit2 = cm3_calc_peripheral_BitBandAddress( ControlRegister2, BitNumber2 );
        }

        switch ( Type ) { // enable individual USART interrupt source

            case E_ttc_interrupt_type_USART_Cts: {               // CR3.Bits.CTSIE
                Config->LowLevel.EnableBits.CTSIE = EnableBit1;
                break;
            }
            case E_ttc_interrupt_type_USART_Idle: {              // CR1.Bits.IDLEIE
                Config->LowLevel.EnableBits.IDLEIE = EnableBit1;
                break;
            }
            case E_ttc_interrupt_type_USART_TxComplete: {        // CR1.Bits.TCIE
                Config->LowLevel.EnableBits.TCIE = EnableBit1;
                break;
            }
            case E_ttc_interrupt_type_USART_TransmitDataEmpty: { // CR1.Bits.TXEIE
                Config->LowLevel.EnableBits.TXEIE = EnableBit1;
                break;
            }
            case E_ttc_interrupt_type_USART_RxNE: {              // CR1.Bits.RXNEIE
                Config->LowLevel.EnableBits.RXNEIE = EnableBit1;
                break;
            }
            case E_ttc_interrupt_type_USART_LinBreak: {          // CR2.Bits.LBDIE
                Config->LowLevel.EnableBits.LBDIE = EnableBit1;
                break;
            }
            case E_ttc_interrupt_type_USART_Error: {             // CR1.Bits.PEIE, CR3.Bits.EIE
                Config->LowLevel.EnableBits.PEIE = EnableBit1;
                Config->LowLevel.EnableBits.EIE  = EnableBit2;
                break;
            }
            default: ttc_assert_halt_origin( ec_interrupt_InvalidImplementation ); break; // invalid type given!
        }
    }
    if ( 1 ) { // enable selected NVIC IRQ channel
        t_u8 IRQ_Channel = interrupt_stm32l1xx_IRQChannel_USART[Config->PhysicalIndex];
        interrupt_cortexm3_enable( IRQ_Channel, 1 );
    }
    if ( 1 ) { // reading from SR and then from DR will reset flags in SR
        volatile t_u32 Dummy = Config->LowLevel.BaseRegister->SR.All;
        Dummy = Config->LowLevel.BaseRegister->DR.All;
        ( void ) Dummy;
    }
    if ( 1 ) { // configure NVIC
        t_u8 IRQ_Channel = interrupt_stm32l1xx_IRQChannel_USART[Config->PhysicalIndex];

        // set interrupt priority of corresponding IRQ channel in NVIC
        interrupt_cortexm3_priority( IRQ_Channel, 0, 3 );
    }

    TTC_RETURN( ec_interrupt_OK );
}
void                     _interrupt_stm32l1xx_deinit_usart( volatile t_register_stm32l1xx_usart* BaseRegister ) {
    Assert_INTERRUPT_Writable( ( void* ) BaseRegister, E_ttc_assert_origin_auto );

    // disable all interrupt sources of this USART

    t_register_stm32l1xx_usart_cr1  CR1; *( ( t_u32* ) & CR1 ) = *( ( volatile t_u32* ) &BaseRegister->CR1 );
    t_register_stm32l1xx_usart_cr2  CR2; *( ( t_u32* ) & CR2 ) = *( ( volatile t_u32* ) &BaseRegister->CR2 );
    t_register_stm32l1xx_usart_cr3  CR3; *( ( t_u32* ) & CR3 ) = *( ( volatile t_u32* ) &BaseRegister->CR3 );

    // disable all interrupt bits for this usart (will be enabled by interrupt_stm32l1xx_enable_usart() )
    CR1.Bits.PEIE   = 0;
    CR1.Bits.TXEIE  = 0;
    CR1.Bits.TCIE   = 0;
    CR1.Bits.RXNEIE = 0;
    CR1.Bits.IDLEIE = 0;
    CR2.Bits.LBDIE  = 0;
    CR3.Bits.EIE    = 0;
    CR3.Bits.CTSIE  = 0;

    // update registers
    *( ( volatile t_u32* ) &BaseRegister->CR1 ) = *( ( volatile t_u32* ) & CR1 );
    *( ( volatile t_u32* ) &BaseRegister->CR2 ) = *( ( volatile t_u32* ) & CR2 );
    *( ( volatile t_u32* ) &BaseRegister->CR3 ) = *( ( volatile t_u32* ) & CR3 );

    TTC_RETURN;
}
void                      interrupt_stm32l1xx_deinit_all_usart() {
    _interrupt_stm32l1xx_deinit_usart( & register_stm32l1xx_USART1 );
    _interrupt_stm32l1xx_deinit_usart( & register_stm32l1xx_USART2 );
    _interrupt_stm32l1xx_deinit_usart( & register_stm32l1xx_USART3 );
    _interrupt_stm32l1xx_deinit_usart( & register_stm32l1xx_UART4 );
    _interrupt_stm32l1xx_deinit_usart( & register_stm32l1xx_UART5 );
}
void                      interrupt_stm32l1xx_deinit_usart( t_ttc_interrupt_config_usart* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

    Assert_INTERRUPT( Config->PhysicalIndex < TTC_USART_AMOUNT, E_ttc_assert_origin_auto );
    Assert_INTERRUPT_Writable( ( void* ) Config->LowLevel.BaseRegister, E_ttc_assert_origin_auto );

    // disable all interrupt sources of this USART
    _interrupt_stm32l1xx_deinit_usart( Config->LowLevel.BaseRegister );
}
void                     _interrupt_stm32l1xx_enable_usart( t_ttc_interrupt_config_usart* Config, e_ttc_interrupt_type Type, t_bool Enable ) {
    /** enables or disables interrupt source of given type by using atomic bit band access to configuration register.
     *
     */

    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL
    t_physical_index interrupt_stm32l1xx_PhysicalIndex = Config->PhysicalIndex;
    Assert_INTERRUPT( interrupt_stm32l1xx_PhysicalIndex < TTC_USART_AMOUNT, E_ttc_assert_origin_auto );

    volatile t_register_stm32l1xx_usart* BaseRegister = Config->LowLevel.BaseRegister;
    Assert_INTERRUPT_Writable( ( void* ) BaseRegister, E_ttc_assert_origin_auto );

    // enable/disable interrupt source using atomic, precalculated bit banding access
    volatile t_u32* EnableFlag;
    switch ( Type ) { // select interrupt enable flag for given type

        case E_ttc_interrupt_type_USART_Cts: {               // CR3.Bits.CTSIE
            EnableFlag = Config->LowLevel.EnableBits.CTSIE;
            break;
        }
        case E_ttc_interrupt_type_USART_Idle: {              // CR1.Bits.IDLEIE
            EnableFlag = Config->LowLevel.EnableBits.IDLEIE;
            break;
        }
        case E_ttc_interrupt_type_USART_TxComplete: {        // CR1.Bits.TCIE
            EnableFlag = Config->LowLevel.EnableBits.TCIE;
            break;
        }
        case E_ttc_interrupt_type_USART_TransmitDataEmpty: { // CR1.Bits.TXEIE
            EnableFlag = Config->LowLevel.EnableBits.TXEIE;
            break;
        }
        case E_ttc_interrupt_type_USART_RxNE: {              // CR1.Bits.RXNEIE
            EnableFlag = Config->LowLevel.EnableBits.RXNEIE;
            break;
        }
        case E_ttc_interrupt_type_USART_LinBreak: {          // CR2.Bits.LBDIE
            EnableFlag = Config->LowLevel.EnableBits.LBDIE;
            break;
        }
        case E_ttc_interrupt_type_USART_Error: {             // CR1.Bits.PEIE, CR3.Bits.EIE
            EnableFlag = Config->LowLevel.EnableBits.PEIE;

            // second flag is handled right here to reduce overall code size
            Assert_INTERRUPT_EXTRA_Writable( ( void* ) Config->LowLevel.EnableBits.EIE, E_ttc_assert_origin_auto );
            *( Config->LowLevel.EnableBits.EIE )  = 0;
            break;
        }
        default: EnableFlag = NULL; ttc_assert_halt_origin( ec_interrupt_InvalidImplementation ); break; // invalid type given!
    }

    // atomically enable or dissable interrupts for given type using bitband address
    Assert_INTERRUPT_Writable( ( void* ) EnableFlag, E_ttc_assert_origin_auto );
    *EnableFlag = ( Enable ) ? 1 : 0;
}
void                      interrupt_stm32l1xx_disable_usart( t_ttc_interrupt_config_usart* Config, e_ttc_interrupt_type Type ) {
    _interrupt_stm32l1xx_enable_usart( Config, Type, FALSE );
}
void                      interrupt_stm32l1xx_enable_usart( t_ttc_interrupt_config_usart* Config, e_ttc_interrupt_type Type ) {
    _interrupt_stm32l1xx_enable_usart( Config, Type, TRUE );
}
t_bool                    interrupt_stm32l1xx_is_enabled_usart( t_ttc_interrupt_config_usart* Config, e_ttc_interrupt_type Type ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)
    t_bool IsEnabled = FALSE;
    switch ( Type ) {
        case E_ttc_interrupt_type_USART_Cts: {
            if ( Config->LowLevel.EnableBits.CTSIE != 0 ) { IsEnabled = TRUE; }

            break;
        }
        case E_ttc_interrupt_type_USART_Error: {
            if ( *Config->LowLevel.EnableBits.PEIE != 0 ) { IsEnabled = TRUE; }
            if ( *Config->LowLevel.EnableBits.EIE != 0 ) { IsEnabled = TRUE; }

            break;
        }
        case E_ttc_interrupt_type_USART_Idle: {
            if ( *Config->LowLevel.EnableBits.IDLEIE != 0 ) { IsEnabled = TRUE; }

            break;
        }
        case E_ttc_interrupt_type_USART_TxComplete: {
            if ( *Config->LowLevel.EnableBits.TCIE != 0 ) { IsEnabled = TRUE; }

            break;
        }
        case E_ttc_interrupt_type_USART_TransmitDataEmpty: {
            if ( *Config->LowLevel.EnableBits.TXEIE != 0 ) { IsEnabled = TRUE; }

            break;
        }
        case E_ttc_interrupt_type_USART_RxNE: {
            if ( *Config->LowLevel.EnableBits.RXNEIE != 0 ) { IsEnabled = TRUE; }

            break;
        }
        case E_ttc_interrupt_type_USART_LinBreak: {
            if ( *Config->LowLevel.EnableBits.LBDIE != 0 ) { IsEnabled = TRUE; }

            break;
        }
        default: break;
    }

    TTC_RETURN( IsEnabled );   // will perform stack overflow check and return given value
}

t_register_stm32l1xx_usart_sr interrupt_stm32l1xx_StatusRegister; // set by USART*_IRQHandler()
t_physical_index              interrupt_stm32l1xx_PhysicalIndex;  // set by USART*_IRQHandler()
t_ttc_interrupt_config_usart* interrupt_stm32l1xx_ConfigUsart;    // set by USART*_IRQHandler()

void USART_General_IRQHandler() {
    Assert_INTERRUPT( interrupt_stm32l1xx_ConfigUsart != NULL, E_ttc_assert_origin_auto );  // should never happen: the usart indexed by interrupt_stm32l1xx_PhysicalIndex has not been initialized!

    // CortexM3 interrupts are level triggered and will reactivate as long as interrupt flag ist active and enabled
    if ( interrupt_stm32l1xx_StatusRegister.Bits.RXNE && interrupt_stm32l1xx_ConfigUsart->isr_ReceiveDataNotEmpty  && *( interrupt_stm32l1xx_ConfigUsart->LowLevel.EnableBits.RXNEIE ) ) {
        interrupt_stm32l1xx_ConfigUsart->isr_ReceiveDataNotEmpty( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_ReceiveDataNotEmpty );
    }
    if ( interrupt_stm32l1xx_StatusRegister.Bits.TXE  && interrupt_stm32l1xx_ConfigUsart->isr_TransmitDataEmpty    && *( interrupt_stm32l1xx_ConfigUsart->LowLevel.EnableBits.TXEIE ) ) {
        interrupt_stm32l1xx_ConfigUsart->isr_TransmitDataEmpty( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_TransmitDataEmpty );
    }
    if ( interrupt_stm32l1xx_StatusRegister.Bits.CTS  && interrupt_stm32l1xx_ConfigUsart->isr_ClearToSend          && *( interrupt_stm32l1xx_ConfigUsart->LowLevel.EnableBits.CTSIE ) ) {
        interrupt_stm32l1xx_ConfigUsart->isr_ClearToSend( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_ClearToSend );
    }
    if ( interrupt_stm32l1xx_StatusRegister.Bits.TC   && interrupt_stm32l1xx_ConfigUsart->isr_TransmissionComplete && *( interrupt_stm32l1xx_ConfigUsart->LowLevel.EnableBits.TCIE ) ) {
        interrupt_stm32l1xx_ConfigUsart->isr_TransmissionComplete( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_TransmissionComplete );
    }
    if ( interrupt_stm32l1xx_StatusRegister.Bits.IDLE && interrupt_stm32l1xx_ConfigUsart->isr_IdleLine             && *( interrupt_stm32l1xx_ConfigUsart->LowLevel.EnableBits.IDLEIE ) ) {
        interrupt_stm32l1xx_ConfigUsart->isr_IdleLine( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_IdleLine );
    }
    if ( interrupt_stm32l1xx_StatusRegister.Bits.LBD  && interrupt_stm32l1xx_ConfigUsart->isr_LinBreakDetected     && *( interrupt_stm32l1xx_ConfigUsart->LowLevel.EnableBits.LBDIE ) ) {
        interrupt_stm32l1xx_ConfigUsart->isr_LinBreakDetected( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_LinBreakDetected );
    }

    if ( ( ( *( ( t_u8* ) &interrupt_stm32l1xx_StatusRegister ) ) & 0xf ) && // error occured: gather data + call error handler
            ( interrupt_stm32l1xx_ConfigUsart->isr_Error != NULL )           // error-handler has been defined
       ) {
        t_ttc_interrupt_usart_errors Error; *( ( t_u8* ) &Error ) = 0;
        if ( interrupt_stm32l1xx_StatusRegister.Bits.FE )   { Error.Framing = 1; }
        if ( interrupt_stm32l1xx_StatusRegister.Bits.ORE )  { Error.Overrun = 1; }
        if ( interrupt_stm32l1xx_StatusRegister.Bits.PE )   { Error.Parity  = 1; }
        //if (interrupt_stm32l1xx_StatusRegister.Bits.NE)   Error.Noise   = 1;

        interrupt_stm32l1xx_ConfigUsart->isr_Error( interrupt_stm32l1xx_PhysicalIndex, Error, interrupt_stm32l1xx_ConfigUsart->Argument_Error );
    }
}
void USART1_IRQHandler() {
    // reading from SR and then from DR will reset flags in SR
    interrupt_stm32l1xx_StatusRegister.All = register_stm32l1xx_USART1.SR.All;
    interrupt_stm32l1xx_PhysicalIndex      = 0;
    interrupt_stm32l1xx_ConfigUsart        = A( ttc_interrupt_config_usart, 0 );
    USART_General_IRQHandler();
}
void USART2_IRQHandler() {
    // reading from SR and then from DR will reset flags in SR
    interrupt_stm32l1xx_StatusRegister.All = register_stm32l1xx_USART2.SR.All;
    interrupt_stm32l1xx_PhysicalIndex      = 1;
    interrupt_stm32l1xx_ConfigUsart        = A( ttc_interrupt_config_usart, 1 );
    USART_General_IRQHandler();
}
void USART3_IRQHandler() {
    // reading from SR and then from DR will reset flags in SR
    interrupt_stm32l1xx_StatusRegister.All = register_stm32l1xx_USART3.SR.All;
    interrupt_stm32l1xx_PhysicalIndex      = 2;
    interrupt_stm32l1xx_ConfigUsart        = A( ttc_interrupt_config_usart, 2 );
    USART_General_IRQHandler();
}

#endif //}EXTENSION_ttc_usart
//------------------------------RTC (REAL TIME CLOCK)---------------------------------------//

#ifdef EXTENSION_ttc_rtc
#warning interrupt_stm32l1xx_*_rtc() functions are pretty old and untested!

void interrupt_stm32l1xx_enable_rtc( t_ttc_interrupt_config_rtc* Config ) {

    Assert_INTERRUPT( Config->PhysicalIndex < TTC_RTC_AMOUNT, E_ttc_assert_origin_auto );

    if ( Config->Common.Type == E_ttc_interrupt_type_RTC_AlarmA ) {

        EXTI_ClearITPendingBit( EXTI_Line17 );

        RTC_ITConfig( RTC_IT_ALRA, DISABLE );


        register_stm32l1xx_RTC.WPR.All = 0xCA;
        register_stm32l1xx_RTC.WPR.All = 0x53;

        /* Clear RTC AlarmA Flags */
        RTC_ClearITPendingBit( RTC_IT_ALRA );

        /* Clear RTC Alarm A flag */
        RTC_ClearFlag( RTC_FLAG_ALRAF );

        register_stm32l1xx_RTC.WPR.All = 0xFF;

        /* Enable AlarmA interrupt */
        RTC_ITConfig( RTC_IT_ALRA, ENABLE );
    }
    else if ( Config->Common.Type == E_ttc_interrupt_type_RTC_AlarmB ) {
        EXTI_ClearITPendingBit( EXTI_Line17 );

        RTC_ITConfig( RTC_IT_ALRB, DISABLE );


        register_stm32l1xx_RTC.WPR.All = 0xCA;
        register_stm32l1xx_RTC.WPR.All = 0x53;

        /* Clear RTC AlarmB Flags */
        RTC_ClearITPendingBit( RTC_IT_ALRB );

        /* Clear RTC AlarmB flag */
        RTC_ClearFlag( RTC_FLAG_ALRBF );

        register_stm32l1xx_RTC.WPR.All = 0xFF;

        /* Enable AlarmB interrupt */
        RTC_ITConfig( RTC_IT_ALRB, ENABLE );
    }
    else if ( Config->Common.Type == E_ttc_interrupt_type_RTC_Tamper ) {

        EXTI_ClearITPendingBit( EXTI_Line19 );

        RTC_ITConfig( RTC_IT_TAMP, DISABLE );


        register_stm32l1xx_RTC.WPR.All = 0xCA;
        register_stm32l1xx_RTC.WPR.All = 0x53;

        /* Clear Tamper 1 pin interrupt pending bit */
        RTC_ClearITPendingBit( RTC_IT_TAMP1 );
        RTC_ClearFlag( RTC_FLAG_TAMP1F );

        register_stm32l1xx_RTC.WPR.All = 0xFF;

        /* Enable the Tamper interrupt */
        RTC_ITConfig( RTC_IT_TAMP, ENABLE );
    }
    else if ( Config->Common.Type == E_ttc_interrupt_type_RTC_WakeUP ) {

        EXTI_ClearITPendingBit( EXTI_Line20 );

        /* Disable WakeUp interrupt */
        RTC_ITConfig( RTC_IT_WUT, DISABLE );


        register_stm32l1xx_RTC.WPR.All = 0xCA;
        register_stm32l1xx_RTC.WPR.All = 0x53;

        /* Clear RTC WakeUP Flags */
        RTC_ClearITPendingBit( RTC_IT_WUT );
        RTC_ClearFlag( RTC_FLAG_WUTF );

        register_stm32l1xx_RTC.WPR.All = 0xFF;

        /* Enable WakeUp interrupt */
        RTC_ITConfig( RTC_IT_WUT, ENABLE );
    }
    else if ( Config->Common.Type == E_ttc_interrupt_type_RTC_TimeStamp ) {

        EXTI_ClearITPendingBit( EXTI_Line19 );

        RTC_ITConfig( RTC_IT_TS, DISABLE );


        register_stm32l1xx_RTC.WPR.All = 0xCA;
        register_stm32l1xx_RTC.WPR.All = 0x53;

        /* Clear RTC TimeStamp Flags */
        RTC_ClearITPendingBit( RTC_IT_TS );
        RTC_ClearFlag( RTC_FLAG_TSF );
        RTC_ClearFlag( RTC_FLAG_TSOVF );

        register_stm32l1xx_RTC.WPR.All = 0xFF;

        /* Enable TimeStamp interrupt */
        RTC_ITConfig( RTC_IT_TS, ENABLE );
    }
    else if ( Config->Common.Type == E_ttc_interrupt_type_RTC_Pin_line_wakeup ) {


    }
    else {

        Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
    }

}
e_ttc_interrupt_errorcode interrupt_stm32l1xx_alarm_rtc() {

    //Enable NVIC interrupt channel for RTC
    NVIC_InitTypeDef NVIC_InitStructure;
    EXTI_InitTypeDef EXTI_InitStructure;


    /* Alarm Interrupt Configuration */
    /* EXTI configuration */
    EXTI_ClearITPendingBit( EXTI_Line17 );
    EXTI_InitStructure.EXTI_Line = EXTI_Line17;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init( &EXTI_InitStructure );

    /* Enable the RTC Alarm Interrupt */
    NVIC_PriorityGroupConfig( NVIC_PriorityGroup_2 );
    NVIC_InitStructure.NVIC_IRQChannel = RTC_Alarm_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init( &NVIC_InitStructure );

    return ( ec_interrupt_OK );

}
e_ttc_interrupt_errorcode interrupt_stm32l1xx_wakeup_rtc() {

    //Enable NVIC interrupt channel for RTC
    NVIC_InitTypeDef NVIC_InitStructure;
    EXTI_InitTypeDef EXTI_InitStructure;


    /* Alarm Interrupt Configuration */
    /* EXTI configuration */
    EXTI_ClearITPendingBit( EXTI_Line20 );
    EXTI_InitStructure.EXTI_Line = EXTI_Line20;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Event;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init( &EXTI_InitStructure );

    /* Enable the RTC Alarm Interrupt */

    NVIC_PriorityGroupConfig( NVIC_PriorityGroup_1 );
    NVIC_InitStructure.NVIC_IRQChannel = RTC_WKUP_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init( &NVIC_InitStructure );

    return ( ec_interrupt_OK );

}
e_ttc_interrupt_errorcode interrupt_stm32l1xx_tamper_rtc() {

    NVIC_InitTypeDef NVIC_InitStructure;
    EXTI_InitTypeDef  EXTI_InitStructure;

    /* Enable The external line19 interrupt */
    EXTI_ClearITPendingBit( EXTI_Line19 );
    EXTI_InitStructure.EXTI_Line = EXTI_Line19;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init( &EXTI_InitStructure );

    /* Enable TAMPER IRQChannel */
    NVIC_PriorityGroupConfig( NVIC_PriorityGroup_1 );
    NVIC_InitStructure.NVIC_IRQChannel = TAMPER_STAMP_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init( &NVIC_InitStructure );


    return ( e_ttc_interrupt_errorcode )0;


}
e_ttc_interrupt_errorcode interrupt_stm32l1xx_timestamp_rtc() {

    return ec_interrupt_OK;
}
e_ttc_interrupt_errorcode interrupt_stm32l1xx_pin_line_wakeup_rtc() {

    // Enable GPIOA's AHB interface clock
    register_stm32l1xx_RCC.AHBENR.Bits.GPIOA_EN = 1;
    //X sysclock_stm32l1xx_RCC_AHBPeriphClockCmd( RCC_AHBPeriph_GPIOA, ENABLE );

    // Enable SYSCFG's APB interface clock
    register_stm32l1xx_RCC.APB2ENR.Bits.SYSCFGEN = 1;
    //X sysclock_stm32l1xx_RCC_APB2PeriphClockCmd( RCC_APB2Periph_SYSCFG, ENABLE );

    //    GPIO_InitTypeDef GPIO_InitStructure;

    //    Configure PA0 pin in input mode
    //    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
    //    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
    //    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
    //    GPIO_Init(GPIOA, &GPIO_InitStructure);
    ttc_gpio_init( E_ttc_gpio_pin_a0, E_ttc_gpio_mode_input_floating, E_ttc_gpio_speed_max );

    /* Connect EXTI Line0 to PA0 pin */
    SYSCFG_EXTILineConfig( EXTI_PortSourceGPIOA, EXTI_PinSource0 );

    EXTI_InitTypeDef EXTI_InitStructure;

    /* Configure EXTI line0 */
    EXTI_InitStructure.EXTI_Line = EXTI_Line0;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init( &EXTI_InitStructure );

    return ec_interrupt_OK;
}
e_ttc_interrupt_errorcode interrupt_stm32l1xx_init_rtc( t_ttc_interrupt_config_rtc* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

    Assert_INTERRUPT( Config->PhysicalIndex < TTC_RTC_AMOUNT, E_ttc_assert_origin_auto );

    /* Configure Alarm intrruption */
    e_ttc_interrupt_errorcode Error = -1;

    if ( Config->Common.Type == E_ttc_interrupt_type_RTC_AlarmA ) {

        Error = interrupt_stm32l1xx_alarm_rtc();

    }
    else if ( Config->Common.Type == E_ttc_interrupt_type_RTC_AlarmB ) {

        Error = interrupt_stm32l1xx_alarm_rtc();


    }
    else if ( Config->Common.Type == E_ttc_interrupt_type_RTC_Tamper ) {


        Error = interrupt_stm32l1xx_tamper_rtc();

    }
    else if ( Config->Common.Type == E_ttc_interrupt_type_RTC_WakeUP ) {

        Error = interrupt_stm32l1xx_wakeup_rtc();


    }
    else if ( Config->Common.Type == E_ttc_interrupt_type_RTC_TimeStamp ) {

        Error = interrupt_stm32l1xx_timestamp_rtc();

    }
    else if ( Config->Common.Type == E_ttc_interrupt_type_RTC_Pin_line_wakeup ) {
        Error = interrupt_stm32l1xx_pin_line_wakeup_rtc();

    }
    else {

        Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

    }

    return Error;
}
void interrupt_stm32l1xx_deinit_rtc( t_ttc_interrupt_config_rtc* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL



}
void interrupt_stm32l1xx_disable_rtc( t_ttc_interrupt_config_rtc* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

}
void RTC_General_IRQHandler() {


}
void TAMPER_STAMP_IRQHandler() {
    RTC_General_IRQHandler();

    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void RTC_WKUP_IRQHandler() {

    RTC_General_IRQHandler();


    if ( RTC_GetITStatus( RTC_IT_WUT ) != RESET ) {

        EXTI_ClearITPendingBit( EXTI_Line20 ); //OK

        /* Disable WakeUp interrupt */
        RTC_ITConfig( RTC_IT_WUT, DISABLE );

        register_stm32l1xx_RTC.WPR.All = 0xCA;
        register_stm32l1xx_RTC.WPR.All = 0x53;

        /* Clear RTC WakeUP Flags */
        RTC_ClearITPendingBit( RTC_IT_WUT );
        RTC_ClearFlag( RTC_FLAG_WUTF );

        register_stm32l1xx_RTC.WPR.All = 0xFF;

        /* Enable WakeUp interrupt */
        RTC_ITConfig( RTC_IT_WUT, ENABLE );
    }
}
void RTC_Alarm_IRQHandler() {

    RTC_General_IRQHandler();

    if ( RTC_GetITStatus( RTC_IT_ALRA ) != RESET ) {
        EXTI_ClearITPendingBit( EXTI_Line17 );

        RTC_ITConfig( RTC_IT_ALRA, DISABLE );

        register_stm32l1xx_RTC.WPR.All = 0xCA;
        register_stm32l1xx_RTC.WPR.All = 0x53;

        /* Clear RTC AlarmA Flags */
        RTC_ClearITPendingBit( RTC_IT_ALRA );
        /* Clear RTC Alarm A flag */
        RTC_ClearFlag( RTC_FLAG_ALRAF );

        register_stm32l1xx_RTC.WPR.All = 0xFF;

        /* Enable AlarmA interrupt */
        RTC_ITConfig( RTC_IT_ALRA, ENABLE );

    }
    else if ( RTC_GetITStatus( RTC_IT_ALRB ) != RESET ) {

        RTC_ITConfig( RTC_IT_ALRB, DISABLE );

        register_stm32l1xx_RTC.WPR.All = 0xCA;
        register_stm32l1xx_RTC.WPR.All = 0x53;

        /* Clear RTC AlarmA Flags */
        RTC_ClearITPendingBit( RTC_IT_ALRB );
        /* Clear RTC Alarm A flag */
        RTC_ClearFlag( RTC_FLAG_ALRBF );

        register_stm32l1xx_RTC.WPR.All = 0xFF;

        /* Enable AlarmA interrupt */
        RTC_ITConfig( RTC_IT_ALRB, ENABLE );
    }

}
#endif


//------------------------------I2C---------------------------------------//
#ifdef EXTENSION_ttc_i2c
e_ttc_interrupt_errorcode interrupt_stm32l1xx_init_i2c( t_ttc_interrupt_config_i2c* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL
    t_register_stm32l1xx_i2c* I2C_Base = NULL;

    switch ( Config->PhysicalIndex ) {
        case 1: I2C_Base = ( t_register_stm32l1xx_i2c* )&register_stm32l1xx_I2C1;
            interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannel = I2C1_EV_IRQn | I2C1_ER_IRQn;
            break;
        case 2: I2C_Base = ( t_register_stm32l1xx_i2c* )&register_stm32l1xx_I2C2;
            interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannel = I2C2_EV_IRQn | I2C2_ER_IRQn;
            break;
        default: Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
            break;
    }
    I2C_Base->CR2.Bits.ITBUF_EN = 1;
    Config->LowLevel.BaseRegister = I2C_Base;


    /* Enable and set I2Cx Interrupt */
    interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannelPreemptionPriority = 0x0F;
    interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannelSubPriority = 0x0F;
    interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init( &interrupt_stm32l1xx_NVIC_Cfg );

    return ( e_ttc_interrupt_errorcode ) 0;
}

void interrupt_stm32l1xx_deinit_i2c( t_ttc_interrupt_config_i2c* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL
}
void interrupt_stm32l1xx_enable_i2c( t_ttc_interrupt_config_i2c* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

    /* Enable and set USARTn Interrupt to the lowest priority */
    interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannelPreemptionPriority = 0x0F;
    interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannelSubPriority = 0x0F;
    interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannelCmd = ENABLE;

    t_register_stm32l1xx_i2c* I2C_Base = Config->LowLevel.BaseRegister;

    I2C_Base->CR2.Bits.ITBUF_EN = 1;
    switch ( Config->PhysicalIndex ) {
        case 1: //X I2C_Base = (t_register_stm32l1xx_i2c*)&register_stm32l1xx_I2C1;
            interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannel = I2C1_EV_IRQn | I2C1_ER_IRQn;
            break;
        case 2: //X I2C_Base = (t_register_stm32l1xx_i2c*)&register_stm32l1xx_I2C2;
            interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannel = I2C2_EV_IRQn | I2C2_ER_IRQn;
            break;
        default: Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
            break;
    }

    switch ( Config->Common.Type ) { // select trigger according to Type
        case E_ttc_interrupt_type_I2C_StartBit:
            Assert_INTERRUPT( A( ttc_interrupt_config_i2c, Config->PhysicalIndex )->isr_StartBitSent != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_I2C_AddrSent:
            Assert_INTERRUPT( A( ttc_interrupt_config_i2c, Config->PhysicalIndex )->isr_AddressSent != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_I2C_10_HDR:
            Assert_INTERRUPT( A( ttc_interrupt_config_i2c, Config->PhysicalIndex )->isr_10BitHeader != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_I2C_Stop:
            Assert_INTERRUPT( A( ttc_interrupt_config_i2c, Config->PhysicalIndex )->isr_StopReceived != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_I2C_DB_TF:
            Assert_INTERRUPT( A( ttc_interrupt_config_i2c, Config->PhysicalIndex )->isr_DataByteTransferFinished != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_I2C_RxNE:
            Assert_INTERRUPT( A( ttc_interrupt_config_i2c, Config->PhysicalIndex )->isr_ReceiveBufferNotEmpty != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_I2C_TxE:
            Assert_INTERRUPT( A( ttc_interrupt_config_i2c, Config->PhysicalIndex )->isr_TransmitBufferEmpty != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_I2C_BusError:
            Assert_INTERRUPT( A( ttc_interrupt_config_i2c, Config->PhysicalIndex )->isr_BusError != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_I2C_ArbitrationLoss:
            Assert_INTERRUPT( A( ttc_interrupt_config_i2c, Config->PhysicalIndex )->isr_ArbitrationLoss != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_I2C_AckFail:
            Assert_INTERRUPT( A( ttc_interrupt_config_i2c, Config->PhysicalIndex )->isr_AckFail != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_I2C_Overrun:
            Assert_INTERRUPT( A( ttc_interrupt_config_i2c, Config->PhysicalIndex )->isr_Overrun != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_I2C_PECError:
            Assert_INTERRUPT( A( ttc_interrupt_config_i2c, Config->PhysicalIndex )->isr_PECError != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_I2C_Timeout:
            Assert_INTERRUPT( A( ttc_interrupt_config_i2c, Config->PhysicalIndex )->isr_Timeout != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_I2C_SMBUSAlert:
            Assert_INTERRUPT( A( ttc_interrupt_config_i2c, Config->PhysicalIndex )->isr_SMBusAlert != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        default: Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
    }


    NVIC_Init( &interrupt_stm32l1xx_NVIC_Cfg );
}
void interrupt_stm32l1xx_disable_i2c( t_ttc_interrupt_config_i2c* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL
}
#endif

//------------------------------NOT IMPLEMENTED----------------------------//

#ifdef EXTENSION_ttc_spi
e_ttc_interrupt_errorcode interrupt_stm32l1xx_init_spi( t_ttc_interrupt_config_spi* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

    ttc_assert_halt_origin( E_ttc_assert_origin_auto ); // ToDo: implement!

    //#warning missing implementation for interrupt_stm32l1xx_init_spi()!
    return ( e_ttc_interrupt_errorcode ) 0;
}
void interrupt_stm32l1xx_deinit_spi( t_ttc_interrupt_config_spi* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

    ttc_assert_halt_origin( E_ttc_assert_origin_auto ); // ToDo: implement!
    //#warning missing implementation for interrupt_stm32l1xx_deinit_spi()!


}
void interrupt_stm32l1xx_enable_spi( t_ttc_interrupt_config_spi* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

    ttc_assert_halt_origin( E_ttc_assert_origin_auto ); // ToDo: implement!
    //#warning missing implementation for interrupt_stm32l1xx_enable_spi()!


}
void interrupt_stm32l1xx_disable_spi( t_ttc_interrupt_config_spi* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

    ttc_assert_halt_origin( E_ttc_assert_origin_auto ); // ToDo: implement!
    //#warning missing implementation for interrupt_stm32l1xx_disable_spi()!


}
#endif
#ifdef EXTENSION_ttc_timer


e_ttc_interrupt_errorcode interrupt_stm32l1xx_init_timer( t_ttc_interrupt_config_timer* Config ) {

    t_register_stm32l1xx_timer* TIMER_Base;
    ( void )TIMER_Base;     //Avoid the warning


    NVIC_PriorityGroupConfig( NVIC_PriorityGroup_1 );

    switch ( Config->PhysicalIndex ) { // physical index starts is the logical index

        case 1:
            TIMER_Base = ( t_register_stm32l1xx_timer* )TIM2;
            interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannel = TIM2_IRQn;
            break;
        case 2: TIMER_Base = ( t_register_stm32l1xx_timer* )TIM3;
            interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannel = TIM3_IRQn;
            break;
        case 3: TIMER_Base = ( t_register_stm32l1xx_timer* )TIM4;
            interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM4_IRQn;
            break;
        case 5: TIMER_Base = ( t_register_stm32l1xx_timer* )TIM6;
            interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM6_IRQn;
            break;
        case 6: TIMER_Base = ( t_register_stm32l1xx_timer* )TIM7;
            interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM7_IRQn;
            break;
        case 8: TIMER_Base = ( t_register_stm32l1xx_timer* )TIM9;
            interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM9_IRQn;
            break;
        case 9: TIMER_Base = ( t_register_stm32l1xx_timer* )TIM10;
            interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM10_IRQn;
            break;
        case 10: TIMER_Base = ( t_register_stm32l1xx_timer* )TIM11;
            interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM11_IRQn;
            break;

        default: { Assert_INTERRUPT( 0, E_ttc_assert_origin_auto ); }
    }

    /* Enable and set TIMx Interrupt to the lowest priority */
    interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannelPreemptionPriority = 0;
    interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannelSubPriority = 0;
    interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init( &interrupt_stm32l1xx_NVIC_Cfg );

    NVIC_EnableIRQ( interrupt_stm32l1xx_NVIC_Cfg.NVIC_IRQChannel );

    return ec_interrupt_OK;

}

void interrupt_stm32l1xx_enable_timer( t_ttc_interrupt_config_timer* Config ) {
    t_register_stm32l1xx_timer* TIMER_Base = NULL;

    switch ( Config->PhysicalIndex ) {

        case 1:

            TIMER_Base = ( t_register_stm32l1xx_timer* ) TIM2;
            register_stm32l1xx_RCC.APB1ENR.Bits.TIM2_EN = 1;
            break;

        case 2:

            TIMER_Base = ( t_register_stm32l1xx_timer* ) TIM3;
            register_stm32l1xx_RCC.APB1ENR.Bits.TIM3_EN = 1;
            break;

        case 3:

            TIMER_Base = ( t_register_stm32l1xx_timer* ) TIM4;
            register_stm32l1xx_RCC.APB1ENR.Bits.TIM4_EN = 1;
            break;

        case 5:
            TIMER_Base = ( t_register_stm32l1xx_timer* ) TIM6;
            register_stm32l1xx_RCC.APB1ENR.Bits.TIM6_EN = 1;

            break;
        case 6:

            TIMER_Base = ( t_register_stm32l1xx_timer* ) TIM7;
            register_stm32l1xx_RCC.APB1ENR.Bits.TIM7_EN = 1;

        case 8:

            TIMER_Base = ( t_register_stm32l1xx_timer* ) TIM9;
            register_stm32l1xx_RCC.APB2ENR.Bits.TIM9EN = 1;
            break;

        case 9:
            TIMER_Base = ( t_register_stm32l1xx_timer* ) TIM10;
            register_stm32l1xx_RCC.APB2ENR.Bits.TIM10EN = 1;

            break;
        case 10:

            TIMER_Base = ( t_register_stm32l1xx_timer* ) TIM11;
            register_stm32l1xx_RCC.APB2ENR.Bits.TIM11EN = 1;
            break;
    }

    switch ( Config->Common.Type ) { // select trigger according to Type
        case E_ttc_interrupt_type_TIMER_Updating:

            /* Disable the Interrupt sources */
            TIMER_Base->DIER.All &= ( t_u16 )~0x0001;

            /* Clear Interrupt Flag */
            TIMER_Base->SR.All = ( t_u16 )~0x0001;

            /* Clear Update Flag */
            TIMER_Base->SR.All = ( t_u16 )~0x0001;

            /* Enable Interrupt */
            TIMER_Base->DIER.All |= ( ( t_u16 )0x0001 );

            break;

        case E_ttc_interrupt_type_TIMER_CC1:

            /* Disable the Interrupt sources */
            TIMER_Base->DIER.All &= ( t_u16 )~0x0002;

            /* Clear Interrupt Flag */
            TIMER_Base->SR.All = ( t_u16 )~0x0002;

            /* Clear Update Flag */
            TIMER_Base->SR.All = ( t_u16 )~0x0002;

            /* Enable Interrupt */
            TIMER_Base->DIER.All |= ( ( t_u16 )0x0002 );
            break;

        case E_ttc_interrupt_type_TIMER_CC2:
            /* Disable the Interrupt sources */
            TIMER_Base->DIER.All &= ( t_u16 )~0x0004;

            /* Clear Interrupt Flag */
            TIMER_Base->SR.All = ( t_u16 )~0x0004;

            /* Clear Update Flag */
            TIMER_Base->SR.All = ( t_u16 )~0x0004;

            /* Enable Interrupt */
            TIMER_Base->DIER.All |= ( ( t_u16 )0x0004 );
            break;

        case E_ttc_interrupt_type_TIMER_CC3:
            /* Disable the Interrupt sources */
            TIMER_Base->DIER.All &= ( t_u16 )~0x0008;

            /* Clear Interrupt Flag */
            TIMER_Base->SR.All = ( t_u16 )~0x0008;

            /* Clear Update Flag */
            TIMER_Base->SR.All = ( t_u16 )~0x0008;

            /* Enable Interrupt */
            TIMER_Base->DIER.All |= ( ( t_u16 )0x0008 );
            break;

        case E_ttc_interrupt_type_TIMER_CC4:
            /* Disable the Interrupt sources */
            TIMER_Base->DIER.All &= ( t_u16 )~0x0010;

            /* Clear Interrupt Flag */
            TIMER_Base->SR.All = ( t_u16 )~0x0010;

            /* Clear Update Flag */
            TIMER_Base->SR.All = ( t_u16 )~0x0010;

            /* Enable Interrupt */
            TIMER_Base->DIER.All |= ( ( t_u16 )0x0010 );
            break;

        case E_ttc_interrupt_type_TIMER_Triggering:
            /* Disable the Interrupt sources */
            TIMER_Base->DIER.All &= ( t_u16 )~0x0040;

            /* Clear Interrupt Flag */
            TIMER_Base->SR.All = ( t_u16 )~0x0040;

            /* Clear Update Flag */
            TIMER_Base->SR.All = ( t_u16 )~0x0040;

            /* Enable Interrupt */
            TIMER_Base->DIER.All |= ( ( t_u16 )0x0040 );
            break;
        default:
            Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
            break;
    }
}


void TIMER_General_IRQHandler( t_physical_index interrupt_stm32l1xx_PhysicalIndex, volatile t_register_stm32l1xx_timer* TIMER ) {

    //ttc_gpio_set(TTC_LIGHT2);

    Assert_INTERRUPT( TIMER != NULL, E_ttc_assert_origin_auto );
    t_register_stm32l1xx_timx_sr interrupt_stm32l1xx_StatusRegister;
    interrupt_stm32l1xx_StatusRegister.All = ( t_u16 )TIMER->SR.All; // reading from SR and then from DR will reset flags in SR
    t_ttc_interrupt_config_timer* interrupt_stm32l1xx_ConfigUsart = A( ttc_interrupt_config_timer, interrupt_stm32l1xx_PhysicalIndex );
    Assert_INTERRUPT( interrupt_stm32l1xx_ConfigUsart != NULL, E_ttc_assert_origin_auto );  // should never happen: the usart indexed by interrupt_stm32l1xx_PhysicalIndex has not been initialized!

    if ( interrupt_stm32l1xx_StatusRegister.Bits.UIF )
    { interrupt_stm32l1xx_ConfigUsart->isr_Update( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_Update ); }
    /*
        if(interrupt_stm32l1xx_StatusRegister.Bytes.CC1IF)
           interrupt_stm32l1xx_ConfigUsart->isr_CC1IF(interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_CC1IF);
        if(interrupt_stm32l1xx_StatusRegister.Bytes.CC2IF)
           interrupt_stm32l1xx_ConfigUsart->isr_CC2IF(interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_CC2IF);
        if(interrupt_stm32l1xx_StatusRegister.Bytes.CC3IF)
           interrupt_stm32l1xx_ConfigUsart->isr_CC3IF(interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_CC3IF);
        if(interrupt_stm32l1xx_StatusRegister.Bytes.CC4IF)
           interrupt_stm32l1xx_ConfigUsart->isr_CC4IF(interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_CC4IF);
        if(interrupt_stm32l1xx_StatusRegister.Bytes.TIF)
           interrupt_stm32l1xx_ConfigUsart->isr_Trigger(interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_Trigger);
        if(interrupt_stm32l1xx_StatusRegister.Bytes.BIF)
           interrupt_stm32l1xx_ConfigUsart->isr_Break(interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_Break);
        if(interrupt_stm32l1xx_StatusRegister.Bytes.CC1OF)
           interrupt_stm32l1xx_ConfigUsart->isr_CC10F(interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_CC10F);
        if(interrupt_stm32l1xx_StatusRegister.Bytes.CC2OF)
           interrupt_stm32l1xx_ConfigUsart->isr_CC20F(interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_CC20F);
        if(interrupt_stm32l1xx_StatusRegister.Bytes.CC3OF)
           interrupt_stm32l1xx_ConfigUsart->isr_CC30F(interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_CC30F);
        if(interrupt_stm32l1xx_StatusRegister.Bytes.CC4OF)
           interrupt_stm32l1xx_ConfigUsart->isr_CC40F(interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_CC40F);
        */
}
void TIM2_IRQHandler() {
    TIMER_General_IRQHandler( 1, ( t_register_stm32l1xx_timer* )TIM2 );
    register_stm32l1xx_TIMER2.SR.All &= ~0x01; // clear all interrupt flags

}
void TIM3_IRQHandler() {
    TIMER_General_IRQHandler( 2, ( t_register_stm32l1xx_timer* )TIM3 );
    register_stm32l1xx_TIMER3.SR.All &= ~0x01; // clear all interrupt flags
}
void TIM4_IRQHandler() {
    TIMER_General_IRQHandler( 3, ( t_register_stm32l1xx_timer* )TIM4 );
    register_stm32l1xx_TIMER4.SR.All &= ~0x01; // clear all interrupt flags

}
void TIM6_IRQHandler() {
    TIMER_General_IRQHandler( 5, ( t_register_stm32l1xx_timer* )TIM6 );
    register_stm32l1xx_TIMER6.SR.All &= ~0x01; // clear all interrupt flags

}
void TIM7_IRQHandler() {
    TIMER_General_IRQHandler( 6, ( t_register_stm32l1xx_timer* )TIM7 );
    register_stm32l1xx_TIMER7.SR.All &= ~0x01; // clear all interrupt flags

}
void TIM9_IRQHandler() {
    TIMER_General_IRQHandler( 8, ( t_register_stm32l1xx_timer* ) TIM9 );
    register_stm32l1xx_TIMER9.SR.All = 0; // clear all interrupt flags
}
void TIM10_IRQHandler() {
    TIMER_General_IRQHandler( 9, ( t_register_stm32l1xx_timer* ) TIM10 );
    register_stm32l1xx_TIMER10.SR.All = 0; // clear all interrupt flags

}

void TIM11_IRQHandler() {
    TIMER_General_IRQHandler( 10, ( t_register_stm32l1xx_timer* ) TIM11 );
    register_stm32l1xx_TIMER11.SR.All = 0;
}

#endif

void interrupt_stm32l1xx_prepare() {




}


/*
    void interrupt_stm32l1xx_deinit_all_can() {




    }

    void interrupt_stm32l1xx_deinit_all_i2c() {




    }




    }
    void interrupt_stm32l1xx_deinit_all_rtc() {




    }
    void interrupt_stm32l1xx_deinit_all_spi() {




    }
    void interrupt_stm32l1xx_deinit_all_timer() {




    }

    */

//Overwrite weak functions from startup_stm32l1xx_md.s

//{-----------------------------------------interrupt_stm32l1xx_ConfigUsart-----------------------------------------//

//--------------------------------------Implemented-----------------------------------------//

#ifdef EXTERNSION_ttc_can  //{

t_bool interrupt_stm32l1xx_is_enabled_can( t_ttc_interrupt_config_can* Config, e_ttc_interrupt_type Type ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)

#warning Function interrupt_stm32l1xx_is_enabled_can() is empty.

    TTC_RETURN( 0 );   // will perform stack overflow check and return given value
}

#endif //}can
#ifdef EXTENSION_ttc_rtc   //{

t_bool interrupt_stm32l1xx_is_enabled_rtc( t_ttc_interrupt_config_rtc* Config, e_ttc_interrupt_type Type ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)

#warning Function interrupt_stm32l1xx_is_enabled_rtc() is empty.

    TTC_RETURN( 0 );   // will perform stack overflow check and return given value
}

#endif //}EXTENSION_TTC_rtc
#ifdef EXTENSION_ttc_spi   //{

t_bool interrupt_stm32l1xx_is_enabled_spi( t_ttc_interrupt_config_spi* Config, e_ttc_interrupt_type Type ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)

#warning Function interrupt_stm32l1xx_is_enabled_spi() is empty.

    TTC_RETURN( 0 );   // will perform stack overflow check and return given value
}

#endif //}EXTENSION_TTC_spi
#ifdef EXTENSION_ttc_timer //{

t_bool interrupt_stm32l1xx_is_enabled_timer( t_ttc_interrupt_config_timer* Config, e_ttc_interrupt_type Type ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)

#warning Function interrupt_stm32l1xx_is_enabled_timer() is empty.

    TTC_RETURN( 0 );   // will perform stack overflow check and return given value
}

#endif //}EXTENSION_TTC_timer
#ifdef EXTENSION_ttc_i2c   //{

t_bool interrupt_stm32l1xx_is_enabled_i2c( t_ttc_interrupt_config_i2c* Config, e_ttc_interrupt_type Type ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)

#warning Function interrupt_stm32l1xx_is_enabled_i2c() is empty.

    TTC_RETURN( 0 );   // will perform stack overflow check and return given value
}
void I2C_General_IRQHandler( t_physical_index interrupt_stm32l1xx_PhysicalIndex, t_register_stm32l1xx_i2c* I2Cx ) {
    t_ttc_interrupt_config_i2c* interrupt_stm32l1xx_ConfigUsart = A( ttc_interrupt_config_i2c, interrupt_stm32l1xx_PhysicalIndex );
    Assert_INTERRUPT( interrupt_stm32l1xx_ConfigUsart != NULL, E_ttc_assert_origin_auto );  // should never happen: the usart indexed by interrupt_stm32l1xx_PhysicalIndex has not been initialized!

    t_register_stm32l1xx_i2c_sr1 interrupt_stm32l1xx_StatusRegister = I2Cx->SR1;

    if ( interrupt_stm32l1xx_StatusRegister.Bits.SB == 1 ) {
        interrupt_stm32l1xx_ConfigUsart->isr_StartBitSent( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_StartBitSent );
        interrupt_stm32l1xx_StatusRegister.Bits.SB = 0;
    }
    else if ( interrupt_stm32l1xx_StatusRegister.Bits.ADDR == 1 ) {
        interrupt_stm32l1xx_ConfigUsart->isr_AddressSent( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_AddressSent );
        interrupt_stm32l1xx_StatusRegister.Bits.ADDR = 0;
    }
    else if ( interrupt_stm32l1xx_StatusRegister.Bits.BTF == 1 ) {
        interrupt_stm32l1xx_ConfigUsart->isr_DataByteTransferFinished( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_10BitHeader );
        interrupt_stm32l1xx_StatusRegister.Bits.BTF = 0;
    }
    else if ( interrupt_stm32l1xx_StatusRegister.Bits.ADD10 == 1 ) {
        interrupt_stm32l1xx_ConfigUsart->isr_10BitHeader( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_10BitHeader );
        interrupt_stm32l1xx_StatusRegister.Bits.ADD10 = 0;
    }
    else if ( interrupt_stm32l1xx_StatusRegister.Bits.STOPF == 1 ) {
        interrupt_stm32l1xx_ConfigUsart->isr_StopReceived( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_StopReceived );
        interrupt_stm32l1xx_StatusRegister.Bits.STOPF = 0;
    }
    else if ( interrupt_stm32l1xx_StatusRegister.Bits.RXNE == 1 ) {
        interrupt_stm32l1xx_ConfigUsart->isr_ReceiveBufferNotEmpty( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_ReceiveBufferNotEmpty );
        interrupt_stm32l1xx_StatusRegister.Bits.RXNE = 0;
    }
    else if ( interrupt_stm32l1xx_StatusRegister.Bits.TxE == 1 ) {
        interrupt_stm32l1xx_ConfigUsart->isr_TransmitBufferEmpty( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_TransmitBufferEmpty );
        interrupt_stm32l1xx_StatusRegister.Bits.TxE = 0;
    }
    else if ( interrupt_stm32l1xx_StatusRegister.Bits.BERR == 1 ) {
        interrupt_stm32l1xx_ConfigUsart->isr_BusError( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_BusError );
        interrupt_stm32l1xx_StatusRegister.Bits.BERR = 0;
    }
    else if ( interrupt_stm32l1xx_StatusRegister.Bits.ARLO == 1 ) {
        interrupt_stm32l1xx_ConfigUsart->isr_ArbitrationLoss( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_ArbitrationLoss );
        interrupt_stm32l1xx_StatusRegister.Bits.ARLO = 0;
    }
    else if ( interrupt_stm32l1xx_StatusRegister.Bits.AF == 1 ) {
        interrupt_stm32l1xx_ConfigUsart->isr_AckFail( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_AckFail );
        interrupt_stm32l1xx_StatusRegister.Bits.AF = 0;
    }
    else if ( interrupt_stm32l1xx_StatusRegister.Bits.OVR == 1 ) {
        interrupt_stm32l1xx_ConfigUsart->isr_Overrun( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_Overrun );
        interrupt_stm32l1xx_StatusRegister.Bits.OVR = 0;
    }
    else if ( interrupt_stm32l1xx_StatusRegister.Bits.PECERR == 1 ) {
        interrupt_stm32l1xx_ConfigUsart->isr_PECError( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_PECError );
        interrupt_stm32l1xx_StatusRegister.Bits.PECERR = 0;
    }
    else if ( interrupt_stm32l1xx_StatusRegister.Bits.TIMEOUT == 1 ) {
        interrupt_stm32l1xx_ConfigUsart->isr_Timeout( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_Timeout );
        interrupt_stm32l1xx_StatusRegister.Bits.TIMEOUT = 0;
    }
    else if ( interrupt_stm32l1xx_StatusRegister.Bits.SMBALERT == 1 ) {
        interrupt_stm32l1xx_ConfigUsart->isr_SMBusAlert( interrupt_stm32l1xx_PhysicalIndex, interrupt_stm32l1xx_ConfigUsart->Argument_SMBusAlert );
        interrupt_stm32l1xx_StatusRegister.Bits.SMBALERT = 0;
    }

    I2Cx->SR1 = interrupt_stm32l1xx_StatusRegister;
}
void I2C1_EV_IRQHandler() {
    I2C_General_IRQHandler( 1, ( t_register_stm32l1xx_i2c* ) I2C1 );
}
void I2C1_ER_IRQHandler() {
    I2C_General_IRQHandler( 1, ( t_register_stm32l1xx_i2c* ) I2C1 );
}
void I2C2_EV_IRQHandler() {
    I2C_General_IRQHandler( 2, ( t_register_stm32l1xx_i2c* )I2C2 );
}
void I2C2_ER_IRQHandler() {
    I2C_General_IRQHandler( 2, ( t_register_stm32l1xx_i2c* ) I2C2 );
}

#endif //}EXTENSION_ttc_i2c

//----------------------------------Not Implemented-----------------------------------------//


void WWDG_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void PVD_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void FLASH_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}
void RCC_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}
void DMA1_Channel1_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}
void DMA1_Channel2_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}
void DMA1_Channel3_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}
void DMA1_Channel4_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}
void DMA1_Channel5_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}
void DMA1_Channel6_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}
void DMA1_Channel7_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}
void ADC1_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}
void USB_HP_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}
void USB_LP_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}
void DAC_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}
void COMP_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}
void LCD_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}
void SPI1_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}
void SPI2_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}
void USB_FS_WKUP_IRQHandler() {
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );

}

// Note: If create_DeviceDriver.pl adds function+macro blocks below, you have to move them into corresponding #ifdef EXTENSION_ttc_* section above to avoid compile errors!
// Hint: Edit this file with jEdit and set fold marks to { } !

//InsertFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//}Function Definitions
//{ Private Functions (ideally) ************************************************

//InsertPrivateFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//}Private Functions
