/** light_gpio.c ***************************************{
 *
 *                          The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Low-level driver for ttc_light.c.
 *
 *  control multiple light emitting diodes (led)
 *
 *  Created from template device_architecture.c revision 40 at 20191218 12:21:38 UTC
 *
 *  Note: See ttc_light.h for description of high-level light implementation.
 *  Note: See light_gpio.h for description of gpio light implementation.
 *
 *  Authors: Gregor Rebel
}*/

/** Includes ********************************************************{
 *
 * C-Sources include their corresponding header file and all other header files
 * that are required to compile this source file.
 * Includes that provide datatypes being used by function prototypes belong into
 * "light_gpio.h".
 */

#include "light_gpio.h"
#include "../ttc_gpio.h"              // get/set/clear gpio pins
#include "../ttc_heap.h"              // dynamic memory allocation (e.g. malloc() )
#include "../ttc_memory.h"            // basic memory checks
#include "../ttc_task.h"              // stack checking (provides TTC_RETURN() macro)
//#include "../ttc_pwm.h"               // pulse width modulation
#include "light_common.h"             // generic functions shared by low-level drivers of all architectures
//InsertIncludes above (DO NOT DELETE THIS LINE!)

//}Includes
/** Global Variables ************************************************{
 *
 * Always prefix your global variables to reflect the filename to which they belong to!
 * Example:
 * file:     my_cool_file.c
 * variable: t_u8 mcf_Index = 1;
 */



const t_ttc_light_features light_gpio_Features = {
    /** example features
              .MinBitRate       = 4800,
              .MaxBitRate       = 230400,
            */
    // set more feature values...

    .BrightnessMin  = 0,    // minimum selectable brightness
    .BrightnessMax  = 255,  // maximum selectable brightness
    .BrightnessStep = 255   // minimum increasement step of brightness
};

//InsertGlobalVariables above (DO NOT DELETE THIS LINE!)

//}GlobalVariables
/** Private Function Declarations ***********************************{
 *
 * Declare all functions here that should not be called from outside.
 * You may prefix private functions by an underscore to help identify them as private:
 * E.g.: void _privateCalc();
 */

//InsertPrivateFunctionDeclarations above (DO NOT DELETE THIS LINE!)

//}PrivateFunctions
/** Function Definitions ********************************************{
 */

void                  _light_gpio_switch( t_light_gpio_channel_config* GpioChannelConfig, t_bool On ) {
    Assert_LIGHT_EXTRA_Writable( GpioChannelConfig, E_ttc_assert_origin_auto );
    e_ttc_gpio_pin  Pin = GpioChannelConfig->Pin;
    Assert_LIGHT_EXTRA( Pin > 0, E_ttc_assert_origin_auto );

    if ( GpioChannelConfig->LowActive ) {  // will have to pull gpio low to switch on light
        if ( On ) {
            ttc_gpio_clr( Pin );
        }
        else {
            ttc_gpio_set( Pin );
        }
    }
    else {                                 // will have to push gpio highto switch on light
        if ( On ) {
            ttc_gpio_set( Pin );
        }
        else {
            ttc_gpio_clr( Pin );
        }
    }
}
void                   light_gpio_configuration_check( t_ttc_light_config* Config ) {
    Assert_LIGHT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)

    for ( volatile t_u8 ChannelIndex = 1; ChannelIndex <= Config->AmountChannels; ChannelIndex++ ) {
        t_ttc_light_channel_config* ChannelConfig = light_common_get_channel_configuration( Config, ChannelIndex );

        Assert_LIGHT_Writable( ChannelConfig->LowLevel, E_ttc_assert_origin_auto ); // light_gpio_load_defaults() should have allocated dynamic memory and set this pointer!
        Assert_LIGHT( ChannelConfig->LowLevel->gpio.Pin != 0, E_ttc_assert_origin_auto ); // missing gpio pin
        Assert_LIGHT( ChannelConfig->LowLevel->gpio.ModeGPIO != 0, E_ttc_assert_origin_auto ); // missing gpio pin mode
    }
}
e_ttc_light_errorcode  light_gpio_deinit( t_ttc_light_config* Config ) {
    light_gpio_configuration_check( Config );

    for ( t_u8 ChannelIndex = 1; ChannelIndex <= Config->AmountChannels; ChannelIndex++ ) { //
        t_ttc_light_channel_config* ChannelConfig = light_common_get_channel_configuration( Config, ChannelIndex );
        volatile e_ttc_gpio_pin PortPin = ChannelConfig->LowLevel->gpio.Pin;
        ttc_gpio_deinit( PortPin );
    }

    TTC_RETURN( 0 );   // will perform stack overflow check and return given value
}
e_ttc_light_errorcode  light_gpio_init( t_ttc_light_config* Config ) {
    light_gpio_configuration_check( Config );
    for ( t_u8 ChannelIndex = 1; ChannelIndex <= Config->AmountChannels; ChannelIndex++ ) { // initialize gpio pins for all channels
        t_ttc_light_channel_config* ChannelConfig = light_common_get_channel_configuration( Config, ChannelIndex );
        ttc_gpio_init( ChannelConfig->LowLevel->gpio.Pin, ChannelConfig->LowLevel->gpio.ModeGPIO, E_ttc_gpio_speed_max );

        if ( ChannelConfig->LowLevel->gpio.LowActive ) { // initialize gpio for low-active LED
            ttc_gpio_set( ChannelConfig->LowLevel->gpio.Pin ); // disable LED
        }
        else {                                        // initialize gpio for high-active LED
            ttc_gpio_clr( ChannelConfig->LowLevel->gpio.Pin ); // disable LED
        }
    }

    TTC_RETURN( 0 );   // will perform stack overflow check and return given value
}
e_ttc_light_errorcode  light_gpio_load_defaults( t_ttc_light_config* Config ) {
    Assert_LIGHT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // always check incoming pointers!

    Config->Architecture = E_ttc_light_architecture_gpio;         // set type of architecture

    // we yet do not know how many channels have been defined: prepare for all supported channels
    t_light_gpio_channel_config gpio[LIGHT_GPIO_MAX_AMOUNT_CHANNELS];
    ttc_memory_set( gpio, 0, sizeof( gpio[0] ) * LIGHT_GPIO_MAX_AMOUNT_CHANNELS );

    for ( t_u8 ChannelIndex = 0; ChannelIndex < LIGHT_GPIO_MAX_AMOUNT_CHANNELS; ChannelIndex++ ) { // set default gpio mode
        gpio[ChannelIndex].ModeGPIO  = E_ttc_gpio_mode_output_push_pull;
    }

    switch ( Config->LogicalIndex ) { // physical index of light device from static configuration
#ifdef TTC_LIGHT1
        case  1: {
            gpio[0].Pin       = TTC_LIGHT1_GPIO1;
#ifdef TTC_LIGHT1_GPIO12
            gpio[1].Pin       = TTC_LIGHT1_GPIO12;
#endif
#ifdef TTC_LIGHT1_GPIO13
            gpio[2].Pin       = TTC_LIGHT1_GPIO13;
#endif
#ifdef TTC_LIGHT1_GPIO14
            gpio[3].Pin       = TTC_LIGHT1_GPIO14;
#endif
#ifdef TTC_LIGHT1_LOWACTIVE
            gpio[0].LowActive = TTC_LIGHT1_LOWACTIVE;
#endif
#ifdef TTC_LIGHT1_MODE_GPIO1
            gpio[0].ModeGPIO  = TTC_LIGHT1_MODE_GPIO;
#endif
            break;
        }
#endif
#ifdef TTC_LIGHT2
        case 2: {
            gpio[0].Pin       = TTC_LIGHT2_GPIO1;
#ifdef TTC_LIGHT2_GPIO12
            gpio[1].Pin       = TTC_LIGHT2_GPIO12;
#endif
#ifdef TTC_LIGHT2_GPIO13
            gpio[2].Pin       = TTC_LIGHT2_GPIO13;
#endif
#ifdef TTC_LIGHT2_GPIO14
            gpio[3].Pin       = TTC_LIGHT2_GPIO14;
#endif
#ifdef TTC_LIGHT2_LOWACTIVE
            gpio[0].LowActive = TTC_LIGHT2_LOWACTIVE;
#endif
#ifdef TTC_LIGHT2_MODE_GPIO1
            gpio[0].ModeGPIO  = TTC_LIGHT2_MODE_GPIO;
#endif
            break;
        }
#endif
#ifdef TTC_LIGHT3
        case 3: {
            gpio[0].Pin       = TTC_LIGHT3_GPIO1;
#ifdef TTC_LIGHT3_GPIO12
            gpio[1].Pin       = TTC_LIGHT3_GPIO12;
#endif
#ifdef TTC_LIGHT3_GPIO13
            gpio[2].Pin       = TTC_LIGHT3_GPIO13;
#endif
#ifdef TTC_LIGHT3_GPIO14
            gpio[3].Pin       = TTC_LIGHT3_GPIO14;
#endif
#ifdef TTC_LIGHT3_LOWACTIVE
            gpio[0].LowActive = TTC_LIGHT3_LOWACTIVE;
#endif
#ifdef TTC_LIGHT3_MODE_GPIO1
            gpio[0].ModeGPIO  = TTC_LIGHT3_MODE_GPIO;
#endif
            break;
        }
#endif
#ifdef TTC_LIGHT4
        case 4: {
            gpio[0].Pin       = TTC_LIGHT4_GPIO1;
#ifdef TTC_LIGHT4_GPIO12
            gpio[1].Pin       = TTC_LIGHT4_GPIO12;
#endif
#ifdef TTC_LIGHT4_GPIO13
            gpio[2].Pin       = TTC_LIGHT4_GPIO13;
#endif
#ifdef TTC_LIGHT4_GPIO14
            gpio[3].Pin       = TTC_LIGHT4_GPIO14;
#endif
#ifdef TTC_LIGHT4_LOWACTIVE
            gpio[0].LowActive = TTC_LIGHT4_LOWACTIVE;
#endif
#ifdef TTC_LIGHT4_MODE_GPIO1
            gpio[0].ModeGPIO  = TTC_LIGHT4_MODE_GPIO;
#endif
            break;
        }
#endif
#ifdef TTC_LIGHT5
        case 5: {
            gpio[0].Pin       = TTC_LIGHT5_GPIO;
#ifdef TTC_LIGHT5_GPIO2
            gpio[1].Pin       = TTC_LIGHT5_GPIO2;
#endif
#ifdef TTC_LIGHT5_GPIO3
            gpio[2].Pin       = TTC_LIGHT5_GPIO3;
#endif
#ifdef TTC_LIGHT5_GPIO4
            gpio[3].Pin       = TTC_LIGHT5_GPIO4;
#endif
#ifdef TTC_LIGHT5_LOWACTIVE
            gpio[0].LowActive = TTC_LIGHT5_LOWACTIVE;
#endif
#ifdef TTC_LIGHT5_MODE_GPIO1
            gpio[0].ModeGPIO  = TTC_LIGHT5_MODE_GPIO;
#endif
            break;
        }
#endif
#ifdef TTC_LIGHT6
        case 6: {
            gpio[0].Pin       = TTC_LIGHT6_GPIO;
#ifdef TTC_LIGHT6_GPIO2
            gpio[1].Pin       = TTC_LIGHT6_GPIO2;
#endif
#ifdef TTC_LIGHT6_GPIO3
            gpio[2].Pin       = TTC_LIGHT6_GPIO3;
#endif
#ifdef TTC_LIGHT6_GPIO4
            gpio[3].Pin       = TTC_LIGHT6_GPIO4;
#endif
#ifdef TTC_LIGHT6_LOWACTIVE
            gpio[0].LowActive = TTC_LIGHT6_LOWACTIVE;
#endif
#ifdef TTC_LIGHT6_MODE_GPIO1
            gpio[0].ModeGPIO  = TTC_LIGHT6_MODE_GPIO;
#endif
            break;
        }
#endif
#ifdef TTC_LIGHT7
        case 7: {
            gpio[0].Pin       = TTC_LIGHT7_GPIO;
#ifdef TTC_LIGHT7_GPIO2
            gpio[1].Pin       = TTC_LIGHT7_GPIO2;
#endif
#ifdef TTC_LIGHT7_GPIO3
            gpio[2].Pin       = TTC_LIGHT7_GPIO3;
#endif
#ifdef TTC_LIGHT7_GPIO4
            gpio[3].Pin       = TTC_LIGHT7_GPIO4;
#endif
#ifdef TTC_LIGHT7_LOWACTIVE
            gpio[0].LowActive = TTC_LIGHT7_LOWACTIVE;
#endif
#ifdef TTC_LIGHT7_MODE_GPIO1
            gpio[0].ModeGPIO  = TTC_LIGHT7_MODE_GPIO;
#endif
            break;
        }
#endif
#ifdef TTC_LIGHT8
        case 8: {
            gpio[0].Pin       = TTC_LIGHT8_GPIO;
#ifdef TTC_LIGHT8_GPIO2
            gpio[1].Pin       = TTC_LIGHT8_GPIO2;
#endif
#ifdef TTC_LIGHT8_GPIO3
            gpio[2].Pin       = TTC_LIGHT8_GPIO3;
#endif
#ifdef TTC_LIGHT8_GPIO4
            gpio[3].Pin       = TTC_LIGHT8_GPIO4;
#endif
#ifdef TTC_LIGHT8_LOWACTIVE
            gpio[0].LowActive = TTC_LIGHT8_LOWACTIVE;
#endif
#ifdef TTC_LIGHT8_MODE_GPIO1
            gpio[0].ModeGPIO  = TTC_LIGHT8_MODE_GPIO;
#endif
            break;
        }
#endif
#ifdef TTC_LIGHT9
        case 9: {
            gpio[0].Pin       = TTC_LIGHT9_GPIO;
#ifdef TTC_LIGHT9_GPIO2
            gpio[1].Pin       = TTC_LIGHT9_GPIO2;
#endif
#ifdef TTC_LIGHT9_GPIO3
            gpio[2].Pin       = TTC_LIGHT9_GPIO3;
#endif
#ifdef TTC_LIGHT9_GPIO4
            gpio[3].Pin       = TTC_LIGHT9_GPIO4;
#endif
#ifdef TTC_LIGHT9_LOWACTIVE
            gpio[0].LowActive = TTC_LIGHT9_LOWACTIVE;
#endif
#ifdef TTC_LIGHT9_MODE_GPIO1
            gpio[0].ModeGPIO  = TTC_LIGHT9_MODE_GPIO;
#endif
            break;
        }
#endif
#ifdef TTC_LIGHT10
        case 10: {
            gpio[0].Pin       = TTC_LIGHT10_GPIO1;
#ifdef TTC_LIGHT10_GPIO2
            gpio[1].Pin       = TTC_LIGHT10_GPIO2;
#endif
#ifdef TTC_LIGHT10_GPIO3
            gpio[2].Pin       = TTC_LIGHT10_GPIO3;
#endif
#ifdef TTC_LIGHT10_GPIO4
            gpio[3].Pin       = TTC_LIGHT1_GPIO14;
#endif
#ifdef TTC_LIGHT10_LOWACTIVE
            gpio[0].LowActive = TTC_LIGHT10_LOWACTIVE;
#endif
#ifdef TTC_LIGHT10_MODE_GPIO1
            gpio[0].ModeGPIO  = TTC_LIGHT10_MODE_GPIO;
#endif
            break;
        }
#endif

        default: ttc_assert_halt_origin( E_ttc_assert_origin_auto ); break; // unknown architecture driver (maybe you missed to activate a low-level driver?
    }

    // Config->AmountChannels =  amount of configured channel gpios
    for ( t_u8 AmountChannels = 0; ( AmountChannels < LIGHT_GPIO_MAX_AMOUNT_CHANNELS ) && ( gpio[AmountChannels].Pin ); ) {
        AmountChannels++;
        Config->AmountChannels = AmountChannels;
    }
    ttc_assert_test_origin( ( Config->AmountChannels > 0 ) && ( Config->AmountChannels <= LIGHT_GPIO_MAX_AMOUNT_CHANNELS ), E_ttc_assert_origin_auto ); // invalid amount of channels. Check constant configuration!

    if ( Config->LowLevelConfig == NULL ) { // allocate new low-level configuration
        Config->LowLevelConfig = ttc_heap_alloc_zeroed( sizeof( t_light_gpio_config ) );
    }
    else {                                  // reset existing low-level configuration
        ttc_memory_set( Config->LowLevelConfig, 0, sizeof( Config->LowLevelConfig->gpio ) * Config->AmountChannels );
    }

    for ( t_u8 ChannelIndex = 1; ChannelIndex <= Config->AmountChannels; ChannelIndex++ ) { // load default channel settings
        t_ttc_light_channel_config* ChannelConfig = light_common_get_channel_configuration( Config, ChannelIndex );

        if ( ChannelConfig->LowLevel == NULL ) {
            ChannelConfig->LowLevel = ttc_heap_alloc_zeroed( sizeof( ChannelConfig->LowLevel->gpio ) );
        }
        else {
            ttc_memory_set( ChannelConfig->LowLevel, 0, sizeof( ChannelConfig->LowLevel->gpio ) );
        }

        // copy gpio port settings
        ChannelConfig->LowLevel->gpio.LowActive = gpio[ChannelIndex - 1].LowActive;
        ChannelConfig->LowLevel->gpio.ModeGPIO  = gpio[ChannelIndex - 1].ModeGPIO;
        ChannelConfig->LowLevel->gpio.Pin       = gpio[ChannelIndex - 1].Pin;

        // assign feature set
        ChannelConfig->Init.Features = &light_gpio_Features;
    }

    TTC_RETURN( 0 ); // will perform stack overflow check and return given value
}
void                   light_gpio_off( t_ttc_light_config* Config ) {
    Assert_LIGHT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)

    for ( t_u8 ChannelIndex = Config->AmountChannels; ChannelIndex > 0; ChannelIndex-- ) {
        t_ttc_light_channel_config* ChannelConfig = light_common_get_channel_configuration( Config, ChannelIndex );
        _light_gpio_switch( &( ChannelConfig->LowLevel->gpio ), FALSE );
    }
}
void                   light_gpio_on( t_ttc_light_config* Config ) {
    Assert_LIGHT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)

    for ( t_u8 ChannelIndex = Config->AmountChannels; ChannelIndex > 0; ChannelIndex-- ) {
        t_ttc_light_channel_config* ChannelConfig = light_common_get_channel_configuration( Config, ChannelIndex );
        if ( ChannelConfig->Init.Brightness == 0xff ) { // full on: simply switch on constantly
            _light_gpio_switch( &( ChannelConfig->LowLevel->gpio ), TRUE );
        }
        else if ( ChannelConfig->Init.Brightness > ChannelConfig->Init.Features->BrightnessMin ) {
            TODO( "Implement PWM dimming!" );
        }
    }
}
void                   light_gpio_prepare() {
    // nothing to do here
}
void                   light_gpio_reset( t_ttc_light_config* Config ) {
    light_gpio_deinit( Config );
}
void                   light_gpio_brightness_update( t_ttc_light_config* Config ) {
    Assert_LIGHT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)
    u_ttc_light_type_config* Light = Config->Light;
    Assert_LIGHT_EXTRA_Writable( Light, E_ttc_assert_origin_auto );
    Assert_LIGHT_EXTRA_Writable( Light->channels4.Channel, E_ttc_assert_origin_auto );

    t_ttc_light_channel_config* Channel;
    switch ( Config->AmountChannels ) {
        case 4:
            Channel = &( Light->channels4.Channel[3] );
            _light_gpio_switch( &( Channel->LowLevel->gpio ), ( Channel->Init.Brightness == Channel->Init.Features->BrightnessMax ) ? TRUE : FALSE );
        // falling through to case 3..
        case 3:
            Channel = &( Light->channels4.Channel[2] );
            _light_gpio_switch( &( Channel->LowLevel->gpio ), ( Channel->Init.Brightness == Channel->Init.Features->BrightnessMax ) ? TRUE : FALSE );
        // falling through to case 2..
        case 2:
            Channel = &( Light->channels4.Channel[1] );
            _light_gpio_switch( &( Channel->LowLevel->gpio ), ( Channel->Init.Brightness == Channel->Init.Features->BrightnessMax ) ? TRUE : FALSE );
        // falling through to case 1..
        case 1:
            Channel = &( Light->channels4.Channel[0] );
            _light_gpio_switch( &( Channel->LowLevel->gpio ), ( Channel->Init.Brightness == Channel->Init.Features->BrightnessMax ) ? TRUE : FALSE );
            break;
        default: ttc_assert_halt_origin( E_ttc_assert_origin_auto ); break; // invalid/unsupported amount of channels.
    }

    // will switch on individual light channel only if brightness value == BrightnessMax
}
//InsertFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//}FunctionDefinitions
/** Private Functions ***********************************************{
 *
 * Private functions are declared in this c-file. They can be called from outside
 * via an extern declaration (or by ignoring the corresponding compiler warning).
 * Though it is possible to call them from outside, it is not intended to do so!
 * Private functions may be changed, renamed or deleted without notification by
 * the maintainer of this file.
 */

//InsertPrivateFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//}Private Functions
