/** example_ttc_<device>.c **********************************************{
 *
 *                              The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Example for <architecture_description>
 *
 *  Basic flow of booting examples:
 *  (1) ttc_extensions.c:ttc_extensions_start() calls example_ttc_<device>_prepare() to 
 *      - initialize all required devices
 *      - initializes its global and local static variables
 *      - create all required task or statemachine instances
 *      - return immediately 
 *
 *  Multitasking is available:
 *  (2) Scheduler is started
 *  (3) Tasks being created by example_ttc_<device>_prepare() get executed.
 *      Each task uses its private stack as given to ttc_task_create() before.
 *
 *  Multitasking is NOT available:
 *  (2) Minimalistic main loop is started
 *  (3) Main loop calls all registered statemachine functions in an endless loop.
 *      All statemachine functions use the same global stack.
 *
 *  Created from template <TEMPLATE_FILE> revision 27 at <DATE>
 *
 *  Authors: Gregor Rebel
 *  
}*/

//{ Includes *************************************************************

// Include all header files being required to compile this file.
#include "example_ttc_<device>.h"
#include "../compile_options.h"       // dynamically created file defining all "COMPILE_OPTS += -D..." constants from makefiles
#include "../ttc-lib/ttc_<device>.h"  // high-level driver of device to use
#include "../ttc-lib/ttc_basic.h"     // basic datatypes and definitions
#include "../ttc-lib/ttc_heap.h"      // dynamic memory allocations and safe arrays
#include "../ttc-lib/ttc_task.h"      // allows to use application in single- and multitasking setup
#include "../ttc-lib/ttc_systick.h"   // exact delays and timeouts

//}Includes
//{ Global Variables *****************************************************

// Note: Adding a unique prefix to all global and static variable makes it
//       easier to get an overview of memory usage via _/calculateMemoryUsage.pl

//}Global Variables
//{ Private Function Declarations ****************************************
//
// Private functions should only be called from this source code, though it is possible.
// Declaring a function as private removes it from list of public functions and makes it
// easier for other developers to understand your application interface.
//

/** Task running this example
 *
 * H2: Description of Example
 * Task function _example_<device>_task() PLACE_DESCRIPTION_HERE!
 *
 * H2: What is a Task?
 * A task is a function being registered via ttc_task_create().
 *
 * H3: Tasks can operate in one of two modes:
 * - Single Tasking Mode
 *   If no multitasking scheduler has been activated, then all registered task functions will be called endlessly from a super loop.
 *   Each task function has to implement a statemachine that will check for new state and return fast to give cpu to other tasks.
 *
 * - Multi Tasking Mode
 *   If a multitasking scheduler (e.g. FreeRTOS) has been activated, then a central scheduler will actively switch among all registered tasks.
 *   Each task runs a statemachine in a endless loop. 
 *   It is advised that each task calls ttc_task_yield() to give cpu to other tasks while waiting for events. 
 *   
 * The ToolChain allows to write task functions that support both modes.
 * The advantage is that such multi mode task functions can be reused more easily.
 *
 * H3: Basic structure of multi mode task functions
 *   <CODE>
 *   task_foo(void *Argument) {
 *     t_my_data* MyData = (t_my_data*) Argument;
 *     if (MyData->FirstRunCompleted == 0) {
 *       // first run: initialize task
 *       MyData->FirstRunCompleted = 1;
 *     }
 *  
 *     do {
 *        // run statemachine
 *     } while (ttc_task_can_yield());
 *   }
 * </CODE>
 * 
 * Note: This task is created automatically by example_ttc_<device>_prepare() at system startup.
 *
 * @param Argument  passed argument #4 from ttc_task_create() call.
 */
void _example_<device>_task(void *Argument);

//}Private Function Declarations
//{ Function Definitions *************************************************

void example_ttc_<device>_prepare() {                   // prepare this example for being started
    /** Note: The _prepare() function is called in single task mode before the scheduler is started.
      *       It has to return to allow all extensions to prepare.
      *       After all extensions finished their preparation, the created tasks are run.
      *       If no multitasking scheduler has been activated, all created tasks are run one after 
      *       another in an endless loop. See ttc_task_start_scheduler() for details.
      *       This scheme allows to enable multiple statemachine functions in a flexible way.  
      */

    if (1) { // create one instance of your example (copy and adjust for multiple instances) 
      
      // LogicalIndex = 1: use device defined as TTC_<DEVICE>1 in makefile
      t_u8 LogicalIndex = 1;
      
      // allocate data of instance as local static (allows compiler to detect if it fits into RAM)
      static t_example_<device>_data <device>_data;
      
      // reset data
      ttc_memory_set(&<device>_data, 0, sizeof(<device>_data));
      
      // loading configuration of first <device> device
      <device>_data.Config_<device> = ttc_<device>_get_configuration(LogicalIndex);
      
      // change configuration before initializing device
      //
      // <device>_data.Config_<device>->  = ;
      //...
      
      // initializing our <device> device here makes debugging easier (still single-tasking)
      // Logical index is stored inside each device configuration
      ttc_<device>_init(LogicalIndex);
  
      // Register a function to be run after all system has finished booting.
      // ttc_task_create() works in single- and multitasking environments.
      ttc_task_create(
                       _example_<device>_task,          // function to start as thread
                       "tExample_<DEVICE>",             // thread name (just for debugging)
                       256,                             // stack size (adjust to amount of local variables of _example_<device>_task() AND ALL ITS CALLED FUNTIONS!) 
                       &<device>_data, // passed as argument to _example_<device>_task()
                       1,                               // task priority (higher values mean more process time)
                       NULL                             // can return a handle to created task
                     );
    }
}

void _example_<device>_task(void *Argument) {         // example task will be run when multitasking scheduler (e.g. FreeRTOS) has been started
    Assert_<DEVICE>_Writable(Argument, E_ttc_assert_origin_auto); // always check pointer arguments with Assert() before dereferencing them!
    t_example_<device>_data* Data = ( t_example_<device>_data* ) Argument; // we're expecting this pointer type

    // get pointer to device configuration
    t_ttc_<device>_config* Config_<device> = Data->Config_<device>;
    
    do {
        switch (Data->State) {
            case E_example_<device>_state_Reset: { // first run: initialize task data 
                ttc_systick_delay_init( &Data->Delay, 0 ); // zero delay will expire immediatelx 
                
                // set state to reach on next _example_<device>_task() call
                Data->State = E_example_<device>_state_Initialized;
            }
            case E_example_<device>_state_Initialized: {
                if ( ttc_systick_delay_expired( & Data->Delay ) ) { // example periodic delay will expire on first run
                    ttc_systick_delay_init( & Data->Delay, 
                                            500000 // 500000us == half a second   
                                          ); // set new delay time
            
                    // implement state here ...
                    
                    // set state to reach on next _example_<device>_task() call
                    //Data->State = E_example_<device>_state_...; 
                }
            }
        }
    } while ( ttc_task_can_yield() ); // multitasking: give cpu to other tasks; singletasking: exit while after one iteration
}

//}Function Definitions
