/** { ttc_light.c ****************************************************
 *
 *                          The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  High-Level driver for light devices.
 *
 *  Implementation of high-level interface.
 *  This file implements all functionality that is common to all supported
 *  architectures.
 *
 *  Most functions of this driver are defined as macros and being implemented
 *  in interfaces/ttc_light_interface.c or in low-level drivers light/light_*.c.
 *
 *  See corresponding ttc_light.h for documentation of all high-level functions.
 *
 *  Created from template ttc_device.c revision 54 at 20191218 12:21:38 UTC
 *
 *  Authors: Gregor Rebel
 *
}*/

//{ Includes *******************************************************************
//
// C-Sources include their corresponding header file and all other header files
// that are required to compile this source file.
// Includes that provide datatypes being used by function prototypes belong into
// "ttc_light.h".
//
#include "ttc_light.h"          // automatically includes current low-level driver
#include "light/light_common.h" // common light functions usable by low- and high-level driver
#include "ttc_heap.h"           // dynamic memory allocationand safe arrays
#include "ttc_task.h"           // disable/ enable multitasking scheduler
#include "ttc_list.h"           // single linked lists
#include "ttc_systick.h"        // universal delays
#ifdef EXTENSION_ttc_interrupt
    #include "ttc_interrupt.h"  // globally disable/ enable interrupts
#endif

//}Includes

#if TTC_LIGHT_AMOUNT == 0
    #warning No LIGHT devices defined, did you forget to activate something? - Define at least TTC_LIGHT1 as one from e_ttc_light_architecture in your makefile!
#endif

/** { Global Variables *****************************************************
 *
 * Global variables defined here are common for all architectures of light devices.
 *
 */


// for each initialized device, a pointer to its generic and architecture specific definitions is stored
A_define( t_ttc_light_config*, ttc_light_configs, TTC_LIGHT_AMOUNT );

// ==TRUE: ttc_light_task_blinking() has been started

t_ttc_light_task_blinking ttc_light_private_TaskBlinking; // private data of _ttc_light_task_blinking()


//}Global Variables

//InsertGlobalVariables above (DO NOT DELETE THIS LINE!)

//} Global Variables
//{ private Function prototypes ********************************************
//
// Private functions can normally only be accessed from inside this file.
// Declaring helper functions as private makes your public interface more understandable.
// Convention: Private functions start with an underscore "_".
// Example:    void _ttc_light(t_ttc_light_config* Config)

/** Checks + corrects entries in given Config to meet limits of current architecture
 *
 * @param LogicalIndex    1.. logical index of LIGHT device. Each logical device <n> is defined via TTC_LIGHT<n>* constants in compile_options.h and extensions.active/makefile
 */
void _ttc_light_configuration_check( t_u8 LogicalIndex );

/** task controlling all automatically blinking LIGHTs
 *
 * Different implementations exist depending on current hardware platform and low-level drivers:
 * - Simple on/off keying uses least cpu time.
 * - Software controlled PWM uses most cpu time.
 * - DMA controlled PWM requires working ttc_pwm driver for current hardware.
 *
 * @param Argument (void*)
 */
void _ttc_light_task_blinking( void* Argument );

/** Finds next non-zero time and its index for given blinking configuration and start index
 *
 * Usage:
 * t_u8 State = E_ttc_light_blinking_state_Overflow;                 // start value
 * t_u32 Time;                            // will be loaded with next valid time value
 * t_ttc_light_blinking Blinking = { ... }; // some blinking definition
 *
 * while (1) {
 *   do {
 *     State = _ttc_light_find_next_state( State, &Time, Blinking );
 *     // implement current blinking division..
 *   } while (State != E_ttc_light_blinking_state_Overflow);
 *
 *   // do something else and then restart period...
 * }
 *
 * @param State  -1..4 == E_ttc_light_blinking_state_Overflow,0,1,2,3,4 start index in Time[] minus one (E_ttc_light_blinking_state_Overflow -> Time[0], 0 -> Time[1], ...)
 * @param Blinking   full (all Time[] entries != 0) or partial (some Time[] entries == 0) blinking definition
 * @return =E_ttc_light_blinking_state_Overflow: no further non-zero time was found (*Time=0); 0..4 = index of next Blinking->times.array[] entry to use
 */
t_u8 _ttc_light_find_next_state( t_u8 State, const t_ttc_light_blinking* Blinking );

/** Manage time delays for each period section of single light
 *
 * New, yet uncontrolled, light is detected and initialized.
 * Current Time[] delay is checked.
 * If Expired then next, non-zero Time[] entry and its index are selected and delay is started.
 * LIGHT state is changed according to current Time[] index.
 *
 * @param Argument      (t_ttc_light_task_blinking*) task data
 * @param PreviousItem  (t_ttc_light_config*) previous list item
 * @param Item          (t_ttc_light_config*) current list item
 * @return  ==FALSE: do not remove item from list
 */
BOOL _ttc_light_blinking_transition( void* Argument, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item );
//InsertPrivateFunctionDeclarations above (DO NOT DELETE THIS LINE!)

//}PrivateFunctions
//{ Function definitions ***************************************************

void                        ttc_light_blink( t_u8 LogicalIndex, t_u8 ChannelIndex, const t_ttc_light_blinking* Blinking ) {
    t_ttc_light_config* Config = ttc_light_get_configuration( LogicalIndex );

    t_ttc_light_channel_config* ConfigChannel = ttc_light_get_channel_configuration( LogicalIndex, ChannelIndex );
    ConfigChannel->Init.Blinking = Blinking;

    if ( Blinking != NULL ) {  // enable automatic blinking for this light
        Assert_LIGHT_Readable( ( void* ) Blinking, E_ttc_assert_origin_auto ); // always check incoming pointer arguments

        ttc_task_critical_begin();
        if ( ttc_light_private_TaskBlinking.State == E_ttc_light_task_State_Reset ) { // task not running: create it
            ttc_light_private_TaskBlinking.State = E_ttc_light_task_State_Starting;
            ttc_list_init_cast( &( ttc_light_private_TaskBlinking.ListActiveLIGHTs ),  "ListActiveLIGHTs" );
            ttc_list_init_cast( &( ttc_light_private_TaskBlinking.ListWaitingLIGHTs ), "ListWaitingLIGHTs" );
            ttc_task_create( _ttc_light_task_blinking, "ttc_light_task_blinking", 128, &ttc_light_private_TaskBlinking, 0, NULL );
        }
        if ( !Config->Flags.IsQueued4Blinking ) {     // light is not yet controlled by _ttc_light_task_blinking(): add it to list of controlled lights
            Config->Flags.IsQueued4Blinking = 1;
            ttc_list_push_back_single_cast( &( ttc_light_private_TaskBlinking.ListWaitingLIGHTs ), Config );
        }
        ttc_task_critical_end();
    }
    else {                     // disable automatic blinking for this light
        ConfigChannel->Init.Blinking = NULL; // LIGHT configuration will automatically being removed from ListWaitingLIGHTs or ListActiveLIGHTs by _ttc_light_blinking_transition()
        //? ttc_light_off( LogicalIndex );
    }
}
void                        ttc_light_brightness1( t_u8 LogicalIndex, t_u8 Level ) {
    t_ttc_light_config* Config = ttc_light_get_configuration( LogicalIndex );

    t_ttc_light_channel_config* Channel = Config->Light->channels1.Channel;
    Channel[0].Init.Brightness = Level;

    _driver_light_brightness_update( Config );
}
void                        ttc_light_brightness2( t_u8 LogicalIndex, t_u8 Level1, t_u8 Level2 ) {
    t_ttc_light_config* Config = ttc_light_get_configuration( LogicalIndex );

    t_ttc_light_channel_config* Channel = Config->Light->channels2.Channel;

    if ( Config->AmountChannels >= 2 ) {
        Channel[1].Init.Brightness = Level2;
    }
    Channel[0].Init.Brightness = Level1;

    _driver_light_brightness_update( Config );
}
void                        ttc_light_brightness3( t_u8 LogicalIndex, t_u8 Red, t_u8 Green, t_u8 Blue ) {
    t_ttc_light_config* Config = ttc_light_get_configuration( LogicalIndex );

    t_ttc_light_channel_config* Channel = Config->Light->channels3.Channel;

    if ( Config->AmountChannels >= 2 ) {
        if ( Config->AmountChannels >= 3 ) {
            Channel[2].Init.Brightness = Blue;
        }
        Channel[1].Init.Brightness = Green;
    }
    Channel[0].Init.Brightness = Red;

    _driver_light_brightness_update( Config );
}
void                        ttc_light_brightness4( t_u8 LogicalIndex, t_u8 Red, t_u8 Green, t_u8 Blue, t_u8 White ) {
    t_ttc_light_config* Config = ttc_light_get_configuration( LogicalIndex );

    t_ttc_light_channel_config* Channel = Config->Light->channels4.Channel;

    if ( Config->AmountChannels >= 2 ) {
        if ( Config->AmountChannels >= 3 ) {
            if ( Config->AmountChannels >= 4 ) {
                Channel[3].Init.Brightness = White;
            }
            Channel[2].Init.Brightness = Blue;
        }
        Channel[1].Init.Brightness = Green;
    }
    Channel[0].Init.Brightness = Red;

    _driver_light_brightness_update( Config );
}
void                        ttc_light_brightness_update( t_u8 LogicalIndex ) {
    t_ttc_light_config* Config = ttc_light_get_configuration( LogicalIndex );

    // high-level code being executed before calling low-level driver...

    // pass function call to interface or low-level driver function
    _driver_light_brightness_update( Config );

    // high-level code being executed after calling low-level driver..
}
void                        ttc_light_deinit( t_u8 LogicalIndex ) {
    ttc_task_critical_begin(); // avoid concurrent access to driver configuration

    t_ttc_light_config* Config = ttc_light_get_configuration( LogicalIndex );

    if ( Config->Flags.Initialized ) {
        for ( t_u8 ChannelIndex = 1; ChannelIndex <= Config->AmountChannels; ChannelIndex++ ) { //
            t_ttc_light_channel_config* ConfigChannel = ttc_light_get_channel_configuration( LogicalIndex, ChannelIndex );
            ConfigChannel->Init.Brightness = ConfigChannel->Init.Features->BrightnessMax;
        }
        e_ttc_light_errorcode Result = _driver_light_deinit( Config );
        if ( Result == E_ttc_light_errorcode_OK )
        { Config->Flags.Initialized = 0; }
    }

    ttc_task_critical_end(); // other tasks now may access this driver
}
t_u8                        ttc_light_get_max_index() {
    TTC_RETURN( TTC_LIGHT_AMOUNT ); // will perform stack overflow check + return value
}
t_ttc_light_config*         ttc_light_get_configuration( t_u8 LogicalIndex ) {
    Assert_LIGHT( LogicalIndex > 0, E_ttc_assert_origin_auto ); // logical index starts at 1
    Assert_LIGHT( LogicalIndex <= TTC_LIGHT_AMOUNT, E_ttc_assert_origin_auto ); // invalid index given (did you configure engnough ttc_light devices in your makefile?)
    t_ttc_light_config* Config = A( ttc_light_configs, LogicalIndex - 1 ); // will assert if outside array bounds

    if ( !Config ) {
        //? ttc_interrupt_critical_begin(); // avoid concurrent access to driver configuration

        Config = A( ttc_light_configs, LogicalIndex - 1 ); // have to reload with disabled interrupts to avoid racing conditions
        if ( !Config ) {
            Config = A( ttc_light_configs, LogicalIndex - 1 ) = ttc_heap_alloc_zeroed( sizeof( t_ttc_light_config ) );
            Config->Flags.Initialized = 0;             // make sure that deinit() is not called during load_defaults()
            ttc_light_load_defaults( LogicalIndex );
        }

        //? ttc_interrupt_critical_end(); // other tasks now may access this driver
    }

    Assert_LIGHT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // memory corrupted?
    TTC_RETURN( Config ); // will perform stack overflow check + return value
}
t_ttc_light_channel_config* ttc_light_get_channel_configuration( t_u8 LogicalIndex, t_u8 ChannelIndex ) {
    Assert_LIGHT( ChannelIndex > 0, E_ttc_assert_origin_auto ); // logical index starts at 1. Check implementation of caller!

    t_ttc_light_config* Config = ttc_light_get_configuration( LogicalIndex );
    return light_common_get_channel_configuration( Config, ChannelIndex );
}
e_ttc_light_errorcode       ttc_light_init( t_u8 LogicalIndex ) {
    ttc_task_critical_begin(); // avoid concurrent access to driver configuration
    t_ttc_light_config* Config = ttc_light_get_configuration( LogicalIndex );

    // This value was not available during ttc_light_prepare() as ttc_systick_prepare() is called later
    ttc_light_private_TaskBlinking.MinimumDelay = ttc_systick_delay_get_minimum() * 4;

    // set some general values
    for ( t_u8 ChannelIndex = 1; ChannelIndex <= Config->AmountChannels; ChannelIndex++ ) { // limit brightness on all channels to defined range
        t_ttc_light_channel_config* ConfigChannel = ttc_light_get_channel_configuration( LogicalIndex, ChannelIndex );
        ConfigChannel->Init.Brightness = ttc_basic_limit( ConfigChannel->Init.Brightness, ConfigChannel->Init.Features->BrightnessMin, ConfigChannel->Init.Features->BrightnessMax );
    }

    // check configuration to meet all limits of current architecture
    _ttc_light_configuration_check( LogicalIndex );

    e_ttc_light_errorcode Error = _driver_light_init( Config );
    if ( ! Error ) // == 0
    { Config->Flags.Initialized = 1; }

    // run additional high-level initialization
    for ( t_u8 ChannelIndex = 1; ChannelIndex <= Config->AmountChannels; ChannelIndex++ ) { // limit brightness on all channels to defined range
        t_ttc_light_channel_config* ConfigChannel = ttc_light_get_channel_configuration( LogicalIndex, ChannelIndex );
        ttc_light_blink( LogicalIndex, ChannelIndex, ConfigChannel->Init.Blinking );
    }

    Assert_LIGHT( Config->Architecture != 0, E_ttc_assert_origin_auto ); // Low-Level driver must set this value!
    ttc_task_critical_end(); // other tasks now may access this driver

    TTC_RETURN( Error ); // will perform stack overflow check + return value
}
void                        ttc_light_init_all() {

    for ( t_u16 IndexLIGHT = 1; IndexLIGHT <= ttc_light_get_max_index(); IndexLIGHT++ ) { // initialize all configured LIGHTs
        ttc_light_init( IndexLIGHT );
    }
}
e_ttc_light_errorcode       ttc_light_load_defaults( t_u8 LogicalIndex ) {
    t_ttc_light_config* Config = ttc_light_get_configuration( LogicalIndex );

    u_ttc_light_architecture* LowLevelConfig = NULL;
    u_ttc_light_type_config*  Light          = NULL;
    if ( Config->Flags.Initialized ) {
        ttc_light_deinit( LogicalIndex );

        // rescue pointer to dynamic allocated memory
        LowLevelConfig = Config->LowLevelConfig;
        Light          = Config->Light;
    }

    // reset full configuration except pointer to light configuration
    ttc_memory_set( Config, 0, sizeof( t_ttc_light_config ) );
    Config->LogicalIndex = LogicalIndex;

    // restore pointer to dynamic allocated memory
    Config->LowLevelConfig = LowLevelConfig;
    Config->Light          = Light;


    switch ( LogicalIndex ) { // load architecture from constant configuration
#ifdef TTC_LIGHT1
        case  1: Config->Architecture = TTC_LIGHT1;  break;
#endif
#ifdef TTC_LIGHT2
        case  2: Config->Architecture = TTC_LIGHT2;  break;
#endif
#ifdef TTC_LIGHT3
        case  3: Config->Architecture = TTC_LIGHT3;  break;
#endif
#ifdef TTC_LIGHT4
        case  4: Config->Architecture = TTC_LIGHT4;  break;
#endif
#ifdef TTC_LIGHT5
        case  5: Config->Architecture = TTC_LIGHT5;  break;
#endif
#ifdef TTC_LIGHT6
        case  6: Config->Architecture = TTC_LIGHT6;  break;
#endif
#ifdef TTC_LIGHT7
        case  7: Config->Architecture = TTC_LIGHT7;  break;
#endif
#ifdef TTC_LIGHT8
        case  8: Config->Architecture = TTC_LIGHT8;  break;
#endif
#ifdef TTC_LIGHT9
        case  9: Config->Architecture = TTC_LIGHT9;  break;
#endif
#ifdef TTC_LIGHT10
        case 10: Config->Architecture = TTC_LIGHT10; break;
#endif
        default: ttc_assert_halt_origin( E_ttc_assert_origin_auto ); break; // unknown/unsupported architecture. Did you enable all required GPIO low-level drivers? Did you add a TTC_LIGHT<LogicalIndex> define for given LogicalIndex to your makefile?
    }

    // Let low-level driver initialize remaining fields
    e_ttc_light_errorcode Error = ttc_light_interface_load_defaults( Config );

    // load generic default values for all channels
    for ( t_u8 ChannelIndex = 1; ChannelIndex <= Config->AmountChannels; ChannelIndex++ ) { //
        t_ttc_light_channel_config* ConfigChannel = ttc_light_get_channel_configuration( LogicalIndex, ChannelIndex );
        ConfigChannel->Init.Brightness = ConfigChannel->Init.Features->BrightnessMax;
    }

    // check if low-level driver returned a valid configuration
    _ttc_light_configuration_check( LogicalIndex );

    TTC_RETURN( Error ); // will perform stack overflow check + return value
}
void                        ttc_light_on( t_u8 LogicalIndex ) {
    t_ttc_light_config* volatile Config = ttc_light_get_configuration( LogicalIndex );

    // high-level code being executed before calling low-level driver...

    // pass function call to interface or low-level driver function
    _driver_light_on( Config );

    // high-level code being executed after calling low-level driver..
    Config->Flags.On = 1;
}
void                        ttc_light_off( t_u8 LogicalIndex ) {
    t_ttc_light_config* Config = ttc_light_get_configuration( LogicalIndex );

    // high-level code being executed before calling low-level driver...

    // pass function call to interface or low-level driver function
    _driver_light_off( Config );

    // high-level code being executed after calling low-level driver..
    Config->Flags.On = 0;
}
void                        ttc_light_prepare() {
    // This function is automatically called from ttc_exensions:ttc_extensions_start() at system startup
    // add your startup code here (Singletasking!)

    A_reset( ttc_light_configs, TTC_LIGHT_AMOUNT ); // safe arrays are placed in data section and must be initialized manually

    ttc_memory_set( &ttc_light_private_TaskBlinking, 0, sizeof( ttc_light_private_TaskBlinking ) );

    if ( 0 ) // optional: register this device for a sysclock change update
    { ttc_sysclock_register_for_update( ttc_light_sysclock_changed ); }

    _driver_light_prepare();
}
void                        ttc_light_reset( t_u8 LogicalIndex ) {
    t_ttc_light_config* Config = ttc_light_get_configuration( LogicalIndex );

    _driver_light_reset( Config );
}
void                        ttc_light_sysclock_changed() {

    if ( 0 ) { // nothing to do on changed sysclock (at the moment)
        // deinit + reinit all initialized LIGHT devices
        for ( t_u8 LogicalIndex = 1; LogicalIndex < ttc_light_get_max_index(); LogicalIndex++ ) {
            t_ttc_light_config* Config = ttc_light_get_configuration( LogicalIndex );
            if ( Config->Flags.Initialized ) {
                ttc_light_deinit( LogicalIndex );
                ttc_light_init( LogicalIndex );
            }
        }
    }
}
void                        ttc_light_toggle( t_u8 LogicalIndex ) {
    t_ttc_light_config* Config = ttc_light_get_configuration( LogicalIndex );

    if ( Config->Flags.On ) { // light is currently switched on:  switch it off
        ttc_light_off( LogicalIndex );
    }
    else {                    // light is currently switched off: switch it on
        ttc_light_on( LogicalIndex );
    }
}
//InsertFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//}Function definitions
//{ private Function definitions *******************************************
//
// Convention: Private functions start with an underscore "_".
// Example:    void _ttc_light(t_u8 LogicalIndex) {  }

void   _ttc_light_configuration_check( t_u8 LogicalIndex ) {
    t_ttc_light_config* Config = ttc_light_get_configuration( LogicalIndex );

    // perform some generic plausibility checks on Config...
    Assert_LIGHT( LogicalIndex == Config->LogicalIndex, E_ttc_assert_origin_auto ); // configuration must store corresponding logical index!
    Assert_LIGHT( ( Config->Architecture > E_ttc_light_architecture_None ) && ( Config->Architecture < E_ttc_light_architecture_unknown ), E_ttc_assert_origin_auto ); // architecture not set properly! Low-Level driver must set this field!
    Assert_LIGHT( ( Config->AmountChannels > 0 ) && ( Config->AmountChannels <= 4 ), E_ttc_assert_origin_auto ); // invalid amount of channels (should have been set by low-level light_*_load_defaults() function!)
    Assert_LIGHT_Writable( Config->Light, E_ttc_assert_origin_auto ); // missing light configuration (should have been allocated by low-level light_*_load_defaults() function!)
    Assert_LIGHT( ( Config->Light->Type > 0 ) && ( Config->Light->Type < E_ttc_light_type_Unknown ), E_ttc_assert_origin_auto ); // invalid/unsupported light type found (should have been set by low-level light_*_load_defaults() function!)

    for ( t_u8 ChannelIndex = 1; ChannelIndex <= Config->AmountChannels; ChannelIndex++ ) { // check configuration of individual channels
        t_ttc_light_channel_config* ConfigChannel = ttc_light_get_channel_configuration( LogicalIndex, ChannelIndex );

        Assert_LIGHT_EXTRA_Readable( ConfigChannel->Init.Features, E_ttc_assert_origin_auto ); // low-level driver did not load this pointer with a address of its feature set. Declare and define a constant instance of t_ttc_filesystem_features and assign a reference to it in current filesystem_XXX_load_defaults()!
        Assert_LIGHT_EXTRA( ConfigChannel->Init.Features->BrightnessStep > 0, E_ttc_assert_origin_auto ); // must not be zero. Check provided feature set!
        Assert_LIGHT_EXTRA( ConfigChannel->Init.Features->BrightnessMin < ConfigChannel->Init.Features->BrightnessMax, E_ttc_assert_origin_auto ); // must must be lower than max. Check provided feature set!

        if ( ConfigChannel->Init.Blinking ) {
            Assert_LIGHT_EXTRA_Readable( ConfigChannel->Init.Blinking, E_ttc_assert_origin_auto ); // blinking definition has been set but seems to point to an unreadable address. Check implementation of low-level driver!
        }
        Assert_LIGHT( ( ConfigChannel->Init.Brightness >= ConfigChannel->Init.Features->BrightnessMin ) && ( ConfigChannel->Init.Brightness <= ConfigChannel->Init.Features->BrightnessMax ), E_ttc_assert_origin_auto ); // brightness level is outside of valid range defined by Features. Check implementation of caller and/or low-level driver!
    }

    /** Example plausibility checks

    if (Config->BitRate > Features->MaxBitRate)   Config->BitRate = Features->MaxBitRate;
    if (Config->BitRate < Features->MinBitRate)   Config->BitRate = Features->MinBitRate;

    */
    // add more architecture independent checks...

    // let low-level driver check this configuration too
    _driver_light_configuration_check( Config );
}
t_u8   _ttc_light_find_next_state( t_u8 State, const t_ttc_light_blinking* Blinking ) {
    Assert_LIGHT_Readable( Blinking, E_ttc_assert_origin_auto );

    switch ( State ) {
        case E_ttc_light_blinking_state_Overflow:
        case E_ttc_light_blinking_state_T4_Off:      if ( Blinking->times.array[0] > 0 ) { State = E_ttc_light_blinking_state_T0_Off;      break; } // using index and delay time 0
        case E_ttc_light_blinking_state_T0_Off:      if ( Blinking->times.array[1] > 0 ) { State = E_ttc_light_blinking_state_T1_RampUp;   break; } // using index and delay time 1
        case E_ttc_light_blinking_state_T1_RampUp:   if ( Blinking->times.array[2] > 0 ) { State = E_ttc_light_blinking_state_T2_Full;     break; } // using index and delay time 2
        case E_ttc_light_blinking_state_T2_Full:     if ( Blinking->times.array[3] > 0 ) { State = E_ttc_light_blinking_state_T3_RampDown; break; } // using index and delay time 3
        case E_ttc_light_blinking_state_T3_RampDown: if ( Blinking->times.array[4] > 0 ) { State = E_ttc_light_blinking_state_T4_Off;      break; } // using index and delay time 4

            ttc_assert_test_origin( State != E_ttc_light_blinking_state_Overflow, E_ttc_assert_origin_auto ); // All Time[] entries seem to be zero. This makes no sense. Check your Blinking definition!
            State = E_ttc_light_blinking_state_Overflow; // indicate overrun
            break;
        default: ttc_assert_halt_origin( E_ttc_assert_origin_auto ); // Invalid State value given. Check implementation of caller!
    }

    Assert_LIGHT_EXTRA( ( State == E_ttc_light_blinking_state_Overflow ) || ( State < 5 ), E_ttc_assert_origin_auto ); // check value before returning it (extra service for caller)
    return State;
}
t_bool _ttc_light_blinking_state( t_ttc_light_channel_config* ConfigChannel, t_u8 AmountSteps ) {

    t_bool BrightnessChanged = FALSE;
    switch ( ConfigChannel->PrivateBlinking.State ) { // change LIGHT state according to current period phase == Time[] index
        case E_ttc_light_blinking_state_T0_Off:
        case E_ttc_light_blinking_state_T4_Off:
            ConfigChannel->Init.Brightness = ConfigChannel->Init.Features->BrightnessMin;
            ConfigChannel->PrivateBlinking.Delta = 0;
            break;
        case E_ttc_light_blinking_state_T1_RampUp:
        case E_ttc_light_blinking_state_T3_RampDown:
            if ( ConfigChannel->PrivateBlinking.Delta == 0 ) { // first call in T1/T3: calculate brightness delta per call
                t_s16 Delta;
                if ( ConfigChannel->PrivateBlinking.State == E_ttc_light_blinking_state_T1_RampUp ) {
                    Delta = ConfigChannel->Init.Features->BrightnessMax - ConfigChannel->Init.Features->BrightnessMin;
                }
                else { //                                 == E_ttc_light_blinking_state_T3_RampDown
                    Delta = ConfigChannel->Init.Features->BrightnessMin - ConfigChannel->Init.Features->BrightnessMax;
                }
                Delta /= AmountSteps;
                ConfigChannel->PrivateBlinking.Delta = ( t_s8 ) ttc_basic_limit( Delta, -127, 127 );
                ConfigChannel->Init.Brightness = ConfigChannel->Init.Features->BrightnessMin;
            }
            else { // consecutive call: ramp up/down channel brightness
                ConfigChannel->Init.Brightness += ConfigChannel->PrivateBlinking.Delta;
                BrightnessChanged = TRUE;
            }
            break;
        case E_ttc_light_blinking_state_T2_Full:
            ConfigChannel->Init.Brightness = ConfigChannel->Init.Features->BrightnessMax;
            ConfigChannel->PrivateBlinking.Delta = 0;
            break;
        default: ttc_assert_halt_origin( E_ttc_assert_origin_auto ); break; // invalid/unsupported State. Check implementation of ttc_light.c!
    }

    return BrightnessChanged;
}
t_bool _ttc_light_blinking_transition( void* Argument, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item ) {
    ( void ) PreviousItem;
    t_ttc_light_config*        volatile ConfigLight = ( t_ttc_light_config* ) Item;
    t_ttc_light_task_blinking* volatile TaskData    = ( t_ttc_light_task_blinking* ) Argument;
    Assert_LIGHT_Writable( ConfigLight, E_ttc_assert_origin_auto );
    Assert_LIGHT_Writable( TaskData,    E_ttc_assert_origin_auto );

    t_u8   AmountUnusedChannels = 0;
    t_bool BrightnessChanged    = FALSE;
    for ( t_u8 ChannelIndex = ConfigLight->AmountChannels; ChannelIndex > 0; ChannelIndex-- ) {
        t_ttc_light_channel_config* ConfigChannel = light_common_get_channel_configuration( ConfigLight, ChannelIndex );

        const t_ttc_light_blinking* Blinking = ConfigChannel->Init.Blinking;
        if ( Blinking != NULL ) { // have a blinking definition for current channel
            if ( ConfigLight->Flags.On ) { // light has been switched on: control it's blinking
                if ( !ConfigLight->Flags.IsBlinking ) { // detected new LIGHT: reset it's period
                    ConfigLight->Flags.IsBlinking   = 1;
                    ConfigChannel->PrivateBlinking.State = E_ttc_light_blinking_state_Overflow; // start at array index -1 (->_ttc_light_find_next_state() )
                    ttc_systick_delay_init( &( ConfigChannel->PrivateBlinking.Delay ), 0 ); // zero delay will expire immediately
                }
                if ( ttc_systick_delay_expired( &( ConfigChannel->PrivateBlinking.Delay ) ) ) { // delay expired: switch to new Time[] index
                    // find next non zero blinking time for this channel
                    e_ttc_light_blinking_state NextState = _ttc_light_find_next_state( ConfigChannel->PrivateBlinking.State, Blinking );
                    if ( NextState == E_ttc_light_blinking_state_Overflow ) { // end of period: restart
                        NextState = _ttc_light_find_next_state( NextState, Blinking ); // restart period from beginning
                        TaskData->State = E_ttc_light_task_State_ScanLIGHTs;  // scan for new LIGHTs at begin of period to keep all LIGHTs with same frequency in sync
                    }
                    ConfigChannel->PrivateBlinking.State = NextState;
                    if ( NextState != E_ttc_light_blinking_state_Overflow ) { // state valid: calculate next delay time
                        // calculate microsecond delay
                        VOLATILE_LIGHT t_u32 Time_us;
                        VOLATILE_LIGHT t_u8 AmountSteps;
                        Time_us = Blinking->times.array[NextState];
                        Time_us *= Blinking->Multiplier;
                        Time_us *= 1000;

                        if ( ( NextState == E_ttc_light_blinking_state_T1_RampUp ) ||
                                ( NextState == E_ttc_light_blinking_state_T3_RampDown )
                           ) { // calculate amount of steps required for ramp
                            AmountSteps = ConfigChannel->Init.Features->BrightnessMax - ConfigChannel->Init.Features->BrightnessMin;
                            if ( AmountSteps > ConfigChannel->Init.Features->BrightnessStep ) {
                                AmountSteps /= ConfigChannel->Init.Features->BrightnessStep;
                            }
                            else { // avoid integer division as it would return zero
                                AmountSteps = 1;
                            }
                        }
                        else { // AmountSteps = 1
                            AmountSteps = 1;
                        }
                        Time_us /= AmountSteps;
                        if ( Time_us < ttc_light_private_TaskBlinking.MinimumDelay ) {
                            Time_us = ttc_light_private_TaskBlinking.MinimumDelay;
                        }
                        ttc_systick_delay_init( &( ConfigChannel->PrivateBlinking.Delay ), Time_us );

                        if ( _ttc_light_blinking_state( ConfigChannel, AmountSteps ) ) {
                            BrightnessChanged = TRUE;
                        }
                    }
                }
            }
            else {                 // light has been switched off: stop blinking
                if ( ConfigLight->Flags.IsBlinking )
                { ConfigLight->Flags.IsBlinking = 0; } // LIGHT period will have to be reset when it is switched on again
            }
        }
        else {
            AmountUnusedChannels++;
        }
    }
    if ( BrightnessChanged ) {
        ttc_light_brightness_update( ConfigLight->LogicalIndex );
    }

    BOOL RemoveItemFromList = FALSE;
    if ( AmountUnusedChannels == ConfigLight->AmountChannels ) { // have no blinking definition for any channel: mark this item for list removal
        RemoveItemFromList = TRUE; // blinking has been disabled for this LIGHT: remove item from list
        ConfigLight->Flags.IsBlinking        = 0;
        ConfigLight->Flags.IsQueued4Blinking = 0;
    }
    else { // update light channels with new brightness
        _driver_light_brightness_update( ConfigLight );
    }

    return RemoveItemFromList;
}
void   _ttc_light_task_blinking( void* Argument ) {
    Assert_LIGHT_Writable( Argument, E_ttc_assert_origin_auto ); // always check incoming pointer arguments
    t_ttc_light_task_blinking* TaskData = ( t_ttc_light_task_blinking* ) Argument;

    do {
        switch ( TaskData->State ) {
            case E_ttc_light_task_State_Starting: {
                ttc_systick_delay_init( &( TaskData->Delay ), 0 ); // zero delay will expire on next call
                TaskData->State = E_ttc_light_task_State_ScanLIGHTs;
                break;
            }
            case E_ttc_light_task_State_ScanLIGHTs: { // append all items from

                while ( ttc_list_get_amount_cast( &( TaskData->ListWaitingLIGHTs ) ) > 0 ) { // have lights waiting to get activated: move them into ListActiveLIGHTs
                    t_ttc_light_config* LIGHT = ( t_ttc_light_config* ) ttc_list_pop_front_single_try_cast( &( TaskData->ListWaitingLIGHTs ) );
                    if ( LIGHT ) {
                        ttc_list_push_back_single_cast( &( TaskData->ListActiveLIGHTs ), LIGHT );
                    }
                    else
                    { ttc_task_yield(); } // list is used by another task: give cpu to other tasks
                }

                // not breaking but continuing with next state
                TaskData->State = E_ttc_light_task_State_ControlLIGHTs;
            }
            case E_ttc_light_task_State_ControlLIGHTs: {
                if ( ttc_list_get_amount_cast( &( TaskData->ListActiveLIGHTs ) ) > 0 ) { // have active lights waiting for
                    TODO( "only wait for earliest delay to expire (faster than always checking all delays)" );
                    //if ( ttc_systick_delay_expired( &( TaskData->Delay ) ) ) { // earliest delay expired: process all active lights
                    // run _ttc_light_blinking_transition(TaskData, ...) for every item in list
                    ttc_list_iterate_cast( &( TaskData->ListActiveLIGHTs ), _ttc_light_blinking_transition, TaskData, 1, 0, -1 );
                    //}
                }
                else { // no active lights: scan for new LIGHTs
                    TaskData->State = E_ttc_light_task_State_ScanLIGHTs;
                }
                break;
            }
            default: ttc_assert_halt_origin( E_ttc_assert_origin_auto ); break; // invalid/unsupported state detected
        }
    }
    while ( ttc_task_can_yield() );
}
//InsertPrivateFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//}PrivateFunctions
