#ifndef TTC_LIGHT_H
#define TTC_LIGHT_H
/** { ttc_light.h *******************************************************
 *
 *                          The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  High-Level driver and documentation for light devices.
 *  The functions defined in this file provide a hardware independent interface
 *  for your application.
 *
 *  The basic usage scenario for devices:
 *  1) check:       Assert_LIGHT(tc_light_get_max_index() > 0, E_ttc_assert_origin_auto);
 *  2) configure:   ttc_light_get_configuration(LogicalIndex);
 *  3) initialize:  ttc_light_init(LogicalIndex);
 *  4) use:         ttc_light_XXX(LogicalIndex);
 *
 *  Device Index
 *  Devices are addressed by a logical index with no correlation to the order of physical devices.
 *  Logical indices start at 1 to simply detect uninitialized values.
 *  Every board defines, which devices can be used under which logical index.
 *  The logical indices are defined in board makefiles (makefile.100_board_*).
 *
 *  Structures, Enums and Defines being required by high-level light and application.
 *
 *  Created from template ttc_device.h revision 39 at 20191218 12:21:38 UTC
 *
 *  Authors: Gregor Rebel
 *
 */
/** Description of ttc_light (Do not delete this line!)
 *
 * H2: Universal Light Controller
 *
 * A ttc_light device can control a wide range of light sources from single LEDs to LED strips and studio lights.
 * Light sources can be connected via GPIO PWM or communication lines.
 * Multi-color light sources are controlled by individual brightness channels for each color.
 * 1..4 individual 8 bit brightness channels are supported for each ttc_light device.
 * Automatic dimming and blinking can be configured for each channel individually.
 *
 * H2: Configuring Lights
 *
 * ttc_light requires that all light devices are configured via constant defines in a makefile.
 * Light devices are typcially defined in 100_board_* or board extension makefiles.
 *
 * It is possible to place TTC_LIGHT<n>_ defines in a *_types.h header file and include it
 * from some central file like ttc_basic_types.h, but this is not the intended usage.
 *
 * H3: Each LIGHT<x> requires at least one constant define
 * #) TTC_LIGHT<x>
 *    == one from e_ttc_light_architecture
 *    Defines which low level driver will handle this light device.
 *
 * Example TTC_LIGHT definition
 *    COMPILE_OPTS += -DTTC_LIGHT1=E_ttc_light_architecture_gpio
 *    COMPILE_OPTS += -DTTC_LIGHT1_GPIO1=E_ttc_gpio_pin_c6
 *    COMPILE_OPTS += -DTTC_LIGHT1_LOWACTIVE=0
 *
 * Additional defines may be required by corresponding low-level driver.
 * Check header files of low-level drivers ttc-lib/light/light_*.h for more details!
}*/

#ifndef EXTENSION_ttc_light
    #error Missing extension! Please add this to your activate_project.sh script: activate.500_ttc_light.sh
#endif

//{ Includes *************************************************************
//
// Header files include all header files that are required to include this
// header file. Includes that provide function declarations should be placed
// in "ttc_light.c"
//
#include "ttc_basic.h"  // Include First! (basic datatypes and definitions)
#include "ttc_memory_types.h"
#include "ttc_list_types.h"      // single linked lists and iterators
#include "ttc_systick_types.h"   // universal systick based delays
#include "interfaces/ttc_light_interface.h" // multi architecture support
//InsertIncludes above (DO NOT DELETE THIS LINE!)

//} Includes
//{ Macros ***************************************************************

//InsertMacros above (DO NOT DELETE THIS LINE!)

//}Macros
//{ Defines/ TypeDefs ****************************************************

//InsertDefines above (DO NOT DELETE THIS LINE!)

//} Defines
//{ Structures/ Enums ****************************************************
// Structures and Enums required by High-Level light driver go here only...

//InsertEnums above (DO NOT DELETE THIS LINE!)

typedef enum {
    E_ttc_light_task_State_Reset = 0,
    E_ttc_light_task_State_Starting,
    E_ttc_light_task_State_ScanLIGHTs,
    E_ttc_light_task_State_ControlLIGHTs,

    E_ttc_light_task_State_Invalid
} e_ttc_light_task_states;
typedef struct {
    struct { // ttc_list storing configurations of all LIGHTs waiting to be controlled by _ttc_light_task_blinking() on start of next period
        TTC_LIST_HEADER( t_ttc_light_config );
    } ListWaitingLIGHTs;
    struct { // ttc_list storing configurations of all LIGHTs being actively controlled by _ttc_light_task_blinking()
        TTC_LIST_HEADER( t_ttc_light_config );
    } ListActiveLIGHTs;

    e_ttc_light_task_states State;         // current state of task
    t_ttc_systick_delay     Delay;         // delay until next LIGHT action
    t_u32                   MinimumDelay;  // minimum delay time for ttc_systick_delay() (microseconds). Increasing minimum delay will reduce cpu overhead of dimming and blinking (ramp up/down)
} t_ttc_light_task_blinking;
//InsertTypeDefs above (DO NOT DELETE THIS LINE!)

//} Structures/ Enums
/** Function Declarations ************************************************{
 *
 * The functions declared below provide the main interface for
 * light devices on all supported architectures.
 * Check light/light_* files for a list of low-level drivers.
 * In most cases, only one low-level driver may be active during compilation.
 * Low-level drivers get activated by calling an activate.450_*.sh script.
 * A complete list of all available activate scripts can be found in your
 * activate_project.sh file inside your project folder.
 *
 */

/** Prepares light Driver for basic operation.
 *
 * Note: This function is called automatically at system start before the scheduler has been started.
 */
void ttc_light_prepare();
void _driver_light_prepare();

/** returns amount of devices available in current configuration
 *
 * @return amount of available devices
 */
t_u8 ttc_light_get_max_index();

/** returns configuration of indexed device (asserts if no valid configuration was found)
 *
 * @param LogicalIndex    logical index of LIGHT device. Each logical device <n> is defined via TTC_LIGHT<n>* constants in compile_options.h and extensions.active/makefile
 * @return                configuration of indexed device
 */
t_ttc_light_config* ttc_light_get_configuration( t_u8 LogicalIndex );


/** Delivers access to configuration data of individual channel
 *
 * ttc_light devices each support one or more channels.
 * The size of configuration structure varies for the different light types.
 * Using this function to access t_ttc_light_config.Light is safer than accessing this union directly.
 *
 * Typical light provide either one (monochromatic) or three (full color rgb) channels.
 * Four channel lights may provide red+green+blue+white colors.
 *
 * Note: This function will not assert if ChannelIndex > AmountChannels, but it will assert if ChannelIndex == 0.
 *
 * @param LogicalIndex (t_u8)  Logical index of light instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_LIGHT<n> lines in extensions.active/makefile
 * @param ChannelIndex (t_u8)  Logical index of channel to configure (1..ttc_light_get_configuration(LogicalIndex)->AmountChannels)
 * @return             (t_ttc_light_channel_config*)  ==NULL: desired channel not available for this light; !=NULL: configuration data (call ttc_light_init() after changing fields!)
 */
t_ttc_light_channel_config* ttc_light_get_channel_configuration( t_u8 LogicalIndex, t_u8 ChannelIndex );

/** initialize indexed device for use
 *
 * @param LogicalIndex    logical index of LIGHT device. Each logical device <n> is defined via TTC_LIGHT<n>* constants in compile_options.h and extensions.active/makefile
 * @return                == 0: light device has been initialized successfully; != 0: error-code
 */
e_ttc_light_errorcode ttc_light_init( t_u8 LogicalIndex );

/** shutdown indexed device
 *
 * @param LogicalIndex    logical index of LIGHT device. Each logical device <n> is defined via TTC_LIGHT<n>* constants in compile_options.h and extensions.active/makefile
 */
void ttc_light_deinit( t_u8 LogicalIndex );

/** load default setting values for indexed device
 *
 * Settings of indexed device are being reset to their default values.
 * Will automatically call ttc_light_deinit() if device has been initialized.
 *
 * @param LogicalIndex  logical index of LIGHT device. Each logical device <n> is defined via TTC_LIGHT<n>* constants in compile_options.h and extensions.active/makefile
 * @return                == 0: default values have been loaded successfully for indexed light device; != 0: error-code
 */
e_ttc_light_errorcode  ttc_light_load_defaults( t_u8 LogicalIndex );

/** reset configuration of indexed device and connected hardware
 *
 * @param LogicalIndex    logical index of LIGHT device. Each logical device <n> is defined via TTC_LIGHT<n>* constants in compile_options.h and extensions.active/makefile

 */
void ttc_light_reset( t_u8 LogicalIndex );

/** Reinitializes all initialized devices after changed systemcloc profile.
 *
 * Note: This function is called automatically from _ttc_sysclock_call_update_functions()
 */
void ttc_light_sysclock_changed();

/** Initialize all configured LIGHTs for operation
 *
 * Calling this function will apply all LIGHT configurations to make LIGHTs operational.
 * Individual LIGHTs will still be switched off.
 *
 */
void ttc_light_init_all();

/** switch on single light to preconfigured brightness level
 *
 * @param Config       (t_ttc_light_config*)  Configuration of light device as returned by ttc_light_get_configuration()
 * @param LogicalIndex (t_u8)               Logical index of light instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_LIGHT<n> lines in extensions.active/makefile
 */
void ttc_light_on( t_u8 LogicalIndex );
void _driver_light_on( t_ttc_light_config* Config );

/** switch off single light
 *
 * Config->Brightness will not be changed
 *
 * @param Config       (t_ttc_light_config*)  Configuration of light device as returned by ttc_light_get_configuration()
 * @param LogicalIndex (t_u8)               Logical index of light instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_LIGHT<n> lines in extensions.active/makefile
 */
void ttc_light_off( t_u8 LogicalIndex );
void _driver_light_off( t_ttc_light_config* Config );

/** toggle single light on/off
 *
 * If switched on, LIGHT will be configured for configured brightness.
 *
 * @param LogicalIndex (t_u8)  Logical index of light instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_LIGHT<n> lines in extensions.active/makefile
 */
void ttc_light_toggle( t_u8 LogicalIndex );

/** Configure automatic blinking for a single light
 *
 * When the first light is configured for automatic blinking, an extra task is created to control blinking of all lights.
 * The maximum achievable frequency and amount of lights depends on low-level drivers for current hardware.
 *
 * @param LogicalIndex (t_u8)                   Logical index of light instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_LIGHT<n> lines in extensions.active/makefile
 * @param ChannelIndex (t_u8)                   >0; Logical index of channel to assign blinking to; ==0; assign same blinking to all channels of this light
 * @param Blinking     (t_ttc_light_blinking*)  ==NULL: disabled automatic blinking mode (also switches of light); !=NULL: pointer to blinking description (must stay persistent!)
 */
void ttc_light_blink( t_u8 LogicalIndex, t_u8 ChannelIndex, const t_ttc_light_blinking* Blinking );

/** set new brightness levels for first channel only
 *
 * @param Config       (t_ttc_light_config*)  Configuration of light device as returned by ttc_light_get_configuration()
 * @param LogicalIndex (t_u8)                 Logical index of light instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_LIGHT<n> lines in extensions.active/makefile
 * @param Level        (t_u8)                 new brightness level for channel 1
 */
void ttc_light_brightness1( t_u8 LogicalIndex, t_u8 Level );

/** set new brightness levels for first two channels at once
 *
 * Note: Low level driver must not check Config->AmountChannels as this already has been done by caller!
 * Note: High level driver will copy new brightness levels into Config->Light->channels2.Channel[].Init.Brightness after calling low-level driver.
 *
 * @param Config       (t_ttc_light_config*)  Configuration of light device as returned by ttc_light_get_configuration()
 * @param LogicalIndex (t_u8)                 Logical index of light instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_LIGHT<n> lines in extensions.active/makefile
 * @param Level1       (t_u8)                 new brightness level for channel 1
 * @param Level2       (t_u8)                 new brightness level for channel 2
 */
void ttc_light_brightness2( t_u8 LogicalIndex, t_u8 Level1, t_u8 Level2 );

/** set new brightness levels for first three channels at once
 *
 * Note: Low level driver must not check Config->AmountChannels as this already has been done by caller!
 * Note: High level driver will copy new brightness levels into Config->Light->channels2.Channel[].Init.Brightness after calling low-level driver.
 *
 * @param Config       (t_ttc_light_config*)  Configuration of light device as returned by ttc_light_get_configuration()
 * @param LogicalIndex (t_u8)                 Logical index of light instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_LIGHT<n> lines in extensions.active/makefile
 * @param Red          (t_u8)                 new brightness level for channel 1
 * @param Blue         (t_u8)                 new brightness level for channel 2
 * @param Green        (t_u8)                 new brightness level for channel 3
 */
void ttc_light_brightness3( t_u8 LogicalIndex, t_u8 Red, t_u8 Green, t_u8 Blue );

/** set new brightness levels for first four channels at once
 *
 * Note: Low level driver must not check Config->AmountChannels as this already has been done by caller!
 * Note: High level driver will copy new brightness levels into Config->Light->channels2.Channel[].Init.Brightness after calling low-level driver.
 *
 * @param Config       (t_ttc_light_config*)  Configuration of light device as returned by ttc_light_get_configuration()
 * @param LogicalIndex (t_u8)                 Logical index of light instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_LIGHT<n> lines in extensions.active/makefile
 * @param Red          (t_u8)                 new brightness level for channel 1
 * @param Blue         (t_u8)                 new brightness level for channel 2
 * @param Green        (t_u8)                 new brightness level for channel 3
 * @param White        (t_u8)                 new brightness level for channel 4
 */
void ttc_light_brightness4( t_u8 LogicalIndex, t_u8 Red, t_u8 Green, t_u8 Blue, t_u8 White );

/** Sends latest Brightness levels in all channels to light controller
 *
 * Call this function to update brightness after changing individual channel brightness values in Config->Light->channels4!
 * Low-level driver will update light brightness in all channels.
 *
 * @param Config       (t_ttc_light_config*)  Configuration of light device as returned by ttc_light_get_configuration()
 * @param LogicalIndex (t_u8)                 Logical index of light instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_LIGHT<n> lines in extensions.active/makefile
 */
void ttc_light_brightness_update( t_u8 LogicalIndex );
void _driver_light_brightness_update( t_ttc_light_config* Config );

//InsertFunctionDeclarations above (DO NOT DELETE THIS LINE!)
//}Function Declarations
/** Prototypes of low-level driver functions ****************************{
 *
 * Functions declared with prefix _driver_light_ are passed to interfaces/ttc_light_interface.h and to
 * all currently activated low-level drivers.
 * If a driver declaration is placed directly below its ttc_* pendant, it will share its documentation.
 * Driver declarations with different argument list should be bundled with their own documentation in this section.
 *
 * If you add a _driver_* prototype, use create_DeviceDriver.pl to automatically add empty functions
 * in all existing low-level drivers by issuing this inside git/TheToolChain/TTC-Library/:
 * cd templates/; ./create_DeviceDriver.pl light UPDATE
 */


/** Checks + corrects entries in given Config to meet limits of current architecture
 *
 * Note: High-level function ttc_light_configuration_check() has already checked Config against
 *       all fields being found in t_ttc_light_features.
 *
 * @param Config        Configuration of light device
 * @return              Fields of *Config have been adjusted if necessary
 */
void _driver_light_configuration_check( t_ttc_light_config* Config );

/** shutdown single LIGHT unit device
 * @param Config        Configuration of light device
 * @return              == 0: LIGHT has been shutdown successfully; != 0: error-code
 */
e_ttc_light_errorcode _driver_light_deinit( t_ttc_light_config* Config );

/** initializes single LIGHT unit for operation
 * @param Config        Configuration of light device
 * @return              == 0: LIGHT has been initialized successfully; != 0: error-code
 */
e_ttc_light_errorcode _driver_light_init( t_ttc_light_config* Config );

/** loads configuration of single light unit with default values
 *
 * Note: Must allocate Config->LowLevelConfig according to connected light device!
 *
 * @param Config        Configuration of light device
 * @return  == 0:       configuration was loaded successfully
 */
e_ttc_light_errorcode _driver_light_load_defaults( t_ttc_light_config* Config );

/** reset configuration of indexed device and connected hardware
 *
 * @param Config        Configuration of light device
 */
void _driver_light_reset( t_ttc_light_config* Config );
//InsertDriverPrototypes above (DO NOT DELETE THIS LINE!)

//}

#endif //TTC_LIGHT_H
