/** ttc_list.c ***********************************************{
 *
 * Written by Gregor Rebel 2013
 *
 * Architecture independent support for single linked lists.
 *
 * Check ->ttc_list.h for a detailed description!
 *
}*/
#include "ttc_list.h"
#include "ttc_semaphore.h"
#include "ttc_mutex.h"
#include "ttc_heap.h"

//{ declare private functions (no locking/ argument checking) *

// Define TTC_LIST_STATISTICS_HISTORY in your makefile or right
// here to memorize pointers to lists being operated on.
#ifdef TTC_LIST_STATISTICS_HISTORY
    #define SIZE_LIST_HISTORY 70
#else
    #define SIZE_LIST_HISTORY 10
#endif
#if SIZE_LIST_HISTORY > 0

/** rotating array storing last processed lists
 *
 * This history of list content allows to see what happened to your list during
 * last SIZE_LIST_HISTORY amount of operations in a debug session.
 *
 * Display number of current index in ListHistory[]
 * (gdb) p LatestListHistory - ListHistory
 *
 * Display latest recorded list values
 * (gdb) p *(LatestListHistory-0)
 *
 * Display second latest recorded list values
 * (gdb) p *(LatestListHistory-1)
 *
 */
t_ttc_list ListHistory[SIZE_LIST_HISTORY];
t_ttc_list* LatestListHistory = ListHistory - 1;  // always points to latest valid entry in ListHistory ( E.g.: p *(LatestListHistory-0) , p *(LatestListHistory-1) , ...)

void         _TTC_LIST_ADD_HISTORY( t_ttc_list* List ) {

    LatestListHistory++;
    if ( LatestListHistory >= ListHistory + SIZE_LIST_HISTORY )
    { LatestListHistory = ListHistory; }
    ttc_memory_copy( LatestListHistory, List, sizeof( t_ttc_list ) );

}
#define TTC_LIST_ADD_HISTORY(LIST) _TTC_LIST_ADD_HISTORY(LIST)

#else
#define TTC_LIST_ADD_HISTORY(LIST)
#endif

/** completes all pending  on given list
 *
 * Note: This function is private and must not be called from outside!
 * Note: This function completes in constant time O(1)
 *
 * @param List     pointer to already created list
 */
void _ttc_list_complete_all_pending_isr( t_ttc_list* List );

/** Common implementation for ttc_list_iterate(), ttc_list_iterate_from()
 *
 * Runs through given list items and calls given iterate function for each one.
 * Can remove individual items from list and call a remove function.
 *
 * @param List             pointer to memory space to be used to manage list
 * @param IterateFunction  function pointer: will be called for each list item. Returns TRUE if current item shall be removed from list.
 *                         Note: IterateFunction() may not append item to other lists or release it. It may return a function pointer that can process the item later (see @return below)!
 *                         @return ==0: do not remove item; ==1: current item shall be removed from list;
 *                                 >1: function to call with item as argument AFTER removing it from current list.
 *                                     Signature of function being called: void RemoveItem(t_ttc_list_item*)
 * @param Argument         argument passed to IterateFunction()
 * @param Step             Allows to skip items:
 *                         == 1: iterate all items 1, 2, ..., last
 *                         == 2: iterate every second item 1, 3, ... last
 * @param Start            pointer to list item for which IterateFunction is called first
 * @param End              pointer to list item for which IterateFunction is called last (==NULL: run to end of list)
 * @param Amount           maximum amount of items to iterate (==-1: not limited)
 */

void _ttc_list_iterate_from( t_ttc_list* List, void* ( *IterateFunction )( void* Argument, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item ), void* Argument, t_u16 SkipFirst, t_s16 Steps, t_ttc_list_item* Start, t_ttc_list_item* End, t_u16 Amount );

/** adds given Item or linked list of items at end of given list
 *
 * Note: This function is private and must not be called from outside!
 * Note: This function completes in constant time O(1)
 * Note: This function will not update List->ttc_list.AmountItems.
 *
 * @param List     pointer to already created list
 * @param Item     pointer to item to be added to list
 * @return         == tle_OK: Item has been successfully added to List
 */
void _ttc_list_push_back_single( t_ttc_list* List, t_ttc_list_item* Item );

/** adds given Item or linked list of items at head of given list
 *
 * Note: This function is private and must not be called from outside!
 * Note: This function completes in constant time O(1)
 * Note: This function will not update List->ttc_list.AmountItems.
 *
 * @param List     pointer to already created list
 * @param Item     pointer to item to be added to list
 * @return         == tle_OK: Item has been successfully added to List
 */
void _ttc_list_push_front_single( t_ttc_list* List, t_ttc_list_item* Item );

/** adds given linked list of items at end of given list
 *
 * Note: This function is private and must not be called from outside!
 * Note: This function completes in constant time O(1)
 * Note: This function will not update List->ttc_list.AmountItems.
 *
 * @param List       pointer to already created list
 * @param FirstItem  pointer to first item in list of items to be added to list
 * @param LastItem   pointer to last item in list of items to be added to list
 */
void _ttc_list_push_back_multiple( t_ttc_list* List, t_ttc_list_item* FirstItem, t_ttc_list_item* LastItem );

/** adds given linked list of items before head of given list
 *
 * Note: This function is private and must not be called from outside!
 * Note: This function completes in constant time O(1)
 * Note: This function will not update List->ttc_list.AmountItems.
 *
 * @param List       pointer to already created list
 * @param FirstItem  pointer to first item in list of items to be added to list
 * @param LastItem   pointer to last item in list of items to be added to list
 */
void _ttc_list_push_front_multiple( t_ttc_list* List, t_ttc_list_item* FirstItem, t_ttc_list_item* LastItem );

/** removes first Item from given list and delivers its reference
 *
 * Note: This function is private and must not be called from outside!
 * Note: This function completes in constant time O(1)
 * Note: This function will not update List->ttc_list.AmountItems.
 *
 * @param List     pointer to already created list
 * @param Item     !=NULL: loaded with pointer to item removed from given list or NULL if list is empty; ==NULL: removed item is not stored
 */
void _ttc_list_pop_front_single( t_ttc_list* List, t_ttc_list_item** Item );

/** removes linked list of first items from front of given list and delivers reference to first and last one
 *
 * Note: This function is private and must not be called from outside!
 * Note: This function completes in constant time O(Amount)
 * Note: This function will not update List->ttc_list.AmountItems.
 *
 * @param List      pointer to already created list
 * @param ListSize  amount of items currently stored in list
 * @param Amount    amount of items to remove from list (==-1: means all items in list)
 * @param FirstItem loaded with pointer to first item of items removed from given list or NULL if list is empty
 * @param LastItem  loaded with pointer to last item of items removed from given list or NULL if list is empty
 * @return          actual amount of items being removed from list
 */
t_base _ttc_list_pop_front_multiple( t_ttc_list* List, t_base ListSize, t_base Amount, t_ttc_list_item** FirstItem, t_ttc_list_item** LastItem );

/** removes single Item from given list
 *
 * Note: This function is private and must not be called from outside!
 * Note: This function completes in constant time O(1)
 * Note: This function will not update List->ttc_list.AmountItems.
 *
 * @param List          pointer to already created list
 * @param PreviousItem  !=NULL: pointer to item previous to item to remove; ==NULL: Item to remove is first item
 * @param Item          pointer to item to be removed from given list
 * @param NextItem      !=NULL: pointer to item following item to remove (==Item->ttc_list_item.Next); ==NULL: Item to remove is last item;
 */
void _ttc_list_remove_item( t_ttc_list* List, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item, t_ttc_list_item* NextItem );

//} private functions
//{ functions *************************************************

t_ttc_list*      ttc_list_create( const char* Name ) {
    t_ttc_list* NewList = ttc_heap_alloc( sizeof( t_ttc_list ) );
    Assert_LIST( NewList != NULL, E_ttc_assert_origin_auto );  // could not create list (no implementation/ not egnough memory)

    ttc_list_init( NewList, Name );

    return NewList;
}
void             ttc_list_init( t_ttc_list* List, const char* Name ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!

    ttc_memory_set( ( void* ) & ( List->ttc_list ), 0, sizeof( List->ttc_list ) );
    ttc_mutex_init( &( List->ttc_list.Lock ) );
    ttc_semaphore_init( &( List->ttc_list.AmountItems ) );
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    List->ttc_list.Debug.Stats.Name = Name;
#else
    ( void ) Name;
#endif
}
e_ttc_list_error ttc_list_complete_all_pending_isr( t_ttc_list* List ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!
    Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto );  // invalid list (did you initialize it?)

    if ( ttc_mutex_is_locked( &( List->ttc_list.Lock ) ) ) // list is locked by task: abort completion
    { return tle_ListInUseByTask; }

    _ttc_list_complete_all_pending_isr( List );

    return tle_OK;
}
t_base           ttc_list_get_amount( t_ttc_list* List ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );         // invalid pointer given!
    Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto );  // invalid list (did you initialize it?)

    return ttc_semaphore_available( &( List->ttc_list.AmountItems ) );
}
void             ttc_list_push_back_single( t_ttc_list* List, t_ttc_list_item* Item ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );         // invalid pointer given!
    Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto );  // invalid list (did you initialize it?)
    Assert_LIST_VALID( Item, E_ttc_assert_origin_auto );         // invalid pointer given!

    ttc_mutex_lock( &( List->ttc_list.Lock ), -1 );

    _ttc_list_push_back_single( List, Item );
    TTC_LIST_STATS_INCREASE( List, AmountPushBack );
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    if ( Item->ttc_list_item.Next != NULL )  {
        Assert_LIST_VALID( Item->ttc_list_item.Next, E_ttc_assert_origin_auto );
    }
#endif
#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
    Item->ttc_list_item.Debug.Caller = __builtin_return_address( 0 );  // store address of calling function for debugging
#endif

    //? ttc_task_critical_begin();
    ttc_mutex_unlock( &( List->ttc_list.Lock ) );
    ttc_semaphore_give( &( List->ttc_list.AmountItems ), 1 ); // must increase semaphore AFTER unlocking mutex (waiting task will try to pull right now)
    //? ttc_task_critical_end();
}
e_ttc_list_error ttc_list_push_back_single_isr( t_ttc_list* List, t_ttc_list_item* Item ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!
#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
    Assert_LIST( ( Item->ttc_list_item.Debug.Owner == NULL ) || ( ttc_memory_is_constant( Item->ttc_list_item.Debug.Owner ) ), E_ttc_assert_origin_auto ); // Item still belongs to a list. Remove it from this list before freeing it!
#endif

    if ( ! ttc_mutex_is_locked( &( List->ttc_list.Lock ) ) ) { // not locked by task: add items to list
        if ( List->ttc_list.Pending_Push_Amount )         // still data pending from last isr-push
        { _ttc_list_complete_all_pending_isr( List ); }
        _ttc_list_push_back_single( List, Item );
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
        Item->ttc_list_item.Debug.Caller = __builtin_return_address( 0 );  // store address of calling function for debugging
#endif
        TTC_LIST_STATS_INCREASE( List, AmountPushBackISR );
        ttc_semaphore_give_isr( &( List->ttc_list.AmountItems ), 1 );
    }
    else {                                            // already locked by task: add to list of pending items
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
        TTC_LIST_STATS_INCREASE( List, AmountPending );
#endif
        if ( List->ttc_list.Pending_Push_Back_Last ) { // append at end of list
            List->ttc_list.Pending_Push_Back_Last->ttc_list_item.Next = Item;
        }
        else {                                // append as first item
            List->ttc_list.Pending_Push_Back_First = Item;
        }
        //? while ( Item->ttc_list_item.Next )
        //? { Item = Item->ttc_list_item.Next; }
        List->ttc_list.Pending_Push_Back_Last = Item;
        List->ttc_list.Pending_Push_Amount++;
        Item->ttc_list_item.Next = TTC_LIST_TERMINATOR;
#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
        Item->ttc_list_item.Debug.Owner = List;  // item now belongs to a list
#endif
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
        Item->ttc_list_item.Debug.Caller = __builtin_return_address( 0 );
#endif

        return tle_ListInUseByTask;
    }

    return tle_OK;
}
void             ttc_list_push_back_multiple( t_ttc_list* List, t_ttc_list_item* FirstItem, t_ttc_list_item* LastItem, t_base Amount ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!
    Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto ); // invalid list (did you initialize it?)
    Assert_LIST_VALID( FirstItem, E_ttc_assert_origin_auto );   // invalid pointer given!
    Assert_LIST_VALID( LastItem, E_ttc_assert_origin_auto );    // invalid pointer given!

    ttc_mutex_lock( &( List->ttc_list.Lock ), -1 );

    _ttc_list_push_back_multiple( List, FirstItem, LastItem );
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    FirstItem->ttc_list_item.Debug.Caller = __builtin_return_address( 0 );  // store address of calling function for debugging
#endif
    ttc_semaphore_give( &( List->ttc_list.AmountItems ), Amount );

    Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto );  // invalid list (did you initialize it?)
    Assert_LIST_VALID( List->ttc_list.Last, E_ttc_assert_origin_auto );
    Assert_LIST_EXTRA( ttc_list_get_amount( List ) == ttc_list_items_count( List->ttc_list.First ), E_ttc_assert_origin_auto ); // bug in size counter implementation

    ttc_mutex_unlock( &( List->ttc_list.Lock ) );
}
e_ttc_list_error ttc_list_push_back_multiple_isr( t_ttc_list* List, t_ttc_list_item* FirstItem, t_ttc_list_item* LastItem, t_base Amount ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!
    Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto ); // invalid list (did you initialize it?)
    Assert_LIST_VALID( FirstItem, E_ttc_assert_origin_auto );   // invalid pointer given!
    Assert_LIST_VALID( LastItem, E_ttc_assert_origin_auto );    // invalid pointer given!

    if ( ! ttc_mutex_is_locked( &( List->ttc_list.Lock ) ) ) { // not locked by task: add items to list
        if ( List->ttc_list.Pending_Push_Amount )         // still data pending from last isr-push
        { _ttc_list_complete_all_pending_isr( List ); }
        _ttc_list_push_back_multiple( List, FirstItem, LastItem );
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
        FirstItem->ttc_list_item.Debug.Caller = __builtin_return_address( 0 );  // store address of calling function for debugging
#endif
        ttc_semaphore_give( &( List->ttc_list.AmountItems ), Amount );
        Assert_LIST_EXTRA( ttc_list_get_amount( List ) == ttc_list_items_count( List->ttc_list.First ), E_ttc_assert_origin_auto ); // bug in size counter implementation
    }
    else {                                            // already locked by task: add to list of pending items
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
        t_ttc_list_item* Item = FirstItem;
        do { // check all items to be added
            Assert_LIST_VALID_EXTRA( Item, E_ttc_assert_origin_auto );
#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
            Assert_LIST( FirstItem->ttc_list_item.Debug.Owner == NULL, E_ttc_assert_origin_auto ); // Item still belongs to a list. Remove it from this list before freeing it!
#endif

            if ( Item != TTC_LIST_TERMINATOR )  {
#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
                Item->ttc_list_item.Debug.Owner = List;  // item now belongs to a list
#endif
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
                Item->ttc_list_item.Debug.Caller = __builtin_return_address( 0 );
#endif
            }
            Item = Item->ttc_list_item.Next;
        }
        while ( ( Item != TTC_LIST_TERMINATOR ) && ( Item != LastItem ) );
        TTC_LIST_STATS_INCREASE( List, AmountPending );
#endif

        if ( List->ttc_list.Pending_Push_Back_Last ) { // append at end of list
            List->ttc_list.Pending_Push_Back_Last->ttc_list_item.Next = FirstItem;
        }
        else {                                // append as first item
            List->ttc_list.Pending_Push_Back_First = FirstItem;
        }
        List->ttc_list.Pending_Push_Back_Last = LastItem;
        List->ttc_list.Pending_Push_Amount    += Amount;

        return tle_ListInUseByTask;
    }

    return tle_OK;
}
void             ttc_list_push_front_single( t_ttc_list* List, t_ttc_list_item* Item ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!
    Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto );  // invalid list (did you initialize it?)
    Assert_LIST_VALID( Item, E_ttc_assert_origin_auto );        // invalid pointer given!

    ttc_mutex_lock( &( List->ttc_list.Lock ), -1 );

    _ttc_list_push_front_single( List, Item );
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Item->ttc_list_item.Debug.Caller = __builtin_return_address( 0 );  // store address of calling function for debugging
#endif
    TTC_LIST_STATS_INCREASE( List, AmountPushFront );
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    if ( ( Item->ttc_list_item.Next ) != TTC_LIST_TERMINATOR ) {
        Assert_LIST_VALID( Item->ttc_list_item.Next, E_ttc_assert_origin_auto );
    }
#endif

    ttc_mutex_unlock( &( List->ttc_list.Lock ) );
    ttc_semaphore_give( &( List->ttc_list.AmountItems ), 1 );
}
e_ttc_list_error ttc_list_push_front_single_isr( t_ttc_list* List, t_ttc_list_item* Item ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!

    if ( ! ttc_mutex_is_locked( &( List->ttc_list.Lock ) ) ) { // not locked by task: add items to list
        if ( List->ttc_list.Pending_Push_Amount )              // still data pending from last isr-push
        { _ttc_list_complete_all_pending_isr( List ); }
        _ttc_list_push_front_single( List, Item );
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
        Item->ttc_list_item.Debug.Caller = __builtin_return_address( 0 );  // store address of calling function for debugging
#endif
        TTC_LIST_STATS_INCREASE( List, AmountPushFrontISR );
        ttc_semaphore_give( &( List->ttc_list.AmountItems ), 1 );
    }
    else {                                            // already locked by task: add to list of pending items
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
        TTC_LIST_STATS_INCREASE( List, AmountPending );
#endif
#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
        Assert_LIST( ( Item->ttc_list_item.Debug.Owner == NULL ) || ( ttc_memory_is_constant( Item->ttc_list_item.Debug.Owner ) ), E_ttc_assert_origin_auto ); // Item still belongs to a list. Remove it from this list before freeing it!
#endif

        if ( List->ttc_list.Pending_Push_Front_Last ) { // append at end of list
            List->ttc_list.Pending_Push_Front_Last->ttc_list_item.Next = Item;
        }
        else {                         // append as first item
            List->ttc_list.Pending_Push_Front_First = Item;
        }
        while ( Item->ttc_list_item.Next )
        { Item = Item->ttc_list_item.Next; }
        List->ttc_list.Pending_Push_Front_Last = Item;
        List->ttc_list.Pending_Push_Amount++;
#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
        Item->ttc_list_item.Debug.Owner = List;  // item now belongs to a list
#endif
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
        Item->ttc_list_item.Debug.Caller = __builtin_return_address( 0 );
#endif

        return tle_ListInUseByTask;
    }

    return tle_OK;
}
t_ttc_list_item* ttc_list_peek_front( t_ttc_list* List ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!

    if ( ttc_list_get_amount( List ) ) { // list stores at least one item

        t_ttc_list_item* Item = List->ttc_list.First;
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
        Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto );  // invalid list (did you initialize it?)
        Assert_LIST_VALID( Item, E_ttc_assert_origin_auto );
#endif
        if ( !Item ) { // no item in regular list: check if we have items from interrupt service
            Item = List->ttc_list.Pending_Push_Front_First;

            if ( !Item ) {
                Item = List->ttc_list.Pending_Push_Back_First;
            }
        }

        return Item;
    }
    return NULL;
}
t_ttc_list_item* ttc_list_peek_index( t_ttc_list* List, t_base Index ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!

    t_u32 AmountItems = ttc_list_get_amount( List );
    if ( Index < AmountItems ) { // list stores egnough items Index = 0..(AmountItems-1)

        t_ttc_list_item* Item = List->ttc_list.First;
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
        Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto );  // invalid list (did you initialize it?)
        Assert_LIST_VALID( Item, E_ttc_assert_origin_auto );
#endif
        while ( Index > 0 ) {
            Index--;
            if ( Item != NULL ) {
                Item = Item->ttc_list_item.Next;
            }
        }
        return Item;
    }
    return NULL;
}
t_base           ttc_list_pop_front_multiple( t_ttc_list* List, t_base Amount, t_ttc_list_item** FirstItem, t_ttc_list_item** LastItem ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );
    Assert_LIST_VALID( FirstItem, E_ttc_assert_origin_auto );
    Assert_LIST_VALID( LastItem, E_ttc_assert_origin_auto );

    t_base ListSize = ttc_list_size( List );
    t_base AmountRemoved;
    if ( ( ( Amount == -1 ) && ( List->ttc_list.Pending_Push_Amount ) ) ||                       // about to pop all items + we have pending ones
            ( ( ListSize < Amount ) && ( ListSize + List->ttc_list.Pending_Push_Amount >= Amount ) ) // list too short but pending items would fullfill our needs
       ) { // complete all pending items now (only done rarely as it locks interrupts)
        ttc_interrupt_critical_begin();             // must lock interrupts before running an isr
        AmountRemoved = ttc_list_pop_front_multiple_isr( List, Amount, FirstItem, LastItem );
        ttc_interrupt_critical_end();
    }

    while ( ttc_list_size( List ) == 0 ) { // list is empty: wait for data
        ttc_semaphore_take( &( List->ttc_list.AmountItems ), Amount, -1 );
    }

    ttc_mutex_lock( &( List->ttc_list.Lock ), -1 );
    ListSize = ( Amount == -1 ) ? // must only provide list size if popping all items
               ( ttc_list_get_amount( List ) + Amount ) :
               -1;

    AmountRemoved = _ttc_list_pop_front_multiple( List, ListSize, Amount, FirstItem, LastItem );
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    if ( *FirstItem ) {
        t_ttc_list_item* CurrentItem = *FirstItem;
        do {
            CurrentItem->ttc_list_item.Debug.Caller = __builtin_return_address( 0 ); // store address of calling function for debugging
            CurrentItem = CurrentItem->ttc_list_item.Next;
        }
        while ( ( CurrentItem != TTC_LIST_TERMINATOR ) && ( CurrentItem != *LastItem ) );
    }
#endif
    ttc_mutex_unlock( &( List->ttc_list.Lock ) );

    return AmountRemoved;
}
e_ttc_list_error ttc_list_pop_front_multiple_isr( t_ttc_list* List, t_base Amount, t_ttc_list_item** FirstItem, t_ttc_list_item** LastItem ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );
    Assert_LIST_VALID( FirstItem, E_ttc_assert_origin_auto );
    Assert_LIST_VALID( LastItem, E_ttc_assert_origin_auto );

    if ( ttc_mutex_is_locked( &( List->ttc_list.Lock ) ) )
    { return tle_ListInUseByTask; }          // list is locked by a task

    if ( List->ttc_list.Pending_Push_Amount )         // still data pending from last isr-push
    { _ttc_list_complete_all_pending_isr( List ); }

    if ( ttc_semaphore_take_isr( &( List->ttc_list.AmountItems ), Amount ) >= tsme_Error )
    { return tle_ListIsEmpty; }

    t_base ListSize = ( Amount == -1 ) ? // must only provide list size if popping all items
                      ListSize = ( ttc_list_get_amount( List ) + Amount ) :
                                 -1;

    _ttc_list_pop_front_multiple( List,
                                  ListSize,
                                  Amount,
                                  FirstItem,
                                  LastItem
                                );

#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    if ( *FirstItem ) {
        t_ttc_list_item* CurrentItem = *FirstItem;
        do {
            CurrentItem->ttc_list_item.Debug.Caller = __builtin_return_address( 0 ); // store address of calling function for debugging
            CurrentItem = CurrentItem->ttc_list_item.Next;
        }
        while ( ( CurrentItem != TTC_LIST_TERMINATOR ) && ( CurrentItem != *LastItem ) );
    }
#endif
    return tle_OK;
}
t_ttc_list_item* ttc_list_pop_front_single_isr( t_ttc_list* List ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!

    // we may use static variables as interrupts are never interrupted!
    static t_ttc_list_item* Item = NULL;

    if ( ttc_mutex_is_locked( &( List->ttc_list.Lock ) ) )
    { return ( t_ttc_list_item* ) 1; }       // list is locked by a task

    if ( List->ttc_list.Pending_Push_Amount )         // still data pending from last isr-push
    { _ttc_list_complete_all_pending_isr( List ); }

    if ( ttc_semaphore_take_isr( &( List->ttc_list.AmountItems ), 1 ) >= tsme_Error )
    { return NULL; }

    _ttc_list_pop_front_single( List, &Item );
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Item->ttc_list_item.Debug.Caller = __builtin_return_address( 0 );  // store address of calling function for debugging
#endif
    TTC_LIST_STATS_INCREASE( List, AmountPopFrontISR );

    return Item;
}
t_ttc_list_item* ttc_list_pop_front_single_try( t_ttc_list* List ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!
    t_ttc_list_item* Item;

    e_ttc_semaphore_error Error = ttc_semaphore_take_try( &( List->ttc_list.AmountItems ), 1 );
    if ( Error ) { // no items in regular list. Maybe we have some pending items?
        if ( List->ttc_list.Pending_Push_Amount ) { // complete all pending items now (only done rarely as it locks interrupts)
            ttc_interrupt_critical_begin();             // must lock interrupts before running an isr
            Item = ttc_list_pop_front_single_isr( List ); // isr will complete all pending items
            ttc_interrupt_critical_end();
            return Item;
        }
        return NULL;
    }

    ttc_mutex_lock( &( List->ttc_list.Lock ), -1 );

    _ttc_list_pop_front_single( List, &Item );
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Item->ttc_list_item.Debug.Caller = __builtin_return_address( 0 );  // store address of calling function for debugging
#endif
    TTC_LIST_STATS_INCREASE( List, AmountPopFront );

    ttc_mutex_unlock( &( List->ttc_list.Lock ) );
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto );  // invalid list (did you initialize it?)
    Assert_LIST_VALID( Item, E_ttc_assert_origin_auto );
#endif

    return Item;
}
t_ttc_list_item* ttc_list_pop_front_single_wait( t_ttc_list* List ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!

    t_ttc_list_item* Item;
    if ( ( ttc_list_size( List ) == 0 ) && List->ttc_list.Pending_Push_Amount ) { // list is empty but items are pending: complete all pending items now (only done rarely as it locks interrupts)
        ttc_interrupt_critical_begin();           // must lock interrupts before running an isr
        Item = ttc_list_pop_front_single_isr( List ); // complete all pending items
        ttc_interrupt_critical_end();
        if ( Item ) // got item from pending items: return it now
        { return Item; }
    }

    e_ttc_semaphore_error Error = ttc_semaphore_take( &( List->ttc_list.AmountItems ), 1, -1 ); ( void ) Error;
    Assert_LIST( Error == tsme_OK, E_ttc_assert_origin_auto );

    ttc_mutex_lock( &( List->ttc_list.Lock ), -1 );
    _ttc_list_pop_front_single( List, &Item );
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Item->ttc_list_item.Debug.Caller = __builtin_return_address( 0 );  // store address of calling function for debugging
#endif
    TTC_LIST_STATS_INCREASE( List, AmountPopFront );
    ttc_mutex_unlock( &( List->ttc_list.Lock ) );
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto );  // invalid list (did you initialize it?)
    Assert_LIST_VALID( Item, E_ttc_assert_origin_auto );
#endif

    return Item;
}
void             ttc_list_iterate( t_ttc_list* List, void* ( *IterateFunction )( void* Argument, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item ), void* Argument, t_s16 Steps, t_u16 SkipFirst, t_u16 Amount ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );             // invalid pointer given!

    ttc_mutex_lock( &( List->ttc_list.Lock ), -1 );
    _ttc_list_iterate_from( List, IterateFunction, Argument, SkipFirst, Steps, List->ttc_list.First, List->ttc_list.Last, Amount );
    ttc_mutex_unlock( &( List->ttc_list.Lock ) );
}
BOOL             ttc_list_iterate_try( t_ttc_list* List, void* ( *IterateFunction )( void* Argument, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item ), void* Argument, t_s16 Steps, t_u16 SkipFirst, t_u16 Amount ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );             // invalid pointer given!

    if ( ! ttc_mutex_lock_try( &( List->ttc_list.Lock ) ) ) { // obtained lock successfully
        _ttc_list_iterate_from( List, IterateFunction, Argument, SkipFirst, Steps, List->ttc_list.First, List->ttc_list.Last, Amount );
        ttc_mutex_unlock( &( List->ttc_list.Lock ) );
        return TRUE;
    }

    return FALSE; // could not obtain lock
}
void             ttc_list_iterate_from( t_ttc_list* List, void* ( *IterateFunction )( void* Argument, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item ), void* Argument, t_s16 Steps, t_ttc_list_item* Start, t_ttc_list_item* End, t_u16 Amount ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );             // invalid pointer given!

    ttc_mutex_lock( &( List->ttc_list.Lock ), -1 );
    _ttc_list_iterate_from( List, IterateFunction, Argument, 0, Steps, Start, End, Amount );
    ttc_mutex_unlock( &( List->ttc_list.Lock ) );
}
BOOL             ttc_list_iterate_from_try( t_ttc_list* List, void* ( *IterateFunction )( void* Argument, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item ), void* Argument, t_s16 Steps, t_ttc_list_item* Start, t_ttc_list_item* End, t_u16 Amount ) {
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );             // invalid pointer given!

    if ( ! ttc_mutex_lock_try( &( List->ttc_list.Lock ) ) ) { // obtained lock successfully
        _ttc_list_iterate_from( List, IterateFunction, Argument, 0, Steps, Start, End, Amount );
        ttc_mutex_unlock( &( List->ttc_list.Lock ) );
        return TRUE;
    }

    return FALSE; // could not obtain lock
}
t_base           ttc_list_items_count( t_ttc_list_item* FirstItem ) {
    if ( FirstItem == TTC_LIST_TERMINATOR )
    { return 0; }

    t_u16 Counter = 0; // Once your lists grow longer than 65535 items, you may increase size of this variable
    t_ttc_list_item* volatile CurrentItem = FirstItem;

#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
    t_ttc_list* List = FirstItem->ttc_list_item.Debug.Owner;
    if ( List ) {
        Assert_LIST_Writable( List, E_ttc_assert_origin_auto ); // Item claims to belong to a list, but its not located in RAM. Check if someone has modified this item!
    }
#endif

    while ( CurrentItem != TTC_LIST_TERMINATOR ) {
        Assert_LIST_Writable( CurrentItem, E_ttc_assert_origin_auto );
#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
        Assert_LIST( CurrentItem->ttc_list_item.Debug.Owner == List, E_ttc_assert_origin_auto ); // Item says that it does not belong to same list as first item. Check if someone has modified this item!
#endif
        CurrentItem = CurrentItem->ttc_list_item.Next;
        Counter++;
        Assert_LIST( Counter != 0, E_ttc_assert_origin_auto ); // endless loop detected in single linked list of list items. Check implementation of ttc_list and your application!
    }

    return Counter;
}
t_base           ttc_list_items_count_safe( t_ttc_list_item* FirstItem, t_base MaxSize ) {
    if ( FirstItem == TTC_LIST_TERMINATOR )
    { return 0; }

    t_base Counter = 0;
    t_ttc_list_item* volatile CurrentItem = FirstItem;

#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
    t_ttc_list* List = FirstItem->ttc_list_item.Debug.Owner;
    if ( List ) {
        Assert_LIST_Writable( List, E_ttc_assert_origin_auto ); // Item claims to belong to a list, but its not located in RAM. Check if someone has modified this item!
    }
#endif

    while ( ( CurrentItem != TTC_LIST_TERMINATOR ) && MaxSize ) {
        Assert_LIST_Writable( CurrentItem, E_ttc_assert_origin_auto );
        CurrentItem = CurrentItem->ttc_list_item.Next;
#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
        Assert_LIST( ( List == NULL ), E_ttc_assert_origin_auto ); // Item says that it does not belong to same list as first item. Check if someone has modified this item!
#endif
        Counter++;
        MaxSize--;
    }

    return Counter;
}

//}functions
//{ private functions *****************************************

void         _ttc_list_complete_all_pending_isr( t_ttc_list* List ) {
    Assert_LIST( ttc_interrupt_check_inside_isr(), E_ttc_assert_origin_auto ); // this function must be called with disable interrupts or from interrupt service routine only. Check implementation of caller!
    if ( List->ttc_list.Pending_Push_Front_First ) { // complete pending pushes to list head
        _ttc_list_push_front_multiple( List, List->ttc_list.Pending_Push_Front_First, List->ttc_list.Pending_Push_Front_Last );
        List->ttc_list.Pending_Push_Front_First  = NULL;
        List->ttc_list.Pending_Push_Front_Last   = NULL;
    }
    if ( List->ttc_list.Pending_Push_Back_First ) {  // complete pending pushes to list end
        _ttc_list_push_back_multiple( List,
                                      List->ttc_list.Pending_Push_Back_First,
                                      List->ttc_list.Pending_Push_Back_Last
                                    );
        List->ttc_list.Pending_Push_Back_First  = NULL;
        List->ttc_list.Pending_Push_Back_Last   = NULL;
    }

    // adjust list size
    ttc_semaphore_give_isr( &( List->ttc_list.AmountItems ), List->ttc_list.Pending_Push_Amount );
    Assert_LIST_EXTRA( ttc_list_get_amount( List ) == ttc_list_items_count( List->ttc_list.First ), E_ttc_assert_origin_auto ); // bug in size counter implementation
    List->ttc_list.Pending_Push_Amount = 0;

    TTC_LIST_STATS_INCREASE( List, AmountPendingCompletion );
}
void         _ttc_list_iterate_from( t_ttc_list* List, void* ( *IterateFunction )( void* Argument, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item ), void* Argument, t_u16 SkipFirst, t_s16 Steps, t_ttc_list_item* Start, t_ttc_list_item* End, t_u16 Amount ) {
    Assert_LIST_VALID_EXTRA( List, E_ttc_assert_origin_auto );             // invalid pointer given!
    Assert_LIST_Readable( IterateFunction, E_ttc_assert_origin_auto ); // invalid pointer given!
    Assert_LIST_EXTRA( Steps > 0, E_ttc_assert_origin_auto ); // invalid argument (backward stepping might be added in the future). Check caller code!
    if ( !Start )
    { return; }                    // nothing to do (ttc_list does not use NULL as item pointers)
    if ( !End )
    { End = TTC_LIST_TERMINATOR; } // ttc_list uses a special value to terminate last item in list

    t_ttc_list_item* Item = Start;

    t_ttc_list_item* PreviousItem = NULL;
    while ( SkipFirst && ( Item != TTC_LIST_TERMINATOR ) ) { // skip first items until start item
        Assert_LIST_EXTRA_Writable( Item, E_ttc_assert_origin_auto ); // invalid item pointer. Check list implementation! (Memory corrupted?)
        t_ttc_list_item* NextItem = Item->ttc_list_item.Next;

        SkipFirst--;
        if ( 1 ) { // proceed to next item
            PreviousItem = Item;
            Item         = NextItem; // this is correct next pointer even if IterateFunction() changed Item.Next

            t_s16 Count = Steps;
            while ( ( Item != TTC_LIST_TERMINATOR ) && ( --Count > 0 ) ) {  // skip additional items
                Assert_LIST_Writable( Item->ttc_list_item.Next, E_ttc_assert_origin_auto ); // next pointer must point to RAM. Memory corrupted? Check implementation of ttc_list and your application!
                PreviousItem = Item;
                Item = Item->ttc_list_item.Next;
            }
        }
    }

    // iterate forward First -> Last
    while ( Amount && ( Item != TTC_LIST_TERMINATOR ) ) {
        Assert_LIST_Writable( Item, E_ttc_assert_origin_auto ); // invalid item pointer. Check list implementation! (Memory corrupted?)
        t_ttc_list_item* NextItem = Item->ttc_list_item.Next;
        Assert_LIST_VALID( NextItem,   E_ttc_assert_origin_auto ); // invalid next pointer
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
        Assert_LIST( Item->ttc_list_item.Debug.Owner == List, E_ttc_assert_origin_auto ); // someone outside has modified this item. Check implementation of ttc_list and your application!
#endif

        void( *RemoveItem )( void* Argument, t_ttc_list_item * Item ) = ( void( * )( void*, t_ttc_list_item* ) ) IterateFunction( Argument, PreviousItem, Item ); // call iterate function
        Assert_LIST( Item->ttc_list_item.Next == NextItem, E_ttc_assert_origin_auto ); // IterateFunction() is not allowed to modify Item. Check implementation of IterateFunction()!
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
        Assert_LIST( Item->ttc_list_item.Debug.Owner == List,     E_ttc_assert_origin_auto ); // IterateFunction() is not allowed to modify Item. Check implementation of IterateFunction()!
#endif
        if ( RemoveItem ) { // removing current item
            e_ttc_semaphore_error Error = ttc_semaphore_take_try( &( List->ttc_list.AmountItems ), 1 ); ( void ) Error;
            Assert_LIST_EXTRA( !Error, E_ttc_assert_origin_auto ); // list should be locked before calling this function. Check list implementation!

            // remove item from its list (allows to add it to other lists or to release it to its pool)
            _ttc_list_remove_item( List, PreviousItem, Item, NextItem );

            if ( 1 != ( t_base ) RemoveItem ) { // function pointer given: execute given function
                Assert_LIST_Readable( RemoveItem, E_ttc_assert_origin_auto ); // returned pointer does not seem to point to executable code. Check implementation of IterateFunction()!
                RemoveItem( Argument, Item );
            }
            Item = PreviousItem;
        }
        if ( Amount != -1 )
        { Amount--; }

        if ( 1 ) { // proceed to next item
            PreviousItem = Item;
            Item         = NextItem; // this is correct next pointer even if IterateFunction() changed Item.Next

            t_s16 Count = Steps;
            while ( ( Item != TTC_LIST_TERMINATOR ) && ( --Count > 0 ) ) {  // skip additional items
                Assert_LIST_Writable( Item->ttc_list_item.Next, E_ttc_assert_origin_auto ); // next pointer must point to RAM. Memory corrupted? Check implementation of ttc_list and your application!
                PreviousItem = Item;
                Item = Item->ttc_list_item.Next;
            }
        }
    }
}
void         _ttc_list_push_back_single( t_ttc_list* List, t_ttc_list_item* Item ) {
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!
    Assert_LIST_VALID( Item, E_ttc_assert_origin_auto );
    Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto );  // invalid list (did you initialize it?)
    Assert_LIST_EXTRA( ttc_list_get_amount( List ) == ttc_list_items_count( List->ttc_list.First ), E_ttc_assert_origin_auto ); // bug in size counter implementation. Caller must add 1 to List->ttc_list.AmountItem after calling this function!
    Assert_LIST( List->ttc_list.Debug.LockedBy == tlo_None, E_ttc_assert_origin_auto );  // outer lock has been broken
    List->ttc_list.Debug.LockedBy = tlo_PushBackSingle;
    TTC_LIST_ADD_HISTORY( List );
#endif

#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
    Assert_LIST( ( Item->ttc_list_item.Debug.Owner == NULL ) || ( ttc_memory_is_constant( Item->ttc_list_item.Debug.Owner ) ), E_ttc_assert_origin_auto ); // Item still belongs to a list. Remove it from this list before freeing it!
#endif

    // push of multiple items would not allow to terminate within constant time!
    Item->ttc_list_item.Next = TTC_LIST_TERMINATOR;
#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
    Item->ttc_list_item.Debug.Owner = List;  // item now belongs to a list
#endif

    if ( List->ttc_list.Last ) { // list is not empty: append item at end of list
        List->ttc_list.Last->ttc_list_item.Next = Item;
        List->ttc_list.Last       = Item;
    }
    else {              // list is empty: set list of new items as new list
        List->ttc_list.First = Item;
        List->ttc_list.Last  = Item;
    }

#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Assert_LIST_VALID( Item->ttc_list_item.Next, E_ttc_assert_origin_auto );
    Assert_LIST_VALID( List->ttc_list.Last->ttc_list_item.Next, E_ttc_assert_origin_auto );
    Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto );  // invalid list (did you initialize it?)
    Assert_LIST_VALID( List->ttc_list.Last, E_ttc_assert_origin_auto );
    Assert_LIST( ( ttc_list_get_amount( List ) + 1 ) == ttc_list_items_count( List->ttc_list.First ), E_ttc_assert_origin_auto ); // bug in size counter implementation
    List->ttc_list.Debug.LockedBy        = tlo_None;
    TTC_LIST_ADD_HISTORY( List );
#endif
}
void         _ttc_list_push_back_multiple( t_ttc_list* List, t_ttc_list_item* FirstItem, t_ttc_list_item* LastItem ) {
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!
    Assert_LIST_VALID( FirstItem, E_ttc_assert_origin_auto );
    Assert_LIST_VALID( LastItem, E_ttc_assert_origin_auto );
    if ( FirstItem ) {
        Assert_LIST_EXTRA_Writable( FirstItem, E_ttc_assert_origin_auto ); // If first item is set, last item must be set too. Check implementation of ttc_list!
    }
    Assert_LIST_EXTRA( ttc_list_get_amount( List ) == ttc_list_items_count( List->ttc_list.First ), E_ttc_assert_origin_auto ); // bug in size counter implementation. Caller must add Amount to List->ttc_list.AmountItem after calling this function!
    Assert_LIST( List->ttc_list.Debug.LockedBy == tlo_None, E_ttc_assert_origin_auto );  // outer lock has been broken
    List->ttc_list.Debug.LockedBy = tlo_PushBackMultiple;
    TTC_LIST_STATS_INCREASE( List, AmountPushBackMultiple );
    TTC_LIST_ADD_HISTORY( List );
#endif

    if ( List->ttc_list.Last ) { // list is not empty: append item at end of list
        List->ttc_list.Last->ttc_list_item.Next = FirstItem;
        List->ttc_list.Last       = LastItem;
    }
    else {              // list is empty: set list of new items as new list
        List->ttc_list.First = FirstItem;
        List->ttc_list.Last  = LastItem;
    }

#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
    do { // set ->ttc_list_item.Debug.Owner field of all items
        Assert_LIST_VALID_EXTRA( FirstItem, E_ttc_assert_origin_auto );
        Assert_LIST( ( FirstItem->ttc_list_item.Debug.Owner == NULL ) || ( FirstItem->ttc_list_item.Debug.Owner == List ), E_ttc_assert_origin_auto ); // Item still belongs to a list. Remove it from this list before freeing it!
        FirstItem->ttc_list_item.Debug.Owner = List;
        FirstItem = FirstItem->ttc_list_item.Next;
    }
    while ( ( FirstItem != LastItem ) && ( FirstItem != TTC_LIST_TERMINATOR ) );
#endif

#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    List->ttc_list.Debug.LockedBy        = tlo_None;
    TTC_LIST_ADD_HISTORY( List );
#endif
}
void         _ttc_list_push_front_single( t_ttc_list* List, t_ttc_list_item* Item ) {
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!
    Assert_LIST_VALID( Item, E_ttc_assert_origin_auto );
    Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto );  // invalid list (did you initialize it?)
    Assert_LIST_EXTRA( ttc_list_get_amount( List ) == ttc_list_items_count( List->ttc_list.First ), E_ttc_assert_origin_auto ); // bug in size counter implementation. Caller must add 1 to List->ttc_list.AmountItem after calling this function!
    Assert_LIST( List->ttc_list.Debug.LockedBy == tlo_None, E_ttc_assert_origin_auto );  // outer lock has been broken
    List->ttc_list.Debug.LockedBy = tlo_PushFrontSingle;
    TTC_LIST_ADD_HISTORY( List );
#endif

#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
    Assert_LIST( ( Item->ttc_list_item.Debug.Owner == NULL ) || ( ttc_memory_is_constant( Item->ttc_list_item.Debug.Owner ) ), E_ttc_assert_origin_auto ); // Item still belongs to a list. Remove it from this list before freeing it!
#endif

    if ( List->ttc_list.First ) { // list is not empty: append item at head of list
        Item->ttc_list_item.Next  = List->ttc_list.First;
        List->ttc_list.First = Item;
    }
    else {               // list is empty: set list of new items as new list
        List->ttc_list.First = Item;
        List->ttc_list.Last  = Item;

        // push of multiple items would not allow to terminate within constant time!
        Item->ttc_list_item.Next = TTC_LIST_TERMINATOR;
    }
#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
    Item->ttc_list_item.Debug.Owner = List;  // item now belongs to a list
#endif

#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    if ( ( Item->ttc_list_item.Next ) != TTC_LIST_TERMINATOR ) {
        Assert_LIST_VALID( Item->ttc_list_item.Next, E_ttc_assert_origin_auto );
    }
    if ( ( List->ttc_list.Last->ttc_list_item.Next ) != TTC_LIST_TERMINATOR ) {
        Assert_LIST_VALID( List->ttc_list.Last->ttc_list_item.Next, E_ttc_assert_origin_auto );
    }
    Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto );  // invalid list (did you initialize it?)
    Assert_LIST_VALID( List->ttc_list.Last,  E_ttc_assert_origin_auto );
    Assert_LIST( ( ttc_list_get_amount( List ) + 1 ) == ttc_list_items_count( List->ttc_list.First ), E_ttc_assert_origin_auto ); // bug in size counter implementation
    List->ttc_list.Debug.LockedBy        = tlo_None;
    TTC_LIST_ADD_HISTORY( List );
#endif
}
void         _ttc_list_push_front_multiple( t_ttc_list* List, t_ttc_list_item* FirstItem, t_ttc_list_item* LastItem ) {
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!
    Assert_LIST_VALID( FirstItem, E_ttc_assert_origin_auto );
    Assert_LIST_VALID( LastItem, E_ttc_assert_origin_auto );
    if ( FirstItem ) {
        Assert_LIST_EXTRA_Writable( FirstItem, E_ttc_assert_origin_auto ); // If first item is set, last item must be set too. Check implementation of ttc_list!
    }
    Assert_LIST_EXTRA( ttc_list_get_amount( List ) == ttc_list_items_count( List->ttc_list.First ), E_ttc_assert_origin_auto ); // bug in size counter implementation. Caller must add 1 to List->ttc_list.AmountItem after calling this function!
    Assert_LIST( List->ttc_list.Debug.LockedBy == tlo_None, E_ttc_assert_origin_auto );  // outer lock has been broken
    List->ttc_list.Debug.LockedBy = tlo_PushFrontMultiple;
    TTC_LIST_STATS_INCREASE( List, AmountPushFront );
    TTC_LIST_ADD_HISTORY( List );
#endif

    if ( List->ttc_list.Last ) { // list is not empty: append item before first item in list
        LastItem->ttc_list_item.Next = List->ttc_list.First;
        List->ttc_list.First    = FirstItem;
    }
    else {              // list is empty: set list of new items as new list
        List->ttc_list.First = FirstItem;
        List->ttc_list.Last  = LastItem;
    }

#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    do { // check all items to be added
#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
        Assert_LIST( FirstItem->ttc_list_item.Debug.Owner == NULL, E_ttc_assert_origin_auto ); // Item still belongs to a list. Remove it from this list before freeing it!
        FirstItem->ttc_list_item.Debug.Owner = List; // item belongs to this list now
#endif
        FirstItem = FirstItem->ttc_list_item.Next;
        Assert_LIST_VALID_EXTRA( FirstItem, E_ttc_assert_origin_auto );
    }
    while ( ( FirstItem != TTC_LIST_TERMINATOR ) && ( FirstItem != LastItem ) );

    List->ttc_list.Debug.LockedBy        = tlo_None;
    TTC_LIST_ADD_HISTORY( List );
#endif
}
void         _ttc_list_pop_front_single( t_ttc_list* List, t_ttc_list_item** Item ) {
    t_ttc_list_item* FirstItem = List->ttc_list.First;

#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!
    Assert_LIST_VALID( Item, E_ttc_assert_origin_auto );
    Assert_LIST( ( ttc_list_get_amount( List ) + 1 ) == ttc_list_items_count( FirstItem ), E_ttc_assert_origin_auto ); // bug in size counter implementation. Caller must take 1 from List->ttc_list.AmountItem before calling this function!
    Assert_LIST( List->ttc_list.Debug.LockedBy == tlo_None, E_ttc_assert_origin_auto );  // outer lock has been broken
    List->ttc_list.Debug.LockedBy = tlo_PopFrontSingle;
    TTC_LIST_ADD_HISTORY( List );
#endif

#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Assert_LIST_VALID( FirstItem, E_ttc_assert_origin_auto );
#endif
    if ( FirstItem ) {   // list stores items: remove first one
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
        if ( ( List->ttc_list.First->ttc_list_item.Next ) != TTC_LIST_TERMINATOR ) {
            Assert_LIST_VALID( List->ttc_list.First->ttc_list_item.Next, E_ttc_assert_origin_auto );
        }
#endif
        if ( List->ttc_list.First->ttc_list_item.Next == TTC_LIST_TERMINATOR ) { // list is empty now
            List->ttc_list.First = NULL;
            List->ttc_list.Last  = NULL;
        }
        else
        { List->ttc_list.First = List->ttc_list.First->ttc_list_item.Next; }
    }

    if ( Item ) {
        *Item = FirstItem; // store item in given pointer

#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
        FirstItem->ttc_list_item.Debug.Owner = NULL; // item is free now
#endif
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
        FirstItem->ttc_list_item.Debug.Caller = __builtin_return_address( 0 );  // store address of calling function for debugging
#endif
    }

#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto );  // invalid list (did you initialize it?)
    Assert_LIST_EXTRA( ttc_list_get_amount( List ) == ttc_list_items_count( List->ttc_list.First ), E_ttc_assert_origin_auto ); // bug in size counter implementation
    List->ttc_list.Debug.LockedBy        = tlo_None;
    TTC_LIST_ADD_HISTORY( List );
#endif
}
t_base       _ttc_list_pop_front_multiple( t_ttc_list* List, t_base ListSize, t_base Amount, t_ttc_list_item** FirstItem, t_ttc_list_item** LastItem ) {
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Assert_LIST( List, E_ttc_assert_origin_auto );
    Assert_LIST( FirstItem, E_ttc_assert_origin_auto );
    Assert_LIST( LastItem, E_ttc_assert_origin_auto );
    Assert_LIST_EXTRA( ( ttc_list_get_amount( List ) + Amount ) == ttc_list_items_count( List->ttc_list.First ), E_ttc_assert_origin_auto ); // bug in size counter implementation. Caller must take Amount from List->ttc_list.AmountItem before calling this function!
    Assert_LIST( List->ttc_list.Debug.LockedBy == tlo_None, E_ttc_assert_origin_auto );  // outer lock has been broken
    List->ttc_list.Debug.LockedBy = tlo_PopFrontMultiple;
    TTC_LIST_STATS_INCREASE( List, AmountPopFront );
    TTC_LIST_ADD_HISTORY( List );
#endif

    if ( List->ttc_list.First == NULL ) { // list is empty
        *FirstItem = NULL;
        *LastItem  = NULL;
        return 0;
    }
    if ( Amount == -1 ) {      // request for all items in list
        Amount      = ListSize;
        *FirstItem  = List->ttc_list.First;
        *LastItem   = List->ttc_list.Last;
        List->ttc_list.First = NULL;
        List->ttc_list.Last  = NULL;

#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
        t_ttc_list_item* CurrentItem = List->ttc_list.First;
        if ( CurrentItem ) { // free all items from this list
            do {
                CurrentItem->ttc_list_item.Debug.Owner = NULL; // Item is free now
                CurrentItem = CurrentItem->ttc_list_item.Next;
            }
            while ( CurrentItem != TTC_LIST_TERMINATOR );
        }
#endif
    }
    else {                     // request for limited amount of items
        if ( Amount > ListSize ) { // list has less items than requested
            Amount = ListSize;
        }
        t_ttc_list_item* CurrentItem = List->ttc_list.First;
        *FirstItem = CurrentItem;

        t_base Count = Amount;
        while ( Count-- > 0 ) { // collect requested amount of items (find last one)
            Assert_LIST_VALID_EXTRA( CurrentItem, E_ttc_assert_origin_auto );
            t_ttc_list_item* NextItem = CurrentItem->ttc_list_item.Next;
#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
            CurrentItem->ttc_list_item.Debug.Owner = NULL; // Item is free now
#endif
            CurrentItem = NextItem;
        }
        *LastItem   = CurrentItem;
        List->ttc_list.First = CurrentItem;
        if ( CurrentItem == TTC_LIST_TERMINATOR ) // list is empty now
        { List->ttc_list.First = NULL; List->ttc_list.Last = NULL; }
    }

#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto );  // invalid list (did you initialize it?)
    Assert_LIST_EXTRA( ttc_list_get_amount( List ) == ttc_list_items_count( List->ttc_list.First ), E_ttc_assert_origin_auto ); // bug in size counter implementation
    List->ttc_list.Debug.LockedBy        = tlo_None;
    TTC_LIST_ADD_HISTORY( List );
#endif

    return Amount;
}
void         _ttc_list_remove_item( t_ttc_list* List, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item, t_ttc_list_item* NextItem ) {
    Assert_LIST_Writable( List, E_ttc_assert_origin_auto );    // invalid argument: Check implementation of caller!
    Assert_LIST_Writable( Item, E_ttc_assert_origin_auto );    // invalid argument: Check implementation of caller!

#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Assert_LIST_VALID( List, E_ttc_assert_origin_auto );        // invalid pointer given!
    Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto );
    Assert_LIST_VALID( PreviousItem, E_ttc_assert_origin_auto );
    Assert_LIST_Writable( Item, E_ttc_assert_origin_auto );
    Assert_LIST_EXTRA( ( ttc_list_get_amount( List ) + 1 ) == ttc_list_items_count( List->ttc_list.First ), E_ttc_assert_origin_auto ); // bug in size counter implementation. Caller must take 1 from List->ttc_list.AmountItem before calling this function!
    Assert_LIST( List->ttc_list.Debug.LockedBy == tlo_None, E_ttc_assert_origin_auto );  // outer lock has been broken
    List->ttc_list.Debug.LockedBy = tlo_RemoveItem;
    TTC_LIST_STATS_INCREASE( List, AmountRemoveItem );
    TTC_LIST_ADD_HISTORY( List );
#endif

    if ( List->ttc_list.First ) {   // list stores >=1 items: remove given one
        if ( PreviousItem ) {       // redirect link of previous item
            PreviousItem->ttc_list_item.Next = NextItem;
            if ( Item == List->ttc_list.Last ) { // removing at list tail
                List->ttc_list.Last = PreviousItem;
            }
        }
        else {                      // removing at list head
            if ( NextItem != TTC_LIST_TERMINATOR )
            { List->ttc_list.First = NextItem; }
            else { // list is empty now
                List->ttc_list.First = NULL;
                List->ttc_list.Last  = NULL;
            }
        }

#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
        Item->ttc_list_item.Debug.Caller = __builtin_return_address( 0 );  // store address of calling function for debugging
#endif
#if (TTC_LIST_ITEMS_IN_ONE_LIST_ONLY == 1)
        Item->ttc_list_item.Debug.Owner = NULL; // Item is free now
#endif

#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
        if ( List->ttc_list.First ) {
            Assert_LIST_VALID( List->ttc_list.First->ttc_list_item.Next, E_ttc_assert_origin_auto );
        }
#endif
    }

#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIST_EXTRA == 1)
    Assert_LIST_VALID( List->ttc_list.First, E_ttc_assert_origin_auto );  // invalid list (did you initialize it?)
    Assert_LIST_EXTRA( ttc_list_get_amount( List ) == ttc_list_items_count( List->ttc_list.First ), E_ttc_assert_origin_auto ); // bug in size counter implementation
    List->ttc_list.Debug.LockedBy        = tlo_None;
    TTC_LIST_ADD_HISTORY( List );
#endif
}

//} private functions
