#ifndef ttc_list_H
#define ttc_list_H

/** ttc_list.h ***********************************************{
 *
 * Written by Gregor Rebel 2013-2019
 *
 * Why to use lists instead of queues?
 * - lists have no size limit (every item brings its own next-pointer)
 * - lists require only as much memory as required (no oversized pre-allocation like queues)
 * - lists can be used from interrupt service routines too
 * - every pool-allocated memory block can be put into a list (-> ttc_memory.h)
 *
 * When not to prefer lists over queues?
 * - if you want to store big amount of very small blocks like single bytes
 *   (every item requires a next-pointer)
 * H2: Extensible Single Linked Lists
 *
 * Lists provide dynamic storage similar to queues or arrays.
 *
 * H3: Advantages of ttc_list over ttc_queue and Arrays
 * *) Unlimited amount of item storage in each list.
 *    To be more precise: All list items that fit into RAM can be stored in one ttc_list instance.
 * *) No static preallocation required.
 *    Empty lists do occupy only small amount of RAM.
 *    The maximum amount of items to be stored must not be known at instancian.
 * *) No memory fragmentation.
 *    No dynamic memory must be freed and reallocated.
 * *) Variable Item Sizes.
 *    List and list-item structures can be extended to carry any amount of user data.
 * *) Lists can be used from interrupt service routines too.
 *    ttc_list provides special *_isr() functions to be called from interrupt service routines.
 * *) Every memory block allocated via ttc_heap_pool_*() can be put into a list (-> ttc_heap.h).
 *    The t_ttc_heap_block_from_pool is itself a t_ttc_list_item structure.
 * *) Full Debug Visibility
 *    Despite being a universal single linked list implementation, all extra user data field can
 *    bee seen in the debugger (GDB, DDD, ...),
 * *) Extra Debug Support
 *    *) Individual List Items Tracking.
 *    *) History of last calling Functions.
 *    *) General Access Statistics.
 *
 * H3: Disadvantages of ttc_list compared to ttc_queue and Arrays
 * *) Constant RAM usage per List Item.
 *    Storing only few bytes in each list item can increase overall RAM usage a lot.
 *    User has to balance between flexibility and RAM usage.
 * *) No Direct Access by Index
 *    Access by index (n-th element of list) take runtime O(n) instead of O(1) for arrays.
 *    Further improvements can reduce this to O(log n) by increasing RAM usage.
 * *) Only optimized in one Direction (single linked).
 *    It is possible to extend ttc_list to use double linked lists without changing user application.
 *    This may be realized via low-level drivers in the future.
 *
 * H2: Example: Traditional User Defined List
 * The example defines its own list and list-item structures and embeds a t_ttc_list / t_ttc_list_item struct.
 * This alllows to use full compiler checks for ttc_list_*() calls but reduces visibilty during debugger sessions.
 * <CODE>
 *     // we define our own special list items
 *     typedef struct s_my_list_item {
 *         t_ttc_list_item ttc_list_item; // must be first entry!
 *
 *         // may add extra fields being stored in each list item here
 *         t_u8 MyData;
 *     } t_my_list_item;
 *
 *     // now we define our own list for items of type t_my_list_item
 *     typedef struct {
 *         t_ttc_list ttc_list; // must be first entry!
 *
 *         // may add extra fields here
 *     } t_my_list;
 *
 *     t_my_list_item Item1;
 *     t_my_list_item Item2;
 *     t_my_list_item Item3;
 *
 *     t_ttc_list MyList;
 *     ttc_list_init(&MyList);
 *     ttc_list_push_back_single(&MyList.ttc_list, &Item1.ttc_list_item);
 *     ttc_list_push_back_single(&MyList.ttc_list, &Item2.ttc_list_item);
 *     ttc_list_push_back_single(&MyList.ttc_list, &Item3.ttc_list_item);
 *
 *     // Here we have to type cast but we know best, which items are stored in our list
 *     t_my_list_item* FirstItem  = (t_my_list_item*) ttc_list_peek_front(&MyList.ttc_list);
 *     t_my_list_item* ThirdItem  = (t_my_list_item*) ttc_list_peek_index_cast( &MyList.ttc_list, 3);
 *
 *     // If concurrent accesses to list data are no concern, then we can directly read from list structure
 *     // As the Next pointer (in this example) only points to t_ttc_list_item, we have to typecast
 *     // Doing this during a debugger session is painfull!
 *     t_my_list_item* SecondItem = (t_my_list_item*) FirstItem->ttc_list.Next;
 *                     ThirdItem  = (t_my_list_item*) FirstItem->ttc_list.Next->Next;
 *
 *     // Using Next pointers requires additional type casting (difficult to do during debug session and dangerous)
 *     t_u8 MyData = ( (t_my_list_item*) FirstItem->ttc_list.Next )->MyData;
 * </CODE>
 *
 * H2: Example: Debugger friendly User Defined List
 * The example defines its own list and list-item structure.
 * Using user defined structures for list head and list items requires to typecast pointers when calling
 * ttc_list_*() functions. For ease of use, every ttc_list_*() has a corresponding ttc_list_*_cast() macro
 * which will type cast your pointers automatically.
 * This will of course disable compiler checks. On the other hand, you will now be able to see all
 * user data fields in your debugger.
 * You may want to debug example_leds in DDD to see the advantage.
 * <CODE>
 *     // we define our own special list items
 *     typedef struct s_my_list_item {
 *         TTC_LIST_ITEM_HEADER(struct s_my_list_item); // must be first entry!
 *
 *         // may add extra fields being stored in each list item here
 *         t_u8 MyData;
 *     } t_my_list_item;
 *
 *     // now we define our own list for items of type t_my_list_item
 *     typedef struct {
 *         TTC_LIST_HEADER(t_my_list_item); // must be first entry!
 *
 *         // may add extra fields here
 *     } t_my_list;
 *
 *     t_my_list_item Item1;
 *     t_my_list_item Item2;
 *     t_my_list_item Item3;
 *
 *     t_ttc_list MyList;
 *     ttc_list_init_cast(&MyList);
 *     ttc_list_push_back_single_cast(&MyList, &Item1);
 *     ttc_list_push_back_single_cast(&MyList, &Item2);
 *     ttc_list_push_back_single_cast(&MyList, &Item3);
 *
 *     // Here we have to type cast but we know best, which items are stored in our list
 *     t_my_list_item* FirstItem  = (t_my_list_item*) ttc_list_peek_front_cast(&MyList);
 *     t_my_list_item* ThirdItem  = (t_my_list_item*) ttc_list_peek_index_cast( &MyList, 3);
 *
 *     // If concurrent accesses to list data are no concern, then we can directly read from list structure.
 *     // No typecasts are required this way as we always use pointers to t_my_list_item.
 *     // During a debug session, we can directly expand all sub structures using a double click only.
 *     t_my_list_item* SecondItem = FirstItem->ttc_list.Next;
 *                     ThirdItem  = FirstItem->ttc_list.Next->Next;
 *
 *     // Now we can use Next pointers without type casting
 *     t_u8 MyData = FirstItem->ttc_list.Next->MyData;
 * </CODE>
 */
/** H2: Description of ttc_list (Do not delete this line!)
 *
 * Why to use lists instead of queues?
 * - Lists have no size limit (every item brings its own next-pointer).
 * - Lists require only as much memory as required (no oversized pre-allocation like queues).
 * - Lists have constant access times too.
 *
 * When not to prefer lists over queues?
 * - If you want to store many small blocks like single bytes
 *   (every item requires at least one next-pointer).
 *
 * Lists can be used as
 * - Input/ Output Queues of dynamic Length
 * - Storage for Memory Blocks
 * - Communication Channel Task<->Task and Task<->Interrupt Service Routines
 * - Task Synchronization
 *
 *
 * H2: Features of ttc_list
 *
 * - Each item can carry a pointer to its list.
 *   Extra self tests then can check if an item should be stored in two lists at the same time.
 *   This typically indicates an error condition that is otherwise difficult to detect.
 *   See TTC_LIST_ITEMS_IN_ONE_LIST_ONLY in ttc_lity_types.h for details!
 *
 * - If a task waits to pull from an empty list, it is put to sleep and awaken after next push.
 *   The same applies for user application in single tasking. An interrupt service routine can
 *   use a *_push_*_isr() function to wake up the user application.
 *
 * - Special _isr() functions are available for interrupt service routines.
 *   An interrupt service routine cannot use the generic *_push_*()/ *_pull_*() functions.
 *   If a user space application currently is accessing a ttc_list, then dara corruption would occur.
 *   The special *_isr() functions will never block and avoid data corruption.
 *
 * H2: Examples for ttc_list
 *
 * H3: Example 1 - Embedding List Item
 *
 * Every memory block starting with a t_ttc_list_item field can be added to a ttc_list.
 *   Simply add this as the first entry to your struct definition:
 *       #include "ttc_list_types.h"
 *       struct {
 *          t_ttc_list_item Item; // required as first entry!
 *          ...
 *       } t_foo;
 *
 *  Using t_ttc_list_item makes debugging a little more difficult as the next pointer does not know its
 *  enclosing struct. You can increase debugability by embedding Next and a special Macro into your structure:
 *
 * H3: Example 2 - Embedding Next Pointer
 *
 *  #include "ttc-lib/ttc_list_item_types.h"  // structures of individual list items
 *  typedef struct s_my_universal_item {
 *    struct s_my_universal_item* Next;  // points to next item in single-linked list (MUST BE FIRST ENTRY IN STRUCT!)
 *    TTC_LIST_ITEM_HEADER;                   // extra list item header                    (MUST BE SECOND ENTRY IN STRUCT!)
 *
 *    // any other fields come below
 *    t_u8 Foo;
 *    ...
 *  } t_my_universal_item;
 *
 *  // count linked list items
 *  t_my_universal_item Item1; ttc_memory_set(&Item1, 0, sizeof(Item1));
 *  t_my_universal_item Item2; ttc_memory_set(&Item2, 0, sizeof(Item2));
 *  t_my_universal_item Item3; ttc_memory_set(&Item3, 0, sizeof(Item3));
 *  Item1.Next = &Item2;
 *  Item2.Next = &Item3;
 *  Item3.Next = NULL;
 *  t_base AmountItems = &Item1;
 *
 * See example_ttc_list for a live implementation using ttc__list!
}*/

//{ includes

#include "ttc_basic.h"  // Include First! (basic datatypes and definitions)
#include "ttc_list_types.h"
#include "ttc_mutex_types.h"
#include "ttc_memory_types.h"
#include "ttc_heap_types.h"
#include "ttc_semaphore.h"

//}includes
//{ Default settings (change via makefile) ******************************

//}Default settings
//{ Types & Structures ***************************************************

//}
//{ Function prototypes **************************************************

/** returns current amount of items stored in given list
 *
 * @param List   pointer to list structure being initialized by ttc_list_init()
 */
t_base ttc_list_get_amount( t_ttc_list* List );
t_base ttc_list_get_amount_cast( t_ttc_list* List );
#define ttc_list_get_amount_cast(List) \
    ttc_list_get_amount((t_ttc_list*) List)

/** counts amount of items linked together starting at given item
 *
 * Note: This functions terminates within O(n) with n = amount of items
 * Note: This functions will not terminate if list contains a loop!
 *
 * @param FirstItem   list item to start counting at
 * @return            amount of items linked together until first NULL-item
 */
t_base ttc_list_items_count( t_ttc_list_item* FirstItem );
t_base ttc_list_items_count_cast( t_ttc_list_item* FirstItem );
#define ttc_list_items_count_cast( FirstItem )  \
    ttc_list_items_count( (t_ttc_list_item*) FirstItem )

/** counts amount of items linked together starting at given item (always returns)
 *
 * Note: This functions terminates within O(n) with n = amount of items
 *
 * @param FirstItem   list item to start counting at
 * @param MaxSize     counting will be stopped when list is longer
 * @return            amount of items linked together until first NULL-item
 */
t_base ttc_list_items_count_safe( t_ttc_list_item* FirstItem, t_base MaxSize );
t_base ttc_list_items_count_safe_cast( t_ttc_list_item* FirstItem, t_base MaxSize );
#define ttc_list_items_count_safe_cast( FirstItem, MaxSize ) \
    ttc_list_items_count_safe( (t_ttc_list_item*) FirstItem, MaxSize )

/** allocates + initializes a new list
 *
 * @param  Name  used for debugging purposes only
 * @return =! NULL: list has been created successfully (use handle to operate on list)
 */
t_ttc_list* ttc_list_create( const char* Name );
t_ttc_list* ttc_list_create_cast( const char* Name );
#define ttc_list_create_cast( Name ) \
    ttc_list_create( Name )

/** initializes a new list in given storage place
 *
 * @param List   pointer to list structure being initialized by ttc_list_init()
 * @param Name   used for debugging purposes only
 * @return =! NULL: list has been created successfully (use handle to operate on list)
 */
void ttc_list_init( t_ttc_list* List, const char* Name );
void ttc_list_init_cast( t_ttc_list* List, const char* Name );
#define ttc_list_init_cast( List, Name ) \
    ttc_list_init( (t_ttc_list*) List, Name )

/** Run iterator function beginning with n-th item in given list
 *
 * Note: List will be locked for push/pop operations during iteration.
 *
 * IterateFunction() can return TRUE to have Item being removed from list.
 * In such a case, it is safe for IterateFunction() to call ttc_heap_pool_block_free() to return
 * memory buffer to its memory pool (if it came from one).

 // Example:

 typedef struct { // a list item storing data
   t_ttc_list_item Item;  // header of all list items
   t_base Value;
   ...
 } t_my_type;

 void release_item(void* Argument, t_ttc_list_item* Item) {
   (void) Argument;                // not used in this example
   ttc_heap_pool_block_free(Item); // send memory buffer back to its pool
 }

 BOOL filter( void* Argument, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item ) {
    t_base Search = (t_base) Argument; // we simply expect this type

   // only works if this item came from a memory pool
   t_ttc_heap_block_from_pool* MemoryBlock = ttc_heap_pool_from_list_item( Item );

   // memory blocks from pool may be casted to anything
   t_my_type* Data = ( t_my_type* ) MemoryBlock;

   if (Data->Value == Search) { // found item with matching value
     return release_item; // Item will be removed from list and returned to its memory pool by calling ttc_heap_pool_block_free( (void*) Item) automatically.
   }

   return FALSE; // Item will remainf in list
 }

 ...
 const t_base Search = 42;

 // Call filter(42, PrevItem, Item) for all items from number (1, 2, .., last)
 ttc_list_iterate(List, filter, Search, 1, 0, -1);

 // Call filter(42, PrevItem, Item) for for up to 5 of every second item starting at fourth item (4, 6, 8, 10, 12)
 ttc_list_iterate(List, filter, Search, 2, 3, 5);

 * @param List             pointer to list structure being initialized by ttc_list_init()
 * @param IterateFunction  function pointer: will be called for each list item.
 *                         Note: IterateFunction() may not append item to other lists or release it. It may return a function pointer that can process the item later (see @return below)!
 *                         @return ==0: do not remove item; ==1: current item shall be removed from list
 *                                 >1: function to call with item as argument AFTER removing it from current list.
 *                                     Signature of function being called: void RemoveItem(void* Argument, t_ttc_list_item* Item)
 * @param Argument         argument passed to IterateFunction() and RemoveItem()
 * @param Step             Allows to skip items:
 *                         == 1: iterate all items 1, 2, ..., last
 *                         == 2: iterate every second item 1, 3, ... last
 * @param SkipFirst        == 0: start at first item; >0: skip first items and start at item number SkipFirst + 1
 * @param Amount           maximum amount of items to iterate (==-1: not limited)
 */
void ttc_list_iterate( t_ttc_list* List, void* ( *IterateFunction )( void* Argument, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item ), void* Argument, t_s16 Step, t_u16 SkipFirst, t_u16 Amount );
void ttc_list_iterate_cast( t_ttc_list* List, void* ( *IterateFunction )( void* Argument, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item ), void* Argument, t_s16 Step, t_u16 SkipFirst, t_u16 Amount );
#define ttc_list_iterate_cast( List, IterateFunction, Argument, Step, SkipFirst, Amount )  \
    ttc_list_iterate( (t_ttc_list*) List, (void* (*)( void*, t_ttc_list_item*, t_ttc_list_item* )) IterateFunction, Argument, Step, SkipFirst, Amount )

/** Try to tun iterator function beginning with n-th item in given list
 *
 * Functionality is basically the same as ttc_list_iterate(). See its description above for details!
 * The only difference is that it will not block if List->Mutex is locked but return FALSE instead.
 * @return         ==TRUE: list was iterated through successfully; ==FALSE: list was locked and no iteration took place.
 */
BOOL ttc_list_iterate_try( t_ttc_list* List, void* ( *IterateFunction )( void* Argument, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item ), void* Argument, t_s16 Steps, t_u16 SkipFirst, t_u16 Amount );
BOOL ttc_list_iterate_try_cast( t_ttc_list* List, void* ( *IterateFunction )( void* Argument, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item ), void* Argument, t_s16 Steps, t_u16 SkipFirst, t_u16 Amount );
#define ttc_list_iterate_try_cast( List, IterateFunction, Argument, Step, SkipFirst, Amount )  \
    ttc_list_iterate_try( (t_ttc_list*) List, (void* (*)( void*, t_ttc_list_item*, t_ttc_list_item* )) IterateFunction, Argument, Step, SkipFirst, Amount )

/** Run iterator function beginning from start item in given list
 *
 * Usage is similar to ttc_list_iterate(). See its description above for an usage example.
 *
 * @param List             pointer to list structure being initialized by ttc_list_init()
 * @param IterateFunction  function pointer: will be called for each list item. Returns TRUE if current item shall be removed from list.
 *                         Note: IterateFunction() may not append item to other lists or release it. It may return a function pointer that can process the item later (see @return below)!
 *                         @return ==0: do not remove item; ==1: current item shall be removed from list
 *                                 >1: function to call with item as argument AFTER removing it from current list.
 *                                     Signature of function being called: void RemoveItem(void* Argument, t_ttc_list_item* Item)
 * @param Argument         argument passed to IterateFunction() and RemoveItem()
 * @param Step             Allows to skip items:
 *                         == 1: iterate all items 1, 2, ..., last
 *                         == 2: iterate every second item 1, 3, ... last
 * @param Start            pointer to list item for which IterateFunction is called first
 * @param End              pointer to list item for which IterateFunction is called last (==TTC_LIST_TERMINATOR or ==NULL: run to end of list)
 * @param Amount           maximum amount of items to iterate (==-1: not limited)
 */
void ttc_list_iterate_from( t_ttc_list* List, void* ( *IterateFunction )( void* Argument, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item ), void* Argument, t_s16 Step, t_ttc_list_item* Start, t_ttc_list_item* End, t_u16 Amount );
void ttc_list_iterate_from_cast( t_ttc_list* List, void* ( *IterateFunction )( void* Argument, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item ), void* Argument, t_s16 Step, t_ttc_list_item* Start, t_ttc_list_item* End, t_u16 Amount );
#define ttc_list_iterate_from_cast( List, IterateFunction, Argument, Step, Start, End, Amount )  \
    ttc_list_iterate_from( (t_ttc_list*) List, (void* (*)( void*, t_ttc_list_item*, t_ttc_list_item* )) IterateFunction, Argument, Step, (t_ttc_list_item*) Start, (t_ttc_list_item*) End, Amount )

/** Try to run iterator function beginning from start item in given list
 *
 * Functionality is basically the same as ttc_list_iterate_from(). See its description above for details!
 * The only difference is that it will not block if List->Mutex is locked but return FALSE instead.
 * @return         ==TRUE: list was iterated through successfully; ==FALSE: list was locked and no iteration took place.
 */
BOOL ttc_list_iterate_from_try( t_ttc_list* List, void* ( *IterateFunction )( void* Argument, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item ), void* Argument, t_s16 Steps, t_ttc_list_item* Start, t_ttc_list_item* End, t_u16 Amount );
BOOL ttc_list_iterate_from_try_cast( t_ttc_list* List, void* ( *IterateFunction )( void* Argument, t_ttc_list_item* PreviousItem, t_ttc_list_item* Item ), void* Argument, t_s16 Steps, t_ttc_list_item* Start, t_ttc_list_item* End, t_u16 Amount );
#define ttc_list_iterate_from_try_cast( List, IterateFunction, Argument, Step, Start, End, Amount )  \
    ttc_list_iterate_from_try( (t_ttc_list*) List, (void* (*)( void*, t_ttc_list_item*, t_ttc_list_item* )) IterateFunction, Argument, Step, (t_ttc_list_item*) Start, (t_ttc_list_item*) End, Amount )

/** returns amount of items stored in List
 *
 * Note: This function completes in constant time O(1)
 *
 * @param List     pointer to already created list
 * @return         Amount of items currently stored in List
 */
t_base ttc_list_size( t_ttc_list* List );
#define ttc_list_size(LIST)  ( ttc_semaphore_available( &((LIST)->ttc_list.AmountItems) ) ) /** completes all pending operations on given list
 *
 * Note: This function may only be called from an interrupt service routine only!
 * Note: This function completes in constant time O(1)
 *
 * @param List     pointer to already created list
 * @return         == tle_OK: all pending operations have been completed
 *                 == tle_ListInUseByTask: list is locked by a task (no operations completed)
 */
e_ttc_list_error ttc_list_complete_all_pending_isr( t_ttc_list* List );
e_ttc_list_error ttc_list_complete_all_pending_isr_cast( t_ttc_list* List );
#define ttc_list_complete_all_pending_isr_cast( List )  \
    ttc_list_complete_all_pending_isr( (t_ttc_list*) List )

/** adds given single Item at end of given list
 * @param List     pointer to already created list
 * @param Item     pointer to item to be added to list
 */
void ttc_list_push_back_single( t_ttc_list* List, t_ttc_list_item* Item );
void ttc_list_push_back_single_cast( t_ttc_list* List, t_ttc_list_item* Item );
#define ttc_list_push_back_single_cast( List, Item ) \
    ttc_list_push_back_single( (t_ttc_list*) List, (t_ttc_list_item*) Item )

/** adds given linked list of items at end of given list
 * @param List       pointer to already created list
 * @param FirstItem  pointer to first item in list of items to be added to list
 * @param LastItem   pointer to last item in list of items to be added to list
 * @param Amount     Amount of items to be added to list
 */
void ttc_list_push_back_multiple( t_ttc_list* List, t_ttc_list_item* FirstItem, t_ttc_list_item* LastItem, t_base Amount );
void ttc_list_push_back_multiple_cast( t_ttc_list* List, t_ttc_list_item* FirstItem, t_ttc_list_item* LastItem, t_base Amount );
#define ttc_list_push_back_multiple_cast( List, FirstItem, LastItem, Amount )  \
    ttc_list_push_back_multiple( (t_ttc_list*) List, (t_ttc_list_item*) FirstItem, (t_ttc_list_item*) LastItem, Amount )

/** adds given single Item at end of given list (does never block)
 *
 * Note: Must be called from interrupt service routine only!
 * Note: function terminates in O(1)
 *
 * @param List     pointer to already created list
 * @param Item     pointer to item to be added to list
 * @return         == tle_OK: Item has been successfully added to List
 *                 == tle_ListInUseByTask: items have been added to pending list and will be later on later calls
 *                                         (you may call ttc_list_push_back_isr(List, NULL) to retry push later
 */
e_ttc_list_error ttc_list_push_back_single_isr( t_ttc_list* List, t_ttc_list_item* Item );
e_ttc_list_error ttc_list_push_back_single_isr_cast( t_ttc_list* List, t_ttc_list_item* Item );
#define ttc_list_push_back_single_isr_cast( List, Item )  \
    ttc_list_push_back_single_isr( (t_ttc_list*) List, (t_ttc_list_item*) Item )

/** adds given linked list of Items at end of given list (does never block)
 *
 * Note: Must be called from interrupt service routine only!
 * Note: function terminates in O(1)
 *
 * @param List       pointer to already created list
 * @param FirstItem  pointer to first item in list of items to be added to list
 * @param LastItem   pointer to last item in list of items to be added to list
 * @param Amount     Amount of items to be added to list
 * @return         == tle_OK: Item has been successfully added to List
 *                 == tle_ListInUseByTask: items have been added to pending list and will be completed on later calls
 *                                         (you may call ttc_list_push_back_isr(List, NULL) to retry push
 */
e_ttc_list_error ttc_list_push_back_multiple_isr( t_ttc_list* List, t_ttc_list_item* FirstItem, t_ttc_list_item* LastItem, t_base Amount );
e_ttc_list_error ttc_list_push_back_multiple_isr_cast( t_ttc_list* List, t_ttc_list_item* FirstItem, t_ttc_list_item* LastItem, t_base Amount );
#define ttc_list_push_back_multiple_isr_cast( List, FirstItem, LastItem, Amount )  \
    ttc_list_push_back_multiple_isr( (t_ttc_list*) List, (t_ttc_list_item*) FirstItem, (t_ttc_list_item*) LastItem, Amount )

/** adds given single Item at head of given list
 *
 * @param List     pointer to already created list
 * @param Item     pointer to item to be added to list
 */
void ttc_list_push_front_single( t_ttc_list* List, t_ttc_list_item* Item );
void ttc_list_push_front_single_cast( t_ttc_list* List, t_ttc_list_item* Item );
#define ttc_list_push_front_single_cast( List, Item )  \
    ttc_list_push_front_single( (t_ttc_list*) List, (t_ttc_list_item*) Item )

/** adds given single Item at head of given list (does never block)
 *
 * Note: Must be called from interrupt service routine only!
 * Note: function terminates in O(1)
 *
 * @param List     pointer to already created list
 * @param Item     pointer to item to be added to list
 * @return         == tle_OK: Item has been successfully added to List
 *                 == tle_ListInUseByTask: items have been added to pending list and will be completed on later calls
 *                                         (you may call ttc_list_push_back_isr(List, NULL) to retry push
 */
e_ttc_list_error ttc_list_push_front_single_isr( t_ttc_list* List, t_ttc_list_item* Item );
e_ttc_list_error ttc_list_push_front_single_isr_cast( t_ttc_list* List, t_ttc_list_item* Item );
#define ttc_list_push_front_single_isr_cast( List, Item )  \
    ttc_list_push_front_single_isr( (t_ttc_list*) List, (t_ttc_list_item*) Item )


/** delivers pointer to first list entry without removing it from list
 *
 * Note: This function provides very fast access to inspect first list item.
 * Note: If list is used by >1 tasks then special care has to be taken to avoid data corruption.
 *       Caller must take care of race conditions with other tasks and interrupt service routines!
 * Note: This function can safely be called from an interrupt service routine.
 *
 * @param List     pointer to already created list
 * @return         !=NULL: pointer to first item; ==NULL: list is empty
 */
t_ttc_list_item* ttc_list_peek_front( t_ttc_list* List );
t_ttc_list_item* ttc_list_peek_front_cast( t_ttc_list* List );
#define ttc_list_peek_front_cast( List ) \
    ttc_list_peek_front( ( t_ttc_list* ) List )

/** delivers read-only pointer to first list entry without removing it from list
 *
 * Note: This function provides very fast access to inspect first list item.
 * Note: If list is used by >1 tasks then special care has to be taken to avoid data corruption.
 *       Caller must take care of race conditions with other tasks and interrupt service routines!
 * Note: This function can safely be called from an interrupt service routine.
 *
 * @param List     pointer to already created list
 * @return         !=NULL: pointer to first item; ==NULL: list is empty
 */
const t_ttc_list_item* ttc_list_peek_front_const( t_ttc_list* List );
#define ttc_list_peek_front_const( List )      (const t_ttc_list_item*) ttc_list_peek_front( List );

const t_ttc_list_item* ttc_list_peek_front_const_cast( t_ttc_list* List );
#define ttc_list_peek_front_const_cast( List ) (const t_ttc_list_item*) ttc_list_peek_front( ( t_ttc_list* ) List )

/** delivers pointer to list entry at given index without removing it from list
 *
 * Note: If list is used by >1 tasks then special care has to be taken to avoid data corruption.
 *       Caller must take care of race conditions with other tasks and interrupt service routines!
 *       Disable multitasking and interrupts if in doubt!
 *       In case you only want to read from list item, check ttc_list_peek_index_const()!
 * Note: This function can safely be called from an interrupt service routine.
 *
 * @param List     pointer to already created list
 * @param Index    0..ttc_list_items_count(List)-1 index of desired item. 0=first item, 1=second item, ...
 * @return         !=NULL: pointer to first item; ==NULL: list is empty
 */
t_ttc_list_item* ttc_list_peek_index( t_ttc_list* List, t_base Index );
t_ttc_list_item* ttc_list_peek_index_cast( t_ttc_list* List, t_base Index );
#define ttc_list_peek_index_cast( List, Index ) \
    ttc_list_peek_index( ( t_ttc_list* ) List, Index )

/** delivers read-only pointer to list entry at given index without removing it from list
 *
 * Note: If list is used by >1 tasks then special care has to be taken to avoid data corruption.
 *       Disable multitasking and interrupts if in doubt!
 * Note: This function can safely be called from an interrupt service routine.
 *
 * @param List     pointer to already created list
 * @param Index    0..ttc_list_items_count(List)-1 index of desired item. 0=first item, 1=second item, ...
 * @return         !=NULL: pointer to first item; ==NULL: list is empty
 */
const t_ttc_list_item* ttc_list_peek_index_const( t_ttc_list* List, t_base Index );
#define ttc_list_peek_index_const( List, Index )      (const t_ttc_list_item*) ttc_list_peek_index( List, Index );

const t_ttc_list_item* ttc_list_peek_index_const_cast( t_ttc_list* List, t_base Index );
#define ttc_list_peek_index_const_cast( List, Index ) (const t_ttc_list_item*) ttc_list_peek_index( ( t_ttc_list* ) List, Index )

/** removes first Item from given list and delivers its reference
 *
 * Note: This function will do endless wait if list is empty!
 *
 * @param List     pointer to already created list
 * @return         pointer to item removed from given list
 */
t_ttc_list_item* ttc_list_pop_front_single_wait( t_ttc_list* List );
t_ttc_list_item* ttc_list_pop_front_single_wait_cast( t_ttc_list* List );
#define ttc_list_pop_front_single_wait_cast( List ) \
    ttc_list_pop_front_single_wait( ( t_ttc_list* ) List )

/** tries to remove first Item from given list and delivers its reference
 *
 * Note: Function returns immediately if list is empty.
 *
 * @param List     pointer to already created list
 * @return         != NULL: pointer to item removed from given list; == NULL: list is empty
 */
t_ttc_list_item* ttc_list_pop_front_single_try( t_ttc_list* List );
t_ttc_list_item* ttc_list_pop_front_single_try_cast( t_ttc_list* List );
#define ttc_list_pop_front_single_try_cast( List ) \
    ttc_list_pop_front_single_try( ( t_ttc_list* ) List )

/** removes linked list of first items from front of given list and delivers reference to first and last one
 *
 * Note: This function completes in constant time O(Amount)
 *
 * @param List      pointer to already created list
 * @param Amount    amount of items to remove from list (==-1: means all items in list)
 * @param FirstItem loaded with pointer to first item of items removed from given list or NULL if list is empty
 * @param LastItem  loaded with pointer to last item of items removed from given list or NULL if list is empty
 * @return          actual amount of items being removed from list
 */
t_base ttc_list_pop_front_multiple( t_ttc_list* List, t_base Amount, t_ttc_list_item** FirstItem, t_ttc_list_item** LastItem );
t_base ttc_list_pop_front_multiple_cast( t_ttc_list* List, t_base Amount, t_ttc_list_item** FirstItem, t_ttc_list_item** LastItem );
#define ttc_list_pop_front_multiple_cast( List, Amount, FirstItem, LastItem )  \
    ttc_list_pop_front_multiple( (t_ttc_list*) List, Amount, (t_ttc_list_item**) FirstItem, (t_ttc_list_item**) LastItem )

/** removes linked list of first items from front of given list and delivers reference to first and last one (does never block)
 *
 * Note: Must be called from interrupt service routine only!
 * Note: This function completes in constant time O(Amount)
 *
 * @param List      pointer to already created list
 * @param Amount    amount of items to remove from list (==-1: means all items in list)
 * @param FirstItem loaded with pointer to first item of items removed from given list or NULL if list is empty
 * @param LastItem  loaded with pointer to last item of items removed from given list or NULL if list is empty
 * @return         == tle_OK: Items have been successfully removed from list
 *                 == tle_ListInUseByTask: list is currently in use by task; try again later
 */
e_ttc_list_error ttc_list_pop_front_multiple_isr( t_ttc_list* List, t_base Amount, t_ttc_list_item** FirstItem, t_ttc_list_item** LastItem );
e_ttc_list_error ttc_list_pop_front_multiple_isr_cast( t_ttc_list* List, t_base Amount, t_ttc_list_item** FirstItem, t_ttc_list_item** LastItem );
#define ttc_list_pop_front_multiple_isr_cast( List, Amount, FirstItem, LastItem )  \
    ttc_list_pop_front_multiple_isr( (t_ttc_list*) List, Amount, (t_ttc_list_item**) FirstItem, (t_ttc_list_item**) LastItem )

/** removes first Items from given list and delivers reference to first item (does never block)
 *
 * Note: Must be called from interrupt service routine only!
 *
 * @param List     pointer to already created list
 * @return         != NULL: pointer to item removed from given list;
 *                 == NULL: list is empty
 *                 == 1:    list is locked by task
 */
t_ttc_list_item* ttc_list_pop_front_single_isr( t_ttc_list* List );
t_ttc_list_item* ttc_list_pop_front_single_isr_cast( t_ttc_list* List );
#define ttc_list_pop_front_single_isr_cast( List ) \
    ttc_list_pop_front_single_isr( ( t_ttc_list* ) List )

//}Function prototypes

#endif
