#ifndef TTC_RANDOM_H
#define TTC_RANDOM_H

/** stm32_random.h ***********************************************
 *
 * Written by Gregor Rebel 2010-2019
 *
 */
/** Description of ttc_random (Do not delete this line!)
 *
 * Architecture independent random number generation providing 16 and 32 bit
 * pseudo random number generators (prng).
 * The currently used prng can be configured by enabling/disabling macro definitions
 * in this header file.
 *
 * The simplest and most portable application example:
 * <CODE>
 *   ttc_random_seed(43);
 *   ...
 *   t_base NewRandom1 = ttc_random_generate();
 *   t_base NewRandom2 = ttc_random_generate();
 * </CODE>
 *
 * Force generation of 16 bit random numbers:
 * <CODE>
 *   ttc_random_seed16(65);
 *   ...
 *   t_u16 NewRandom1 = ttc_random_generate16();
 *   t_u16 NewRandom2 = ttc_random_generate16();
 * </CODE>
 *
 * Force generation of 32 bit random numbers:
 * <CODE>
 *   ttc_random_seed32(3);
 *   ...
 *   t_u32 NewRandom1 = ttc_random_generate32();
 *   t_u32 NewRandom2 = ttc_random_generate32();
 * </CODE>
*/
//{ Defines/ TypeDefs ****************************************************

// To be defined by architecture driver
// RAND_MAX16
//     E.g.: #define RAND_MAX16  (t_u32) -1

//} Defines

//{ Includes *************************************************************

#include "ttc_basic.h"  // Include First! (basic datatypes and definitions)

//} Includes

//{ Function prototypes **************************************************



/** best random number generator for current architecture
 *
 * This macro leads to a fast random number generator for current microcontroller architecture.
 * - On 32 bit architectures, it uses a fast 32 bit generator.
 * - On 16 and 8 bit architectures , it uses a fast 16 bit generator.
 *
 * @return random number 0..RAND_MAX
 */
t_u16 ttc_random_generate( void );
#if (TTC_BASIC_TYPES_DATAWIDTH==32)
    #define RAND_MAX            RAND_MAX16             // maximum generated random number
    #define ttc_random_generate ttc_random_generate32  // using default 32 bit prng
#else
    #define RAND_MAX            RAND_MAX32             // maximum generated random number of current 32 bit prng
    #define ttc_random_generate _ttc_random_generate16 // using default 16 bit prng
#endif


/** Initializes pseudo random number generator with given value
 *
 * The numbers generated by any prng are deterministic.
 * All generated random numbers are the same on every program start
 * on all microcontrollers running the same binary.
 *
 * The trick to generate different random numbers on every program start
 * is to use different start values. This can be the amount of seconds from
 * a long running realtime clock. In general a good seed value can be read
 * from various external sources depending on your current application.
 *
 * Note: It is not required to run ttc_random_seed() for every ttc_random_generate() call.
 *       If you call it too frequently, the generated random numbers will be dominated by
 *       your external source instead of the current prng algorithm.
 *
 * @param Seed  will be used as new starting point
 */
void ttc_random_seed( t_u32 Seed );
#if (TTC_BASIC_TYPES_DATAWIDTH==32)
    #define ttc_random_seed _ttc_random_seed32
#else
    #define ttc_random_seed _ttc_random_seed16
#endif

/** current 16 bit random number generator to use
 *
 * Different implementations have pros and cons.
 * Read referenced documentation for implementation details!
 *
 * @return random number 0..RAND_MAX16
 */
t_u16 ttc_random_generate16( void );
#if (1)
    #define RAND_MAX16            0xffff
    #define ttc_random_generate16 _ttc_random_generate16
#endif

/** current 32 bit random number generator to use
 *
 * Different implementations have pros and cons.
 * Read referenced documentation for implementation details!
 *
 * @return random number 0..RAND_MAX32
 */
t_u16 ttc_random_generate32( void );
#if (1)
    #define RAND_MAX32            0xffffffff
    #define ttc_random_generate32 _ttc_random_generate32_xorshift32
#endif
#if (0)
    #define RAND_MAX32            0x7fffffff
    #define ttc_random_generate32 _ttc_random_generate32_parker_miller
#endif

/** check if pseudo random generator already got a seed value
 *
 * @return (BOOL) ==TRUE: someone already called ttc_random_seed() before
 */
BOOL ttc_random_is_seeded();

/** returns 16-bit random number 0..0xffff
 *
 * Note: Do not directly call this function, but use macro ttc_random_generate() instead!
 *
 * @return random number 0..0xffff
 */
t_u16 _ttc_random_generate16( void );

/** returns 32-bit random number 0..0x7fffffff
 *
 * Calculates 31 bit random number according to Parker-Miller algorithm.
 * -> http://www.math.uni-frankfurt.de/~numerik/lehre/Vorlesungen/Comp_Fin09/skript/n-shell3.pdf p.50
 *
 * @return random number 0..0x7fffffff
 */
t_u32 _ttc_random_generate32_parker_miller( void );

/** returns 32-bit random number 0..0xffffffff
 *
 * Calculates 32 bit random number according to XOR Shift 32 bit algorithm.
 * -> https://en.wikipedia.org/wiki/Xorshift
 *
 * @return random number 0..0xffffffff
 */
t_u32 _ttc_random_generate32_xorshift32( void );

/** initializes 32-bit pseudo random number generator with given value analog to srand()
 *
 * Note: Do not directly call this function, but use macro ttc_random_seed() instead!
 *
 * @param Seed  will be used as new starting point
 */
void _ttc_random_seed32( t_u32 Seed );

/** initializes 16-bit pseudo random number generator with given value analog to srand()
 *
 * Note: Do not directly call this function, but use macro ttc_random_seed() instead!
 *
 * @param Seed  will be used as new starting point
 */
void _ttc_random_seed16( t_u16 Seed );


#ifndef RAND_MAX16
    #  error Missing definition for RAND_MAX16. Activate exactly one 16 bit pseudo random number generator (prng) above!
#endif

#ifndef RAND_MAX32
    #  error Missing definition for RAND_MAX32. Activate exactly one 16 bit pseudo random number generator (prng) above!
#endif

//}Function prototypes
#endif
