/** { ttc_usart.c **********************************************************
 *
 *                          The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  High-Level driver for usart devices.
 *
 *  Implementation of high-level interface.
 *  This file implements all functionality that is common to all supported
 *  architectures.
 *
 *  Created from template ttc_device.c revision 23 at 20140217 09:36:59 UTC
 *
 *  Authors: Gregor Rebel
 *
}*/

#include "ttc_usart.h"
#include "ttc_queue.h"     // efficient queues for individual bytes and pointers
#include "ttc_heap.h"      // dynamic memory and safe arrays

/** { Global Variables *****************************************************
 *
 * Global variables defined here are common for all architectures of usart devices.
 *
 */

#if TTC_USART_AMOUNT == 0
    #warning No USART devices defined, check your makefile! (did you forget to activate something?)
#endif

#ifdef TTC_REGRESSION
    #define TTC_USART_STATISTIC(FIELD) Config->statistics.FIELD
#else
    #define TTC_USART_STATISTIC(FIELD);
#endif


/* Define Testpin Macros
 *
 * The macros below can be used to measure performance of ttc_usart with a digital oscilloscpe.
 * If TTC_USART_TESTPIN_TX has been defined, the RTS pin of every USART will be configured as output_push_pull
 * and will be set during _ttc_usart_isr_tx() when a byte is bend written into usart output register.
 *
 * 1) Define this as one from e_ttc_gpio_pin in your extensions.local/makefile.700_extra_settings file:
 *    TTC_USART_TESTPIN_TX pin to set on entering _ttc_usart_isr_tx() for any USART device
 *
 *    Example content of extensions.local/makefile.700_extra_settings:
 *    COMPILE_OPTS += -DTTC_USART_TESTPIN_TX1=E_ttc_gpio_pin_a15
 *
 * 2) Connect your digital oscilloscope to these pins (assuming TTC_USART1..TTC_USART3 being defined in current board makefile)
 *    TTC_USART_TESTPIN_TX   will be high during _ttc_usart_isr_tx() for any usart device
 *    TTC_USART1_RTS         will be high during _ttc_usart_isr_tx() for logical usart #1 (TTC_USART1)
 *    TTC_USART2_RTS         will be high during _ttc_usart_isr_tx() for logical usart #2 (TTC_USART2)
 *    TTC_USART3_RTS         will be high during _ttc_usart_isr_tx() for logical usart #3 (TTC_USART3)
 *
 * 3) Choose and activate a board with at least three available TTC_USART devices (e.g. 110_board_stm32l100c_discovery)
 * 4) Activate example_ttc_usart.c
 * 5) compile, flash and run your project
 *
 * Then you should be able to see how often _ttc_usart_isr_tx() passes data to each logical USART device.
 * If no DMA transfers are enabled, each _ttc_usart_isr_tx() call will send out exactly one byte.
 */
#define TTC_USART_TESTPIN_TX E_ttc_gpio_pin_a15
#ifdef TTC_USART_TESTPIN_TX
    #define TTC_USART_TESTPINS_SET(Config); ttc_gpio_set(TTC_USART_TESTPIN_TX); if (Config->PortRTS) ttc_gpio_set(Config->PortRTS);
    #define TTC_USART_TESTPINS_CLR(Config); ttc_gpio_clr(TTC_USART_TESTPIN_TX); if (Config->PortRTS) ttc_gpio_clr(Config->PortRTS);
#else
    #define TTC_USART_TESTPINS_SET(Config);
    #define TTC_USART_TESTPINS_CLR(Config);
#endif

// Stores released memory blocks for reuse as transmit or receive blocks for all configured usart interfaces
t_ttc_heap_pool ttc_usart_PoolEmptyBlocks __attribute__( ( aligned( 4 ) ) );
t_ttc_usart_memory_heap_block* MyBlocks = NULL; //DEBUG

// for each initialized device, a pointer to its generic and stm32f1xx definitions is stored
A_define( t_ttc_usart_config*, ttc_usart_configs, TTC_USART_MAX_AMOUNT );

t_u8 ttc_usart_Amount_BlocksAllocated;    // amount of transmit/receive buffers allocated by ttc_usart_get_empty_block() so far
t_u8 ttc_usart_Amount_RxBytesSkipped;     // amount of bytes received by _ttc_usart_isr_rx() which could not be pushed into receive queue

// logical index of USART to use by ttc_usart_stdout_send_string()/ ttc_usart_stdout_send_block()
t_u8 ttc_usart_stdout_index = 0;


//InsertGlobalVariables above (DO NOT DELETE THIS LINE!)

//} Global Variables
/** { Private Function Declarations ***********************************
 *
 * Declare all functions here that should not be called from outside.
 * You may prefix private functions by an underscore to help identify them as private:
 * E.g.: void _privateCalc();
 */

/** Manually process receive and transmit queues
 *
 * Call this function reqularly to ensure that rx- and tx-queue are processed!
 *
 * @param Config         USART configuration as returned by ttc_usart_get_configuration()
 */
void _ttc_usart_process_queues( t_ttc_usart_config* Config );

/** Ensure that delayed transmissions are running
 *
 * Sometimes, delayed transmissions are hanging.
 * Will reenable transmit interrupt if required.
 *
 * @param Config         USART configuration as returned by ttc_usart_get_configuration()
 */
void _ttc_usart_observe_transmits( t_ttc_usart_config* Config );

/** Try to obtain one memory block from shared memory pool
 *
 * @return ==NULL: no memory blocks currently available; !=NULL: memory block from pool
 */
t_ttc_usart_memory_block* _ttc_usart_get_empty_block_try();

/** Add 1 byte to receive queue. Called from low level USART interrupt service routine
 * Note: This function is private and should not be called from outside.
 *
 * @param Config         USART configuration as returned by ttc_usart_get_configuration()
 * @param Byte           received byte to add to receive buffer
 */
void _ttc_usart_isr_rx( t_ttc_usart_config* Config, t_u8 Byte );

/** Send out next byte from transmit queue.
 * Note: This function is private and should not be called from outside.
 * Note: This function must be called from transmit isr function of low-level driver to fetch
 *       next byte to send and to call low-level byte send function.
 *
 * @param Config         USART configuration as returned by ttc_usart_get_configuration()
 * @param USART_Hardware low-level pointer required for fast low-level transmit (passed to _driver_usart_send_byte_isr() )
 * @return               !=0: still bytes in transmit buffer (low-level driver may stay in interrupt service routine for high-data rates=
 */
BOOL _ttc_usart_isr_tx( t_ttc_usart_config* Config, volatile void* USART_Hardware );

/** Send out given amount of raw bytes from buffer.
 * Will not interpret bytes in buffer.
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 * Note: This function blocks until all bytes have been sent!
 *
 * @param LogicalIndex   device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @param Buffer         memory location from which to read data
 * @param Amount         amount of bytes to send from Buffer[]
 * @return               == 0: Buffer has been sent successfully; != 0: error-code
 */
e_ttc_usart_errorcode _ttc_usart_send_raw_blocking( t_u8 LogicalIndex, const t_u8* Buffer, t_u16 Amount );

/** Send out given bytes from buffer until first zero byte or MaxLength reached.
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 * Note: This function blocks until all bytes have been sent!
 *
 * @param LogicalIndex   device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @param Buffer         memory location from which to read data
 * @param MaxLength      no more than this amount of bytes will be send
 * @return               == 0: Buffer has been sent successfully; != 0: error-code
 */
e_ttc_usart_errorcode _ttc_usart_send_string_blocking( t_u8 LogicalIndex, const char* Buffer, t_u16 MaxLength );

/** Send out given data word (8 or 9 bits)
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 * Note: This function blocks until bytes has been sent!
 *
 * @param LogicalIndex   device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @param Byte           8 or 9 bit value to send
 * @return               == 0: Buffer has been sent successfully; != 0: error-code
 */
e_ttc_usart_errorcode _ttc_usart_send_word_blocking( t_u8 LogicalIndex, const t_u16 Word );

/** User space task reading data from receive queue.
 *
 * When the USART receives a byte
 * 1a) An interrupt is risen.
 * 1b) _ttc_usart_isr_rx() fetches byte from USART->DR and pushes it into QueueRx.
 * 2a) _ttc_usart_task_receive() pops all bytes from QueueRx and appends them to CurrentRxBuffer.
 * 2b) If CurrentRxBuffer is full or
 *     byte == Char_EndOfLine
 *     then function_Receive_Block(CurrentRxBuffer) is called to process data.
 *
 * @param Args  t_ttc_usart_config* pointer to data of configured USART
 */
void _ttc_usart_task_receive( void* Args );

/** releases a used Tx/ Rx memory blocks for later reuse
 * @param Block         pointer to a t_ttc_usart_memory_block
 */
void _ttc_usart_release_block( t_ttc_usart_memory_block* Block );

/** releases a used Tx/ Rx memory blocks for later reuse (will not block)
 * @param Block         pointer to a t_ttc_usart_memory_block
 */
void _ttc_usart_release_block_isr( t_ttc_usart_memory_block* Block );

/** returns configuration of indexed usart (allocates + resets new configuration if necessary)
 * for internal use only
 * @param LogicalIndex    device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @return                !=NULL: pointer to struct t_ttc_usart_config
 */
t_ttc_usart_config* _ttc_usart_get_configuration( t_u8 LogicalIndex );
//InsertPrivateFunctionDeclarations above (DO NOT DELETE THIS LINE!)

//}PrivateFunctions
//{ Function definitions ***************************************************

t_u8                      ttc_usart_check_bytes_available( t_u8 LogicalIndex ) {
    t_ttc_usart_config* Config = _ttc_usart_get_configuration( LogicalIndex );

    return _driver_usart_check_bytes_available( Config );
}
t_bool                    ttc_usart_check_transmission_ongoing( t_u8 LogicalIndex ) {
    t_ttc_usart_config* Config = _ttc_usart_get_configuration( LogicalIndex );

    _ttc_usart_observe_transmits( Config );

    return Config->private_tx_isr.BytesRemaining;
}
t_u8                      ttc_usart_get_max_index() {
    return TTC_USART_AMOUNT;
}
void                      ttc_usart_deinit( t_u8 LogicalIndex ) {
    Assert_USART( LogicalIndex, E_ttc_assert_origin_auto );  // logical index starts at 1
    t_ttc_usart_config* Config = ttc_usart_get_configuration( LogicalIndex );

    e_ttc_usart_errorcode Result = _driver_usart_deinit( Config );
    if ( Result == ec_usart_OK )
    { Config->Flags.Initialized = 0; }
}
void                      ttc_usart_flush_tx( t_u8 LogicalIndex ) {
    t_ttc_usart_config* Config = _ttc_usart_get_configuration( LogicalIndex );
    if ( Config->Init.Flags.DelayedTransmits ) { // delayed transmits have been activated for this USART

        // wait for transmit queue to run empty
        while ( ttc_queue_pointer_get_amount( Config->Queue_Tx ) > 0 )
        { ttc_usart_yield(); }
    }
}
t_ttc_usart_config*       ttc_usart_get_configuration( t_u8 LogicalIndex ) {
    Assert_USART( LogicalIndex > 0, E_ttc_assert_origin_auto );                // logical index starts at 1
    Assert_USART( LogicalIndex <= TTC_USART_AMOUNT, E_ttc_assert_origin_auto );  // use ttc_i2c_get_max_index() to get max allowed value
    t_ttc_usart_config* Config = A( ttc_usart_configs, LogicalIndex - 1 );

    if ( !Config ) {
        Config = A( ttc_usart_configs, LogicalIndex - 1 ) = ttc_heap_alloc_zeroed( sizeof( t_ttc_usart_config ) );
        ttc_usart_reset( LogicalIndex );
        ttc_usart_load_defaults( LogicalIndex );
    }

    return Config;
}
t_u8                      ttc_usart_get_max_logicalindex() {

    return TTC_USART_AMOUNT; // -> ttc_usart_types.h
}
e_ttc_usart_errorcode     ttc_usart_get_features( t_u8 LogicalIndex, t_ttc_usart_config* Config ) {
    Assert_USART( LogicalIndex > 0, E_ttc_assert_origin_auto );              // logical index starts at 1
    Assert_USART( LogicalIndex <= TTC_USART_AMOUNT, E_ttc_assert_origin_auto );  // use ttc_i2c_get_max_index() to get max allowed value
    Assert_USART( Config != NULL, E_ttc_assert_origin_auto );

    ttc_memory_set( Config, 0, sizeof( Config ) );
    Config->LogicalIndex = LogicalIndex;
    e_ttc_usart_errorcode Status = _driver_usart_get_features( Config );

    if ( Status == ec_usart_OK ) { // low-level driver returned OK: add platform independent features
        Config->Init.Flags.DelayedTransmits = 1;
        Config->Init.Flags.StayInISR        = 1;

        if ( 1 ) { // disable features requiring unconfigured pins
            switch ( LogicalIndex ) {
                case 1: {
#ifndef TTC_USART1_TX
                    Config->Init.Flags.Transmit    = 0;
#endif
#ifndef TTC_USART1_RX
                    Config->Init.Flags.Receive     = 0;
#endif
#ifndef TTC_USART1_RTS
                    Config->Init.Flags.RtsCts      = 0;
                    Config->Init.Flags.Rts         = 0;
#endif
#ifndef TTC_USART1_CTS
                    Config->Init.Flags.RtsCts      = 0;
                    Config->Init.Flags.Cts         = 0;
#endif
#ifndef TTC_USART1_CK
                    Config->Init.Flags.Synchronous = 0;
#endif

                    break;
                }
                case 2: {
#ifndef TTC_USART2_TX
                    Config->Init.Flags.Transmit    = 0;
#endif
#ifndef TTC_USART2_RX
                    Config->Init.Flags.Receive     = 0;
#endif
#ifndef TTC_USART2_RTS
                    Config->Init.Flags.RtsCts      = 0;
                    Config->Init.Flags.Rts         = 0;
#endif
#ifndef TTC_USART2_CTS
                    Config->Init.Flags.RtsCts      = 0;
                    Config->Init.Flags.Cts         = 0;
#endif
#ifndef TTC_USART2_CK
                    Config->Init.Flags.Synchronous = 0;
#endif

                    break;
                }
                case 3: {
#ifndef TTC_USART3_TX
                    Config->Init.Flags.Transmit    = 0;
#endif
#ifndef TTC_USART3_RX
                    Config->Init.Flags.Receive     = 0;
#endif
#ifndef TTC_USART3_RTS
                    Config->Init.Flags.RtsCts      = 0;
                    Config->Init.Flags.Rts         = 0;
#endif
#ifndef TTC_USART3_CTS
                    Config->Init.Flags.RtsCts      = 0;
                    Config->Init.Flags.Cts         = 0;
#endif
#ifndef TTC_USART3_CK
                    Config->Init.Flags.Synchronous = 0;
#endif

                    break;
                }
                case 4: {
#ifndef TTC_USART4_TX
                    Config->Init.Flags.Transmit    = 0;
#endif
#ifndef TTC_USART4_RX
                    Config->Init.Flags.Receive     = 0;
#endif

                    // UART4, UART5 only provide TX+RX pins
                    Config->Init.Flags.RtsCts      = 0;
                    Config->Init.Flags.Rts         = 0;
                    Config->Init.Flags.Cts         = 0;
                    Config->Init.Flags.Synchronous = 0;

                    break;
                }
                case 5: {
#ifndef TTC_USART5_TX
                    Config->Init.Flags.Transmit    = 0;
#endif
#ifndef TTC_USART5_RX
                    Config->Init.Flags.Receive     = 0;
#endif

                    // UART4, UART5 only provide TX+RX pins
                    Config->Init.Flags.RtsCts      = 0;
                    Config->Init.Flags.Rts         = 0;
                    Config->Init.Flags.Cts         = 0;
                    Config->Init.Flags.Synchronous = 0;

                    break;
                }
                default: return ec_usart_DeviceNotFound;
            }
        }
    }

    return Status;
}
t_ttc_usart_memory_block* ttc_usart_get_empty_block( t_u8 LogicalIndex ) {
    Assert_USART( ttc_interrupt_critical_level() == 0, E_ttc_assert_origin_auto ); // This function may block endlessly if called with disabled interrupts. Change code of caller!
    t_ttc_usart_memory_block* Block = NULL;
    while ( Block == NULL ) {
        Block = ttc_usart_get_empty_block_try( LogicalIndex );
        if ( Block == NULL ) { // got no block: retry + process all other usart devices
            t_ttc_usart_config* Config = ttc_usart_get_configuration( LogicalIndex );
            do {
                if ( ttc_task_is_scheduler_available() ) { // multitasking mode: each task processes its own queues only
                    _ttc_usart_process_queues( Config );
                    ttc_task_yield();
                }
                else {                                     // single-tasking mode: will process queues of all usarts
                    ttc_usart_yield();
                }
                Block = ttc_usart_get_empty_block_try( LogicalIndex );
            }
            while ( Block == NULL );
        }
    }

    Assert_USART_Writable( Block, E_ttc_assert_origin_auto );
#if TTC_HEAP_POOL_STATISTICS == 1
    t_ttc_heap_block_from_pool* PoolBlock = ( t_ttc_heap_block_from_pool* ) Block;
    PoolBlock--; // back to header
    PoolBlock->MyOwner = __builtin_return_address( 0 ); // store address of our caller
#endif

    // have buffer pointing at first element of Data[] by default
    Block->Buffer = Block->Data;

    return Block;
}
t_ttc_usart_memory_block* ttc_usart_get_empty_block_try( t_u8 LogicalIndex ) {
    t_ttc_usart_memory_block* Block = NULL;
    t_ttc_usart_config* Config = ttc_usart_get_configuration( LogicalIndex );
    if ( Config->Init.Amount_PoolBlocksReserved < ttc_heap_pool_get_available( &ttc_usart_PoolEmptyBlocks ) ) {
        TODO( "Implement limitation to Amount_PoolBlocksReserved for each ttc_usart device! (might incur performance penalty)" );
        Block = _ttc_usart_get_empty_block_try( Config );
    }
    return Block;
}
t_u8                      ttc_usart_get_physicalindex( t_u8 LogicalIndex ) {
    Assert_USART( LogicalIndex > 0, E_ttc_assert_origin_auto );  // logical USART-indices start at 1 !
    t_s8 PhysicalIndex = -1;

    switch ( LogicalIndex ) {              // find corresponding USART as defined by makefile.100_board_*
#ifdef TTC_USART1
        case 1: PhysicalIndex = TTC_USART1; break; // TTC_USART1 = 0..TTC_USART_AMOUNT-1
#endif
#ifdef TTC_USART2
        case 2: PhysicalIndex = TTC_USART2; break; // TTC_USART2 = 0..TTC_USART_AMOUNT-1
#endif
#ifdef TTC_USART3
        case 3: PhysicalIndex = TTC_USART3; break; // TTC_USART3 = 0..TTC_USART_AMOUNT-1
#endif
#ifdef TTC_USART4
        case 4: PhysicalIndex = TTC_USART4; break; // TTC_USART4 = 0..TTC_USART_AMOUNT-1
#endif
#ifdef TTC_USART5
        case 5: PhysicalIndex = TTC_USART5; break; // TTC_USART5 = 0..TTC_USART_AMOUNT-1
#endif
        default: ttc_assert_halt_origin( E_ttc_assert_origin_auto ); break; // No TTC_USARTn defined! Check your makefile.100_board_* file!
    }
    Assert_USART( PhysicalIndex < TTC_USART_MAX_AMOUNT, E_ttc_assert_origin_auto );  // given logical USART index is out of range
    Assert_USART( PhysicalIndex != -1, E_ttc_assert_origin_auto );  // error: invalid configuration or no low-level implementation available
    return PhysicalIndex;
}
e_ttc_usart_errorcode     ttc_usart_init( t_u8 LogicalIndex ) {

    if ( LogicalIndex > ttc_usart_get_max_logicalindex() )
    { return ec_usart_DeviceNotFound; }
#if (TTC_TUNABLE_SELFCHECK_ASSERT_USART_EXTRA == 1)
    t_u8 InitialCriticalLevel = ttc_interrupt_critical_level();
#endif
    ttc_task_critical_begin();

    t_ttc_usart_config* Config = ttc_usart_get_configuration( LogicalIndex );
    e_ttc_usart_errorcode volatile_USART ReturnCode = ec_usart_NotImplemented;
    Config->LogicalIndex = LogicalIndex;
    if ( ( Config->Init.function_Receive_Block != NULL ) || ( Config->Init.function_Receive_Single_Byte != NULL ) ) { // blocking of data activated: spawn receive-task
        if ( Config->Queue_Rx == NULL ) { // create receive queue
            Config->Queue_Rx = ttc_queue_byte_create( Config->Init.Size_Queue_Rx );
            ttc_task_create( _ttc_usart_task_receive, "ttc_usart_rx", 128, ( void* ) Config, 10, &( Config->Task_Rx ) );
            Assert_USART_Writable( Config->Task_Rx, E_ttc_assert_origin_auto );  // could not spawn receive task (out of memory?)
        }
    }

    Assert_USART( ( Config->Init.Flags.ParityEven ^ Config->Init.Flags.ParityOdd ) == 0, E_ttc_assert_origin_auto ); // Only one of ParityEven and ParityOdd may be set. Check configuration!

    if ( 1 ) {                         // validate USART_Features
        t_ttc_usart_config USART_Features;
        ttc_memory_set( &USART_Features, 0, sizeof( USART_Features ) );
        USART_Features.LogicalIndex  = LogicalIndex;
        USART_Features.PhysicalIndex = Config->PhysicalIndex;
        e_ttc_usart_errorcode Error = ttc_usart_get_features( LogicalIndex, &USART_Features );
        if ( Error ) { return Error; }

        Assert_USART( sizeof( Config->Init.Flags ) == 4, E_ttc_assert_origin_auto ); // size of bitfield must exactly match typecast below. Change code to fit!
        Assert_USART( USART_Features.Architecture != ta_usart_None, E_ttc_assert_origin_auto ); // low-level driver must set this field!

        // copy architecture from features
        Config->Architecture = USART_Features.Architecture;

        // mask out unavailable flags
        *( ( t_u32* ) & ( Config->Init.Flags ) ) &= *( ( t_u32* ) & ( USART_Features.Init.Flags ) );

        if ( Config->Layout > USART_Features.Layout )
        { Config->Layout = USART_Features.Layout; }
    }
    if ( Config->Flags.Initialized ) { // usart device has been initialized before: reset private structures
        if ( 1 ) { // reset Config->private_tx_isr
            if ( Config->Interrupt_TxComplete ) { // ensure interrupt source is disabled
                ttc_interrupt_disable_usart( Config->Interrupt_TxComplete );
            }
            if ( Config->private_tx_isr.CurrentTxBuffer ) {
                _ttc_usart_release_block( Config->private_tx_isr.CurrentTxBuffer );
            }
            ttc_memory_set( &Config->private_tx_isr, 0, sizeof( Config->private_tx_isr ) );
        }
        if ( 0 ) { // reset Config->private_task_receive
            ttc_memory_set( &Config->private_task_receive, 0, sizeof( Config->private_task_receive ) );
        }
    }

    Config->ActualBitRate = 0;

    // issue low-level initialization
    ReturnCode = _driver_usart_init( Config );

    Assert_USART( ReturnCode == ec_usart_OK, E_ttc_assert_origin_auto ); // low-level driver did return error-code. Check implementation of low-level driver for details!
    Assert_USART( Config->ActualBitRate != 0, E_ttc_assert_origin_auto ); // low-level driver must calculate actual bitrate. Check implementation of low-level driver for details!

    // Low-Level driver stores addresses of USART-Registers for fast
    // Rx/Tx operation (E.g. Config->BaseRegister->DR.All = Data;)
    Assert_USART_Writable( ( void* ) Config->BaseRegister, E_ttc_assert_origin_auto );  // low-level driver must set this field!

    // low-level initialization successfull: init interrupts

    if ( Config->Init.Flags.IrqOnRxNE &&
            ( ( Config->Init.function_Receive_Block != NULL ) || ( Config->Init.function_Receive_Single_Byte != NULL ) )
       ) { // setup interrupt for E_ttc_interrupt_type_USART_RxNE

        Config->Interrupt_RxNE = ttc_interrupt_init( E_ttc_interrupt_type_USART_RxNE,
                                                     Config->PhysicalIndex,
                                                     _driver_usart_isr_rx,
                                                     ( void* ) Config,
                                                     NULL,
                                                     NULL
                                                   );
        Assert_USART( Config->Interrupt_RxNE != NULL, E_ttc_assert_origin_auto );
        Config->ConfigInterrupts = ttc_interrupt_get_configuration_usart( Config->Interrupt_RxNE );
        ttc_interrupt_enable_usart( Config->Interrupt_RxNE );
    }
    if ( Config->Init.Flags.DelayedTransmits ) { // setup interrupt for E_ttc_interrupt_type_USART_TransmitDataEmpty
        if ( Config->Queue_Tx == NULL ) {
            Config->Queue_Tx = ttc_queue_pointer_create( Config->Init.Size_Queue_Tx );
            Assert_USART( Config->Queue_Tx != NULL, E_ttc_assert_origin_auto );
        }

        // this interrupt will be enabled when there is data to send
        Config->Interrupt_TxComplete = ttc_interrupt_init( E_ttc_interrupt_type_USART_TransmitDataEmpty,
                                                           Config->PhysicalIndex,
                                                           _driver_usart_isr_tx,
                                                           ( void* ) Config,
                                                           NULL,
                                                           NULL
                                                         );

        Assert_USART( Config->Interrupt_TxComplete != NULL, E_ttc_assert_origin_auto );  // interrupt could not be initialized
        Config->ConfigInterrupts = ttc_interrupt_get_configuration_usart( Config->Interrupt_TxComplete );
    }
    //ttc_mutex_init( &(Config->Queue_TX_Lock) );

    if ( !Config->Flags.Initialized ) { // first initialization: increase memory pool
        ttc_heap_pool_increase( &ttc_usart_PoolEmptyBlocks, Config->Init.Amount_PoolBlocks );
#if TTC_HEAP_POOL_STATISTICS == 1
        MyBlocks = ( t_ttc_usart_memory_heap_block* ) ttc_usart_PoolEmptyBlocks.Statistics.MyFirstBlock; //DEBUG
#endif
    }

    // register our configuration in global array and mark it as initialized
    A( ttc_usart_configs, LogicalIndex - 1 ) = Config;
    Config->Flags.Initialized = 1;

    // from now on use this USART for default output
    ttc_usart_stdout_set( LogicalIndex );

    /*  ttc_sysclock should now inform us whenever the systemclock changes */
    ttc_sysclock_register_for_update( _ttc_usart_update_to_sysclock );

    ttc_task_critical_end();
    Assert_USART_EXTRA( InitialCriticalLevel == ttc_interrupt_critical_level(), E_ttc_assert_origin_auto ); // amount of  ttc_task_critical_end() calls does not match. Check driver implementation!

#ifdef TTC_USART_TESTPIN_TX
    if ( Config->PortRTS != E_ttc_gpio_pin_none ) {
        ttc_gpio_init( Config->PortRTS, E_ttc_gpio_mode_output_push_pull, E_ttc_gpio_speed_max );
    }
#endif

    return ReturnCode;
}
void                      ttc_usart_load_defaults( t_u8 LogicalIndex ) {
    Assert_USART( LogicalIndex, E_ttc_assert_origin_auto );  // logical index starts at 1
    t_ttc_usart_config* Config = ttc_usart_get_configuration( LogicalIndex );

    if ( Config->Flags.Initialized )
    { ttc_usart_deinit( LogicalIndex ); }

    ttc_memory_set( Config, 0, sizeof( t_ttc_usart_config ) );

    // load generic default values
    Config->LogicalIndex                   = LogicalIndex;
    Config->PhysicalIndex                  = ttc_usart_logical_2_physical_index( LogicalIndex );
    Config->Timeout                        = 200;
    Config->Init.Flags.DelayedTransmits    = 1;
    Config->Init.Size_Queue_Rx             = 20;
    Config->Init.Size_Queue_Tx             = 20;
    Config->Init.Amount_PoolBlocks         = TTC_USART_AMOUNT_MEMORY_BUFFERS_DEFAULT;
    Config->Init.Amount_PoolBlocksReserved = TTC_USART_AMOUNT_MEMORY_BUFFERS_RESERVED_DEFAULT * TTC_USART_AMOUNT; // one global limit for all configured ttc_usart devices has less performance impact than one limit per individual device

    switch ( LogicalIndex ) { // load static pin configuration

#ifdef TTC_USART1
        case  1: { // check configuration of USART #1
#ifdef TTC_USART1_TX
            Config->PortTxD = TTC_USART1_TX;
#else
            Config->PortTxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART1_RX
            Config->PortRxD = TTC_USART1_RX;
#else
            Config->PortRxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART1_RTS
            Config->PortRTS = TTC_USART1_RTS;
#else
            Config->PortRTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART1_CTS
            Config->PortCTS = TTC_USART1_CTS;
#else
            Config->PortCTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART1_CK
            Config->PortCLK = TTC_USART1_CK;
#else
            Config->PortCLK = E_ttc_gpio_pin_none;
#endif
            break;
        }
#endif
#ifdef TTC_USART2
        case  2: { // check configuration of USART #2
#ifdef TTC_USART2_TX
            Config->PortTxD = TTC_USART2_TX;
#else
            Config->PortTxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART2_RX
            Config->PortRxD = TTC_USART2_RX;
#else
            Config->PortRxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART2_RTS
            Config->PortRTS = TTC_USART2_RTS;
#else
            Config->PortRTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART2_CTS
            Config->PortCTS = TTC_USART2_CTS;
#else
            Config->PortCTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART2_CK
            Config->PortCLK = TTC_USART2_CK;
#else
            Config->PortCLK = E_ttc_gpio_pin_none;
#endif
            break;
        }
#endif
#ifdef TTC_USART3
        case  3: { // check configuration of USART #3
#ifdef TTC_USART3_TX
            Config->PortTxD = TTC_USART3_TX;
#else
            Config->PortTxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART3_RX
            Config->PortRxD = TTC_USART3_RX;
#else
            Config->PortRxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART3_RTS
            Config->PortRTS = TTC_USART3_RTS;
#else
            Config->PortRTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART3_CTS
            Config->PortCTS = TTC_USART3_CTS;
#else
            Config->PortCTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART3_CK
            Config->PortCLK = TTC_USART3_CK;
#else
            Config->PortCLK = E_ttc_gpio_pin_none;
#endif
            break;
        }
#endif
#ifdef TTC_USART4
        case  4: { // check configuration of USART #4
#ifdef TTC_USART4_TX
            Config->PortTxD = TTC_USART4_TX;
#else
            Config->PortTxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART4_RX
            Config->PortRxD = TTC_USART4_RX;
#else
            Config->PortRxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART4_RTS
            Config->PortRTS = TTC_USART4_RTS;
#else
            Config->PortRTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART4_CTS
            Config->PortCTS = TTC_USART4_CTS;
#else
            Config->PortCTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART4_CK
            Config->PortCLK = TTC_USART4_CK;
#else
            Config->PortCLK = E_ttc_gpio_pin_none;
#endif
            break;
        }
#endif
#ifdef TTC_USART5
        case  5: { // check configuration of USART #5
#ifdef TTC_USART5_TX
            Config->PortTxD = TTC_USART5_TX;
#else
            Config->PortTxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART5_RX
            Config->PortRxD = TTC_USART5_RX;
#else
            Config->PortRxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART5_RTS
            Config->PortRTS = TTC_USART5_RTS;
#else
            Config->PortRTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART5_CTS
            Config->PortCTS = TTC_USART5_CTS;
#else
            Config->PortCTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART5_CK
            Config->PortCLK = TTC_USART5_CK;
#else
            Config->PortCLK = E_ttc_gpio_pin_none;
#endif
            break;
        }
#endif
#ifdef TTC_USART6
        case  6: { // check configuration of USART #6
#ifdef TTC_USART6_TX
            Config->PortTxD = TTC_USART6_TX;
#else
            Config->PortTxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART6_RX
            Config->PortRxD = TTC_USART6_RX;
#else
            Config->PortRxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART6_RTS
            Config->PortRTS = TTC_USART6_RTS;
#else
            Config->PortRTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART6_CTS
            Config->PortCTS = TTC_USART6_CTS;
#else
            Config->PortCTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART6_CK
            Config->PortCLK = TTC_USART6_CK;
#else
            Config->PortCLK = E_ttc_gpio_pin_none;
#endif
            break;
        }
#endif
#ifdef TTC_USART7
        case  7: { // check configuration of USART #7
#ifdef TTC_USART7_TX
            Config->PortTxD = TTC_USART7_TX;
#else
            Config->PortTxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART7_RX
            Config->PortRxD = TTC_USART7_RX;
#else
            Config->PortRxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART7_RTS
            Config->PortRTS = TTC_USART7_RTS;
#else
            Config->PortRTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART7_CTS
            Config->PortCTS = TTC_USART7_CTS;
#else
            Config->PortCTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART7_CK
            Config->PortCLK = TTC_USART7_CK;
#else
            Config->PortCLK = E_ttc_gpio_pin_none;
#endif
            break;
        }
#endif
#ifdef TTC_USART8
        case  8: { // check configuration of USART #8
#ifdef TTC_USART8_TX
            Config->PortTxD = TTC_USART8_TX;
#else
            Config->PortTxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART8_RX
            Config->PortRxD = TTC_USART8_RX;
#else
            Config->PortRxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART8_RTS
            Config->PortRTS = TTC_USART8_RTS;
#else
            Config->PortRTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART8_CTS
            Config->PortCTS = TTC_USART8_CTS;
#else
            Config->PortCTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART8_CK
            Config->PortCLK = TTC_USART8_CK;
#else
            Config->PortCLK = E_ttc_gpio_pin_none;
#endif
            break;
        }
#endif
#ifdef TTC_USART9
        case  9: { // check configuration of USART #9
#ifdef TTC_USART9_TX
            Config->PortTxD = TTC_USART9_TX;
#else
            Config->PortTxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART9_RX
            Config->PortRxD = TTC_USART9_RX;
#else
            Config->PortRxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART9_RTS
            Config->PortRTS = TTC_USART9_RTS;
#else
            Config->PortRTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART9_CTS
            Config->PortCTS = TTC_USART9_CTS;
#else
            Config->PortCTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART9_CK
            Config->PortCLK = TTC_USART9_CK;
#else
            Config->PortCLK = E_ttc_gpio_pin_none;
#endif
            break;
        }
#endif
#ifdef TTC_USART10
        case 10: { // check configuration of USART #10
#ifdef TTC_USART10_TX
            Config->PortTxD = TTC_USART10_TX;
#else
            Config->PortTxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART10_RX
            Config->PortRxD = TTC_USART10_RX;
#else
            Config->PortRxD = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART10_RTS
            Config->PortRTS = TTC_USART10_RTS;
#else
            Config->PortRTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART10_CTS
            Config->PortCTS = TTC_USART10_CTS;
#else
            Config->PortCTS = E_ttc_gpio_pin_none;
#endif
#ifdef TTC_USART10_CK
            Config->PortCLK = TTC_USART10_CK;
#else
            Config->PortCLK = E_ttc_gpio_pin_none;
#endif
            break;
        }
#endif

        default: ttc_assert_halt_origin( E_ttc_assert_origin_auto ); break; // No TTC_USARTn defined! Check your makefile.100_board_* file!
    }

    //Insert additional generic default values here ...

    _driver_usart_load_defaults( Config );

    // check certain required config fields that have to be filled by low-level driver
    Assert_USART_Writable( ( void* ) Config->BaseRegister, E_ttc_assert_origin_auto );  // low-level driver must assign base address of peripheral register!
}
t_u8                      ttc_usart_logical_2_physical_index( t_u8 LogicalIndex ) {

    switch ( LogicalIndex ) {         // determine base register and other low-level configuration data
#ifdef TTC_USART1
        case 1: return TTC_USART1;
#endif
#ifdef TTC_USART2
        case 2: return TTC_USART2;
#endif
#ifdef TTC_USART3
        case 3: return TTC_USART3;
#endif
#ifdef TTC_USART4
        case 4: return TTC_USART4;
#endif
#ifdef TTC_USART5
        case 5: return TTC_USART5;
#endif
#ifdef TTC_USART6
        case 6: return TTC_USART6;
#endif
#ifdef TTC_USART7
        case 7: return TTC_USART7;
#endif
#ifdef TTC_USART8
        case 8: return TTC_USART8;
#endif
#ifdef TTC_USART9
        case 9: return TTC_USART9;
#endif
#ifdef TTC_USART10
        case 10: return TTC_USART10;
#endif
        // extend as required

        default: break;
    }

    ttc_assert_halt_origin( E_ttc_assert_origin_auto ); // No TTC_USARTn defined! Check your makefile.100_board_* file!
    return -1; // program flow should never go here
}
void                      ttc_usart_prepare() {
    ttc_heap_pool_init( &ttc_usart_PoolEmptyBlocks,
                        //X TTC_USART_BLOCK_SIZE + sizeof( t_ttc_usart_memory_block ), // each memory block has a special usart header
                        sizeof( t_ttc_usart_memory_block ), // each memory block has a special usart header
                        0 // amount will be increased on every ttc_usart_init()
                      );
    ttc_usart_Amount_BlocksAllocated = 0;
    ttc_usart_Amount_RxBytesSkipped  = 0;

#ifdef TTC_USART_TESTPIN_TX
    ttc_gpio_init( TTC_USART_TESTPIN_TX, E_ttc_gpio_mode_output_push_pull, E_ttc_gpio_speed_max );
#endif
    // reset all data entries of safe array
    A_reset( ttc_usart_configs, TTC_USART_MAX_AMOUNT ); // must reset data of safe array manually!

    _driver_usart_prepare();
}
t_u8                      ttc_usart_physical_2_logical_index( t_u8 PhysicalIndex ) {

    switch ( PhysicalIndex ) {         // determine base register and other low-level configuration data
#ifdef TTC_USART1
        case TTC_USART1: return 1;
#endif
#ifdef TTC_USART2
        case TTC_USART2: return 2;
#endif
#ifdef TTC_USART3
        case TTC_USART3: return 3;
#endif
#ifdef TTC_USART4
        case TTC_USART4: return 4;
#endif
#ifdef TTC_USART5
        case TTC_USART5: return 5;
#endif
#ifdef TTC_USART6
        case TTC_USART6: return 6;
#endif
#ifdef TTC_USART7
        case TTC_USART7: return 7;
#endif
#ifdef TTC_USART8
        case TTC_USART8: return 8;
#endif
#ifdef TTC_USART9
        case TTC_USART9: return 9;
#endif
#ifdef TTC_USART10
        case TTC_USART10: return 10;
#endif
        // extend as required

        default: break;
    }

    ttc_assert_halt_origin( E_ttc_assert_origin_auto ); // No TTC_USARTn defined! Check your makefile.100_board_* file!
    return -1; // program flow should never go here
}
e_ttc_usart_errorcode     ttc_usart_register_receive( t_u8 LogicalIndex, t_ttc_usart_memory_block * ( *function_Receive_Block )( void* Argument, t_ttc_usart_config* Config, t_ttc_usart_memory_block* Block ), void* Argument ) {
    t_ttc_usart_config* Config = ttc_usart_get_configuration( LogicalIndex );

    if ( Config->Init.Flags.Receive ) { // USART driver implements receive functionality
        Config->Init.function_Receive_Block          = function_Receive_Block;
        Config->Init.function_Receive_Block_Argument = Argument;

        return ttc_usart_init( LogicalIndex );
    }
    return ec_usart_NotImplemented;
}
e_ttc_usart_errorcode     ttc_usart_reset( t_u8 LogicalIndex ) {
    Assert_USART( LogicalIndex, E_ttc_assert_origin_auto );  // logical index starts at 1
    t_ttc_usart_config* Config = _ttc_usart_get_configuration( LogicalIndex );

    ttc_memory_set( Config, 0, sizeof( Config ) );
    Config->LogicalIndex  = LogicalIndex;
    Config->PhysicalIndex = ttc_usart_get_physicalindex( LogicalIndex );


    e_ttc_usart_errorcode Status = _driver_usart_reset( Config );

    return Status;
}
e_ttc_usart_errorcode     ttc_usart_send_raw( t_u8 LogicalIndex, t_u8* Buffer, t_base Amount ) {
    Assert_USART( Buffer != NULL, E_ttc_assert_origin_auto );
    t_ttc_usart_config* Config = _ttc_usart_get_configuration( LogicalIndex );

    if ( Config->Init.Flags.DelayedTransmits ) { // push buffer to Config->Queue_Tx

        // data must be copied into one or more memory blocks
        t_u16 RemainingBytes = Amount;
        t_u16 Bytes_written  = 0;
        while ( RemainingBytes > 0 ) {
            t_ttc_usart_memory_block* TxBlock = ttc_usart_get_empty_block( LogicalIndex );

            t_u16 Bytes2Copy = RemainingBytes;
            if ( Bytes2Copy > TTC_USART_BLOCK_SIZE )
            { Bytes2Copy = TTC_USART_BLOCK_SIZE; }

            ttc_memory_copy( TxBlock->Buffer, &Buffer[Bytes_written], Bytes2Copy );
            TxBlock->AmountBytes = Bytes2Copy;
            ttc_usart_send_block( LogicalIndex, TxBlock );
            RemainingBytes -= Bytes2Copy;
            Bytes_written += Bytes2Copy;
        }
    }
    else
    { _ttc_usart_send_raw_blocking( LogicalIndex, Buffer, Amount ); }

    return ec_usart_OK;
}
e_ttc_usart_errorcode     ttc_usart_send_raw_const( t_u8 LogicalIndex, t_u8* Buffer, t_base Amount ) {
    Assert_USART( Buffer != NULL, E_ttc_assert_origin_auto );
    t_ttc_usart_config* Config = _ttc_usart_get_configuration( LogicalIndex );

    if ( Config->Init.Flags.DelayedTransmits ) { // push buffer to Config->Queue_Tx
        // data from constant memory is not copied
        t_ttc_usart_memory_block* TxBlock = ttc_usart_get_empty_block( LogicalIndex );
        TxBlock->Buffer = Buffer; // use memory block as a virtual memory block
        TxBlock->AmountBytes   = Amount;
        Assert( TxBlock->AmountBytes > 0, E_ttc_assert_origin_auto ); // given amount of bytes is too large for this function. Adapt implementation of caller!
        ttc_usart_send_block( LogicalIndex, TxBlock );
    }
    else
    { _ttc_usart_send_raw_blocking( LogicalIndex, Buffer, Amount ); }

    return ec_usart_OK;
}
e_ttc_usart_errorcode     ttc_usart_send_string( t_u8 LogicalIndex, const void* Buffer, t_base MaxLength ) {
    Assert_USART( Buffer != NULL, E_ttc_assert_origin_auto );
    t_ttc_usart_config* Config = _ttc_usart_get_configuration( LogicalIndex );

    if ( Config->Init.Flags.DelayedTransmits ) { // push buffer to Config->Queue_Tx
        // data must be copied into one or more memory blocks
        t_u16 TotalBytesCopied = 0;
        const t_u8* Reader = ( t_u8* ) Buffer;
        while ( TotalBytesCopied < MaxLength ) {
            t_ttc_usart_memory_block* TxBlock = ttc_usart_get_empty_block( LogicalIndex );

            t_u8* Writer = TxBlock->Buffer;

            t_u16 MaxBytes2Copy = TTC_USART_BLOCK_SIZE;
            if ( MaxBytes2Copy > ( MaxLength - TotalBytesCopied ) ) {
                MaxBytes2Copy = MaxLength - TotalBytesCopied;
            }

            t_u16 BytesCopied = 0;
            while ( BytesCopied < MaxBytes2Copy ) {
                t_u8 C = *Reader++;
                if ( C != 0 )         // stopping at first zero
                { *Writer++ = C; }
                else {
                    MaxLength = TotalBytesCopied; // will cause outer loop to leave
                    break;
                }

                BytesCopied++;
                TotalBytesCopied++;
            }

            TxBlock->AmountBytes = BytesCopied;
            ttc_usart_send_block( LogicalIndex, TxBlock );
        }
    }
    else
    { _ttc_usart_send_string_blocking( LogicalIndex, ( const char* ) Buffer, MaxLength ); }

    return ec_usart_OK;
}
e_ttc_usart_errorcode     ttc_usart_send_string_const( t_u8 LogicalIndex, const char* Buffer, t_base MaxLength ) {
    Assert_USART( Buffer != NULL, E_ttc_assert_origin_auto );
    t_ttc_usart_config* Config = _ttc_usart_get_configuration( LogicalIndex );

    if ( Config->Init.Flags.DelayedTransmits ) { // push buffer to Config->Queue_Tx
        // data from constant memory is not copied
        t_ttc_usart_memory_block* TxBlock = ttc_usart_get_empty_block( LogicalIndex );
        char* Buffer2 = ( char* ) Buffer;                       // yes, we cast away the constness as we have to store pointer in RAM!
        TxBlock->Buffer = ( t_u8* ) Buffer2;                    // use memory block as a virtual memory block
        TxBlock->AmountBytes   = - ( t_base_signed ) MaxLength; // marking string as zero terminated (see definition of t_ttc_usart_memory_block)
        if ( TxBlock->AmountBytes > 0 )                         // make sure its a negative value
        { TxBlock->AmountBytes = -TxBlock->AmountBytes; }

        ttc_usart_send_block( LogicalIndex, TxBlock );
    }
    else
    { _ttc_usart_send_string_blocking( LogicalIndex, Buffer, MaxLength ); }

    return ec_usart_OK;
}
e_ttc_usart_errorcode     ttc_usart_send_block( t_u8 LogicalIndex, t_ttc_usart_memory_block* Block ) {
    Assert_USART( Block != NULL, E_ttc_assert_origin_auto );
    t_ttc_usart_config* Config = _ttc_usart_get_configuration( LogicalIndex );

    if ( Config->Init.Flags.DelayedTransmits ) { // push buffer to Config->Queue_Tx
        //? while ( ttc_mutex_lock( &( Config->Queue_TX_Lock ), -1 ) != tme_OK ); TODO( "Extra Mutex still required for Queue_Tx?" );
        ttc_queue_pointer_push_back( Config->Queue_Tx, Block );
        TTC_USART_STATISTIC( Amount_Queue_TX += Block->AmountBytes );
        //? ttc_mutex_unlock( &( Config->Queue_TX_Lock ) ); TODO( "Extra Mutex still required for Queue_Tx?" );

        // enable Transmission interrupt to automatically call _ttc_usart_isr_tx()
        ttc_interrupt_enable_usart( Config->Interrupt_TxComplete );
    }
    else {
        _ttc_usart_send_raw_blocking( LogicalIndex, Block->Buffer, Block->AmountBytes );

        // the memory block came from a pool so it is safe to cast it
        ttc_heap_block_release( ( t_ttc_heap_block* ) Block );
    }

    return ec_usart_OK;
}
e_ttc_usart_errorcode     ttc_usart_read_word( t_u8 LogicalIndex, t_u16* Word ) {
    Assert_USART( Word != NULL, E_ttc_assert_origin_auto );
    t_ttc_usart_config* Config = _ttc_usart_get_configuration( LogicalIndex );
    Assert_USART_Writable( ( void* ) Config->BaseRegister, E_ttc_assert_origin_auto );

    return _driver_usart_read_word_blocking( Config->BaseRegister, Word );
}
e_ttc_usart_errorcode     ttc_usart_read_byte( t_u8 LogicalIndex, t_u8* Byte ) {
    Assert_USART( Byte != NULL, E_ttc_assert_origin_auto );
    t_ttc_usart_config* Config = _ttc_usart_get_configuration( LogicalIndex );
    Assert_USART_Writable( ( void* ) Config->BaseRegister, E_ttc_assert_origin_auto );

    return _driver_usart_read_byte( Config->BaseRegister, Byte );
}
e_ttc_usart_errorcode     ttc_usart_stdout_set( t_u8 LogicalIndex ) {
    Assert_USART( LogicalIndex <= TTC_USART_AMOUNT, E_ttc_assert_origin_auto );  // not egnough ttc_usart_configs defined!
    Assert_USART( A( ttc_usart_configs, LogicalIndex - 1 ) != NULL, E_ttc_assert_origin_auto );  // USART not yet initialized!

    ttc_usart_stdout_index = LogicalIndex;
    return ec_usart_OK;
}
void                      ttc_usart_stdout_send_block( t_ttc_usart_memory_block* Block ) {
    if ( ttc_usart_stdout_index ) {
        ttc_usart_send_block( ttc_usart_stdout_index, Block );
    }
}
void                      ttc_usart_stdout_send_string( const t_u8* String, t_base MaxSize ) {
    if ( ttc_usart_stdout_index ) {
        if ( ttc_memory_is_constant( String ) )
        { ttc_usart_send_string_const( ttc_usart_stdout_index, ( const char* ) String, MaxSize ); }
        else
        { ttc_usart_send_string( ttc_usart_stdout_index, ( const char* ) String, MaxSize ); }
    }
}
void                      ttc_usart_yield() {
    if ( ttc_task_is_scheduler_started() ) {
        ttc_task_yield();
    }
    else {                                   // manually process receive and transmit queues
        for ( t_u8 Index = 1; Index <= ttc_usart_get_max_logicalindex(); Index++ ) {
            t_ttc_usart_config* Config = ttc_usart_get_configuration( Index );
            _ttc_usart_process_queues( Config );
        }
    }
}
//InsertFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//}FunctionDefinitions
//{ private Function definitions *******************************************
//
// Convention: Private functions start with an underscore "_".
// Example:    void _ttc_usart(t_u8 LogicalIndex) {  }

// Note: Blocking transmit functions can be used from GDB like "p _ttc_usart_send_string_blocking(...)"

t_ttc_usart_config*       _ttc_usart_get_configuration( t_u8 LogicalIndex ) {
    Assert_USART( LogicalIndex > 0, E_ttc_assert_origin_auto );  // logical USART-indices start at 1 !
    t_ttc_usart_config* Config = A( ttc_usart_configs, LogicalIndex - 1 );
    Assert_USART_Writable( Config, E_ttc_assert_origin_auto );  // USART not yet created; call ttc_usart_get_configuration() before!

    return Config;
}
t_ttc_usart_memory_block* _ttc_usart_get_empty_block_try() {
    t_ttc_usart_memory_block* Block = ( t_ttc_usart_memory_block* ) ttc_heap_pool_block_get_try( &ttc_usart_PoolEmptyBlocks );
    if ( Block ) { // successfully obtained memory block from pool: initialize it
        Assert_USART_Writable( Block, E_ttc_assert_origin_auto );
        Block->AmountBytes = 0;
        Block->Buffer = ( t_u8* )( &( Block->Buffer ) + 1 ); // correct pointer to buffer in case this block has been used as a virtual block

#if TTC_HEAP_POOL_STATISTICS == 1
        t_ttc_heap_block_from_pool* PoolBlock = ( t_ttc_heap_block_from_pool* ) Block;
        PoolBlock--; // back to header
        PoolBlock->MyOwner = __builtin_return_address( 0 ); // store address of our caller
#endif
    }

    TTC_RETURN Block;
}
void                      _ttc_usart_isr_rx( t_ttc_usart_config* Config, t_u8 Byte ) {

    Assert_USART_Writable( Config, E_ttc_assert_origin_auto );
    Assert_USART( Config->Queue_Rx != NULL, E_ttc_assert_origin_auto );
    e_ttc_queue_error PushError;

    if ( Config->Init.function_Activity_Rx_ISR )
    { Config->Init.function_Activity_Rx_ISR( Config, Byte ); }

    // send byte to receive task
    PushError = ttc_queue_byte_push_back_isr( Config->Queue_Rx, Byte );
    TTC_USART_STATISTIC( Config->Amount_ISR_RX++ );
#ifdef TTC_REGRESSION
    Assert_USART( PushError < tqe_Error, E_ttc_assert_origin_auto );  // no bytes should be skipped in a regression test!
#endif

    if ( PushError >= tqe_Error ) {
        ttc_basic_increase_upto( Config->Amount_RxBytesSkipped, 255 );
    }
}
void                      _ttc_usart_isr_tx_fetch_buffer( t_ttc_usart_config* Config ) {
    t_ttc_usart_private_tx_isr* PrivateData = &( Config->private_tx_isr );
    e_ttc_queue_error Error = ttc_queue_pointer_pull_front_isr( Config->Queue_Tx, ( void** ) &PrivateData->CurrentTxBuffer );

    if ( Error == tqe_OK ) { // buffer found: use it
        Assert_USART_Writable( PrivateData->CurrentTxBuffer, E_ttc_assert_origin_auto ); // got invalid pointer from Queue_Tx. Check implementation of ttc_usart and/or ttc_queue_pointer_pull_front_isr()!
        PrivateData->TxBuffer_Reader = PrivateData->CurrentTxBuffer->Buffer;
        PrivateData->TxBuffer_Index  = 0;
        if ( PrivateData->CurrentTxBuffer->AmountBytes < 0 ) { // buffer stores zero terminated string
            if ( PrivateData->CurrentTxBuffer->AmountBytes == -1 ) {
                PrivateData->TxBuffer_Size = PrivateData->CurrentTxBuffer->AmountBytes; // buffer size not known: use maximum unsigned value
            }
            else {
                PrivateData->TxBuffer_Size = -PrivateData->CurrentTxBuffer->AmountBytes; // buffer size known: make it a positive value
            }
            PrivateData->ZeroTerminated  = TRUE;
        }
        else {                                                 // buffer stores binary data
            PrivateData->TxBuffer_Size   = PrivateData->CurrentTxBuffer->AmountBytes;
            PrivateData->ZeroTerminated  = FALSE;
        }
        PrivateData->BytesRemaining = TRUE;
    }
    else {                   // no success in obtaining buffer
        PrivateData->TxBuffer_Reader = NULL;
        PrivateData->CurrentTxBuffer = NULL;
        PrivateData->BytesRemaining  = FALSE; // No more buffers to send: reset flag
    }
}
t_bool                    _ttc_usart_isr_tx( t_ttc_usart_config* Config, volatile void* Argument ) {
    Assert_USART( Config != NULL, E_ttc_assert_origin_auto );
    TTC_USART_TESTPINS_SET( Config );
    t_ttc_usart_private_tx_isr* PrivateData = &( Config->private_tx_isr );

    if ( PrivateData->TxBuffer_Reader == NULL ) { // try to get new buffer from Queue_Tx
        _ttc_usart_isr_tx_fetch_buffer( Config );

        if ( PrivateData->TxBuffer_Reader == NULL ) { // could not obtain a new buffer: return immediately
            //Config->Amount_TxISRsSkipped++; // why was interrupt generated if there is no data to send?
            goto EndOfFunction;
        }
    }
    t_bool ByteValid = FALSE;
    t_u8 Byte;
    do { // fetch byte from available buffers until valid or no more buffers available
        Byte = *PrivateData->TxBuffer_Reader++;
        if ( ( Byte == 0 ) && PrivateData->ZeroTerminated ) { // end of zero terminated string
            _ttc_usart_release_block_isr( PrivateData->CurrentTxBuffer );
            _ttc_usart_isr_tx_fetch_buffer( Config );
        }
        else {
            if ( PrivateData->TxBuffer_Index >= PrivateData->TxBuffer_Size ) {  // buffer size exceeded
                _ttc_usart_release_block_isr( PrivateData->CurrentTxBuffer );
                _ttc_usart_isr_tx_fetch_buffer( Config );
            }
            else { // got a valid byte from buffer
                PrivateData->TxBuffer_Index++;
                ByteValid = TRUE;
            }
        }
    }
    while ( ( !ByteValid ) && ( PrivateData->CurrentTxBuffer != NULL ) );

    if ( ByteValid ) {
        _driver_usart_send_byte_isr( Argument, Byte );
        TTC_USART_STATISTIC( Amount_ISR_TX++ );
    }

EndOfFunction:
    if ( 1 ) { // construct required to avoid strange compiler error "error: a label can only be part of a statement and a declaration is not a statement"
        t_bool StayInISR = FALSE;
        if ( !PrivateData->BytesRemaining ) { // no more data to send: disable interrupt
            ttc_interrupt_disable_usart( Config->Interrupt_TxComplete );
        }
        else {
            if ( Config->Init.Flags.StayInISR ) {
                StayInISR = TRUE;
            }
        }

        TTC_USART_TESTPINS_CLR( Config );
        return StayInISR;
    }
}
void                      _ttc_usart_observe_transmits( t_ttc_usart_config* Config ) {

    if ( Config->private_tx_isr.BytesRemaining &&                                // transmit isr still has bytes to transmit
            Config->Init.Flags.DelayedTransmits &&                               // delayed transmits are enabled
            ( Config->Interrupt_TxComplete != NULL ) &&                          // an interrupt source of type TxComplete has been initialized
            ( ! ttc_interrupt_is_enabled_usart( Config->Interrupt_TxComplete ) ) // interrupt source is currently disabled
       ) {
        // reenable interupt source so that _ttc_usart_isr_tx() can process transmit queue
        ttc_interrupt_enable_usart( Config->Interrupt_TxComplete );
    }
}
void                      _ttc_usart_process_queues( t_ttc_usart_config* Config ) {
    if ( ( !ttc_task_is_scheduler_started() ) && Config->Flags.TaskRxRunning ) { // receive task has been started: run it to check for incoming data
        _ttc_usart_task_receive( Config );
    }
    _ttc_usart_observe_transmits( Config );
}
void                      _ttc_usart_release_block( t_ttc_usart_memory_block* Block ) {
    Assert_USART_Writable( Block, E_ttc_assert_origin_auto );

    ttc_heap_pool_block_free( Block );
}
void                      _ttc_usart_release_block_isr( t_ttc_usart_memory_block* Block ) {
    Assert_USART_Writable( Block, E_ttc_assert_origin_auto );

    ttc_heap_pool_block_free_isr( Block );
}
e_ttc_usart_errorcode     _ttc_usart_send_word_blocking( t_u8 LogicalIndex, const t_u16 Word ) {
    t_ttc_usart_config* Config = _ttc_usart_get_configuration( LogicalIndex );
    Assert_USART_Writable( ( void* ) Config->BaseRegister, E_ttc_assert_origin_auto );

    return _driver_usart_send_word_blocking( Config->BaseRegister, Word );
}
e_ttc_usart_errorcode     _ttc_usart_send_raw_blocking( t_u8 LogicalIndex, const t_u8* Buffer, t_u16 Amount ) {
    t_ttc_usart_config* Config = _ttc_usart_get_configuration( LogicalIndex );
    Assert_USART_Writable( ( void* ) Config->BaseRegister, E_ttc_assert_origin_auto );
    e_ttc_usart_errorcode Error = ec_usart_OK;

    //? while ( ttc_mutex_lock( &( Config->Queue_TX_Lock ), -1 ) != tme_OK );
    while ( Amount-- > 0 ) {
        // Error = _usart_stm32f1xx_send_word_blocking(BaseRegister, (t_u16) *Buffer++, -1);
        Error = _driver_usart_send_word_blocking( Config->BaseRegister, ( t_u16 ) * Buffer++ );
    }
    //? ttc_mutex_unlock( &( Config->Queue_TX_Lock ) );

    return Error;
}
e_ttc_usart_errorcode     _ttc_usart_send_string_blocking( t_u8 LogicalIndex, const char* Buffer, t_u16 MaxLength ) {
    Assert_USART( Buffer != NULL, E_ttc_assert_origin_auto );
    t_ttc_usart_config* Config = _ttc_usart_get_configuration( LogicalIndex );
    Assert_USART_Writable( ( void* ) Config->BaseRegister, E_ttc_assert_origin_auto );

    char C = 0;
    e_ttc_usart_errorcode Error = ec_usart_OK;
    //? ttc_mutex_lock( &( Config->Queue_TX_Lock ), -1 );

    while ( ( MaxLength-- > 0 ) && ( ( C = *Buffer++ ) != 0 ) ) {
        if ( Config->Init.function_Activity_Tx_ISR )
        { Config->Init.function_Activity_Tx_ISR( Config, C ); }

        Error = _driver_usart_send_word_blocking( Config->BaseRegister, ( t_u16 ) C );
        if ( Error != ec_usart_OK )
        { break; }
    }
    //? ttc_mutex_unlock( &( Config->Queue_TX_Lock ) );

    return Error;
}
void                      _ttc_usart_task_receive( void* Args ) {
    t_ttc_usart_config* Config = ( t_ttc_usart_config* ) Args;
    Assert_USART_Writable( Config, E_ttc_assert_origin_auto );

    if ( !Config->Flags.TaskRxRunning ) { // first call: initialize task
        Config->Flags.TaskRxRunning = 1;
        ttc_memory_set( &( Config->private_task_receive ), 0, sizeof( Config->private_task_receive ) );
    }
    else {                                // second call: check single tasking mode
        Assert_USART_EXTRA( ttc_task_is_scheduler_started() == 0, E_ttc_assert_origin_auto ); // this function is only allowed to be called more than once in single tasking mode!
    }

    t_bool ByteReceived = FALSE;
    do {
        t_u8 Byte;
        if ( ttc_task_is_scheduler_started() ) { // multi-tasking; use blocking queue pull which will put task to sleep
            Byte = ttc_queue_byte_pull_front( Config->Queue_Rx );
            ByteReceived = TRUE;
        }
        else {                                   // single-tasking: use non-blocking queue pull
            ByteReceived = ( ttc_queue_byte_pull_front_try( Config->Queue_Rx, &Byte ) == tqe_OK ) ? TRUE : FALSE;
        }
        if ( ByteReceived ) {                    // received a byte: store it in CurrentRxBuffer

            TTC_USART_STATISTIC( Amount_Queue_RX++ ); // count every byte received
            if ( Config->Init.function_Receive_Block != NULL ) {
                t_ttc_usart_memory_block* CurrentRxBuffer = Config->private_task_receive.CurrentRxBuffer;
                if ( Config->private_task_receive.Writer == NULL ) { // no buffer available: fetch new memory block from Queue_EmptyRxBlocks
                    if ( !CurrentRxBuffer ) { // no receive buffer: fetch one from queue
                        CurrentRxBuffer = Config->private_task_receive.CurrentRxBuffer = _ttc_usart_get_empty_block_try();
                    }
                    if ( CurrentRxBuffer ) {  // have a new buffer: point Writer to its start
                        Config->private_task_receive.Writer = Config->private_task_receive.CurrentRxBuffer->Buffer;
                        CurrentRxBuffer->AmountBytes = 0;
                    }
                }
                if ( Config->private_task_receive.Writer == NULL ) { // no buffer available: deliver single byte
                    if ( Config->Init.function_Receive_Single_Byte != NULL ) {
                        Config->Init.function_Receive_Single_Byte( Config, Byte );
                    }
                    else { // no memory block and no single receive function available!
                        Config->Amount_RxBytesSkipped++;
                    }
                }
                else {                                               // store byte in CurrentRxBuffer
                    *Config->private_task_receive.Writer++ = Byte;
                    CurrentRxBuffer->AmountBytes++;

                    if (
                        ( ( Config->Init.Char_EndOfLine < 0x100 ) && ( Config->Init.Char_EndOfLine == Byte ) ) ||
                        ( CurrentRxBuffer->AmountBytes >= TTC_USART_BLOCK_SIZE )
                    ) {
                        // zero terminate string in CurrentRxBuffer->Buffer[]
                        *Config->private_task_receive.Writer = 0;

                        // Hand over filled buffer to receive function being supplied by application.
                        // Buffer will be reused if function_Receive_Block() returns pointer != NULL.
                        t_ttc_usart_memory_block* ReturnedRxBuffer = Config->Init.function_Receive_Block( Config->Init.function_Receive_Block_Argument,
                                                                                                          Config,
                                                                                                          CurrentRxBuffer
                                                                                                        );

                        if ( ( ReturnedRxBuffer != NULL ) && ( ReturnedRxBuffer != CurrentRxBuffer ) )
                        { ttc_assert_halt_origin( E_ttc_assert_origin_auto ); } // Config->Init.function_Receive_Block() returned unknown pointer. It is only allowed to return NULL or CurrentRxBuffer. Fix implementation of Config->Init.function_Receive_Block()!

                        CurrentRxBuffer = Config->private_task_receive.CurrentRxBuffer = ReturnedRxBuffer;
                        Config->private_task_receive.Writer = NULL;
                    }
                }
            }
            else if ( Config->Init.function_Receive_Single_Byte != NULL ) { // deliver byte directly
                Config->Init.function_Receive_Single_Byte( Config, Byte );
            }
            else { Assert_USART( 0, E_ttc_assert_origin_auto ); }  // no function registered to pass received data to
        }
        else {                                   // no byte received: give cpu to other tasks
            ttc_task_yield();
        }
    }
    while ( ByteReceived ||                      // byte received: rerun loop to check for more bytes
            ttc_task_is_scheduler_started()      // multitasking mode: loop endless
          );
}
void                      _ttc_usart_update_to_sysclock() {

    for ( t_u8 Index = ttc_usart_get_max_index(); Index >= 1; Index-- ) {


        t_ttc_usart_config* Config = ttc_usart_get_configuration( Index );

        if ( Config->Flags.Initialized ) {
            ttc_usart_init( Index );
        }
    }

}
//InsertPrivateFunctionDefinitions above (DO NOT DELETE THIS LINE!)


//}FunctionDefinitions

