/** { ttc_usart.h *******************************************************
 *
 *                          The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  High-Level driver and documentation for USART devices.
 *  The functions defined in this file provide a hardware independent interface
 *  for your application.
 *
 *  The basic usage scenario for devices:
 *  1) check:       Assert_USART(tc_usart_get_max_index() > 0, E_ttc_assert_origin_auto);
 *  2) configure:   ttc_usart_get_configuration(LogicalIndex);
 *  3) initialize:  ttc_usart_init(LogicalIndex);
 *  4) use:         ttc_usart_XXX(LogicalIndex);
 *
 *  Device Index
 *  Every device has a physical and a logical index.
 *  The physical index ist simply its hardware address.
 *  Not all physical devices can be used in real hardware.
 *  Every board defines, which devices can be used under which logical index.
 *  The logical indices are defined in board makefiles (makefile.100_board_*).
 *
 *  Structures, Enums and Defines being required by high-level usart and application.
 *
 *  Created from template ttc_device.h revision 23 at 20140217 09:36:59 UTC
 *
 *  Authors: Gregor Rebel
 *
 */
/** Description of ttc_usart (Do not delete this line!)
 *
 * A Universal Synchronous and Asynchronous Receiver and Transmitter (USART) is the
 * swiss army knife in microcontroller communication. It is the most flexible communication
 * interface of modern microcontrollers. With disabled synchronous feature, a UART can send or receive data
 * at a wide range of data rates (300 bit/s .. 2 Mbit/s) over a single pin.
 * Different extensions allow to improve communication distance, amount of bus participants and data integrity.
 *
 *
 * H2: High-Level Features
 *
 * ttc_usart provides a convenient serial communication interface with several high-level features
 * that keeps low-level drivers small and easy to implement:
 * - Send fixed amount of raw bytes.
 * - Send zero terminated ASCII strings.
 * - Send data from constant memory areas without extra copying.
 * - Send memory blocks of raw bytes with automatic buffer release.
 * - Send large data blocks without extra copying using virtual memory blocks.
 * - Registration of external receive function to be called for incoming data.
 * - Registration of external activity functions to be called on every tx/rx byte.
 * - printf() redirection to serial port.
 * - Interrupt driven send and receive (if supported by hardware and low-level driver).
 * - Memory buffer management simplifies application code.
 * - Transmit and receive queues store incoming and outgoing buffers for delayed transmission.
 * - Received data is processed in user space task instead of interrupt service routine.
 * - Automatic adaption to system clock frequency changes (->ttc_sysclock).
 *
 *
 * H2: Configure USART Devices
 *
 * ttc_usart devices are enumerated by their logical index 1..ttc_usart_get_max_index().
 * Each logical index <n> is configured by static defines named TTC_USART<n>_* in the currently activated
 * board makefile. When writing your own board makefile, it should be easy to understand the syntax from
 * existing board makefiles. Also take a look at ttc_usart_types.h which checks all
 * TTC_USART* defines and generates compiler errors if required.
 *
 *
 * H2: Basic ttc_usart Usage
 *
 * #) Obtain and adjust Configuration
 *    // replace LogicalIndex by your preferred logical device index!
 *    t_ttc_usart_config* ConfigUsart = ttc_usart_get_configuration(LogicalIndex);
 *    ConfigUsart->... = ...;
 * #) Initiallize Device for Operation
 *    ttc_usart_init(LogicalIndex);
 * #) Send some Data
 *    ttc_usart_send_string_const(LogicalIndex, "Hello World\n");
 *    // check description of other ttc_usart_send_*() functions for more details!
 * #) Check for Received Data
 *
 *
 * H2: Advanced ttc_usart Usage
 *
 * Every communication with external devices raises complexity of your project.
 * Communicating with one partner after another keeps you on the safe time.
 * But if you have to mange multiple streams (USART, SPI, I2C, keyboard, complex calculations) then your
 * project will get very complex and you have to dive much deeper into ttc_usart.
 *
 * Activate, compile and check example_ttc_usart to see a live example!
 *
 *
 * H2: Redirecting printf() to ttc_usart
 *
 * ttc_string_register_stdout() allows to redirect output of ttc_string_printf() to another device.
 * To redirect ttc_string_printf() to a ttc_usart device, simply
 * // 1. set logical index of default ttc_usart device
 * ttc_usart_stdout_set( 1..ttc_usart_get_max_logicalindex);
 * // 2. register ttc_usart stdout functions to ttc_string device:
 * ttc_string_register_stdout( ttc_usart_stdout_send_block, ttc_usart_stdout_send_string );
 * All printf() calls will then be redirected to ttc_usart device.
 *
 * Note: printf() compatibility has been added just for convenience but it comes with drawbacks, pitfalls and headaches.
 *       It is advised to use ttc_string_snprintf() and local buffers instead!
 *
 * H2: Basic Process of Data Receival
 *
 * Individual bytes are received in an interrupt service routine and buffer in a receive queue.
 * A receive task then copies bytes from queue into memory blocks.
 * If a memory block is full or a predefined end-of-line character has been detected then the whole block is
 * handed to the application for further processing.
 * The advantage of this two level implementation is that the interrupt service routine is small and fast.
 * It allows to store all bytes received in high speed data bursts. The more complex buffer handling is done in a
 * non-timecritical user space task.
 *
 * #) USART device receives a byte
 *    #) RxNE Interrupt is risen
 *    #) usart_*_rx_isr()
 *       #) Reads received byte from usart data register.
 *       #) Passes byte to _ttc_usart_isr_rx()
 *    #) _ttc_usart_isr_rx()
 *       #) may call Config->Init.function_Activity_Rx_ISR()
 *       #) Pushes byte into Config->QueueRx
 * #) _ttc_usart_task_receive()
 *   #) Pops byte from Config->QueueRx
 *   #) Appends byte to CurrentRxBuffer.
 *   #) If CurrentRxBuffer is full or byte == Config->Init.Char_EndOfLine then
 *      #) Application provided receive function is called:
 *         Config->Init.function_Receive_Block(Config->Init.function_Receive_Block_Argument, Config, CurrentRxBuffer)
 *
 * H2: Basic Process of Data Transmission
 *
 * The transmission process differs depending on value of Config->Init.Flags.DelayedTransmits.
 *
 * H3: Config->Init.Flags.DelayedTransmits==1
 *
 * Delayed transmissions store memory blocks to be transmitted in Config->QueueTx.
 * A transmit interrupt service routine then transmits individual bytes from each
 * queued memory block one after another.
 * The application that started the transmission can run in parallel to the ongoing byte sending.
 * This mode is optimal for slow communications (approximately <1Mbit/second).
 *
 * #) Application calls ttc_usart_send_*()
 *    #) TxBlock = ttc_usart_get_empty_block()
 *    #) Given data is referenced in or copied into TxBlock.
 *    #) calls ttc_usart_send_block(TxBlock)
 * #) ttc_usart_send_block(TxBlock)
 *    #) TxBlock is pushed into Config->Queue_Tx.
 *    #) calls ttc_interrupt_enable_usart() to enable transmit interrupt.
 * #) USART transmit buffer is empty.
 *    #) TxE interrupt is risen and calls usart_stm32f1xx_tx_isr()
 *    # usart_stm32f1xx_tx_isr()
 *      #) Obtain address of USART DataRegister
 *      #) Call _ttc_usart_isr_tx() in a loop (optimization for high BitRates and if StayInISR==1)
 *         do {
 *           StayInISR = _ttc_usart_isr_tx()
 *         } while (StayInISR  && TxE)
 *    #) _ttc_usart_isr_tx()
 *       #) if PrivateData->CurrentTxBuffer==NULL then
 *          _ttc_usart_isr_tx_fetch_buffer(): PrivateData->CurrentTxBuffer = ttc_queue_pop(Config->Queue_Tx)
 *       #) Call _driver_usart_send_byte_isr() to sent a single byte
 *       #) If no bytes remain to be sent then
 *          ttc_interrupt_disable_usart() disables transmit interrupt
 *
 * H3: Config->Init.Flags.DelayedTransmits==0
 *
 * Direct transmission does not require any interrupt support.
 * All ttc_usart_send_*() fucntions will block until all data has been sent.
 * Config->Init.Amount_PoolBlocks can be lowered to reduce RAM usage.
 * This mode is optimal for fast communications (approximately >1Mbit/second).
 *
 * #) Application calls ttc_usart_send_*()
 *    #) TxBlock = ttc_usart_get_empty_block()
 *    #) Given data is referenced in or copied into TxBlock.
 *    #) _ttc_usart_send_raw_blocking() will block until complete data is sent.
 *
}*/

#ifndef TTC_USART_H
#define TTC_USART_H

#ifndef EXTENSION_ttc_usart
    #  error Missing extension! Please add this to your activate_project.sh script: activate.500_ttc_usart.sh
#endif

//{ Includes *************************************************************

#include "ttc_basic.h"  // Include First! (basic datatypes and definitions)
#include "ttc_memory.h"
#include "ttc_interrupt.h"
#include "interfaces/ttc_usart_interface.h" // multi architecture support
//InsertIncludes above (DO NOT DELETE THIS LINE!)

//} Includes
//{ Macros ***************************************************************

//InsertMacros above (DO NOT DELETE THIS LINE!)

//}Macros
//{ Defines/ TypeDefs ****************************************************

//InsertDefines above (DO NOT DELETE THIS LINE!)

//} Defines
//{ Structures/ Enums ****************************************************
// Structures and Enums required by High-Level usart only go here...

//InsertEnums above (DO NOT DELETE THIS LINE!)

//InsertTypeDefs above (DO NOT DELETE THIS LINE!)

//} Structures/ Enums
/** Function prototypes *************************************************{
 *
 * The functions declared below provide the main interface for
 * usart devices on all supported architectures.
 * Check usart/usart_* files for a list of low-level drivers.
 * In most cases, only one low-level driver may be active during compilation.
 * Low-level drivers get activated by calling an activate.450_*.sh script.
 * A complete list of all available activate scripts can be found in your
 * activate_project.sh file inside your project folder.
 *
 */

/** Prepares usart Driver for basic operation.
 *
 * Note: This function is called automatically at system start before the scheduler has been started.
 */
void ttc_usart_prepare();

/** returns amount of devices available in current configuration
 *
 * @return amount of available devices
 */
t_u8 ttc_usart_get_max_index();

/** returns configuration of indexed device (asserts if no valid configuration was found)
 *
 * @param LogicalIndex    index of device to init (1..ttc_USART_get_max_LogicalIndex())
 * @return                configuration of indexed device
 */
t_ttc_usart_config* ttc_usart_get_configuration( t_u8 LogicalIndex );

/** initialize indexed device for use
 *
 * @param LogicalIndex    device index of device to init (1..ttc_USART_get_max_LogicalIndex())
 * @return              == 0: USART has been reset successfully; != 0: error-code
 */
e_ttc_usart_errorcode ttc_usart_init( t_u8 LogicalIndex );

/** shutdown indexed device
 *
 * @param LogicalIndex    device index of device to shutdown (1..ttc_USART_get_max_LogicalIndex())
 */
void ttc_usart_deinit( t_u8 LogicalIndex );

/** load default setting values for indexed device
 *
 * Seetings of indexed device are being reset to their default values.
 * Will automatically call ttc_usart_deinit() if device has been initialized.
 *
 * @param LogicalIndex  logical index of usart device (1..ttc_usart_get_max_index() )
 */
void ttc_usart_load_defaults( t_u8 LogicalIndex );

/** maps from logical to physical device index
 *
 * High-level usarts (ttc_usart_*() functions) use logical device indices.
 * Logical index n is mapped via "COMPILE_OPTS += -DTTC_USARTn=p" lines in the makefile to physical index p.
 *
 * Boards usually connect only some of the available devices to connectors. Therefore, some devices may not be
 * usable. The mapping from logical to physical indices allows to create a consecutive range of indices in any
 * order.
 *
 * @param LogicalIndex  logical index of usart device (1..ttc_usart_get_max_index() )
 * @return              physical index of usart device (0 = first physical usart device, ...)
 */
t_u8 ttc_usart_logical_2_physical_index( t_u8 LogicalIndex );

/** maps from physical to logical device index
 *
 * This is a reverse function to ttc_usart_logical_2_physical_index()
 *
 * @param  PhysicalIndex  physical index of usart device (0 = first physical usart device, ...)
 * @return                logical index of usart device (1..ttc_usart_get_max_index() )
 */
t_u8 ttc_usart_physical_2_logical_index( t_u8 PhysicalIndex );

/** reset configuration of indexed device and connected hardware
 *
 * @param LogicalIndex    device index of device to init (1..ttc_USART_get_max_LogicalIndex())
 * @return              == 0: USART has been reset successfully; != 0: error-code
 */
e_ttc_usart_errorcode ttc_usart_reset( t_u8 LogicalIndex );

/** Prepares USART-interface for operation. Must be called before any other function!
 */
void ttc_usart_prepare();

/** returns amount of USART devices available on current uC
 * @return amount of available USART devices
 */
t_u8 ttc_usart_get_max_logicalindex();

/** returns configuration of indexed usart (asserts if no valid configuration was read)
 *
 * @param LogicalIndex    device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @return                !=NULL: pointer to struct t_ttc_usart_config
 */
t_ttc_usart_config* ttc_usart_get_configuration( t_u8 LogicalIndex ) ;

/** fills out given USART_Generic with default values for indexed USART
 * @param LogicalIndex    device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @return  == 0:         *USART_Generic has been initialized successfully; != 0: error-code
 */
e_ttc_usart_errorcode ttc_usart_reset( t_u8 LogicalIndex );

/** fills out given USART configuration with maximum valid values for indexed USART
 *
 * @param LogicalIndex   device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @param Config         USART configuration as returned by ttc_usart_get_configuration()
 * @return  == 0:        *USART_Generic has been initialized successfully; != 0: error-code
 */
e_ttc_usart_errorcode ttc_usart_get_features( t_u8 LogicalIndex, t_ttc_usart_config* Config );

/** Delivers a memory block usable as transmit or receive buffer from preallocated pool of available usart memory block.
 *
 * During ttc_usart_prepare() a memory pool is allocated for all configured usarts TTC_USART<1>..TTC_USART<TTC_USART_AMOUNT>.
 *
 * Note: This function will block until a new buffer is available.
 * Note: In in single tasking mode, call ttc_usart_get_empty_block_try() instead to avoid endless blocking!
 *
 *
 * @param LogicalIndex   device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @return               != NULL: pointer to new or reused memory block; == NULL: currently no buffer available (-> increase USART_Generic->Size_Queue_Tx or wait until tx-buffer is released)
 */
t_ttc_usart_memory_block* ttc_usart_get_empty_block( t_u8 LogicalIndex );

/** returns size of payload area of each memory block being returned by ttc_usart_get_empty_block().
 *
 * All memory blocks of all initialized ttc_usart devices are held in a commond pool of equal sized memory blocks.
 * The size of each memory block is set at compile time via constant TTC_USART_BLOCK_SIZE.
 * You should not define this value too high to reduce RAM usage.
 * The ttc_usart_send_*() function can automatically send larger strings by using multiple buffers.
 *
 * You may also use memory blocks as Virtual Blocks to avoid unnecessary data copying.
 * See description of ->ttc_usart_send_block() for details!
 *
 * @return size of t_ttc_usart_memory_block.Buffer[] of each memory block returned by ttc_usart_get_empty_block().
 */
t_u16 ttc_usart_get_empty_block_size();
#define ttc_usart_get_empty_block_size()  ((t_u16) TTC_USART_BLOCK_SIZE)

/** Delivers a memory block usable as transmit or receive buffer from preallocated pool of available usart memory block.
 *
 * Note: See ->ttc_usart_get_empty_block() for more details of how to use memory blocks with ttc_usart!
 *
 * @param LogicalIndex   device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @return               != NULL: pointer to new or reused memory block; == NULL: currently no buffer available (-> increase USART_Generic->Size_Queue_Tx or wait until tx-buffer is released)
 */
t_ttc_usart_memory_block* ttc_usart_get_empty_block_try( t_u8 LogicalIndex );

/** initializes single USART with current configuration
 * @param LogicalIndex    device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @return                == 0: USART has been initialized successfully; != 0: error-code
 */
e_ttc_usart_errorcode ttc_usart_init( t_u8 LogicalIndex );

/** registers given function as a receiver for incoming data
 *
 * Note: This function will automatically call ttc_usart_init() to init/reinit USART
 *
 * @param LogicalIndex            device index of USART to use (1..ttc_usart_get_max_logicalindex())
 * @param function_Receive_Block  function to call for each incoming data block (will contain one or more bytes)
 *                                function_Receive_Block() can return value of Block to allow to reuse it for new received data
 *                                If function_Receive_Block() wants to use the memory block longer then has to return NULL
  *                               and call ttc_heap_block_release(Block) to release it later.
 * @param Argument                will be passed as first argument to function_Receive_Block()
 * @return == 0: function has been registered and USART reinitialized successfully; != 0: error-code
 */
e_ttc_usart_errorcode ttc_usart_register_receive( t_u8 LogicalIndex, t_ttc_usart_memory_block * ( *function_Receive_Block )( void* Argument, t_ttc_usart_config* Config, t_ttc_usart_memory_block* Block ), void* Argument );

/** Stores given raw buffer to be send in parallel. (Function copies data and returns immediately if egnough memory blocks are available)
 *
 * H2: Sending without Blocking
 * Sending data via this function will block sender whenever not egnough memory blocks are available to store given amount of bytes.
 * Checking amount of available blocks before sending is not 100% safe.
 * If non-blocking sending is important, you should use ->ttc_usart_send_block() instead!
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 *
 * @param LogicalIndex     device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @param Buffer           memory location from which to read data
 * @param Amount           amount of bytes to send from Buffer[] (even zero bytes are sent)
 * @return                 == 0: Buffer has been stored for delivery successfully; != 0: error-code
 */
e_ttc_usart_errorcode ttc_usart_send_raw( t_u8 LogicalIndex, t_u8* Buffer, t_base Amount );

/** Stores given raw buffer to be send in parallel. (Function stores pointer to data and returns immediately)
 *
 * This function is especially optimized for constant data (e.g. strings in ROM) and avoids extra copying into buffers.
 *
 * H2: Sending without Blocking
 * Sending data via this function will block sender whenever no single memory block is available.
 * Checking amount of available blocks before sending is not 100% safe.
 * If non-blocking sending is important, you should use ->ttc_usart_send_block() instead!
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 *
 * @param LogicalIndex     device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @param Buffer           memory location from which to read data (must not change!)
 * @param Amount           amount of bytes to send from Buffer[] (even zero bytes are sent)
 * @return                 == 0: Buffer has been stored for delivery successfully; != 0: error-code
 */
e_ttc_usart_errorcode ttc_usart_send_raw_const( t_u8 LogicalIndex, t_u8* Buffer, t_base Amount );

/** Stores given string buffer to be send in parallel. (Function copies data and returns immediately)
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 *
 * @param LogicalIndex     device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @param Buffer           memory location from which to read data
 * @param MaxLength        no more than this amount of bytes will be send
 * @return                 == 0: Buffer has been stored for delivery successfully; != 0: error-code
 */
e_ttc_usart_errorcode ttc_usart_send_string( t_u8 LogicalIndex, const void* Buffer, t_base MaxLength );

/** Stores given string buffer to be send in parallel. (Function stores pointer to data and returns immediately)
 *
 * This function is especially optimized for constant data (e.g. strings in ROM) and avoids extra copying into buffers.
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 *
 * @param LogicalIndex     device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @param Buffer           memory location from which to read data
 * @param MaxLength        no more than this amount of bytes will be send
 * @return                 == 0: Buffer has been stored for delivery successfully; != 0: error-code
 */
e_ttc_usart_errorcode ttc_usart_send_string_const( t_u8 LogicalIndex, const char* Buffer, t_base MaxLength );

/** Universal delayed sending of data from given memory block.
 *
 * This function copies no data and has least overhead of all delayed functions.
 * After all data from Block has been transmitted, the Block is appended to an internal queue.
 *
 * H2: Using Memory Blocks from foreign Pool
 *
 * ttc_usart_send_block() will also work with any memory block being allocated via ttc_heap_pool_get()
 * and initialized as t_ttc_usart_memory_block.
 *
 * Example Usage of foreign Memory Pool:
 <CODE>

    // Define Dimensions of new Memory Pool
    const t_u16 MyPayloadSize = 16; // payload size of each memory block in pool
    const t_u16 AmountBlocks  = 10; // amount of memory blocks to allocate

    // Create Memory Pool (increase payload size by t_ttc_usart_memory_block)
    t_ttc_heap_pool* MyPool = ttc_heap_pool_create( MyPayloadSize + sizeof( t_ttc_usart_memory_block ), AmountBlocks );

    // Prepare one block from our pool for transmission
    t_ttc_usart_memory_block* MyBlock = ttc_heap_pool_block_get_try( MyPool );

    // calculacte memory address right behind MyBlock->Buffer
    MyBlock->Buffer      = (t_u8*) &( MyBlock->Buffer ) + 1;

    // Below you find different variants how to fill memory block before sending it!

    // Variant A: Typecast Buffer[] to some struct and fill with data to send
    typedef struct { t_u8 A; t_u16 B; t_u8 C; } t_my_struct;
    t_my_struct* MyStruct = ( t_my_struct* ) MyBlock->Buffer;
    MyStruct->A = 1;
    MyStruct->B = 2;
    MyStruct->C = 3;
    MyBlock->AmountBytes = sizeof( t_my_struct );


    // Variant B: Copy constant string into buffer and calculate payload size using pointer arithmetics.
    t_u8* EndOfString = ttc_string_snprintf( MyBlock->Buffer, MyPayloadSize, "Hello" );
    MyBlock->AmountBytes = EndOfString - MyBlock->Buffer;


    // Variant C: Point Buffer onto constant string (zero-copy operation).
    //            Use negative size to indicate zero termination (constant C-strings are zero terminated).
    MyBlock->Buffer = "Hello";
    MyBlock->AmountBytes = -MyPayloadSize;


    // finally send prepared memory block from MyPool
    // MyBlock will be returned to MyPool after successfull transmission.
    // transmissionComplete() will be called if registered.
    ttc_usart_send_block( <Logical_Index_Usart>, MyBlock );

 * </CODE>
 *
 *
 * H3: Using Virtual Blocks to send large Buffers without Copying
 *
 * Memory blocks can also be used as virtual blocks  which do not store real data, but reference
 * data in another buffer.
 * Principle of Virtual Blocks
 * *) Point Buffer[] to existing struct and set payload size to send without copying data.
 * *) Size of payload area can be zero for this but requires special free() function.
 * *) Requires signalling via global variable to keep buffer until it has been transmitted.
 *
 * <CODE>
    // Create a new Memory Pool whiTH extra small Blocks
    // Note: A single virtual block can transport up to 2GB of data!
    t_ttc_heap_pool MyVirtualPool;
    ttc_heap_pool_init(&MyVirtualPool, sizeof(t_ttc_usart_memory_block), 10);

    // Flag variable indicating end of transfer (must persist during whole transmission!)
    t_bool AllDataSent = FALSE;

    // implement a small function to be called whenever a block from MyPool is released.
    t_ttc_heap_block_from_pool* transmissionComplete( t_ttc_heap_block_from_pool* Block ) {
        // flag variable must be set only here
        AllDataSent = TRUE;

        // return Block unchanged to have it returned to its pool
        return Block;
    }

    // our simple function can work for both types
    ttc_heap_pool_register_free(MyVirtualPool, transmissionComplete, transmissionComplete);

    // prepare amount of data structs to be send
    #define AMOUNT_ENTRIES 10
    t_my_struct MyStruct[AMOUNT_ENTRIES];
    MyStruct[0].A = 1; MyStruct[0].B = 2; MyStruct[0].C = 3;
    //...

    // obtain memory block from our pool and reference it to our data
    t_ttc_usart_memory_block* MyBlock = ttc_heap_pool_block_get(&MyVirtualPool);
    MyBlock->Buffer = (t_u8*) &(MyStruct[0]);
    MyBlock->AmountBytes = AMOUNT_ENTRIES * sizeof(t_my_struct);

    // Send complete MyStruct[AMOUNT_ENTRIES] array without copying it into extra buffers.
    // Note: MyStruct[AMOUNT_ENTRIES] must not be changed before AllDataSent == TRUE.
    ttc_usart_send_block(<Logical_Index_Usart>, MyBlock);

    //...

 * </CODE>
 * @param LogicalIndex   device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @param Block          stores data to be send; It is mandatory to allocate Block with ttc_usart_get_empty_block()!
 * @return               == 0: Buffer has been stored for delivery successfully; != 0: error-code
 */
e_ttc_usart_errorcode ttc_usart_send_block( t_u8 LogicalIndex, t_ttc_usart_memory_block* Block );

/** Waits until transmit queue has run empty on given USART
 *
 * Note: USART must be initialized before!
 * Note: This function blocks until all bytes have been sent!
 * Note: You may want to call ttc_task_critical_begin() before to be sure that no other task transmits in parallel.
 *
 * @param LogicalIndex      device index of USART to init (1..ttc_usart_get_max_logicalindex())
 */
void ttc_usart_flush_tx( t_u8 LogicalIndex );

/** Reads single data word from input buffer (8 or 9 bits)
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 * Note: This function blocks until a valid word has been received or timeout occurs
 *
 * @param LogicalIndex   device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @param Word           pointer to 16 bit buffer where to store Word
 * @return               == 0: Word has been read successfully; != 0: error-code
 */
e_ttc_usart_errorcode ttc_usart_read_word( t_u8 LogicalIndex, t_u16* Word );

/** Handle background tasks and give cpu to other processes
 *
 * This function can be called whenever ttc_usart functions wait for something.
 * In multitasking mode, if simply hands over cpu to other tasks.
 * In singletasking mode, it calls internal task functions to transmit data from Tx-Queue and
 * to store incoming bytes in Rx-Queue.
 */
void ttc_usart_yield();

/** returns amount of bytes being held in receive buffer
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 *
 * @param LogicalIndex   device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @param Config         USART configuration as returned by ttc_usart_get_configuration()
 * @return               == 0: no bytes available; >0: amount of bytes available
 */
t_u8 ttc_usart_check_bytes_available( t_u8 LogicalIndex );
t_u8 _driver_usart_check_bytes_available( t_ttc_usart_config* Config );

/** Reads single data byte from input buffer.
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 * Note: This function blocks until a valid byte has been received or timeout occurs
 *
 * @param LogicalIndex   device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @param Word           pointer to 16 bit buffer where to store Word
 * @return               == 0: Word has been read successfully; != 0: error-code
 */
e_ttc_usart_errorcode ttc_usart_read_byte( t_u8 LogicalIndex, t_u8* Byte );

/** translates from logical index of functional unit to real hardware index (0=USART1, 1=USART2, ...)
 * @param LogicalIndex    logical device index of USART to init (1..ttc_usart_get_max_logicalindex())
 * @return                real device index of USART to init (0..MAX)
 */
t_u8 ttc_usart_get_physicalindex( t_u8 LogicalIndex );

/** Defines which logical device shall be used by ttc_usart_stdout_send_string()/ ttc_usart_stdout_send_block()
 *
 * @param LogicalIndex   >0: logical device index of USART (must be initialized before!); ==0: disable default USART
 * @return               != 0: error message
 */
e_ttc_usart_errorcode ttc_usart_stdout_set( t_u8 LogicalIndex );

/** Will send given memory block to default USART being set by ttc_usart_stdout_set()
 *
 * Note: Requires to call ttc_usart_stdout_set() once before to define logical index of USART device to use!
 *
 * @param Block         pointer to a t_ttc_usart_memory_block (will be released after sending automatically)
 */
void ttc_usart_stdout_send_block( t_ttc_usart_memory_block* Block );

/** Will send given string to default USART being set by ttc_usart_stdout_set()
 *
 * Note: Requires to call ttc_usart_stdout_set() once before to define logical index of USART device to use!
 *
 * @param Block         pointer to a t_ttc_usart_memory_block (will be released after sending automatically)
 */
void ttc_usart_stdout_send_string( const t_u8* String, t_base MaxSize );

/** Checks if usart still has data to transmit
 *
 *
 *
 * @param LogicalIndex (t_u8)  Logical index of usart instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_USART<n> lines in extensions.active/makefile
 * @return             (t_bool) ==TRUE: usart device is still transmitting data.
 */
t_bool ttc_usart_check_transmission_ongoing( t_u8 LogicalIndex );

//InsertFunctionDeclarations above (DO NOT DELETE THIS LINE!)

//} Function prototypes
/** Prototypes of low-level driver functions ****************************{
 *
 * Functions declared with prefix _driver_heap_ are passed to interfaces/ttc_heap_interface.h and to
 * all currently activated low-level drivers.
 * If a driver declaration is placed directly below its ttc_* pendant, it will share its documentation.
 * Driver declarations with different argument list should be bundled with their own documentation in this section.
 *
 * If you add a _driver_* prototype, use create_DeviceDriver.pl to automatically add empty functions
 * in all existing low-level drivers by issuing this inside git/TheToolChain/TTC-Library/:
 * cd templates/; ./create_DeviceDriver.pl heap UPDATE
 */

/** Prepares usart driver for basic operation.
 *
 * Note: This function is called automatically at system start before the scheduler has been started.
 */
void _driver_usart_prepare();

/** returns configuration of indexed device (asserts if no valid configuration was found)
 *
 * @param Config         USART configuration as returned by ttc_usart_get_configuration()
 * @return               configuration of indexed device
 */
t_ttc_usart_config* _driver_usart_get_configuration( t_ttc_usart_config* Config );

/** fills out given Config with maximum valid values for indexed USART
 *
 * Note: Must set Config->Architecture!
 *
 * @param Config         USART configuration as returned by ttc_usart_get_configuration()
 * @return  == 0:        *Config has been initialized successfully; != 0: error-code
 */
e_ttc_usart_errorcode _driver_usart_get_features( t_ttc_usart_config* Config );

/** shutdown single USART unit device
 * @param Config         USART configuration as returned by ttc_usart_get_configuration()
 * @return               == 0: USART has been shutdown successfully; != 0: error-code
 */
e_ttc_usart_errorcode _driver_usart_deinit( t_ttc_usart_config* Config );

/** initializes single USART unit for operation
 *
 * Note: Low-level driver must calculate and set Config->ActualBitRate to real bitrate!
 *
 * @param Config         USART configuration as returned by ttc_usart_get_configuration()
 * @return               == 0: USART has been initialized successfully; != 0: error-code
 */
e_ttc_usart_errorcode _driver_usart_init( t_ttc_usart_config* Config );

/** loads configuration of indexed USART unit with default values
 * @param Config         USART configuration as returned by ttc_usart_get_configuration()
 * @return  == 0:        configuration was loaded successfully
 */
e_ttc_usart_errorcode _driver_usart_load_defaults( t_ttc_usart_config* Config );

/** reset configuration of indexed device and connected hardware
 *
 * @param Config         USART configuration as returned by ttc_usart_get_configuration()
 * @return               == 0: USART has been reset successfully; != 0: error-code
 */
e_ttc_usart_errorcode _driver_usart_reset( t_ttc_usart_config* Config );

/** Reads single data word from input buffer (8 or 9 bits)
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 * Note: This function blocks until a valid word has been received or timeout occurs
 *
 * @param Register      pointer to low-level register for fast hardware operation
 * @param Word           pointer to 16 bit buffer where to store Word
 * @return               == 0: Word has been read successfully; != 0: error-code
 */
e_ttc_usart_errorcode _driver_usart_read_word_blocking( volatile t_ttc_usart_register* Register, t_u16* Word );

/** Reads single data byte from input buffer (8 bits)
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 * Note: This function blocks until a valid word has been received or timeout occurs
 *
 * @param Register      pointer to low-level register for fast hardware operation
 * @param Byte          pointer to 8 bit buffer where to store Word
 * @return               == 0: Word has been read successfully; != 0: error-code
 */
e_ttc_usart_errorcode _driver_usart_read_byte( volatile t_ttc_usart_register* Register, t_u8* Byte );

/** Send single data word to output buffer (8 or 9 bits)
 *
 * Note: USART must be initialized before!
 * Note: This function is partially thread safe (use it for each USART from one thread only)!
 * Note: This function blocks until word has been sent or timeout occurs
 *
 * @param Register      pointer to low-level register for fast hardware operation
 * @param Word          pointer to 16 bit buffer where to store Word
 * @return              == 0: Word has been read successfully; != 0: error-code
 */
e_ttc_usart_errorcode _driver_usart_send_word_blocking( volatile t_ttc_usart_register* Register, const t_u16 Word );

/** low level send routine especially to be called from _ttc_usart_isr_tx()
 *
 * @param Register       pointer to hardware register for fast access
 * @param Word           data word to send (8 or 9 bits)
 */
//void _driver_usart_send_byte_isr( volatile t_ttc_usart_register_tx* Register, const t_u16 Word );

/** Reinitializes all already initialized USARTs to reflect a changed system clock.
 *
 * Note: This function is normally only called from ttc_sysclock facility when system clock has changed.
 *
 */
void _ttc_usart_update_to_sysclock();

//InsertDriverPrototypes above (DO NOT DELETE THIS LINE!)


//}

#endif //TTC_USART_H
