/** { usart_stm32f1xx.c ************************************************
 *
 *                          The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Low-Level driver for usart devices on stm32f1xx architectures.
 *  Implementation of low-level driver.
 *
 *  Created from template device_architecture.c revision 20 at 20140217 09:36:59 UTC
 *
 *  Note: See ttc_usart.h for description of stm32f1xx independent USART implementation.
 *
 *  Authors: Gregor Rebel
}*/

#include "usart_stm32f1xx.h"
#include "../ttc_assert.h"
#include "../ttc_cpu.h"
#include "../ttc_task.h"
#include "../ttc_interrupt.h"
#include "../ttc_gpio.h"
#include "../ttc_sysclock.h"

// these are defined by ttc_usart.c
extern void _ttc_usart_isr_rx( t_ttc_usart_config* Config, t_u8 Byte );
extern BOOL _ttc_usart_isr_tx( t_ttc_usart_config* Config, volatile void* USART_Hardware );
t_base ttc_usart_Timeout = 0;


volatile t_register_stm32f1xx_usart* _usart_stm32f1xx_get_base_register( t_u8 PhysicalIndex ) {
    switch ( PhysicalIndex ) {              // find corresponding USART as defined by makefile.100_board_*
        case 0: return &register_stm32f1xx_USART1; break;
        case 1: return &register_stm32f1xx_USART2; break;
        case 2: return &register_stm32f1xx_USART3; break;
        case 3: return &register_stm32f1xx_UART4;  break;
        case 4: return &register_stm32f1xx_UART5;  break;
        default: Assert_USART( 0, E_ttc_assert_origin_auto ); break; // No TTC_USARTn defined! Check your makefile.100_board_* file!
    }
    return NULL;
}

//{ Private Function prototypes ******************************************
//
// Convention: Private functions start with an underscore "_".
// Example:    void _usart_stm32f1xx_foo(t_ttc_usart_config* Config)

/** Sends out given byte.
 * @param BaseRegister        Base register of USART device to use
 * @param Word           pointer to 16 bit buffer where to store Word

 *
 * Note: This low-level function should not be called from outside!
 */
e_ttc_usart_errorcode _usart_stm32f1xx_send_word_blocking( volatile t_register_stm32f1xx_usart* BaseRegister, const t_u16 Word );

/** low level send routine especially to be called from _ttc_usart_isr_tx()
 *
 * @param BaseRegister        Base register of USART device to use
 * @param Word           data word to send (8 or 9 bits)
 */
void _usart_stm32f1xx_send_byte_isr( volatile t_register_stm32f1xx_usart* BaseRegister, const t_u16 Word );

/** Wait until word has been received or timeout occured.
 * Note: This low-level function is private and should not be called from outside!
 *
 * @param BaseRegister        Base register of USART device to use

 */
e_ttc_usart_errorcode _usart_stm32f1xx_wait_for_RXNE( volatile t_register_stm32f1xx_usart* BaseRegister );

/** wait until transmit buffer is empty
 * Note: This low-level function is private and should not be called from outside!
 *
 * @param BaseRegister        Base register of USART device to use
 */
e_ttc_usart_errorcode _usart_stm32f1xx_wait_for_TXE( volatile t_register_stm32f1xx_usart* BaseRegister );

/** Enable the selected USART by setting the UE bit in the CR1 register
 *
 * @param BaseRegister        Base register of USART device to use
 */
void _usart_stm32f1xx_enable( volatile t_register_stm32f1xx_usart* BaseRegister, t_bool Enable );

e_ttc_usart_errorcode _usart_stm32f1xx_init( t_ttc_usart_config* Config );

e_ttc_usart_errorcode _usart_stm32f1xx_configure_bitrate( t_ttc_usart_config* Config );

//InsertPrivateFunctionDeclarations above (DO NOT DELETE THIS LINE!)

//}PrivateFunctions

//{ Function definitions *******************************************************

t_u8                  usart_stm32f1xx_check_bytes_available( t_ttc_usart_config* Config ) {
    Assert_USART_Writable( Config, E_ttc_assert_origin_auto ); // pointers must not be NULL

    if ( Config->BaseRegister->SR.Bits.RXNE )
    { return 1; }
    return 0;
}
e_ttc_usart_errorcode usart_stm32f1xx_deinit( t_ttc_usart_config* Config ) {
    Assert_USART_Writable( Config, E_ttc_assert_origin_auto ); // pointers must not be NULL


    return ( e_ttc_usart_errorcode ) 0;
}
t_ttc_usart_config*   usart_stm32f1xx_get_configuration( t_ttc_usart_config* Config ) {
    Assert_USART_Writable( Config, E_ttc_assert_origin_auto ); // pointers must not be NULL


    return ( t_ttc_usart_config* ) 0;
}
e_ttc_usart_errorcode usart_stm32f1xx_get_features( t_ttc_usart_config* Config ) {
    // assumption: *Config has been zeroed

    t_u8 PhysicalIndex = Config->PhysicalIndex;
    Assert_USART( Config != NULL, E_ttc_assert_origin_auto );
    if ( PhysicalIndex >= TTC_USART_MAX_AMOUNT ) { return ec_usart_DeviceNotFound; }
    Config->BaseRegister = _usart_stm32f1xx_get_base_register( PhysicalIndex );

    ttc_memory_set( &( Config->Init.Flags ), 0, sizeof( Config->Init.Flags ) );
    Config->Architecture = ta_usart_stm32f1xx;

    Config->Init.Flags.Receive          = 1;
    Config->Init.Flags.Transmit         = 1;
    Config->Init.Flags.DelayedTransmits = 1; // ToDo: Fix Assert()-issue when activating this bit
    Config->Init.Flags.SendBreaks       = 1;
    Config->Init.Flags.ParityEven       = 1;
    Config->Init.Flags.ParityOdd        = 1;
    Config->Init.Flags.IrqOnRxNE        = 1;

    if ( 0 ) { // remaining bits
        Config->Init.Flags.RtsCts           = 0; // (ToDo: implement)
        Config->Init.Flags.SingleWire       = 0; // (ToDo: implement)
        Config->Init.Flags.SmartCard        = 0; // (ToDo: implement)
        Config->Init.Flags.Synchronous      = 0; // (ToDo: implement)
        Config->Init.Flags.LIN              = 0; // (ToDo: implement)
        Config->Init.Flags.IrDA             = 0; // (ToDo: implement)
        Config->Init.Flags.IrDALowPower     = 0; // (ToDo: implement)
        Config->Init.Flags.SmartCardNACK    = 0; // (ToDo: implement)
        Config->Init.Flags.RxDMA            = 0; // (ToDo: implement)
        Config->Init.Flags.TxDMA            = 0; // (ToDo: implement)
        Config->Init.Flags.Rts              = 0; // (ToDo: implement)
        Config->Init.Flags.IrqOnCts         = 0; // (ToDo: implement)
        Config->Init.Flags.IrqOnError       = 0; // (ToDo: implement)
        Config->Init.Flags.IrqOnIdle        = 0; // (ToDo: implement)
        Config->Init.Flags.IrqOnTxComplete  = 0; // (ToDo: implement)
        Config->Init.Flags.IrqOnParityError = 0; // (ToDo: implement)
    }

    Config->Init.WordLength      = 9;
    Config->Init.HalfStopBits    = 4;
    Config->Init.BitRate        = 115200;

    switch ( PhysicalIndex ) { // find amount of available remapping-layouts (-> RM0008 p. 176)
        case 0:
            Config->Layout = 1;   // USART1 and USART2 provide 1 alternate pin layout
            break;
        case 1:
            if ( ttc_cpu_get_configuration( 1 )->Info->Amount_Pins >= 100 )
            { Config->Layout = 1; } // USART1 and USART2 provide 1 alternate pin layout
            else
            { Config->Layout = 0; } // USART1 and USART2 provide 0 alternate pin layout
            break;
        case 2:
            if ( ttc_cpu_get_configuration( 1 )->Info->Amount_Pins >= 100 )
            { Config->Layout = 2; } // USART3 provides 2 alternate pin layout
            else if ( ttc_cpu_get_configuration( 1 )->Info->Amount_Pins >= 64 )
            { Config->Layout = 1; } // USART3 provides 1 alternate pin layout
            else
            { Config->Layout = 0; } // USART3 provides 0 alternate pin layout
            break;
        case 3:
        case 4:
            // USART4 and USART5 lack some features (-> ST RM0038 Table 138 on p. 651)
            Config->Init.Flags.Rts            = 0;
            Config->Init.Flags.RtsCts         = 0;
            Config->Init.Flags.SmartCard      = 0;
            Config->Init.Flags.SmartCardNACK  = 0;
            Config->Init.Flags.Synchronous    = 0;
            Config->Init.Flags.IrqOnCts       = 0;
            Config->Layout                    = 0; // no alternate pin layout

            break;
        default: ttc_assert_halt( ); break; // No TTC_USARTn defined! Check your makefile.100_board_* file!
    }

    return ec_usart_OK;
}
t_u8                  usart_stm32f1xx_get_physicalindex( t_ttc_usart_config* Config ) {
    Assert_USART( Config != NULL, E_ttc_assert_origin_auto );

    return Config->PhysicalIndex;
}
e_ttc_usart_errorcode usart_stm32f1xx_init( t_ttc_usart_config* Config ) {
    // assumption: *Config.USART_Arch has been zeroed

    t_u8 LogicalIndex = Config->LogicalIndex;
    Assert_USART( LogicalIndex > 0, E_ttc_assert_origin_auto ); // logical index starts at 1
    Assert_USART( Config != NULL, E_ttc_assert_origin_auto );
    if ( LogicalIndex > TTC_USART_AMOUNT )
    { return ec_usart_DeviceNotFound; }
    if ( ( Config->Init.WordLength   < 8 ) || ( Config->Init.WordLength   > 9 ) )
    { return ec_usart_InvalidWordSize; }
    if ( ( Config->Init.HalfStopBits < 1 ) || ( Config->Init.HalfStopBits > 4 ) )
    { return ec_usart_InvalidStopBits; }

    ttc_usart_Timeout = Config->Timeout;
    if ( 0 ) {                                 // validate USART_Features (moved into ttc_usart_init() )
        t_ttc_usart_config USART_Features;
        ttc_memory_set( &USART_Features, 0, sizeof( USART_Features ) );
        USART_Features.LogicalIndex  = LogicalIndex;
        USART_Features.PhysicalIndex = Config->PhysicalIndex;
        e_ttc_usart_errorcode Error = usart_stm32f1xx_get_features( &USART_Features );
        if ( Error ) { return Error; }

        //X Config->Init.Flags.All &= USART_Features.Flags.All; // mask out unavailable flags
        Assert_USART_EXTRA( sizeof( Config->Init.Flags ) == 4, E_ttc_assert_origin_auto ); // size of bitfield must exactly match typecast below. Change code to fit!

        // mask out unavailable flags
        *( ( t_u32* ) & ( Config->Init.Flags ) ) &= *( ( t_u32* ) & ( USART_Features.Init.Flags ) );
        TODO( "Above Required? Movable to ttc_usart_init?" )

        if ( Config->Layout > USART_Features.Layout )
        { Config->Layout = USART_Features.Layout; }
    }
    switch ( Config->PhysicalIndex ) { // enable + reset device
        case 0: {
            register_stm32f1xx_RCC.APB2ENR.USART1_EN  = 1; // power on USART device
            register_stm32f1xx_RCC.APB2RSTR.USART1RST = 1; // reset USART device
            register_stm32f1xx_RCC.APB2RSTR.USART1RST = 0; // reset USART device
            //X register_stm32f1xx_USART1.CR1.Bits.UE     = 1; // enable unit
            break;
        }
        case 1: {
            register_stm32f1xx_RCC.APB1ENR.USART2_EN  = 1; // power on USART device
            register_stm32f1xx_RCC.APB1RSTR.USART2RST = 1; // reset USART device
            register_stm32f1xx_RCC.APB1RSTR.USART2RST = 0; // reset USART device
            //X register_stm32f1xx_USART2.CR1.Bits.UE     = 1; // enable unit
            break;
        }
        case 2: {
            register_stm32f1xx_RCC.APB1ENR.USART3_EN  = 1; // power on USART device
            register_stm32f1xx_RCC.APB1RSTR.USART3RST = 1; // reset USART device
            register_stm32f1xx_RCC.APB1RSTR.USART3RST = 0; // reset USART device
            //X register_stm32f1xx_USART3.CR1.Bits.UE     = 1; // enable unit
            break;
        }
        case 3: {
            register_stm32f1xx_RCC.APB1ENR.UART4_EN   = 1; // power on USART device
            register_stm32f1xx_RCC.APB1RSTR.UART4RST  = 1; // reset USART device
            register_stm32f1xx_RCC.APB1RSTR.UART4RST  = 0; // reset USART device
            //X register_stm32f1xx_UART4.CR1.Bits.UE      = 1; // enable unit
            break;
        }
        case 4: {
            register_stm32f1xx_RCC.APB1ENR.UART5_EN   = 1; // power on USART device
            register_stm32f1xx_RCC.APB1RSTR.UART5RST  = 1; // reset USART device
            register_stm32f1xx_RCC.APB1RSTR.UART5RST  = 0; // reset USART device
            //X register_stm32f1xx_UART5.CR1.Bits.UE      = 1; // enable unit
            break;
        }
        default: ttc_assert_halt_origin( E_ttc_assert_origin_auto ); break; // unknown physical index. Check implementation of caller!
    }

    Config->Layout = 0xff; // start with invalid value
    switch ( Config->PhysicalIndex ) { // determine pin layout (->RM0008 p.176)
        case 0:  { // USART1
            if ( Config->PortTxD ) { // using TX pin to identify remap layout
                if ( Config->PortTxD == E_ttc_gpio_pin_b6 ) { Config->Layout = 1; }
                else                                        { Config->Layout = 0; }
            }
            else {                   // using RX pin to identify remap layout
                Assert_USART( Config->PortRxD != 0, E_ttc_assert_origin_auto ); // either PortTX or PortRX must be configured. Check implementation of caller or static configuration in your makefile!
                if ( Config->PortRxD == E_ttc_gpio_pin_b7 ) { Config->Layout = 1; }
                else                                        { Config->Layout = 0; }
            }

            if ( Config->Layout > 0 ) { // test alternate pin remapping (-> ST RM0041 STM32F100xx advanced ARM-based 32-bit MCUs p.118)
                Assert_USART( ( !Config->PortRxD ) || ( Config->PortRxD == E_ttc_gpio_pin_b7 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                Assert_USART( ( !Config->PortTxD ) || ( Config->PortTxD == E_ttc_gpio_pin_b6 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
            }
            else {
                Assert_USART( ( !Config->PortRxD ) || ( Config->PortRxD == E_ttc_gpio_pin_a10 ), E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                Assert_USART( ( !Config->PortTxD ) || ( Config->PortTxD == E_ttc_gpio_pin_a9 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
            }
            // test unmappable pins
            Assert_USART( ( !Config->PortCTS ) || ( Config->PortCTS == E_ttc_gpio_pin_a11 ), E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
            Assert_USART( ( !Config->PortRTS ) || ( Config->PortRTS == E_ttc_gpio_pin_a12 ), E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
            Assert_USART( ( !Config->PortCLK ) || ( Config->PortCLK == E_ttc_gpio_pin_a8 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
            break;
        }
        case 1:  { // USART2
            if ( Config->PortTxD ) { // using TX pin to identify remap layout
                if ( Config->PortTxD == E_ttc_gpio_pin_d5 ) { Config->Layout = 1; }
                else                                        { Config->Layout = 0; }
            }
            else {                   // using RX pin to identify remap layout
                Assert_USART( Config->PortRxD != 0, E_ttc_assert_origin_auto ); // either PortTX or PortRX must be configured. Check implementation of caller or static configuration in your makefile!
                if ( Config->PortRxD == E_ttc_gpio_pin_d6 ) { Config->Layout = 1; }
                else                                        { Config->Layout = 0; }
            }

            if ( Config->Layout > 0 ) { // test alternate pin remapping (-> ST RM0041 STM32F100xx advanced ARM-based 32-bit MCUs p.118)
                Assert_USART( ( !Config->PortTxD ) || ( Config->PortTxD == E_ttc_gpio_pin_d5 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                Assert_USART( ( !Config->PortRxD ) || ( Config->PortRxD == E_ttc_gpio_pin_d6 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                Assert_USART( ( !Config->PortCTS ) || ( Config->PortCTS == E_ttc_gpio_pin_d3 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                Assert_USART( ( !Config->PortRTS ) || ( Config->PortRTS == E_ttc_gpio_pin_d4 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                Assert_USART( ( !Config->PortCLK ) || ( Config->PortCLK == E_ttc_gpio_pin_d7 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
            }
            else {
                Assert_USART( ( !Config->PortTxD ) || ( Config->PortTxD == E_ttc_gpio_pin_a2 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                Assert_USART( ( !Config->PortRxD ) || ( Config->PortRxD == E_ttc_gpio_pin_a3 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                Assert_USART( ( !Config->PortCTS ) || ( Config->PortCTS == E_ttc_gpio_pin_a0 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                Assert_USART( ( !Config->PortRTS ) || ( Config->PortRTS == E_ttc_gpio_pin_a1 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                Assert_USART( ( !Config->PortCLK ) || ( Config->PortCLK == E_ttc_gpio_pin_a4 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
            }
            break;
        }
        case 2:  { // USART3
            if ( Config->PortTxD ) {    // using TX pin to identify remap layout (-> ST RM0041 STM32F100xx advanced ARM-based 32-bit MCUs p.118)
                switch ( Config->PortTxD ) {
                    case E_ttc_gpio_pin_b10: Config->Layout = 0; break; // no remap
                    case E_ttc_gpio_pin_c10: Config->Layout = 1; break; // partial remap
                    case E_ttc_gpio_pin_d8:  Config->Layout = 2; break; // full remap
                    default: break;
                }
            }
            else {                      // using RX pin to identify remap layout
                Assert_USART( Config->PortRxD != 0, E_ttc_assert_origin_auto ); // either PortTX or PortRX must be configured. Check implementation of caller or static configuration in your makefile!
                switch ( Config->PortRxD ) {
                    case E_ttc_gpio_pin_b11: Config->Layout = 0; break; // no remap
                    case E_ttc_gpio_pin_c11: Config->Layout = 1; break; // partial remap
                    case E_ttc_gpio_pin_d9:  Config->Layout = 2; break; // full remap
                    default: break;
                }
            }
            if ( TTC_TUNABLE_SELFCHECK_ASSERT_USART == 1 ) { // test alternate pin remapping (-> ST RM0041 STM32F100xx advanced ARM-based 32-bit MCUs p.118)
                switch ( Config->Layout ) {
                    case 0: { // no pin remapping
                        Assert_USART( ( !Config->PortTxD ) || ( Config->PortTxD == E_ttc_gpio_pin_b10 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                        Assert_USART( ( !Config->PortRxD ) || ( Config->PortRxD == E_ttc_gpio_pin_b11 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                        Assert_USART( ( !Config->PortCTS ) || ( Config->PortCTS == E_ttc_gpio_pin_b13 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                        Assert_USART( ( !Config->PortRTS ) || ( Config->PortRTS == E_ttc_gpio_pin_b14 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                        Assert_USART( ( !Config->PortCLK ) || ( Config->PortCLK == E_ttc_gpio_pin_b12 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                        break;
                    }
                    case 1: { // partial pin remapping
                        Assert_USART( ( !Config->PortTxD ) || ( Config->PortTxD == E_ttc_gpio_pin_c10 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                        Assert_USART( ( !Config->PortRxD ) || ( Config->PortRxD == E_ttc_gpio_pin_c11 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                        Assert_USART( ( !Config->PortCTS ) || ( Config->PortCTS == E_ttc_gpio_pin_b13 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                        Assert_USART( ( !Config->PortRTS ) || ( Config->PortRTS == E_ttc_gpio_pin_b14 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                        Assert_USART( ( !Config->PortCLK ) || ( Config->PortCLK == E_ttc_gpio_pin_c12 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                        break;
                    }
                    case 2: { // full pin remapping
                        Assert_USART( ( !Config->PortTxD ) || ( Config->PortTxD == E_ttc_gpio_pin_d8 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                        Assert_USART( ( !Config->PortRxD ) || ( Config->PortRxD == E_ttc_gpio_pin_d9 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                        Assert_USART( ( !Config->PortCTS ) || ( Config->PortCTS == E_ttc_gpio_pin_d11 ), E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                        Assert_USART( ( !Config->PortRTS ) || ( Config->PortRTS == E_ttc_gpio_pin_d12 ), E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                        Assert_USART( ( !Config->PortCLK ) || ( Config->PortCLK == E_ttc_gpio_pin_d10 ), E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
                        break;
                    }
                    default: ttc_assert_halt_origin( E_ttc_assert_origin_auto ); break; // invalid/unsupported USART alternate function remapping
                }
            }
            break;
        }
        case 3:  { // UART4
            Assert_USART( ( !Config->PortTxD ) || ( Config->PortTxD == E_ttc_gpio_pin_c10 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
            Assert_USART( ( !Config->PortRxD ) || ( Config->PortRxD == E_ttc_gpio_pin_c11 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
            break;
        }
        case 4:  { // UART5
            Assert_USART( ( !Config->PortTxD ) || ( Config->PortTxD == E_ttc_gpio_pin_c12 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
            Assert_USART( ( !Config->PortRxD ) || ( Config->PortRxD == E_ttc_gpio_pin_d2 ),  E_ttc_assert_origin_auto ); // invalid pin configuration. Check implementation of caller or static configuration in your makefile!
            break;
        }
        default: { ttc_assert_halt( ); } // invalid/unsupported physical uart index (stm32f1xx has only five uarts). Check TTC_USARTx defines in current board makefile!
    }

    if ( 1 ) {                         // configure GPIOs
        if ( Config->PortTxD )  { ttc_gpio_init( Config->PortTxD, E_ttc_gpio_mode_alternate_function_push_pull, E_ttc_gpio_speed_max ); }
        if ( Config->PortRxD )  { ttc_gpio_init( Config->PortRxD, E_ttc_gpio_mode_input_floating,               E_ttc_gpio_speed_max ); }
        if ( Config->PortRTS )  { ttc_gpio_init( Config->PortRTS, E_ttc_gpio_mode_alternate_function_push_pull, E_ttc_gpio_speed_max ); }
        if ( Config->PortCLK )  { ttc_gpio_init( Config->PortCLK, E_ttc_gpio_mode_alternate_function_push_pull, E_ttc_gpio_speed_max ); }
        if ( Config->PortCTS )  { ttc_gpio_init( Config->PortCTS, E_ttc_gpio_mode_input_floating,               E_ttc_gpio_speed_max ); }
    }

    // enable AFIO clock to allow USART port pin remapping (MAPR register will always read zero if AFIO clock is disabled)
    if ( 1 ) { // enable AFIO clock using struct register access
        register_stm32f1xx_RCC.APB2ENR.AFIO_EN = 1;
    }
    else {     // enable AFIO clock using 32 bit register access
        volatile t_register_stm32f1xx_rcc_apb2_enr APB2ENR;
        *( ( volatile t_u32* )( &APB2ENR ) ) = * ( ( volatile t_u32* )( &register_stm32f1xx_RCC.APB2ENR ) );
        APB2ENR.AFIO_EN = 1;
    }

    t_afio_mapr MAPR;
    // force 32 bit read from register into local variable
    *( ( volatile t_u32* )( &MAPR ) ) = * ( ( volatile t_u32* )( &register_stm32f1xx_AFIO.MAPR ) );

    switch ( Config->PhysicalIndex ) { // configure GPIO port remapping

        case 0: { // USART1
            MAPR.USART1_REMAP = Config->Layout;
            break;
        }
        case 1: { // USART2
            MAPR.USART2_REMAP = Config->Layout;
            break;
        }
        case 2: { // USART3
            switch ( Config->Layout ) { // (-> ST RM0041 STM32F100xx advanced ARM-based 32-bit MCUs p.118)
                case 0: MAPR.USART3_REMAP = 0b00; break; // no remap
                case 1: MAPR.USART3_REMAP = 0b01; break; // partial remap
                case 2: MAPR.USART3_REMAP = 0b11; break; // full remap
                default:  Assert_USART_EXTRA( 0, E_ttc_assert_origin_auto ); // illegal configuration. Check implementation above!
                    break;
            }
            break;
        }

        default: { break; }
    }

    // force 32 bit write to register into local variable
    * ( ( volatile t_u32* )( &register_stm32f1xx_AFIO.MAPR ) ) = *( ( volatile t_u32* )( &MAPR ) );
    Assert_USART( * ( ( volatile t_u32* )( &register_stm32f1xx_AFIO.MAPR ) ) == *( ( volatile t_u32* )( &MAPR ) ), E_ttc_assert_origin_auto ); // could not write to MAPR register. Is the clock enabled?

    _usart_stm32f1xx_configure_bitrate( Config );
    if ( 1 ) {                        // configure rest of USART features
        t_register_stm32f1xx_usart_cr1 volatile CR1; CR1.All = 0;
        t_register_stm32f1xx_usart_cr2 volatile CR2; CR2.All = 0;
        t_register_stm32f1xx_usart_cr3 volatile CR3; CR3.All = 0;

        CR1.Bits.TE     = Config->Init.Flags.Transmit;
        CR1.Bits.RE     = Config->Init.Flags.Receive;
        CR1.Bits.IDLEIE = Config->Init.Flags.IrqOnIdle;
        CR1.Bits.RXNEIE = Config->Init.Flags.IrqOnRxNE;
        CR1.Bits.TCIE   = Config->Init.Flags.IrqOnTxComplete;
        CR1.Bits.PEIE   = Config->Init.Flags.IrqOnParityError;
        CR1.Bits.PCE    = Config->Init.Flags.ParityEven | Config->Init.Flags.ParityOdd;
        if ( CR1.Bits.PCE ) { // parity enabled:  can support word length of 7 or 8 bits (msb is parity)
            Assert_USART( Config->Init.Flags.ParityEven | Config->Init.Flags.ParityOdd, E_ttc_assert_origin_auto ); // only one of even or odd parity may be enabled!
            Config->Init.WordLength = ttc_basic_limit( Config->Init.WordLength, 7, 8 );
            CR1.Bits.M = ( Config->Init.WordLength == 8 ) ? 1 : 0;
        }
        else {                // parity disabled: can support word length of 8 or 9 bits
            Config->Init.WordLength = ttc_basic_limit( Config->Init.WordLength, 8, 9 );
            CR1.Bits.M = ( Config->Init.WordLength == 9 ) ? 1 : 0;
        }

        CR2.Bits.ADD    = Config->Init.LIN_Address;
        CR2.Bits.LBDL   = Config->Init.Flags.LIN_Break11Bits;
        CR2.Bits.LBDIE  = Config->Init.Flags.LIN_IrqOnBreak;
        CR2.Bits.LBCL   = 0; // not used
        CR2.Bits.CPHA   = 1 - Config->Init.Flags.ClockOnFirst;
        CR2.Bits.CPOL   = 1 - Config->Init.Flags.ClockIdleLow;
        CR2.Bits.CLKEN  = Config->Init.Flags.Synchronous;
        CR2.Bits.LINEN  = Config->Init.Flags.LIN;

        switch ( Config->Init.HalfStopBits ) { // -> RM0008 p.795
            case 1: CR2.Bits.STOP = 0b01; break; // 0.5 stop bits
            case 2: CR2.Bits.STOP = 0b00; break; // 1   stop bit
            case 3: CR2.Bits.STOP = 0b11; break; // 1.5 stop bits
            case 4: CR2.Bits.STOP = 0b10; break; // 2   stop bits
            default: Assert_USART( 0, E_ttc_assert_origin_auto ); // invalid configuration
        }

        CR3.Bits.EIE   = Config->Init.Flags.IrqOnError;
        CR3.Bits.IREN  = Config->Init.Flags.IrDA;
        CR3.Bits.IRLP  = Config->Init.Flags.IrDALowPower;
        CR3.Bits.HDSEL = Config->Init.Flags.HalfDuplex;
        CR3.Bits.NACK  = Config->Init.Flags.SmartCardNACK;
        CR3.Bits.SC_EN = Config->Init.Flags.SmartCard;
        CR3.Bits.DMAR  = Config->Init.Flags.RxDMA;
        CR3.Bits.DMAT  = Config->Init.Flags.TxDMA;
        CR3.Bits.RTSE  = Config->Init.Flags.Rts | Config->Init.Flags.RtsCts;
        CR3.Bits.CTSE  = Config->Init.Flags.RtsCts;
        CR3.Bits.CTSIE = Config->Init.Flags.IrqOnCts;

        // update USART configuration registers
        Config->BaseRegister->CR1.All = CR1.All;
        Config->BaseRegister->CR2.All = CR2.All;
        Config->BaseRegister->CR3.All = CR3.All;
    }

    _usart_stm32f1xx_enable( Config->BaseRegister, TRUE );

    return ec_usart_OK;
}
e_ttc_usart_errorcode usart_stm32f1xx_load_defaults( t_ttc_usart_config* Config ) {
    //X     if (!su_Initialized) usart_stm32f1xx_reset_all();
    // assumption: *Config has been zeroed

    Assert_USART_Writable( Config, E_ttc_assert_origin_auto );
    Assert_USART( Config->PhysicalIndex < TTC_CPU_AMOUNT_USART, E_ttc_assert_origin_auto );
    Config->BaseRegister =  _usart_stm32f1xx_get_base_register( Config->PhysicalIndex );
    Config->USART_Arch.BitBand_SR_RxNE = cm3_calc_peripheral_BitBandAddress( &( Config->BaseRegister->SR ), 5 ); // SR.Bits.RXNE
    Assert_USART_Writable( Config->USART_Arch.BitBand_SR_RxNE, E_ttc_assert_origin_auto ); // invalid address. Check implementation above!

    ttc_memory_set( &( Config->Init.Flags ), 0, sizeof( Config->Init.Flags ) );
    Config->Init.Flags.SendBreaks     = 1;
    Config->Init.Flags.Receive        = 1;
    Config->Init.Flags.Transmit       = 1;

    if ( 0 ) { // remaining bits are already reset
        Config->Init.Flags.LIN              = 0;
        Config->Init.Flags.IrDA             = 0;
        Config->Init.Flags.IrDALowPower     = 0;
        Config->Init.Flags.SmartCardNACK    = 0;
        Config->Init.Flags.RxDMA            = 0;
        Config->Init.Flags.TxDMA            = 0;
        Config->Init.Flags.Rts              = 0;
        Config->Init.Flags.IrqOnCts         = 0;
        Config->Init.Flags.IrqOnError       = 0;
        Config->Init.Flags.IrqOnIdle        = 0;
        Config->Init.Flags.IrqOnTxComplete  = 0;
        Config->Init.Flags.IrqOnParityError = 0;
        Config->Init.Flags.ParityEven       = 0;
        Config->Init.Flags.ParityOdd        = 0;
        Config->Init.Flags.RtsCts           = 0;
        Config->Init.Flags.SingleWire       = 0;
        Config->Init.Flags.SmartCard        = 0;
        Config->Init.Flags.Synchronous      = 0;
    }

    Config->Init.WordLength    = 8;
    Config->Init.HalfStopBits  = 2;
    Config->Layout             = 0;
    Config->Init.BitRate       = 115200;

    return ec_usart_OK;
}
void                  usart_stm32f1xx_prepare() {




}
e_ttc_usart_errorcode usart_stm32f1xx_read_byte_blocking( volatile t_register_stm32f1xx_usart* Register, t_u8* Byte ) {

    if ( _usart_stm32f1xx_wait_for_RXNE( Register ) )
    { return ec_usart_TimeOut; }

    *Byte = ( t_u8 ) 0xff & ( ( USART_TypeDef* ) Register )->DR;

    return ec_usart_OK;
}
e_ttc_usart_errorcode usart_stm32f1xx_read_word_blocking( volatile t_register_stm32f1xx_usart* Register, t_u16* Word ) {

    if ( _usart_stm32f1xx_wait_for_RXNE( Register ) )
    { return ec_usart_TimeOut; }

    *Word = 0x1ff & ( ( USART_TypeDef* ) Register )->DR;

    return ec_usart_OK;
}
e_ttc_usart_errorcode usart_stm32f1xx_reset( t_ttc_usart_config* Config ) {
    Assert_USART_Writable( Config, E_ttc_assert_origin_auto ); // pointers must not be NULL

    return usart_stm32f1xx_load_defaults( Config );
}
e_ttc_usart_errorcode usart_stm32f1xx_read_byte( volatile t_ttc_usart_register_rx* Register, t_u8* Byte ) {
    Assert_USART_Writable( ( void* ) Register, E_ttc_assert_origin_auto ); // pointers must not be NULL
    Assert_USART_Writable( Byte, E_ttc_assert_origin_auto ); // pointers must not be NULL

    *Byte = ( t_u8 ) 0xff & ( ( USART_TypeDef* ) Register )->DR;

    return ( e_ttc_usart_errorcode ) 0;
}
e_ttc_usart_errorcode usart_stm32f1xx_send_word_blocking( volatile t_ttc_usart_register_tx* Register, const t_u16 Word ) {
    Assert_USART( Register != NULL, E_ttc_assert_origin_auto );

    return _usart_stm32f1xx_send_word_blocking( Register, Word );
}
void                  usart_stm32f1xx_rx_isr( t_physical_index PhysicalIndex, void* Argument ) {
    Assert_USART( PhysicalIndex < TTC_CPU_AMOUNT_USART, E_ttc_assert_origin_auto ); // valid range: 0 (first physical device unit) .. TTC_CPU_AMOUNT_USART
    t_ttc_usart_config* Config = ( t_ttc_usart_config* ) Argument;
    volatile t_register_stm32f1xx_usart* USART_Base = Config->BaseRegister;

    // reading SR and then DR will clear RxNE flag
    volatile t_u16 Dummy = *( ( t_u16* ) & ( USART_Base->SR ) ); ( void ) Dummy;
    volatile t_u16 Byte  = *( ( t_u16* ) & ( USART_Base->DR ) );

    _ttc_usart_isr_rx( ( t_ttc_usart_config* ) Argument, ( t_u8 )( 0xff & Byte ) ); // provide received byte to registered function
}
void                  usart_stm32f1xx_tx_isr( t_physical_index PhysicalIndex, void* Argument ) {
    Assert_USART( PhysicalIndex < TTC_CPU_AMOUNT_USART, E_ttc_assert_origin_auto ); // valid range: 0 (first physical device unit) .. TTC_CPU_AMOUNT_USART
    Assert_USART( ttc_memory_is_writable( Argument ), E_ttc_assert_origin_auto );

    t_ttc_usart_config* Config = ( t_ttc_usart_config* ) Argument;
    volatile t_register_stm32f1xx_usart* USART_Base = Config->BaseRegister;

    t_bool StayInISR;
    do {
        StayInISR = _ttc_usart_isr_tx( Config, & ( USART_Base->DR ) ); // let high level driver choose next byte to send
        USART_Base->SR.Bits.TXE = 0;
    }
    while ( StayInISR && USART_Base->SR.Bits.TXE ); // stay in isr for high data rates
}
//InsertFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//}FunctionDefinitions
//{ private functions (ideally) ************************************************

e_ttc_usart_errorcode _usart_stm32f1xx_send_word_blocking( volatile t_register_stm32f1xx_usart* BaseRegister, const t_u16 Word ) {
    Assert_USART( BaseRegister != NULL, E_ttc_assert_origin_auto );

    e_ttc_usart_errorcode  Error = _usart_stm32f1xx_wait_for_TXE( BaseRegister );
    if ( Error != ec_usart_OK )
    { return Error; }

    usart_stm32f1xx_send_byte_isr( &( BaseRegister->DR ), Word );
    return ec_usart_OK;
}
e_ttc_usart_errorcode _usart_stm32f1xx_wait_for_RXNE( volatile t_register_stm32f1xx_usart* BaseRegister ) {

    if ( ttc_usart_Timeout != ( t_base ) - 1 ) { // wait until word received or timeout
        t_base TimeOut = ttc_usart_Timeout;
        while ( BaseRegister->SR.Bits.RXNE == 0 ) {
            if ( TimeOut-- == 0 )  { return ec_usart_TimeOut; }
            ttc_task_yield();
        }
    }
    else {             // wait until word received (no timeout)
        while ( BaseRegister->SR.Bits.RXNE == 0 )
        { ttc_task_yield(); }
    }

    return ec_usart_OK;
}
e_ttc_usart_errorcode _usart_stm32f1xx_wait_for_TXE( volatile t_register_stm32f1xx_usart* BaseRegister ) {

    while ( ( BaseRegister->SR.All & USART_FLAG_TXE ) == 0 ) {
        ttc_task_yield();
    }

    return ec_usart_OK;
}
void                  _usart_stm32f1xx_enable( volatile t_register_stm32f1xx_usart* BaseRegister, t_bool Enable ) {

    /* Enable the selected USART by setting the UE bit in the CR1 register using a 32 bit register access.
     *
     * Compiler may optimize struct access into 8 or 16 bit wide memory accesses.
     * STM32F1xx does only support 32 bit wide memory accesses to its peripherals.
     * This is why we load all CR1 bits into local variable, modify it and write it back as 32 bits.
     * The compiler will optimize this into something very similar to
     * BaseRegister->CR1.All |= MASK
     * The ToolChain prefers use of structures over constants. This is because structures can be checked by compiler, while constants cannot.
     */
    t_register_stm32f1xx_usart_cr1  CR1;
    CR1.All = BaseRegister->CR1.All;
    CR1.Bits.UE = ( Enable ) ? 1 : 0;
    BaseRegister->CR1.All = CR1.All;
    Assert_USART( BaseRegister->CR1.Bits.UE == 1, E_ttc_assert_origin_auto ); // cannot switch on usart device. Check low-level driver!
}
e_ttc_usart_errorcode _usart_stm32f1xx_configure_bitrate( t_ttc_usart_config* Config ) {
    Assert_USART_Writable( Config, E_ttc_assert_origin_auto ); // pointers must not be NULL


    t_ttc_sysclock_architecture* volatile Frequencies = &( ttc_sysclock_get_configuration()->LowLevelConfig );

    volatile t_base ClockAPB = ( Config->PhysicalIndex == 0 ) ?
                               Frequencies->frequency_apb2 :  // USART1 is clocked from APB2
                               Frequencies->frequency_apb1;   // all other USARTs are clocked from APB1

    /* step 1: calculate integer part of USARTDIV (->RM0008 p.770)
     *
     * Instead of calculating UsartDiv, we calculate UsartDiv*16 which is an integer number to avoid loosing precision.
     * DIV_Mantissa is then calculated as predecimal of (UsartDiv*16) /16.
     *
     *                      BitRate = ClockAPB / (16*USARTDIV)
     * <=>                 USARTDIV = ClockAPB / (16*BitRate)   | * 16
     * <=>            16 * USARTDIV = ClockAPB / BitRate        | USARTDIV_x16 = USARTDIV * 16
     * <=>             USARTDIV_x16 = ClockAPB / BitRate        | USARTDIV_x16 = 16 * Mantissa + Fraction
     * <=> 16 * Mantissa + Fraction = USARTDIV_x16
     *  => Mantissa = int( USARTDIV_x16 / 16 )
     *  => Fraction = USARTDIV_x16 - 16 * Mantissa
     */
    volatile_USART t_u32 USARTDIV_x16 = ClockAPB / Config->Init.BitRate;
    volatile_USART t_register_stm32f1xx_usart_brr  BRR; BRR.All = 0;
    BRR.Bits.DIV_Mantissa = ( ( 1 << 12 ) - 1 ) & ( USARTDIV_x16 / 16 );
    BRR.Bits.DIV_Fraction = ( ( 1 << 4 )  - 1 ) & ( USARTDIV_x16 - 16 * BRR.Bits.DIV_Mantissa );

    /* DEPRECATED
        volatile_USART t_u32 USARTDIV_x16 = ClockAPB / Config->Init.BitRate;
        volatile_USART t_register_stm32f1xx_usart_brr  BRR; BRR.All = 0;
        BRR.Bits.DIV_Mantissa = USARTDIV_x16 / 16;
        BRR.Bits.DIV_Fraction = USARTDIV_x16 - ( BRR.Bits.DIV_Mantissa * 16 );
    */

    // update bitrate register
    Config->BaseRegister->BRR.All = BRR.All;

    // calculate actual bitrate (RM0008 p. 761)
    volatile_USART t_u32 USARTDIV_x16b = ( ( ( t_u32 ) BRR.Bits.DIV_Mantissa ) * 16 + BRR.Bits.DIV_Fraction );
    Assert_USART( USARTDIV_x16b != 0, E_ttc_assert_origin_auto ); // Attempting to divide by zero. Check above implementation!
    Config->ActualBitRate = ClockAPB / USARTDIV_x16b;

    return ec_usart_OK;
}
//InsertPrivateFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//} private functions
