/*{ example_leds.c ************************************************

  This example shows how to generate an LED heartbeat on any gpio pin by using one task for each pin
  and  one central control task.
  The provided implementation is pretty ineffective in terms of cpu time overhead. But it shows how
  to spawn tasks and communicate using shared variables.

  This example requires FreeRTOS multitasking scheduler to run.

  Its tasks do call ttc_task_msleep() and ttc_task_usleep() which make it unusable in
  single tasking mode.
  This example should work on every board with at least one defined LED (TTC_LIGHT1).
  If no TTC_LIGHT1 is defined, simply add one to this file like this:
  #define TTC_LIGHT1 e_ttc_gpio_pin_a1

  Overview of implemented functions in this example:

  example_leds_prepare()
      1. Allocates and initializes one or more LED configurations as a single linked list.
                  +------------------+    +------------------+
         LEDs --> | t_config_led LED1 |    | t_config_led LED1 |
                  +------------------+    +------------------+
                  | Next ---------------> | Next ---------------> NULL
                  | Brightness       |    | Brightness       |
                  | MaxDuty          |    | MaxDuty          |
                  | PortPin          |    | PortPin          |
                  | LowActive        |    | LowActive        |
                  +------------------+    +------------------+

      2. Spawns thread taskLightShow(LEDs).


  taskLightShow(t_config_led* LEDs)
      foreach t_config_led* CurrentLED in single linked list LEDs {
        1. Initialize gpio port connected to LED
        2. Spawn taskLedPWM(CurrentLED)
      }
      while (1) {
        foreach t_config_led* CurrentLED in single linked list LEDs {
          while (CurrentLED->Ton < 100) { CurrentLED->Ton++; CurrentLED->Toff--; }
          while (CurrentLED->Ton >   0) { CurrentLED->Ton--; CurrentLED->Toff++; }
        }
      }


  taskLedPWM(t_config_led* LED)
      while (1) {
        switch on LED
        wait for LED->Ton
        switch off LED
        wait for LED->Toff
      }

}*/

#include "example_leds.h"
#include "../ttc-lib/ttc_basic.h"     // Basic set of helper functions, datatypes and definitions
#include "../ttc-lib/ttc_systick.h"   // exact timing and delays
#include "../ttc-lib/ttc_gpio.h"      // general purpose input output pins
#include "../ttc-lib/ttc_task.h"      // register functions as tasks or statemachines
#include "../ttc-lib/ttc_memory.h"    // erase/ fill buffers and test pointers
#include "../ttc-lib/ttc_heap.h"      // dynamic memory allocation
#include "../ttc-lib/ttc_random.h"    // pseudo random number generation
#include "../ttc-lib/ttc_list.h"      // generic single linked lists
#include "../ttc-lib/ttc_light_types.h" // datatypes for using TTC_LIGHT constants

//{ Global variables

// Note: Global and static variables require a unique prefix (here "el_") to avoid
//       conflicting symbol names when combined with other sources!


//}
//{ Private Function Declarations ****************************************

//InsertPrivateFunctionDeclarations  above (DO NOT DELETE THIS LINE!)
//}PrivateFunctionDeclarations
//{ Function definitions *************************************************

/* if you don't like the settings on current board, simply copy these lines into your
 * extensions.local/makefile.70_extra_settings file.
 * You may also use your project specific board file
 * extensions.local/makefile.100_board_*
 * and enable the corresponding line in your activate_project.sh.

    # TTC_LIGHT1 is a low active LED being controlled by gpio b3 with max duty 40/215
    COMPILE_OPTS += -DTTC_LIGHT1=E_ttc_light_architecture_gpio
    COMPILE_OPTS += -DTTC_LIGHT1_GPIO1=e_ttc_gpio_pin_b3
    COMPILE_OPTS += -DTTC_LIGHT1_LOWACTIVE=1
    COMPILE_OPTS += -DTTC_LIGHT1_MAX_DUTY=40

    # TTC_LIGHT2 is a low active LED being controlled by gpio b4 with max duty 20/235
    COMPILE_OPTS += -DTTC_LIGHT2=E_ttc_light_architecture_gpio
    COMPILE_OPTS += -DTTC_LIGHT2_GPIO1=e_ttc_gpio_pin_b4
    COMPILE_OPTS += -DTTC_LIGHT2_LOWACTIVE=1
    COMPILE_OPTS += -DTTC_LIGHT2_MAX_DUTY=20

    # TTC_LIGHT3 is a low active LED being controlled by gpio b5 with max duty 20/235
    COMPILE_OPTS += -DTTC_LIGHT3=E_ttc_light_architecture_gpio
    COMPILE_OPTS += -DTTC_LIGHT3_GPIO1=E_ttc_gpio_pin_b5
    COMPILE_OPTS += -DTTC_LIGHT3_LOWACTIVE=1
    COMPILE_OPTS += -DTTC_LIGHT3_MAX_DUTY=20
*/

/** Calculate LED on-time based on current brightness and maximum duty level
 *
 * @param Brightness   0..99
 * @param MaxLevel     0..99
 * @return             0..MaxLevel
 */
t_u8 calcOnTime( t_u8 Brightness, t_u8 MaxLevel ) {
    const t_u8 MaxBrightness = 99;
    if ( Brightness > MaxBrightness ) { Brightness = MaxBrightness; }
    if ( MaxLevel   > MaxLevel )      { MaxLevel   = MaxLevel; }
    return ( t_u8 ) 0xff & ( ( ( t_u32 ) Brightness ) * MaxLevel / MaxBrightness );
}

/** Prepares data structures for given LED
 *
 * @param LEDs_List    !=NULL: pointer to pointer to start of single linked list of LED configurations
 * @param PortPin      gpio port pin being connected to LED
 * @param IsLowActive  ==TRUE: gpio pin must be pulled low to enable LED; ==FALE: ... pushed high ...
 * @param MaxDuty      0...100 maximum percentage of on-time in every pwm cycle (maximum brightness)
 * @return *LEDs_List  new configuration item has been added to single linked list
 */
void prepareLED( t_my_list* LEDs_List, e_ttc_gpio_pin PortPin, BOOL IsLowActive, t_u8 MaxDuty ) {
    Assert_Writable( LEDs_List, E_ttc_assert_origin_auto ); // must point to RAM!

    // allocate memory from heap
    t_config_led* LED = ttc_heap_alloc_zeroed( sizeof( t_config_led ) );

    LED->Brightness = 0;            // starting value for brightness
    LED->MaxDuty    = MaxDuty;      // maximum duty level for this led
    LED->PortPin    = PortPin;      // gpio pin to use
    LED->LowActive  = IsLowActive;  // ==1: gpio pin must be cleared to activate LED; ==0: be set

    ttc_list_push_back_single_cast( &( LEDs_List->ttc_list ), &( LED->ttc_list_item ) );

    // initialize GPIO pin being connected to LED and switch it off
    ttc_gpio_init( LED->PortPin, E_ttc_gpio_mode_output_push_pull, E_ttc_gpio_speed_max );
    if ( LED->LowActive ) {
        //ttc_gpio_init( LED->PortPin, E_ttc_gpio_mode_output_open_drain_pull_down_resistor, E_ttc_gpio_speed_max );
        ttc_gpio_set( LED->PortPin );
    }
    else {
        //ttc_gpio_init( LED->PortPin, E_ttc_gpio_mode_output_push_pull_pull_down_resistor, E_ttc_gpio_speed_max );
        ttc_gpio_clr( LED->PortPin );
    }
}

//InsertPrivateFunctionDefinitions  above (DO NOT DELETE THIS LINE!)

// Set default values for at least three LEDs
#ifndef TTC_LIGHT1
    #define TTC_LIGHT1 0
#endif
#ifndef TTC_LIGHT1_GPIO1
    #define TTC_LIGHT1_GPIO1 0
#endif
#ifndef TTC_LIGHT1_LOWACTIVE
    #define TTC_LIGHT1_LOWACTIVE 0
#endif
#ifndef TTC_LIGHT1_MAX_DUTY
    #define TTC_LIGHT1_MAX_DUTY 20
#endif

#ifndef TTC_LIGHT2
    #define TTC_LIGHT2 0
#endif
#ifndef TTC_LIGHT2_GPIO1
    #define TTC_LIGHT2_GPIO1 0
#endif
#ifndef TTC_LIGHT2_LOWACTIVE
    #define TTC_LIGHT2_LOWACTIVE 0
#endif
#ifndef TTC_LIGHT2_MAX_DUTY
    #define TTC_LIGHT2_MAX_DUTY 20
#endif

#ifndef TTC_LIGHT3
    #define TTC_LIGHT3 0
#endif
#ifndef TTC_LIGHT3_GPIO1
    #define TTC_LIGHT3_GPIO1 0
#endif
#ifndef TTC_LIGHT3_LOWACTIVE
    #define TTC_LIGHT3_LOWACTIVE 0
#endif
#ifndef TTC_LIGHT3_MAX_DUTY
    #define TTC_LIGHT3_MAX_DUTY 20
#endif

typedef struct {
    t_my_list  LEDs_List;                 // a single linked list of LED configurations
    t_bool     taskLightShow_FirstRunFinished; // set to TRUE when taskLightShow() has been started
} t_example_leds_task_data;

void example_leds_prepare() {
    t_example_leds_task_data* TaskData = ttc_heap_alloc_zeroed( sizeof( *TaskData ) );
    ttc_list_init_cast( &TaskData->LEDs_List, "LEDs_List" );

    // Each prepareLED() call will add one item to TaskData->LEDs_List
    if ( TTC_LIGHT1 != 0 ) {
        prepareLED( &( TaskData->LEDs_List ), TTC_LIGHT1_GPIO1, TTC_LIGHT1_LOWACTIVE, TTC_LIGHT1_MAX_DUTY );
    }
    if ( TTC_LIGHT2 != 0 ) {
        prepareLED( &( TaskData->LEDs_List ), TTC_LIGHT2_GPIO1, TTC_LIGHT2_LOWACTIVE, TTC_LIGHT2_MAX_DUTY );
    }
    if ( TTC_LIGHT3 != 0 ) {
        prepareLED( &( TaskData->LEDs_List ), TTC_LIGHT3_GPIO1, TTC_LIGHT3_LOWACTIVE, TTC_LIGHT3_MAX_DUTY );
    }

    ttc_task_create( taskLightShow,    // function to start as thread
                     "taskLightShow",  // thread name (just for debugging)
                     128,               // stack size
                     TaskData,          // passed as argument to taskLightShow()
                     1,                 // task priority (higher values mean more process time)
                     NULL               // can return a handle to created task
                   );
}
void taskLightShow( void* TaskArgument ) {
    t_example_leds_task_data* TaskData = ( t_example_leds_task_data* ) TaskArgument;
    /* LEDs_List stores a single linked, NULL terminated list of LED configurations
     *
     *                     +-------------------+    +-------------------+    +-------------------+
     * LEDs_List.wFirst --> | t_config_led LED1 |    | t_config_led LED2 |    | t_config_led LED3 |
     *                     +-------------------+    +-------------------+    +-------------------+
     *                     | Next ----------------> | Next ----------------> | Next ---------------> NULL
     *                     | Brightness        |    | Brightness        |    | Brightness        |
     *                     | MaxDuty           |    | MaxDuty           |    | MaxDuty           |
     *                     | PortPin           |    | PortPin           |    | PortPin           |
     *                     | LowActive         |    | LowActive         |    | LowActive         |
     *                     +-------------------+    +-------------------+    +-------------------+
     */
    Assert_Writable( &( TaskData->LEDs_List ), E_ttc_assert_origin_auto );
    if ( ! TaskData->taskLightShow_FirstRunFinished ) { // first called for this configuration
        t_config_led* CurrentLED = ( t_config_led* ) ttc_list_peek_front_cast( &( TaskData->LEDs_List ) );

        while ( CurrentLED != NULL ) { // Start one taskLedPWM() for each LED in given list

            ttc_task_create( taskLedPWM,          // function to start as thread
                             "taskLedPWM",        // thread name (just for debugging)
                             128,                 // stack size
                             ( void* ) CurrentLED,// passed as argument to taskLedPWM()
                             1,                   // task priority (higher values mean more process time)
                             NULL                 // can return a handle to created task
                           );

            // proceed to next LED configuration in single linked list
            CurrentLED = CurrentLED->ttc_list_item.Next;
        }
    }
    t_base AmountItems = ttc_list_get_amount_cast( &TaskData->LEDs_List );
    t_u16 PreviousIndex = 0;
    t_u16 NextIndex     = 0;

    do  {
        t_config_led* CurrentLED = NULL;
        t_u8 Brightness = 0;

        if ( 1 ) { // simultaneously ramp all LEDs 0%->100%->0%
            while ( Brightness < 100 ) { // ramp LED brightness up to 100%
                CurrentLED = ( t_config_led* ) ttc_list_peek_front_cast( &( TaskData->LEDs_List ) );
                while ( CurrentLED != NULL ) { // configure all LEDs in given list to same brightness
                    CurrentLED->Brightness = Brightness;
                    CurrentLED->OnTime = -1; // force recalculation
                    CurrentLED = CurrentLED->ttc_list_item.Next;
                }
                Brightness++;
                ttc_task_msleep( 10 );
            }
            ttc_task_msleep( 3000 );
            while ( Brightness > 0 ) { // ramp LED brightness down to 0%
                CurrentLED = ( t_config_led* ) ttc_list_peek_front_cast( &( TaskData->LEDs_List ) );
                while ( CurrentLED != NULL ) {  // configure all LEDs in given list to same brightness
                    CurrentLED->Brightness = Brightness;
                    CurrentLED->OnTime = -1; // force recalculation
                    CurrentLED = CurrentLED->ttc_list_item.Next;
                }
                Brightness--;
                ttc_task_msleep( 10 );
            }
        }
        if ( 0 && ( AmountItems > 1 ) ) { // ramp LED  brightness 0%->100%->0% at random index
            for ( t_u8 Count = 20; Count > 0; Count-- ) {
                PreviousIndex = NextIndex;
                do { // choose next index randomly until it is different from previous one
                    NextIndex = ttc_random_generate16() % AmountItems;
                }
                while ( ( AmountItems > 1 ) && ( NextIndex == PreviousIndex ) );
                CurrentLED = ( t_config_led* ) ttc_list_peek_index_cast( &( TaskData->LEDs_List ), NextIndex );

                const t_u8 Increment = 4;
                while ( CurrentLED->Brightness <= ( 100 - Increment ) ) { // ramp LED brightness up to 100% == MaxDuty
                    CurrentLED->Brightness += Increment;
                    CurrentLED->OnTime = -1; // force recalculation
                    ttc_task_msleep( 10 );
                }
                while ( CurrentLED->Brightness >= Increment ) {   // ramp LED brightness down to 0%
                    CurrentLED->Brightness -= Increment;
                    CurrentLED->OnTime = -1; // force recalculation
                    ttc_task_msleep( 10 );
                }
            }
        }
        if ( 0 && ( AmountItems > 1 ) ) { // cool multidimensional lightning effect

            const t_u8 Amount_Velocity     = 11; // must be odd number (velocity range will be -Amount_Velocity/2 ... 0 ... +Amount_Velocity/2)
            const t_u8 Amount_Acceleration = 11; // must be odd number (acceleration range will be -Amount_Acceleration/2 ... 0 ... +Amount_Acceleration/2)

            if ( !TaskData->taskLightShow_FirstRunFinished ) { // first run: initialize data set
                CurrentLED = ( t_config_led* ) ttc_list_peek_front_cast( &( TaskData->LEDs_List ) );
                while ( CurrentLED ) { // calculate initial brightness, velocity and accelleration for all LEDs in list
                    CurrentLED->Brightness   = ttc_random_generate16() % 50 + 25;
                    CurrentLED->Acceleration = ( ttc_random_generate16() % Amount_Acceleration ) - Amount_Acceleration - 1;
                    CurrentLED->Velocity     = ( ttc_random_generate16() % Amount_Velocity ) - Amount_Velocity - 1;
                    CurrentLED->OnTime = -1; // force recalculation
                    CurrentLED = CurrentLED->ttc_list_item.Next;
                }
            }

            for ( t_u32 Count = 1000; Count > 0; Count-- ) {
                CurrentLED = ( t_config_led* ) ttc_list_peek_front_cast( &( TaskData->LEDs_List ) );
                while ( CurrentLED ) { // calculate new brightness for all leds based on individual velocity and acceleration

                    // add velocity to brightness and reflect if boundary is reached
                    t_s16 NewBrightness = CurrentLED->Brightness;
                    NewBrightness += CurrentLED->Velocity;
                    if ( ( NewBrightness < 0 ) || ( NewBrightness > 100 ) ) { // reflect brightness at upper or lower boundary
                        NewBrightness = -NewBrightness;
                        CurrentLED->Velocity = -CurrentLED->Velocity;
                    }

                    CurrentLED->Brightness = ( t_u8 ) NewBrightness;
                    CurrentLED->OnTime = -1; // force recalculation

                    // add acceleration to velocity and limit it to given range
                    t_s16 NewVelocity = CurrentLED->Velocity;
                    NewVelocity += CurrentLED->Acceleration;
                    if ( NewVelocity < ( -Amount_Velocity / 2 + 1 ) ) { NewVelocity = ( -Amount_Velocity / 2 + 1 ); }
                    if ( NewVelocity > ( Amount_Velocity  / 2 - 1 ) ) { NewVelocity = ( Amount_Velocity / 2 - 1 ); }

                    // add random value to acceleration and limit it to given range
                    t_s16 NewAcceleration = ( ttc_random_generate16() % Amount_Acceleration ) - ( Amount_Acceleration / 2 ) - 1;
                    NewAcceleration += CurrentLED->Acceleration;
                    if ( NewAcceleration < ( -Amount_Acceleration / 2 + 1 ) ) { NewAcceleration = ( -Amount_Acceleration / 2 + 1 ); }
                    if ( NewAcceleration > ( Amount_Acceleration / 2 - 1 ) ) { NewAcceleration = ( Amount_Acceleration / 2 - 1 ); }

                    CurrentLED = CurrentLED->ttc_list_item.Next;
                }
                ttc_task_msleep( 100 );
            }
        }
        TaskData->taskLightShow_FirstRunFinished = TRUE;
    }
    while ( ttc_task_is_scheduler_available() );
}
void taskLedPWM( void* TaskArgument ) {
    Assert_Writable( TaskArgument, E_ttc_assert_origin_auto );                // always check pointer arguments with Assert()!
    t_config_led* LED = ( t_config_led* ) TaskArgument; // we're expecting a t_config_led* pointer
    Assert_Writable( LED, E_ttc_assert_origin_auto );
    do {

        if ( LED->OnTime == -1 )  { // settings have changed: recalculate
            LED->OnTime = calcOnTime( LED->Brightness, LED->MaxDuty );
        }
        t_u32 OnTime  = LED->OnTime;
        t_u32 OffTime = 99 - OnTime;

        // Multiply on- and off-time to see a difference between brightness levels
        OnTime  *= MinimumDelay;
        OffTime *= MinimumDelay;

        if ( OnTime  > 0 ) { // switch  on LED
            if ( LED->LowActive ) { ttc_gpio_clr( LED->PortPin ); }
            else                  { ttc_gpio_set( LED->PortPin ); }

            // give cpu to other tasks while waiting
            ttc_task_usleep( OnTime );
        }
        if ( OffTime > 0 ) { // switch off LED
            if ( LED->LowActive ) { ttc_gpio_set( LED->PortPin ); }
            else                  { ttc_gpio_clr( LED->PortPin ); }

            // give cpu to other tasks while waiting
            ttc_task_usleep( OffTime );
        }
    }
    while ( TRUE );
}

//InsertFunctionDefinitions above (DO NOT DELETE THIS LINE!)
//}FunctionDefinitions
