/** example_ttc_light.c **********************************************{
 *
 *                              The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Example for universal led controller
 *
 *  Basic flow of booting examples:
 *  (1) ttc_extensions.c:ttc_extensions_start() calls example_ttc_light_prepare() to
 *      - initialize all required devices
 *      - initializes its global and local static variables
 *      - create all required task or statemachine instances
 *      - return immediately
 *
 *  Multitasking is available:
 *  (2) Scheduler is started
 *  (3) Tasks being created by example_ttc_light_prepare() get executed.
 *      Each task uses its private stack as given to ttc_task_create() before.
 *
 *  Multitasking is NOT available:
 *  (2) Minimalistic main loop is started
 *  (3) Main loop calls all registered statemachine functions in an endless loop.
 *      All statemachine functions use the same global stack that is maximum of all
 *      stack sizes given to ttc_task_create() before.
 *
 *  Created from template example_ttc_device.c revision 25 at 20200104 17:59:03 UTC
 *
 *  Authors: <AUTHOR>
 *
}*/

//{ Includes *************************************************************

// Include all header files being required to compile this file.
#include "example_ttc_light.h"
#include "../compile_options.h"       // dynamically created file defining all "COMPILE_OPTS += -D..." constants from makefiles
#include "../ttc-lib/ttc_light.h"  // high-level driver of device to use
#include "../ttc-lib/ttc_basic.h"     // basic datatypes and definitions
#include "../ttc-lib/ttc_heap.h"      // dynamic memory allocations and safe arrays
#include "../ttc-lib/ttc_task.h"      // allows to use application in single- and multitasking setup
#include "../ttc-lib/ttc_systick.h"   // exact delays and timeouts

//}Includes
//{ Global Variables *****************************************************

// Note: Adding a unique prefix to all global and static variable makes it
//       easier to get an overview of memory usage via _/calculateMemoryUsage.pl

#define AMOUNT_BLINKING_DEFINITIONS 2
#define PHASE_SHIFT                 20 // shift phase of second blinking
const t_ttc_light_blinking example_ttc_light_Blinking[AMOUNT_BLINKING_DEFINITIONS] = { // multiple blinking definitions
    /* Each blinking definition can be applied to one or more ttc_light devices,
     * The blinking will take effect when device is switched on via ttc_light_on().
     * These blinking definitions are constant which means, they do not occupy any RAM.
     * You may also create a blinking definition in RAM and modify it at runtime.
     */
    {
        // Blinking[0]
        .Multiplier = 5, // millisecond multiplier
        .times = {
            .named.T0_Off      = 0,  // * Multiplier =   0ms
            .named.T1_RampUp   = 0,  // * Multiplier =   0ms
            .named.T2_Full     = 20, // * Multiplier = 100ms
            .named.T3_RampDown = 0,  // * Multiplier =   0ms
            .named.T4_Off      = 20  // * Multiplier = 100ms
        }
    },

    {
        // Blinking[1]
        .Multiplier = 5, // millisecond multiplier
        .times = {
            .named.T0_Off      = PHASE_SHIFT,        // shift phase
            .named.T1_RampUp   = 0,  // * Multiplier =   0ms
            .named.T2_Full     = 20, // * Multiplier = 100ms
            .named.T3_RampDown = 0,  // * Multiplier =   0ms
            .named.T4_Off      = 20 - PHASE_SHIFT// subtracte phase shift to keep frequency constant
        }
    },
};

//}Global Variables
//{ Private Function Declarations ****************************************
//
// Private functions should only be called from this source code, though it is possible.
// Declaring a function as private removes it from list of public functions and makes it
// easier for other developers to understand your application interface.
//

/** Task running this example
 *
 * H2: Description of Example
 * Task function _example_ttc_light_task() implements a statemachine.
 * This statemachine operates all Lights being configured by current board makefile over some time.
 *
 * H3: Different Modes are applied to all Lights
 * #) Manual Blinking
 *    The lights are switched on and off manually via ttc_light_on() and ttc_light_off() calls.
 * #) Automatic Blinking
 *    A blinking definition is assigned to each light.
 *    The blinking itself is automatically controlled by ttc_light.
 *
 * H2: What is a Task?
 * A task is a function being registered via ttc_task_create().
 *
 * H3: Tasks can operate in one of two modes:
 * - Single Tasking Mode
 *   If no multitasking scheduler has been activated, then all registered task functions will be called endlessly from a super loop.
 *   Each task function has to implement a statemachine that will check for new state and return fast to give cpu to other tasks.
 *
 * - Multi Tasking Mode
 *   If a multitasking scheduler (e.g. FreeRTOS) has been activated, then a central scheduler will actively switch among all registered tasks.
 *   Each task runs a statemachine in a endless loop.
 *   It is advised that each task calls ttc_task_yield() to give cpu to other tasks while waiting for events.
 *
 * The ToolChain allows to write task functions that support both modes.
 * The advantage is that such multi mode task functions can be reused more easily.
 *
 * H3: Basic structure of multi mode task functions
 *   <CODE>
 *   task_foo(void *Argument) {
 *     t_my_data* MyData = (t_my_data*) Argument;
 *     if (MyData->FirstRunCompleted == 0) {
 *       // first run: initialize task
 *       MyData->FirstRunCompleted = 1;
 *     }
 *
 *     do {
 *        // run statemachine
 *     } while (ttc_task_can_yield());
 *   }
 * </CODE>
 *
 * Note: This task is created automatically by example_ttc_light_prepare() at system startup.
 *
 * @param Argument  passed argument #4 from ttc_task_create() call.
 */
void _example_ttc_light_task( void* Argument );

//}Private Function Declarations
//{ Function Definitions *************************************************

void example_ttc_light_prepare() {                // prepare this example for being started
    /** Note: The _prepare() function is called in single task mode before the scheduler is started.
      *       It has to return to allow all extensions to prepare.
      *       After all extensions finished their preparation, the created tasks are run.
      *       If no multitasking scheduler has been activated, all created tasks are run one after
      *       another in an endless loop. See ttc_task_start_scheduler() for details.
      *       This scheme allows to enable multiple statemachine functions in a flexible way.
      */

    if ( 1 ) { // create one instance of your example (copy and adjust for multiple instances)

        // allocate data of instance as local static (allows compiler to detect if it fits into RAM)
        static t_example_ttc_light_data led_data;

        // reset data
        ttc_memory_set( &led_data, 0, sizeof( led_data ) );

        // initialize all configured LEDs to their default configuration
        ttc_light_init_all();

        // Register a function to be run after all system has finished booting.
        // ttc_task_create() works in single- and multitasking environments.
        ttc_task_create(
            _example_ttc_light_task,   // function to start as thread
            "tExample_ttc_light",  // thread name (just for debugging)
            256,                 // stack size (adjust to amount of local variables of _example_ttc_light_task() AND ALL ITS CALLED FUNTIONS!)
            &led_data,           // passed as argument to _example_ttc_light_task()
            1,                   // task priority (higher values mean more process time)
            NULL                 // can return a handle to created task
        );
    }
}
//}Function Definitions

//{ Private Function Definitions *************************************************

void _example_ttc_light_task( void* Argument ) {  // example task will be run when multitasking scheduler (e.g. FreeRTOS) has been started
    Assert_LIGHT_Writable( Argument, E_ttc_assert_origin_auto ); // always check pointer arguments with Assert() before dereferencing them!
    t_example_ttc_light_data* Data = ( t_example_ttc_light_data* ) Argument; // we're expecting this pointer type

    do {
        switch ( Data->State ) {
            case E_example_ttc_light_state_Reset: {               // first run: initialize task data

                // add code to be executed on first run here!

                // set next state (active on next function call)
                Data->State = E_example_ttc_light_state_Blinking_Manual1;
                //X Data->State = E_example_ttc_light_state_Blinking_Automatic1;
                break;
            }
            case E_example_ttc_light_state_Blinking_Manual1: {    // prepare manual blinking of all leds
                Data->Counter = 20; // amount of times to toggle lights in next state
                ttc_systick_delay_init( & ( Data->Delay ), 0 ); // zero delay will expire immediately
                Data->State = E_example_ttc_light_state_Blinking_Manual2;
                break;
            }
            case E_example_ttc_light_state_Blinking_Manual2: {    // blink all leds at 1= Hertz
                if ( ttc_systick_delay_expired( & ( Data->Delay ) ) ) { // delay expired: toggle leds
                    ttc_systick_delay_init( & ( Data->Delay ), 50000 ); // 50000 us == 1/20 second

                    if ( Data->Counter > 0 ) {
                        Data->Counter--;
                        for ( t_u8 IndexLed = 1; IndexLed <= ttc_light_get_max_index(); IndexLed++ ) { // toggle all leds
                            ttc_light_toggle( IndexLed );
                        }
                    }
                    else { // counter ran out: jump to next state
                        Data->State = E_example_ttc_light_state_Blinking_ManualEnd;
                    }
                }
                break;
            }
            case E_example_ttc_light_state_Blinking_ManualEnd:
                Data->State = E_example_ttc_light_state_Blinking_Automatic1; break; // set state for next function call

            case E_example_ttc_light_state_Blinking_Automatic1: { // initialize automatic blinking
                // configure all leds for automatic blinking
                for ( t_u8 IndexLed = 1; IndexLed <= ttc_light_get_max_index(); IndexLed++ ) {
                    // configure automatic blinking as one from example_ttc_light_Blinking[] array
                    t_u8 IndexBlinking = IndexLed % AMOUNT_BLINKING_DEFINITIONS; // IndexBlinking = 0..AMOUNT_BLINKING_DEFINITIONS-1
                    for ( t_u8 IndexChannel = ttc_light_get_configuration( IndexLed )->AmountChannels; IndexChannel > 0; IndexChannel-- ) {
                        ttc_light_blink( IndexLed, IndexChannel, &( example_ttc_light_Blinking[IndexBlinking] ) );
                    }

                    // turn led on or we won't see it blinking
                    ttc_light_on( IndexLed );
                }

                ttc_systick_delay_init( & ( Data->Delay ), 5000000 ); // set delay time for next state: 5000000 us == five seconds
                Data->State = E_example_ttc_light_state_Blinking_Automatic2; //  set next state
                break;
            }
            case E_example_ttc_light_state_Blinking_Automatic2: { // let automatic blinking run for some time
                if ( ttc_systick_delay_expired( & ( Data->Delay ) ) ) { // time passed: go to next state
                    Data->State = E_example_ttc_light_state_Blinking_Automatic3;
                }
                break;
            }
            case E_example_ttc_light_state_Blinking_Automatic3: { // remove automatic blinking from all leds and switch them off
                for ( t_u8 IndexLed = 1; IndexLed <= ttc_light_get_max_index(); IndexLed++ ) {
                    for ( t_u8 IndexChannel = ttc_light_get_configuration( IndexLed )->AmountChannels; IndexChannel > 0; IndexChannel-- ) {
                        ttc_light_blink( IndexLed, IndexChannel, NULL );

                        // set brightness on all channels to zero (unsupported channels will be ignored)
                        ttc_light_brightness4( IndexLed, 0, 0, 0, 0 );
                    }
                }

                ttc_systick_delay_init( & ( Data->Delay ), 2000000 ); // set delay time for next state: 2000000 us == two seconds
                Data->State = E_example_ttc_light_state_Blinking_Automatic4; //  set next state
                break;
            }
            case E_example_ttc_light_state_Blinking_Automatic4: { // wait while all leds are off
                if ( ttc_systick_delay_expired( & ( Data->Delay ) ) ) { // time passed: go to next state
                    Data->State = E_example_ttc_light_state_Blinking_AutomaticEnd;
                }
                break;
            }
            case E_example_ttc_light_state_Blinking_AutomaticEnd:
                Data->State = E_example_ttc_light_state_Blinking_Manual1; break; // set state for next function call

            default: ttc_assert_halt_origin( E_ttc_assert_origin_auto ); break; // invalid or unsupported state found!
        }
    }
    while ( ttc_task_can_yield() );   // multitasking: give cpu to other tasks; singletasking: exit while after one iteration
}

//}Private Function Definitions
