/** example_ttc_usart.c **********************************************{
 *
 *                             The ToolChain
 *
 * What is an USART?
 * On microcontrollers, serial ports are often called
 * Universal Synchronous Asynchronous Receiver Transmitter (USART).
 *
 * This example shows a minimalistic way to configure and use a serial port.
 * It will periodically send out a text.
 * Any text being received is send back introduced by "relaying: ".
 *
 * This example supports single- and multitasking.
 *
}*/

#include "../compile_options.h"       // dynamically created collection of all "COMPILE_OPTS +=-D..." definitions from all makefiles
#include "example_ttc_usart.h"
#include "../ttc-lib/ttc_string.h"
#include "../ttc-lib/ttc_usart.h"
#include "../ttc-lib/ttc_heap.h"
#include "../ttc-lib/ttc_systick.h"  // exact delays and timeouts
//{ Global variables


const char* TextHello = "Hello, talk to me!";

//}
//{ Private Function Declarations ****************************************

//InsertPrivateFunctionDeclarations  above (DO NOT DELETE THIS LINE!)
//}PrivateFunctionDeclarations
//{ Function definitions *************************************************

/** Automatically called from _ttc_usart_task_receive() to process incoming data.
 *
 * Note: Read description of ttc_usart_register_receive() for more details!
 *
 * @param Argument     value of third argument given to ttc_usart_register_receive()
 * @param ConfigUsart  pointer to configuration of ttc_usart device which has received data
 * @param Data         memory buffer storing one or more bytes of received data
 */
t_ttc_usart_memory_block* _example_ttc_usart_receiveData( void* Argument, t_ttc_usart_config* ConfigUsart, t_ttc_usart_memory_block* Data );

//InsertPrivateFunctionDefinitions  above (DO NOT DELETE THIS LINE!)
//{ Private Structures ****************************************************

/* Structure of private data being passed to task_ExampleUSART()
 *
 * To support both, single- and multitasking flow, all private task data
 * must be stored in this structure.
 */
#define EU_BUFFER_SIZE 35
typedef struct {
    t_u8                IndexUSART;             // logical index of ttc_usart device to use (1..ttc_usart_get_max_logicalindex())
    t_ttc_systick_delay Delay;                  // required for precise delays
    t_u8                Counter;                // a simple counter that will increase on every transmission of TextHello
    t_bool              TaskStarted;            // must be initialized as zero!
    t_u8                Buffer[EU_BUFFER_SIZE]; // used for ttc_string_snprintf() to compile temporary strings

    // add more data fields here!
} t_task_example_usart_Data;

//}Structures/ Enums

/** configure single usart unit
 *
 * @param IndexUSART  1..ttc_usart_get_max_logicalindex()  logical index of usart unit to initialize
 */
void example_ttc_usart_configure( t_u8 IndexUSART ) {

    t_ttc_usart_config* Config = ttc_usart_get_configuration( IndexUSART );
    Assert_Writable( Config, E_ttc_assert_origin_auto );

    // change some settings (Press F2 on any field to see definition!)
    Config->Init.Flags.IrqOnRxNE = 1;  // enable IRQ on Rx-buffer not empty
    Config->Init.Char_EndOfLine  = 13; // packets end at carriage return
    Config->Init.Flags.Receive   = 1;

    Config->Init.WordLength   = 8;     // raw data length (parity is added if enabled)
    switch ( Config->LogicalIndex ) {  // select bitrate for individual serial ports
        case 1:  Config->Init.BitRate =  460800; Config->Init.Flags.StayInISR = 1; break; // prioritize this USART for max speed
        case 2:  Config->Init.BitRate =  230400; break;
        case 3:  Config->Init.BitRate =  115200; break;
        default: Config->Init.BitRate =   19200; break;
    }

    Config->Init.HalfStopBits = 2;

    /* Configure Parity */
    Config->Init.Flags.ParityEven     = 0;
    Config->Init.Flags.ParityOdd      = 0;
    Config->Init.Flags.SendBreaks     = 0;

    Config->Init.Flags.DelayedTransmits = 1; // ==1: all transmits are queued and sent in interrupt service routine parallel to this task
    //                                          ==0: all transmits are blocking until data is send (can loose rx-data while waiting for transmit!)

    volatile  e_ttc_usart_errorcode Error = ttc_usart_init( IndexUSART ); // note: first bytes might get lost until ttc_usart_register_receive() is called
    Assert( !Error, E_ttc_assert_origin_auto ); // error occured during init
}
void example_ttc_usart_prepare() {
    /** Note: This *_prepare() function is called in single task mode before the scheduler is started.
      *       It has to return to startup multitasking and run the tasks being created here.
      *       This function may run indefinitely if multitasking is not required.
      */

    Assert( ttc_usart_get_max_logicalindex() > 0, E_ttc_assert_origin_auto ); // no USARTs defined by board makefile!

    // initialize all configured usart devices TTC_USART1, ... and instantiate one task for each
    for ( t_u8 IndexUsart = 1; IndexUsart <= ttc_usart_get_max_logicalindex(); IndexUsart++ ) {

        // allocate data to pass to task_ExampleUSART
        t_task_example_usart_Data* TaskData = ttc_heap_alloc_zeroed( sizeof( t_task_example_usart_Data ) );
        TaskData->IndexUSART = IndexUsart; // press F2 on IndexUSART to see its definition!

        // have usart configured
        example_ttc_usart_configure( TaskData->IndexUSART );

        // register function to call for incoming data
        ttc_usart_register_receive( TaskData->IndexUSART, _example_ttc_usart_receiveData, TaskData );

        // instantiate task to handle all usart traffic (will be run after all *_prepare() function ran
        ttc_task_create( task_ExampleUSART, // function to start as task (see below!)
                         "tUSART",          // thread name (just for debugging)
                         512,               // stack size for new task (difficult to determine)
                         TaskData,     // passed as argument to task_ExampleUSART()
                         1,                 // task priority (higher values mean more process time)
                         NULL               // can return a handle to created task
                       );
    }
}
void task_ExampleUSART( void* Argument ) {
    Assert_Writable( Argument, E_ttc_assert_origin_auto ); // will stop if caller provides invalid pointer
    t_task_example_usart_Data* volatile TaskArguments = ( t_task_example_usart_Data* ) Argument; // casting pointer to known type

    if ( !TaskArguments->TaskStarted ) { // first call: initialize task
        // initialize a timer delay for precise timing
        ttc_systick_delay_init( &( TaskArguments->Delay ), 1 ); // set delay to 0s

#ifdef TTC_LIGHT1 // use first LED as status if defined by board makefile
        // initialize GPIO pin connected to first board LED as output
        ttc_gpio_init( TTC_LIGHT1_GPIO1, E_ttc_gpio_mode_output_push_pull, E_ttc_gpio_speed_max );
#endif

        TaskArguments->TaskStarted = TRUE;
    }

    do {
        if ( ttc_systick_delay_expired( &( TaskArguments->Delay ) ) ) { // delay passed: send hello world
            ttc_systick_delay_init( &( TaskArguments->Delay ), 500000 ); // reset delay to half a second = 500ms = 500000us

#ifdef TTC_LIGHT1 // use first LED as status if defined by board makefile
            ttc_gpio_put( TTC_LIGHT1_GPIO1, TaskArguments->Counter & 1 );
#endif
            if ( ! ttc_usart_check_transmission_ongoing( TaskArguments->IndexUSART ) ) { // Avoid to flood QueueTx of slow usarts to reduce response time to incoming data!
                // compiling formatted string into temporary buffer before sending it
                ttc_string_snprintf( TaskArguments->Buffer, EU_BUFFER_SIZE, "example_usart(%i): %i=0x%x ", TaskArguments->IndexUSART, TaskArguments->Counter, TaskArguments->Counter );
                TaskArguments->Counter++;
                ttc_usart_send_string( TaskArguments->IndexUSART, TaskArguments->Buffer, EU_BUFFER_SIZE );

                // transmit some constant strings (zero copy implementation)
                ttc_usart_send_string_const( TaskArguments->IndexUSART, TextHello, -1 );
                ttc_usart_send_string_const( TaskArguments->IndexUSART, " ...\n", -1 );
            }
        }
    }
    while ( ttc_task_can_yield() ); // if scheduler available, give cpu to other processes
}
t_ttc_usart_memory_block* _example_ttc_usart_receiveData( void* Argument, t_ttc_usart_config* ConfigUsart, t_ttc_usart_memory_block* Data ) {
    Assert_Writable( Argument, E_ttc_assert_origin_auto ); // will stop if caller provides invalid pointer
    Assert_Readable( Data, E_ttc_assert_origin_auto );     // will stop if caller provides invalid pointer
    t_task_example_usart_Data* TaskArguments = ( t_task_example_usart_Data* ) Argument; // casting pointer to known type
    ( void ) ConfigUsart; // not used

    if ( 1 ) { // example implementation: just echoing back what we received

        // find start and length of string inside Data->Buffer[] without leading and trailing whitespaces
        t_u8* StrippedBuffer = Data->Buffer;
        t_base StrippedSize  = Data->AmountBytes;
        ttc_string_strip( &StrippedBuffer, &StrippedSize );

        if ( StrippedSize > 0 ) {
            ttc_usart_send_string_const( TaskArguments->IndexUSART, "you said: '", -1 );
            ttc_usart_send_raw( TaskArguments->IndexUSART, StrippedBuffer, StrippedSize ); // not constant: data will be copied
            ttc_usart_send_string_const( TaskArguments->IndexUSART, "'\n", -1 );
        }
    }

    TTC_RETURN( Data ); // returning memory block immediately for reuse as we do not need it any longer
}

//InsertFunctionDefinitions above (DO NOT DELETE THIS LINE!)
//}FunctionDefinitions
